/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XInjectorMonCtl.cpp : Implementation of the CXInjectorMonCtrl ActiveX Control class.

#include "stdafx.h"
#include "XInjectorMon.h"
#include "XInjectorMonCtl.h"
#include "XInjectorMonPpg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CXInjectorMonCtrl, COleControl)


/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CXInjectorMonCtrl, COleControl)
//{{AFX_MSG_MAP(CXInjectorMonCtrl)
ON_WM_CREATE()
ON_WM_SIZE()
ON_WM_DESTROY()
//}}AFX_MSG_MAP
ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
ON_MESSAGE(WMU_DYN_GRAPH,OnShowDynGraphs)
ON_MESSAGE(WMU_UPDATE_TITLE, OnUpdateTitle)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CXInjectorMonCtrl, COleControl)
//{{AFX_DISPATCH_MAP(CXInjectorMonCtrl)
DISP_PROPERTY_EX(CXInjectorMonCtrl, "RefreshDelay", GetNotSupported, SetRefreshDelay, VT_I4)
DISP_FUNCTION(CXInjectorMonCtrl, "XInitialise", XInitialise, VT_EMPTY, VTS_BSTR)
//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CXInjectorMonCtrl, COleControl)
//{{AFX_EVENT_MAP(CXInjectorMonCtrl)
EVENT_CUSTOM("ShowDynGraph", FireShowDynGraph, VTS_BSTR  VTS_BSTR)
EVENT_CUSTOM("ControlClosing", FireControlClosing, VTS_NONE)
EVENT_CUSTOM("UpdateTitle", FireUpdateTitle, VTS_BSTR)
//}}AFX_EVENT_MAP
EVENT_CUSTOM("ShowDynGraph", FireShowDynGraph, VTS_BSTR)
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CXInjectorMonCtrl, 1)
PROPPAGEID(CXInjectorMonPropPage::guid)
END_PROPPAGEIDS(CXInjectorMonCtrl)


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CXInjectorMonCtrl, "XINJECTORMON.XInjectorMonCtrl.1",
                       0xc210c4d, 0xbc9, 0x11d4, 0xbd, 0x8c, 0, 0xc0, 0x4f, 0x89, 0xe3, 0x92)


/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CXInjectorMonCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DXInjectorMon =
    {
        0xc210c4b, 0xbc9, 0x11d4, { 0xbd, 0x8c, 0, 0xc0, 0x4f, 0x89, 0xe3, 0x92 }
    };
const IID BASED_CODE IID_DXInjectorMonEvents =
    {
        0xc210c4c, 0xbc9, 0x11d4, { 0xbd, 0x8c, 0, 0xc0, 0x4f, 0x89, 0xe3, 0x92 }
    };


/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwXInjectorMonOleMisc =
    OLEMISC_ACTIVATEWHENVISIBLE |
    OLEMISC_SETCLIENTSITEFIRST |
    OLEMISC_INSIDEOUT |
    OLEMISC_CANTLINKINSIDE |
    OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CXInjectorMonCtrl, IDS_XINJECTORMON, _dwXInjectorMonOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CXInjectorMonCtrl::CXInjectorMonCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CXInjectorMonCtrl

BOOL CXInjectorMonCtrl::CXInjectorMonCtrlFactory::UpdateRegistry(BOOL bRegister)
{
    // TODO: Verify that your control follows apartment-model threading rules.
    // Refer to MFC TechNote 64 for more information.
    // If your control does not conform to the apartment-model rules, then
    // you must modify the code below, changing the 6th parameter from
    // afxRegApartmentThreading to 0.

    if (bRegister)
        return AfxOleRegisterControlClass(
                   AfxGetInstanceHandle(),
                   m_clsid,
                   m_lpszProgID,
                   IDS_XINJECTORMON,
                   IDB_XINJECTORMON,
                   afxRegApartmentThreading,
                   _dwXInjectorMonOleMisc,
                   _tlid,
                   _wVerMajor,
                   _wVerMinor);
    else
        return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CXInjectorMonCtrl::CXInjectorMonCtrl - Constructor

CXInjectorMonCtrl::CXInjectorMonCtrl()
{
    InitializeIIDs(&IID_DXInjectorMon, &IID_DXInjectorMonEvents);

    // TODO: Initialize your control's instance data here.
}


/////////////////////////////////////////////////////////////////////////////
// CXInjectorMonCtrl::~CXInjectorMonCtrl - Destructor


CXInjectorMonCtrl::~CXInjectorMonCtrl()
{
    // TODO: Cleanup your control's instance data here.
}


/////////////////////////////////////////////////////////////////////////////
// CXInjectorMonCtrl::OnDraw - Drawing function


void CXInjectorMonCtrl::OnDraw(
    CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
    // TODO: Replace the following code with your own drawing code.
    if(!m_InjectorMonDlg.GetSafeHwnd())
    {
        pdc->FillRect(rcBounds, CBrush::FromHandle((HBRUSH)GetStockObject(WHITE_BRUSH)));
        pdc->TextOut(0,0,"XInjectorMon");
    }
}


/////////////////////////////////////////////////////////////////////////////
// CXInjectorMonCtrl::DoPropExchange - Persistence support

void CXInjectorMonCtrl::DoPropExchange(CPropExchange* pPX)
{
    ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
    COleControl::DoPropExchange(pPX);

    // TODO: Call PX_ functions for each persistent custom property.

}


/////////////////////////////////////////////////////////////////////////////
// CXInjectorMonCtrl::OnResetState - Reset control to default state


void CXInjectorMonCtrl::OnResetState()
{
    COleControl::OnResetState();  // Resets defaults found in DoPropExchange

    // TODO: Reset any other control state here.
}


/////////////////////////////////////////////////////////////////////////////
// CXInjectorMonCtrl message handlers


int CXInjectorMonCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (COleControl::OnCreate(lpCreateStruct) == -1)
        return -1;

    SetRedraw(FALSE);

    m_InjectorMonDlg.Create(IDD_INJECTOR_MON_DLG,this);

    return 0;
}

void CXInjectorMonCtrl::OnSize(UINT nType, int cx, int cy)
{
    COleControl::OnSize(nType, cx, cy);

    if(m_InjectorMonDlg.GetSafeHwnd())
        m_InjectorMonDlg.MoveWindow(0,0,cx,cy);
}

void CXInjectorMonCtrl::XInitialise(LPCTSTR szTestRegName)
{
    m_InjectorMonDlg.StartMonitor(szTestRegName);
}

void CXInjectorMonCtrl::SetRefreshDelay(long nNewValue)
{
    m_InjectorMonDlg.SetRefreshDelay(nNewValue);
}

BOOL CXInjectorMonCtrl::PreTranslateMessage(MSG* pMsg)
{
    if(m_InjectorMonDlg.PreTranslateMessage(pMsg))
        return true;

    return COleControl::PreTranslateMessage(pMsg);
}

LRESULT CXInjectorMonCtrl::OnShowDynGraphs(WPARAM wParam,LPARAM lParam)
{
    FireShowDynGraph((LPCTSTR)lParam,(LPCTSTR)wParam);
    return 0;
}

void CXInjectorMonCtrl::OnDestroy()
{
    COleControl::OnDestroy();

    FireControlClosing();
}

LRESULT CXInjectorMonCtrl::OnUpdateTitle(WPARAM wParam,LPARAM lParam)
{
    FireUpdateTitle((LPCTSTR)wParam);
    return 0;
}
