/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// EnvironmentPage.cpp : implementation file
//

#include "stdafx.h"
#include "xotaeditor.h"
#include "EnvironmentPage.h"
#include "..\..\HTMLHelp\XOTAEditor\Environment.inl"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEnvironmentPage property page

IMPLEMENT_DYNCREATE(CEnvironmentPage, CPropertyPage)

CEnvironmentPage::CEnvironmentPage() : CGenericListPage(CEnvironmentPage::IDD)
{
    //{{AFX_DATA_INIT(CEnvironmentPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
    m_strTokenName = "Environment";

    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

}

CEnvironmentPage::~CEnvironmentPage()
{}


void CEnvironmentPage::DoDataExchange(CDataExchange* pDX)
{
    CGenericListPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CEnvironmentPage)
    DDX_Control(pDX, IDC_GRID, m_ctlGrid);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CEnvironmentPage, CGenericListPage)
//{{AFX_MSG_MAP(CEnvironmentPage)
ON_WM_SIZE()
ON_WM_HELPINFO()
ON_BN_CLICKED(IDCANCEL,OnCancel)
ON_BN_CLICKED(IDOK, OnOK)
//}}AFX_MSG_MAP
ON_COMMAND(ID__INSERTROW,OnInsertRow)
ON_COMMAND(ID__DELETEROW,OnDeleteRow)
ON_MESSAGE(WMU_CAN_START_EDITING,OnCanStartEditting)
ON_MESSAGE(WMU_VALIDATE_CELL,OnValidateCell)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEnvironmentPage message handlers

void CEnvironmentPage::OnSize(UINT nType, int cx, int cy)
{
    CGenericListPage::OnSize(nType, cx, cy);

    if(m_ctlGrid.GetSafeHwnd()!=NULL)
    {
        m_ctlGrid.MoveWindow(5,5,cx-10,cy-10);

        int nWidth = m_ctlGrid.GetColumnWidth(enEnvironmentVariablesColumn);
        if(nWidth < cx)
        {
            m_ctlGrid.SetColumnWidth(enEnvironmentVariablesColumn,cx-15);
        }
    }
}

BOOL CEnvironmentPage::OnInitDialog()
{
    CGenericListPage::OnInitDialog();

    const int nNumColumns = 1;
    char szHeaders[nNumColumns][40] = {"Environment variables"};

    m_ctlGrid.SetColumnCount(nNumColumns);
    m_ctlGrid.SetRowCount(1);
    m_ctlGrid.SetFixedRowCount();
    m_ctlGrid.SetEditable();

    for(int nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        m_ctlGrid.SetItemText(0,nColumn, szHeaders[nColumn]);
    }

    RestoreSettings();

    return false;//TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CEnvironmentPage::OnInsertRow()
{
    int nRowCount = m_ctlGrid.GetRowCount();
    m_ctlGrid.SetRowCount(nRowCount+1);
}

int CEnvironmentPage::GetNumDataEntries()
{
    return m_ctlGrid.GetRowCount();
}

//
//  GetDataEntry
//  ============
//
//  Get values from a row in the list box and store put them
//  into a list organised as {Key1,value1,Key2,value2,...}
//
bool CEnvironmentPage::GetDataEntry(int nRow,CStringArray& arstrParams)
{

    //Check that the row required exists
    int nNumRows = m_ctlGrid.GetRowCount();
    if(nRow < 0 || nRow+1 >= nNumRows)
    {
        ASSERT(FALSE);
        return false;
    }

    bool bEmptyRow = true;

    //Go through all the columns...
    CString strValue,strToken;

    int nNumColumns = m_ctlGrid.GetColumnCount();
    for(int nColumn=0;nColumn<nNumColumns;nColumn++)
    {
        switch(nColumn)
        {
        case enEnvironmentVariablesColumn:
            strToken="env";
            break;
        default:
            ASSERT(FALSE);
            break;
        }
        arstrParams.Add(strToken);

        //Get the value for this row and column
        strValue = m_ctlGrid.GetItemText(nRow+1,nColumn);
        arstrParams.Add(strValue);

        //found at least one non empty entry in this row
        if(!strValue.IsEmpty())
            bEmptyRow = false;
    }

    return !bEmptyRow;
}

//Add a data entry to this page
void CEnvironmentPage::AddDataEntry(const CStringArray& arstrParams)
{

    int nNewRow = m_ctlGrid.GetRowCount()+1;
    m_ctlGrid.SetRowCount(nNewRow);

    int nColumn=-1;
    CString strToken,strValue;
    int nNumEntries=arstrParams.GetSize()/2;
    for(int nEntry=0;nEntry<nNumEntries;nEntry++)
    {
        strToken = arstrParams[nEntry*2];

        if(strToken=="env")
            nColumn = enEnvironmentVariablesColumn;
        else
        {
            ASSERT(FALSE);
        }

        if(nColumn != -1)
        {
            strValue = arstrParams[nEntry*2+1];
            m_ctlGrid.SetItemText(nNewRow,nColumn,strValue);
        }
    }
}

void CEnvironmentPage::OnDeleteRow()
{
    CCellRange range = m_ctlGrid.GetSelectedCellRange();

    //remove all the selected rows from the grid
    for(int nRow=range.GetMinRow();nRow<=range.GetMaxRow();nRow++)
    {
        m_ctlGrid.DeleteRow(nRow);
    }

}

BOOL CEnvironmentPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

LRESULT CEnvironmentPage::OnCanStartEditting(WPARAM nRow,LPARAM nCol)
{
    return 1;
}

LRESULT CEnvironmentPage::OnValidateCell(WPARAM nRow,LPARAM nCol)
{
    return 1;
}
