/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CXOutputTree.cpp : implementation file
//

#include "stdafx.h"
#include "XReposTree.h"
#include "CXOutputTree.h"
//#include "PropertiesDlg.h"
#include "StringEx.h"
#include "Server.h"
//
//#include "Architecture.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CXOutputTree dialog


CXOutputTree::CXOutputTree(CWnd* pParent /*=NULL*/)
        : CDialog(CXOutputTree::IDD, pParent)
{
    //{{AFX_DATA_INIT(CXOutputTree)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_strProject="";
    m_pServer = NULL;
}


void CXOutputTree::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CXOutputTree)
    DDX_Control(pDX, IDC_XTREE_OUTPUT, m_XTreeOutput);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CXOutputTree, CDialog)
//{{AFX_MSG_MAP(CXOutputTree)
ON_WM_SIZE()
ON_WM_PAINT()
//}}AFX_MSG_MAP
ON_WM_ERASEBKGND()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXOutputTree message handlers

BEGIN_EVENTSINK_MAP(CXOutputTree, CDialog)
//{{AFX_EVENTSINK_MAP(CXOutputTree)
ON_EVENT(CXOutputTree, IDC_XTREE_OUTPUT, 1 /* LBDblClick */, OnLBDblClickXtreeOutput, VTS_I4 VTS_I2 VTS_BSTR)
ON_EVENT(CXOutputTree, IDC_XTREE_OUTPUT, 2 /* XTreeExpanding */, OnXTreeExpandingXtreeOutput, VTS_I4 VTS_I2 VTS_BSTR)
ON_EVENT(CXOutputTree, IDC_XTREE_OUTPUT, 3 /* XTreeContextClick */, OnXTreeContextClickXtreeOutput, VTS_BSTR VTS_I2 VTS_BSTR)
ON_EVENT(CXOutputTree, IDC_XTREE_OUTPUT, 4 /* XTreeItemProperties */, OnXTreeItemProperties, VTS_BSTR VTS_I2)
ON_EVENT(CXOutputTree, IDC_XTREE_OUTPUT, 5 /* XShowContext */, OnXShowContext, VTS_I2 VTS_BSTR)
//}}AFX_EVENTSINK_MAP
END_EVENTSINK_MAP()

void CXOutputTree::OnLBDblClickXtreeOutput(long hTreeItem, short nNodeType, LPCTSTR szPath)
{
    CString path = szPath;

    if (nNodeType != TESTRUN)
        return;

    int len = path.GetLength();
    int pos = path.Find('|');
    m_TestName = path.Left(pos);
    CString RunTime = path.Right(len - pos-1);
    m_RunName = FindTestRun(m_TestName, RunTime);

    // Check to see if the Audit file is there before invoking the results
    // window.
    if(m_pServer->DoesFileExist(CYRServer::enTestsFolder,m_TestName + "\\" + m_RunName,"TestLog.txt"))
    {
        ::SendNotifyMessage(GetParent()->m_hWnd,WMU_XVIEW_RESULTS,0,0);
    }
    else
    {
        MessageBox("No Audit information found.", "OpenSTA");
    }
}

void CXOutputTree::OnXTreeExpandingXtreeOutput(long hTreeItem, short nNodeType, LPCTSTR szPath)
{
    BeginWaitCursor();

    HTREEITEM hti = (HTREEITEM)hTreeItem;

    switch (nNodeType)
    {
    case TEST:
        {
            AddTestRuns(hti);
            break;
        }
    }

    EndWaitCursor();
}

void CXOutputTree::OnXTreeContextClickXtreeOutput(LPCTSTR strPath, short nType, LPCTSTR strEntry)
{
    CString path = strPath;
    CString menuEntry = strEntry;

    switch (nType)
    {
    case TESTRUN:
        {
            int len = path.GetLength();
            int pos = path.Find('|');
            m_TestName = path.Left(pos);
            CString RunTime = path.Right(len - pos-1);

            m_RunName = FindTestRun(m_TestName, RunTime);

            if (menuEntry == "&Audit")
            {
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_XVIEW_RESULTS,0,0);
            }
            if (menuEntry == "&Report")
            {
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_XVIEW_RESULTS,1,0);
            }
            if (menuEntry == "&History")
            {
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_XVIEW_RESULTS,2,0);
            }
            if (menuEntry == "&Delete")
            {
                LPCTSTR testName = m_TestName;
                LPCTSTR runName = m_RunName;
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DELETE_TESTRUN,(int)testName,(int)runName);
            }
            else if (menuEntry == "&Statistics")
            {
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_OPEN_TESTRUN,0,0);
            }
            else if (menuEntry == "H&ttp Statistics")
            {
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_HTTP_STATS,0,0);
            }
            break;
        }
    case TEST:
        {
            m_TestName = path;

            if (menuEntry == "&Delete All Results")
            {
                LPCTSTR testName = m_TestName;
                ::SendNotifyMessage(GetParent()->m_hWnd,WMU_DEL_RESULTS,0,(int)testName);
            }
            break;
        }

    }
}

BOOL CXOutputTree::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Define node types and context menu entries for each type
    m_XTreeOutput.DefineNodeType(PROJECT,   "", FALSE ,FALSE);
    m_XTreeOutput.DefineNodeType(TESTRUN_FOLDER,    "", FALSE ,FALSE);
    m_XTreeOutput.DefineNodeType(TESTRUN, "&Audit,1|&Report,1|&History,1|&Statistics,1|H&ttp Statistics,1|&Delete,1", FALSE ,FALSE);
    m_XTreeOutput.DefineNodeType(TEST,  "&Delete All Results,1", FALSE ,FALSE);


    // Add bitmaps to the image list
    CBitmap* bmp = new CBitmap;
    bmp->LoadBitmap(IDB_FOLDER);
    m_XTreeOutput.AddImageToList((long)bmp->m_hObject);
    delete bmp;
    bmp = new CBitmap;
    bmp->LoadBitmap(IDB_TEST);
    m_XTreeOutput.AddImageToList((long)bmp->m_hObject);
    delete bmp;
    bmp = new CBitmap;
    bmp->LoadBitmap(IDB_TESTRUN);
    m_XTreeOutput.AddImageToList((long)bmp->m_hObject);
    delete bmp;

    if (m_strProject=="")
    {
        m_XTreeOutput.AddItem(-1,PROJECT,"No Project Selected",0,FALSE);
    }
    else
    {
        AddTests();
    }
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CXOutputTree::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    if( m_XTreeOutput.GetSafeHwnd() != NULL )
    {
        m_XTreeOutput.SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
}

void CXOutputTree::AddTests()
{
    if (m_strProject == "" || m_strProject == "<No Project Selected>")
    {
        return;
    }

    //    ...Recover a list of the tests in this project...
    //    ...and add them to the tree beneath the project.
    //

    CStringArray arstrTests;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,"",arstrTests);

    for(int nNdx=0;nNdx<arstrTests.GetSize();nNdx++)
    {
        // First we see if there are any childre
        BOOL bHasChildren = HasTestRuns(arstrTests[nNdx]);

        // And add in the test item to the tree
        m_XTreeOutput.AddItem(-1, TEST, arstrTests[nNdx], 1,bHasChildren);
    }
}

void CXOutputTree::AddTestRuns(HTREEITEM parent)
{
    //
    // Make sure the parent allows children
    //
    m_XTreeOutput.SetChildren((long)parent, TRUE);

    //
    //    Get path to the item selected in the tree and from there to the test name which is the last
    //    element in the path
    //
    CString path = m_XTreeOutput.GetItemPath((long)parent);

    int nPos = path.ReverseFind('|');
    int nLen = path.GetLength();
    CString testName = path.Right(nLen-nPos-1);

    //    ...and get a list of the test runs.
    CStringArray arstrTestRuns;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,testName,arstrTestRuns);

    //    ...and add them to the tree.
    for(int nNdx=0;nNdx<arstrTestRuns.GetSize();nNdx++)
    {
        CString strTestRun = arstrTestRuns[nNdx];
        PrintRunName(strTestRun);
        arstrTestRuns[nNdx] = strTestRun;
    }

    if(arstrTestRuns.GetSize())
    {
        // Sort runs into date order
        qsort( (void *)arstrTestRuns.GetData(), (size_t)arstrTestRuns.GetSize(), sizeof( CString* ), CompareTestRuns);

        //Add runs to tree
        for(int nRun=0;nRun<arstrTestRuns.GetSize();nRun++)
        {
            m_XTreeOutput.AddItem((long)parent, TESTRUN, (LPCTSTR)arstrTestRuns[nRun], 2, FALSE);
        }

        arstrTestRuns.RemoveAll();
    }
    else
    {
        m_XTreeOutput.SetChildren((long)parent, FALSE);
    }
}

int CXOutputTree::CompareTestRuns( const void *arg1, const void *arg2 )
{
    try
    {
        TRACE("Comparing: %s , %s\n",*(char**)arg1,*(char**)arg2);
        //Test run is organised as "Day-Month-Year Hour-Min-Sec"
        //so we need to compare by year first,then month,then day etc
        CString strRun1(*(char**)arg1);
        CString strRun2(*(char**)arg2);

        const char* pszDay1 = strtok(strRun1.GetBuffer(0),"-");
        const char* pszMonth1 = strtok(NULL,"-");
        const char* pszYear1 = strtok(NULL," ");
        const char* pszTime1 = strtok(NULL," ");

        const char* pszDay2 = strtok(strRun2.GetBuffer(0),"-");
        const char* pszMonth2 = strtok(NULL,"-");
        const char* pszYear2 = strtok(NULL," ");
        const char* pszTime2 = strtok(NULL," ");

        //Compare years
        int nRes = _stricmp(pszYear2,pszYear1);
        if(nRes!=0)
            return nRes;

        //Compare months
        nRes = _stricmp(pszMonth2,pszMonth1);
        if(nRes!=0)
            return nRes;

        //compare days
        nRes = _stricmp(pszDay2,pszDay1);
        if(nRes!=0)
            return nRes;

        //finally compare the times
        return _stricmp(pszTime2,pszTime1);
    }
    catch(...)
    {
        return -1;
    }
}

void CXOutputTree::Activate()
{
    m_XTreeOutput.DeleteAllItems();

    if (m_strProject=="")
    {
        m_XTreeOutput.AddItem(-1,PROJECT,"No Project Selected",0,FALSE);
    }
    else
    {
        AddTests();
    }
}

void CXOutputTree::SetActiveProject(CString strProject)
{
    m_strProject=strProject;
}


void CXOutputTree::PrintRunName(CString &strTestRun)
{
    int pos = strTestRun.ReverseFind('.');

    strTestRun = strTestRun.Left(pos);
}

CString CXOutputTree::FindTestRun(CString TestName, CString RunTime)
{
    CString TestRun="";

    //get list of test runs for this test...
    CStringArray arstrTestRuns;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,TestName,arstrTestRuns);


    BOOL bFound = FALSE;

    for(int nNdx=0;nNdx<arstrTestRuns.GetSize();nNdx++)
    {
        CString strTestRun = arstrTestRuns[nNdx];
        PrintRunName(strTestRun);
        if (strTestRun == RunTime)
        {
            TestRun = arstrTestRuns[nNdx];
            break;
        }
    }

    return TestRun;
}

//
// HasTestRun
// ==========
//
// Stripped down version of FindTestRun - simply to determine if a test has any
// test runs yet.
BOOL CXOutputTree::HasTestRuns(CString TestName)
{
    // Obtain the current Project
    CStringArray arstrRuns;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,TestName,arstrRuns);
    return (arstrRuns.GetSize() ? TRUE : FALSE);
}

void CXOutputTree::GetOpenTestRun(CString &TestName, CString &RunName)
{
    TestName = m_TestName;
    RunName = m_RunName;
}

void CXOutputTree::SetServer(CYRServer* pServer)
{
    ASSERT(pServer != NULL);

    m_pServer = pServer;
}

void CXOutputTree::OnXTreeItemProperties(LPCTSTR strPath, short nType)
{
    /*  CPropertiesDlg  dlg;
        dlg.SetServerInfo(m_pServer, m_strProject);
        dlg.SetItemInfo(strPath, nType);
        dlg.DoModal();  
    */
}

void CXOutputTree::RefreshTree()
{
    // We need to add in any newly created test - so save checking all
    // tests against the current tree items we simply get rid of everything and
    // add them again.
    m_XTreeOutput.DeleteAllItems();
    AddTests();

    m_XTreeOutput.RefreshTree();
}

void CXOutputTree::EnableMenus(BOOL bEnabaled)
{
    m_XTreeOutput.EnableMenus(bEnabaled);
}

void CXOutputTree::OnPaint()
{
    CPaintDC dc(this); // device context for painting

    // TODO: Add your message handler code here

    // Do not call CDialog::OnPaint() for painting messages
}


BOOL CXOutputTree::OnEraseBkgnd(CDC *pDC)
{
    return true;
}
void CXOutputTree::OnXShowContext(short nNodeType, LPCTSTR strPath)
{
    CString path = strPath;

    if (nNodeType != TESTRUN)
        return;

    // Format the tree item name into a test run folder name
    int len = path.GetLength();
    int pos = path.Find('|');
    m_TestName = path.Left(pos);
    CString RunTime = path.Right(len - pos-1);
    m_RunName = FindTestRun(m_TestName, RunTime);

    // Connect to the TestRunFolder
    CStringArray arstrFiles;
    m_pServer->GetFileList(CYRServer::enTestsFolder,m_TestName + "\\" + m_RunName,arstrFiles);

    BOOL bTestLogExists=false;
    BOOL bTestRepExists=false;
    BOOL bTestHisExists=false;
    BOOL bHttpStatsExist=false;

    for(int nNdx=0;nNdx<arstrFiles.GetSize();nNdx++)
    {
        CString &strFile = arstrFiles[nNdx];
        if(strFile == "TestLog.txt")
            bTestLogExists = true;
        else
            if(strFile == "TestRep.txt")
                bTestRepExists = true;
            else
                if(strFile == "TestHis.txt")
                    bTestHisExists = true;
                else
                    if(strFile.Left(4) == "TEW_")
                        bHttpStatsExist = true;
    }

    if(!bTestLogExists)
    {
        m_XTreeOutput.EnableMenuItem(nNodeType, "&Audit", FALSE);
        m_XTreeOutput.EnableMenuItem(nNodeType, "&Statistics", FALSE);
    }
    else
    {
        m_XTreeOutput.EnableMenuItem(nNodeType, "&Audit", TRUE);
        m_XTreeOutput.EnableMenuItem(nNodeType, "&Statistics", TRUE);
    }

    if (!bTestRepExists)
    {
        m_XTreeOutput.EnableMenuItem(nNodeType, "&Report", FALSE);
    }
    else
    {
        m_XTreeOutput.EnableMenuItem(nNodeType, "&Report", TRUE);
    }

    if (!bTestHisExists)
    {
        m_XTreeOutput.EnableMenuItem(nNodeType, "&History", FALSE);
    }
    else
    {
        m_XTreeOutput.EnableMenuItem(nNodeType, "&History", TRUE);
    }

    if (!bHttpStatsExist)
    {
        m_XTreeOutput.EnableMenuItem(nNodeType, "H&ttp Statistics", FALSE);
    }
    else
    {
        m_XTreeOutput.EnableMenuItem(nNodeType, "H&ttp Statistics", TRUE);
    }


}
