/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECResultsDlg.cpp : implementation file
//

#include "stdafx.h"
#include <afx.h>


#include "XResults.h"
#include "CECResultsDlg.h"
#include "..\..\HTMLHelp\XResults\Results.inl"
#include "htmlhelp.h"
#include "resource.h"
#include "server.h"
#include "ctime.hpp"
#include "CyrWrapLib\CECTestOutput.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECResultsDlg dialog


CECResultsDlg::CECResultsDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECResultsDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECResultsDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pServer = NULL;

    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";

}


void CECResultsDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECResultsDlg)
    DDX_Control(pDX, IDC_COMBO_TYPE, m_ComboType);
    DDX_Control(pDX, IDC_TYPE_LABEL, m_LabelType);
    //}}AFX_DATA_MAP
    DDX_Control(pDX, IDC_GRID, m_wndGrid);
}


BEGIN_MESSAGE_MAP(CECResultsDlg, CDialog)
ON_WM_CONTEXTMENU()
//{{AFX_MSG_MAP(CECResultsDlg)
ON_WM_SIZE()
ON_CBN_SELCHANGE(IDC_COMBO_TYPE, OnSelchangeComboType)
ON_WM_HELPINFO()
ON_COMMAND(IDC_FULL_HELP, OnFullHelp)
ON_COMMAND(ID_POPUP_COPY, OnPopupCopy)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECResultsDlg message handlers
//

// @mfunc   OnInitDialog
//
//    Initailize the dialog controls. Need to sub-class the Custom control with
//    the grid window and activate the grid with some intitial settings.
//
BOOL CECResultsDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Set the initial row and column counts
    m_wndGrid.SetRowCount(100);
    m_wndGrid.SetColumnCount(4);
    m_wndGrid.SetFixedRowCount();
    m_wndGrid.EnableToolTips();
    //  m_wndGrid.SetEditable(FALSE);

    return TRUE;    // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

// @mfunc   OnSize
//
//    Handle the sizing of the controls within the dialog.
//

void CECResultsDlg::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    // Position Labels and selectionn combos
    if (m_LabelType.m_hWnd !=NULL)
    {
        m_LabelType.SetWindowPos(NULL, 5,5, 0,0,SWP_NOSIZE);
        m_ComboType.SetWindowPos(NULL, 40,3, cx/3-60,15,SWP_NOZORDER);
    }

    // Position the grid window
    if (m_wndGrid.m_hWnd != NULL)
    {
        m_wndGrid.SetWindowPos(&wndTop, 0,40,cx,cy-40,SWP_NOZORDER);

        m_wndGrid.ExpandLastColumn();
    }

    Invalidate();
}

// @mfunc   SetServer
//
//    Store a pointer to the OpenSTARep .
//
void CECResultsDlg::SetServer(CYRServer *pServer)
{
    m_pServer = pServer;
    ASSERT(m_pServer !=NULL);
}

// @mfunc   Init
//
//    This function handles the initial parameters given when the control is started.
//    From the given project name, the combo boxes are filled with the result sets
//    available. A result set is selected by the initial parameters and is read into
//    the grid window.
//
void CECResultsDlg::Init(CString szProject, CString szTest, CString szRun, int nType)
{

    m_nType = nType;
    m_szTest = szTest;
    m_szRun = szRun;

    //
    // Check for an empty project string. The control cannot be initialised for an invalid
    // project name.
    //
    if (szProject.IsEmpty())
        return;

    //
    // Initialize the result type combo and load in the tests available to select from.
    //
    BOOL bInit = FALSE;
    AddTypes();

    //
    //    Type may be set to either RESULT_AUDIT, RESULT_REPORT or RESULT_HISTORY. These affect the
    //    way in which we add columns.
    //
    switch (nType)
    {
    case RESULT_AUDIT:
        {
            AddAuditColumns ();
            break;
        }

    case RESULT_REPORT:
        {
            AddReportColumns ();
            break;
        }
    case RESULT_HISTORY:
        {
            AddHistoryColumns ();
            break;
        }
    }

    //
    // Select the initial Items in the combo boxes from the given initialization parameters.
    //
    m_ComboType.SetCurSel(nType);

    //
    // Display the results for the initial selection.
    //
    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_UPDATE_TITLE,0,0);
    ReadResults(nType, m_szTest, m_szRun);
}

// @mfunc   AddAuditColumns
//
//   Add the column headers for Audit messages
//
void CECResultsDlg::AddAuditColumns()
{
    CString HeaderString = "Time\tTest\tScript\tMessage\t";
    m_wndGrid.PasteTextFromBuffer(0,0,HeaderString);

}

// @mfunc   AddReportColumns
//
//   Add the column headers for Report messages
//
void CECResultsDlg::AddReportColumns()
{
    CString HeaderString = "Time\tTest\tScript\tMessage\t";
    m_wndGrid.PasteTextFromBuffer(0,0,HeaderString);

}

// @mfunc   AddHistoryColumns
//
//   Add the column headers for History messages
//
void CECResultsDlg::AddHistoryColumns()
{
    CString HeaderString = "Time\tTest\tScript\tMessage\t";
    m_wndGrid.PasteTextFromBuffer(0,0,HeaderString);

}

// @mfunc   AddTypes
//
//   Add the result types to the combo box.
//
void CECResultsDlg::AddTypes()
{
    m_ComboType.AddString("Audit");
    m_ComboType.AddString("Report");
    m_ComboType.AddString("History");
    m_ComboType.SetCurSel(0);
}

void CECResultsDlg::ReadResults(int nType, CString szTest, CString szRun)
{
    //
    // Call the appropriate function based on result type to read the correct result file
    // into the grid.
    //
    int nRows;
    switch (nType)
    {
    case RESULT_AUDIT:
        {
            nRows = ReadAudit( szTest, szRun);
            break;
        }

    case RESULT_REPORT:
        {
            nRows = ReadReport( szTest, szRun);
            break;
        }
    case RESULT_HISTORY:
        {
            nRows = ReadHistory( szTest, szRun);
            break;
        }
    }

    if (nRows == 0)
    {
        m_wndGrid.SetRowCount(2);
        CString none = "(none)\t(none)\t(none)\t(none)\t";
        m_wndGrid.PasteTextFromBuffer(1,0,none);
    }
    else
    {
        m_wndGrid.SetRowCount(nRows+1);
    }

    m_wndGrid.SetColumnTextClr(0, RGB(0,130,60));
    m_wndGrid.SetColumnTextClr(1, RGB(200,50,30));
    m_wndGrid.SetColumnTextClr(2, RGB(255,110,40));
    m_wndGrid.SetColumnTextClr(3, RGB(0,0,128));

    m_wndGrid.ExpandColumnsToFit();
}

int CECResultsDlg::ReadAudit(CString test, CString run)
{
    if(!AddTestRunSuffix(test, run))
        return 0;

    //
    //    Iterate through the audit messages and add them to the grid
    //
    int Row = 0;
    try
    {
        CString strAuditPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
        strAuditPath += test + "\\" + run + "\\TestLog.txt";

        CStdioFile AuditFile(strAuditPath,CFile::modeRead);

        CTTime Time;

        CString strLine;
        while(AuditFile.ReadString(strLine))
        {
            CECTestLog TestLog(strLine,false);

            //
            //    Time of audit message
            //
            Time = TestLog.GetCreatedTime();
            CString strTime = (CString)Time.Format("%d/%m/%Y %H:%M:%S").c_str();

            //
            //    ...and the message
            //
            CString strText = TestLog.GetText().c_str();

            //
            //    ...parse '|' separated message
            //
            CString exename, pid, threadname, scriptname, objindex, msg;
            GetNextField('|', strText, exename);
            GetNextField('|', strText, pid);
            GetNextField('|', strText, threadname);
            GetNextField('|', strText, scriptname);
            GetNextField('|', strText, objindex);
            GetNextField('|', strText, msg);

            CString data;
            data.Format("%s\t%s\t%s\t%s\t",strTime, test, scriptname, msg);
            m_wndGrid.PasteTextFromBuffer(Row+1,0,data);

            //
            //    ...and move onto the next entry
            //
            Row++;
        }
    }
    catch(...)
    {}


    return Row;
}

int CECResultsDlg::ReadHistory(CString strTestName, CString /*run*/)
{
    CStringArray arstrRuns;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,strTestName,arstrRuns);

    int nNumRows = 0;
    CString strRunName;

    for(int nRun=0;nRun<arstrRuns.GetSize();nRun++)
    {
        nNumRows += AddHistoryFromRun(strTestName,arstrRuns[nRun],nNumRows);
    }
    return nNumRows;
}

int CECResultsDlg::AddHistoryFromRun(CString strTest,CString strRun,int nFirstRow)
{
    //
    //    Iterate through the audit messages and add them to the grid
    //
    int Row = nFirstRow;
    CTTime Time;

    try
    {
        CString strHistoryPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
        strHistoryPath += strTest + "\\" + strRun + "\\TestHis.txt";

        CStdioFile HistoryFile(strHistoryPath,CFile::modeRead);

        CString strHistory;
        while(HistoryFile.ReadString(strHistory))
        {
            CECTestHistory TestHistory(strHistory,false);
            //
            //    Time of report message
            //
            Time = TestHistory.GetCreatedTime();
            CString strTime = (CString)Time.Format("%d/%m/%Y %H:%M:%S").c_str();

            //
            //    ...and the message
            //
            CString strText = TestHistory.GetText().c_str();

            //
            //    ...parse '|' separated message
            //
            CString exename, pid, threadname, scriptname, objindex, msg;
            GetNextField('|', strText, exename);
            GetNextField('|', strText, pid);
            GetNextField('|', strText, threadname);
            GetNextField('|', strText, scriptname);
            GetNextField('|', strText, objindex);
            GetNextField('|', strText, msg);

            CString data;
            data.Format("%s\t%s\t%s\t%s\t",strTime,strTest, scriptname, msg);
            m_wndGrid.PasteTextFromBuffer(Row+1,0,data);

            //
            //    ...and move onto the next entry
            //
            Row++;
        }
    }
    catch(...)
    {
    }


    return Row-nFirstRow;
}

int CECResultsDlg::ReadReport(CString test, CString run)
{
    if(!AddTestRunSuffix(test, run))
        return 0;

    //
    //    Iterate through the audit messages and add them to the grid
    //
    int Row = 0;
    CTTime Time;

    try
    {
        CString strRepPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
        strRepPath += test + "\\" + run + "\\TestRep.txt";

        CStdioFile RepFile(strRepPath,CFile::modeRead);

        CString strReport;
        while(RepFile.ReadString(strReport))
        {
            CECTestReport TestReport(strReport,false);

            //
            //    Time of report message
            //
            Time = TestReport.GetCreatedTime();
            CString strTime = (CString)Time.Format("%d/%m/%Y %H:%M:%S").c_str();

            //
            //    ...and the message
            //
            CString strText = TestReport.GetText().c_str();

            //
            //    ...parse '|' separated message
            //
            CString exename, pid, threadname, scriptname, objindex, msg;
            GetNextField('|', strText, exename);
            GetNextField('|', strText, pid);
            GetNextField('|', strText, threadname);
            GetNextField('|', strText, scriptname);
            GetNextField('|', strText, objindex);
            GetNextField('|', strText, msg);

            CString data;
            data.Format("%s\t%s\t%s\t%s\t",strTime, test, scriptname, msg);
            m_wndGrid.PasteTextFromBuffer(Row+1,0,data);

            //
            //    ...and move onto the next entry
            //
            Row++;
        }
    }
    catch(...)
    {
    }


    return Row;
}

int CECResultsDlg::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr == "")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep, 0);

    if (nPos == -1)
    {
        field = inStr;
        inStr = "";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr = inStr.Right(len - nPos - 1);
        return 0;
    }
}

BOOL CECResultsDlg::AddTestRunSuffix(CString testName, CString &testRunName)
{
    CStringArray arstrTestRuns;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,testName,arstrTestRuns);

    CString szRun = testRunName;
    int nLen = szRun.GetLength();

    for(int nNdx=0;nNdx<arstrTestRuns.GetSize();nNdx++)
    {
        if (arstrTestRuns[nNdx].Left(nLen) == testRunName)
        {
            testRunName = arstrTestRuns[nNdx];
            return true;
        }
    }
    return false;
}

void CECResultsDlg::OnSelchangeComboType()
{
    BeginWaitCursor();

    m_nType = m_ComboType.GetCurSel();

    ::SendNotifyMessage(GetParent()->m_hWnd,WMU_UPDATE_TITLE,0,0);

    AddTestRunSuffix(m_szTest, m_szRun);
    ReadResults(m_nType, m_szTest, m_szRun);

    EndWaitCursor();
    Invalidate();
}

BOOL CECResultsDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);
    Invalidate();
    return true;
}


void CECResultsDlg::OnContextMenu(CWnd*, CPoint point)
{
    if (point.x == -1 && point.y == -1)
    {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);

        point = rect.TopLeft();
        point.Offset(5, 5);
    }

    CMenu menu;
    VERIFY(menu.LoadMenu(IDR_HELP_MENU));

    CMenu* pPopup = menu.GetSubMenu(0);
    ASSERT(pPopup != NULL);

    pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,
                           this);
}

void CECResultsDlg::OnFullHelp()
{
    HtmlHelp(m_hWnd, m_szHtmlHelpFile, HH_DISPLAY_TOPIC, (DWORD)"xresults.htm");
}

void CECResultsDlg::OnPopupCopy()
{
    m_wndGrid.OnEditCopy();
}
