/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// @doc     Document
// @module  CYRTreeCtrl | Customized Tree control class which gives added support
//          for associating a type to a set of items.
//
////////////////////////////////////////////////////////////////////////////////


#if !defined(AFX_CYRTREECTRL_H__1C10FA38_B798_11D2_BCF0_00608CB45114__INCLUDED_)
#define AFX_CYRTREECTRL_H__1C10FA38_B798_11D2_BCF0_00608CB45114__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// CYRTreeCtrl.h : header file
//

//
// @struct TreeItem | Type information about a tree item
// @field int | Type | Defined type of the item
// @field CString | Path | Tree path to the item
// @field int | ImageIndex | Index number of the icon in the image list
struct TreeItem
{
    int Type;
    CString Path;
    int ImageIndex;
}
typedef TreeItem;

//
// @struct MenuEntry | Context menu information for a given item type
// @field int | Type | Defined type of the item
// @field CString | Entries | Context menu items
// @field BOOL | bHasProperties | Has Context menu properies
// @field BOOL | bDraggable | Is draggable
struct MenuEntry
{
    int     Type;
    CString Entries;
    BOOL    bHasProperties;
    BOOL    bDraggable;
}
typedef MenuEntry;


// User Events
#define WMU_DBLCLICK_ITEM       (WM_USER+3000)
#define WMU_CONTEXT_MENU_CLICK  (WM_USER+3001)
#define WMU_EXPANDING           (WM_USER+3002)
#define WMU_ITEM_PROPERTIES     (WM_USER+3003)
#define WMU_SHOW_CONTEXT        (WM_USER+3004)
#define WMU_ITEM_DELETE         (WM_USER+3005)
#define WMU_SHOW_HELP           (WM_USER+3006)

#define ID_CONTEXT_USER         50000

/////////////////////////////////////////////////////////////////////////////
// CYRTreeCtrl window

// @class Customized Tree control class
// @base public | CTreeCtrl
class CYRTreeCtrl : public CTreeCtrl
{
    // Construction
    // @access Public Members
public:
    // @cmember Constructor
    CYRTreeCtrl();


    // Drag/Drop members
    bool        m_bDragging;
    CImageList* m_pDragImage;
    HTREEITEM   m_hDragItem;

    // OLE members
    // @cmember OLE Drop Target
    COleDropTarget m_DropTarget;

    // @cmember Performs the copy from dragged item to drop item
    void    CopyItemProperties      (HTREEITEM hNewItem, HTREEITEM hDraggedItem);

    // @cmember Checks if the item can be dropped on
    CWnd*   IsItemCanBeDroppedOn    (HTREEITEM hSource ,CPoint ScreenPoint);

    // @cmember Specifies which item types can be dragged
    BOOL    ItemCanBeDragged        (HTREEITEM hItem);
    // Attributes
public:

    // Operations
public:
    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CYRTreeCtrl)
    //}}AFX_VIRTUAL

    // Implementation
    // @access Public Members
public:
    void EnableMenus(BOOL bEnabled);
    // @cmember Does the item type have a property context-menu item
    BOOL DoesTypeHaveProperties(int type);

    // @cmember Previously selected item
    HTREEITEM m_pOldSel;

    // @cmember Find which item has been clicked on to be expanded
    HTREEITEM GetItemToExpand();

    // @cmember Item to be expanded
    HTREEITEM m_ItemToExpand;

    // @cmember Find the currently selected item
    void GetSelectedTreeItem(CString & Path, int & Type);

    // @cmember Define an item type
    void DefineType(int strType, CString strMenuItems, BOOL bProperties ,BOOL bDraggable);

    // @cmember Add an item to the tree control
    HTREEITEM AddItem(HTREEITEM hti, int NodeType, CString strLabel, int imageIndex,BOOL bHasChildren);

    // @cmember Add a bitmap to the image list
    int AddImageToList(CBitmap* pBmp);

    // @cmember Get the defined type of the current item
    int GetItemType(HTREEITEM hti);

    // @cmember Get the tree path to the current item
    CString GetItemPath(HTREEITEM hti);

    // @cmember Enable/Disable a spcified item on a context menu
    void EnableMenuItem(long nNodeType, LPCTSTR szLabel, BOOL bEnabled);

    // @cmember Show the context menu
    void ShowPopupMenu( CPoint& point ,HTREEITEM hTreeItem);

    // @cmember Store an array of items in the tree
    CArray <TreeItem,TreeItem> m_TreeItems;

    // @cmember Store of Context menu entries for each item type
    CArray <MenuEntry,MenuEntry> m_MenuEntries;

    // @cmember Destructor
    virtual ~CYRTreeCtrl();

    // @cmember Image List
    CImageList m_imageList;

    // @cmember Returns the selected context menu item
    void    GetMenuItemClicked      (CString & Path, int & Type, CString & Entry);

    // @cmember Adds Menu entries to the popup menu
    void    AddAdditionalEntries    (CMenu *pPopup, CString type);

    // @cmember Display a context menu
    void    OnContextMenu           (CPoint point);

    // @cmember Refresh the items in the tree
    void RefreshTree();

    // Generated message map functions
    // @access Protected Members
protected:
    //{{AFX_MSG(CYRTreeCtrl)
    afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
    afx_msg void OnItemexpanding(NMHDR* pNMHDR, LRESULT* pResult);
    afx_msg void OnPopupProperties();
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
    //}}AFX_MSG
    // @cmember Constructor
    afx_msg void OnContextClick (UINT nID);

    DECLARE_MESSAGE_MAP()
private:

    // @access Private Members
private:
    // @cmember Get the Menu entries for the given item type
    MenuEntry*  GetMenuEntry (int type);

    // @cmember The selected menu item
    int m_nMenuIndexClicked;

    // @cmember Construct the drag object from the current tree item
    COleDataSource* GetDragItem (HTREEITEM hTreeItem);
    void RefreshBranch(HTREEITEM child);
    BOOL m_bMenus;

};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CYRTREECTRL_H__1C10FA38_B798_11D2_BCF0_00608CB45114__INCLUDED_)
