/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// XTreeCtl.cpp : Implementation of the CXTreeCtrl ActiveX Control class.

#include "stdafx.h"
#include "XTree.h"
#include "XTreeCtl.h"
#include "XTreePpg.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


IMPLEMENT_DYNCREATE(CXTreeCtrl, COleControl)


/////////////////////////////////////////////////////////////////////////////
// Message map

BEGIN_MESSAGE_MAP(CXTreeCtrl, COleControl)
//{{AFX_MSG_MAP(CXTreeCtrl)
ON_WM_CREATE()
ON_WM_SIZE()
ON_WM_LBUTTONDOWN()
ON_WM_CONTEXTMENU()
ON_WM_ERASEBKGND()
ON_WM_PAINT()
//}}AFX_MSG_MAP
ON_OLEVERB(AFX_IDS_VERB_PROPERTIES, OnProperties)
ON_MESSAGE (WMU_DBLCLICK_ITEM, OnDblClickTreeItem)
ON_MESSAGE (WMU_EXPANDING, OnTreeExpanding)
ON_MESSAGE (WMU_CONTEXT_MENU_CLICK, OnContextMenuClick)
ON_MESSAGE (WMU_ITEM_PROPERTIES ,OnItemProperties)
ON_MESSAGE (WMU_SHOW_CONTEXT, OnShowContext)
ON_MESSAGE (WMU_ITEM_DELETE, OnItemDelete)
ON_MESSAGE (WMU_SHOW_HELP, OnShowHelp)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// Dispatch map

BEGIN_DISPATCH_MAP(CXTreeCtrl, COleControl)
//{{AFX_DISPATCH_MAP(CXTreeCtrl)
DISP_FUNCTION(CXTreeCtrl, "AddItem", AddItem, VT_I4, VTS_I4 VTS_I2 VTS_BSTR VTS_I2 VTS_BOOL)
DISP_FUNCTION(CXTreeCtrl, "AddImageToList", AddImageToList, VT_EMPTY, VTS_HANDLE)
DISP_FUNCTION(CXTreeCtrl, "DeleteAllItems", DeleteAllItems, VT_EMPTY, VTS_NONE)
DISP_FUNCTION(CXTreeCtrl, "GetItemPath", GetItemPath, VT_BSTR, VTS_I4)
DISP_FUNCTION(CXTreeCtrl, "GetItemType", GetItemType, VT_I2, VTS_I4)
DISP_FUNCTION(CXTreeCtrl, "GetSelectedItem", GetSelectedItem, VT_I4, VTS_NONE)
DISP_FUNCTION(CXTreeCtrl, "SetItemImage", SetItemImage, VT_EMPTY, VTS_I4 VTS_I2)
DISP_FUNCTION(CXTreeCtrl, "SetChildren", SetChildren, VT_I2, VTS_I4 VTS_BOOL)
DISP_FUNCTION(CXTreeCtrl, "DefineNodeType", DefineNodeType, VT_EMPTY, VTS_I2 VTS_BSTR VTS_BOOL VTS_BOOL)
DISP_FUNCTION(CXTreeCtrl, "EnableMenuItem", EnableMenuItem, VT_EMPTY, VTS_I4 VTS_BSTR VTS_BOOL)
DISP_FUNCTION(CXTreeCtrl, "RefreshTree", RefreshTree, VT_EMPTY, VTS_NONE)
DISP_FUNCTION(CXTreeCtrl, "EnableMenus", EnableMenus, VT_EMPTY, VTS_BOOL)
//}}AFX_DISPATCH_MAP
DISP_FUNCTION_ID(CXTreeCtrl, "AboutBox", DISPID_ABOUTBOX, AboutBox, VT_EMPTY, VTS_NONE)
END_DISPATCH_MAP()


/////////////////////////////////////////////////////////////////////////////
// Event map

BEGIN_EVENT_MAP(CXTreeCtrl, COleControl)
//{{AFX_EVENT_MAP(CXTreeCtrl)
EVENT_CUSTOM("LBDblClick", FireLBDblClick, VTS_I4  VTS_I2  VTS_BSTR)
EVENT_CUSTOM("XTreeExpanding", FireXTreeExpanding, VTS_I4  VTS_I2  VTS_BSTR)
EVENT_CUSTOM("XTreeContextClick", FireXTreeContextClick, VTS_BSTR  VTS_I2  VTS_BSTR)
EVENT_CUSTOM("XTreeItemProperties", FireXTreeItemProperties, VTS_BSTR  VTS_I2)
EVENT_CUSTOM("XShowContext", FireXShowContext, VTS_I2  VTS_BSTR)
EVENT_CUSTOM("XTreeItemDelete", FireXTreeItemDelete, VTS_BSTR  VTS_I2)
EVENT_CUSTOM("XShowHelp", FireXShowHelp, VTS_NONE)
//}}AFX_EVENT_MAP
END_EVENT_MAP()


/////////////////////////////////////////////////////////////////////////////
// Property pages

// TODO: Add more property pages as needed.  Remember to increase the count!
BEGIN_PROPPAGEIDS(CXTreeCtrl, 1)
PROPPAGEID(CXTreePropPage::guid)
END_PROPPAGEIDS(CXTreeCtrl)


/////////////////////////////////////////////////////////////////////////////
// Initialize class factory and guid

IMPLEMENT_OLECREATE_EX(CXTreeCtrl, "XTREE.XTreeCtrl.1",
                       0xa8692636, 0xdde1, 0x11d2, 0xbc, 0xfb, 0, 0x60, 0x8c, 0xb4, 0x51, 0x14)


/////////////////////////////////////////////////////////////////////////////
// Type library ID and version

IMPLEMENT_OLETYPELIB(CXTreeCtrl, _tlid, _wVerMajor, _wVerMinor)


/////////////////////////////////////////////////////////////////////////////
// Interface IDs

const IID BASED_CODE IID_DXTree =
    {
        0xa8692634, 0xdde1, 0x11d2, { 0xbc, 0xfb, 0, 0x60, 0x8c, 0xb4, 0x51, 0x14 }
    };
const IID BASED_CODE IID_DXTreeEvents =
    {
        0xa8692635, 0xdde1, 0x11d2, { 0xbc, 0xfb, 0, 0x60, 0x8c, 0xb4, 0x51, 0x14 }
    };


/////////////////////////////////////////////////////////////////////////////
// Control type information

static const DWORD BASED_CODE _dwXTreeOleMisc =
    OLEMISC_ACTIVATEWHENVISIBLE |
    OLEMISC_SETCLIENTSITEFIRST |
    OLEMISC_INSIDEOUT |
    OLEMISC_CANTLINKINSIDE |
    OLEMISC_RECOMPOSEONRESIZE;

IMPLEMENT_OLECTLTYPE(CXTreeCtrl, IDS_XTREE, _dwXTreeOleMisc)


/////////////////////////////////////////////////////////////////////////////
// CXTreeCtrl::CXTreeCtrlFactory::UpdateRegistry -
// Adds or removes system registry entries for CXTreeCtrl

BOOL CXTreeCtrl::CXTreeCtrlFactory::UpdateRegistry(BOOL bRegister)
{
    // TODO: Verify that your control follows apartment-model threading rules.
    // Refer to MFC TechNote 64 for more information.
    // If your control does not conform to the apartment-model rules, then
    // you must modify the code below, changing the 6th parameter from
    // afxRegApartmentThreading to 0.

    if (bRegister)
        return AfxOleRegisterControlClass(
                   AfxGetInstanceHandle(),
                   m_clsid,
                   m_lpszProgID,
                   IDS_XTREE,
                   IDB_XTREE,
                   afxRegApartmentThreading,
                   _dwXTreeOleMisc,
                   _tlid,
                   _wVerMajor,
                   _wVerMinor);
    else
        return AfxOleUnregisterClass(m_clsid, m_lpszProgID);
}


/////////////////////////////////////////////////////////////////////////////
// CXTreeCtrl::CXTreeCtrl - Constructor

CXTreeCtrl::CXTreeCtrl()
{
    InitializeIIDs(&IID_DXTree, &IID_DXTreeEvents);

    m_pOldSel=NULL;
}


/////////////////////////////////////////////////////////////////////////////
// CXTreeCtrl::~CXTreeCtrl - Destructor

CXTreeCtrl::~CXTreeCtrl()
{
    // TODO: Cleanup your control's instance data here.
}


/////////////////////////////////////////////////////////////////////////////
// CXTreeCtrl::OnDraw - Drawing function


void CXTreeCtrl::OnDraw(
    CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid)
{
    // TODO: Replace the following code with your own drawing code.
    //  pdc->FillRect(rcBounds, CBrush::FromHandle((HBRUSH)GetStockObject(WHITE_BRUSH)));
    //  pdc->Ellipse(rcBounds);
}


/////////////////////////////////////////////////////////////////////////////
// CXTreeCtrl::DoPropExchange - Persistence support


void CXTreeCtrl::DoPropExchange(CPropExchange* pPX)
{
    ExchangeVersion(pPX, MAKELONG(_wVerMinor, _wVerMajor));
    COleControl::DoPropExchange(pPX);

    // TODO: Call PX_ functions for each persistent custom property.

}


/////////////////////////////////////////////////////////////////////////////
// CXTreeCtrl::OnResetState - Reset control to default state


void CXTreeCtrl::OnResetState()
{
    COleControl::OnResetState();  // Resets defaults found in DoPropExchange

    // TODO: Reset any other control state here.
}


/////////////////////////////////////////////////////////////////////////////
// CXTreeCtrl::AboutBox - Display an "About" box to the user


void CXTreeCtrl::AboutBox()
{
    CDialog dlgAbout(IDD_ABOUTBOX_XTREE);
    dlgAbout.DoModal();
}


/////////////////////////////////////////////////////////////////////////////
// CXTreeCtrl message handlers

int CXTreeCtrl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (COleControl::OnCreate(lpCreateStruct) == -1)
        return -1;

    //
    // Create a tree control
    //
    DWORD dwStyle = TVS_SHOWSELALWAYS |
                    TVS_HASBUTTONS |
                    TVS_LINESATROOT |
                    TVS_HASLINES |
                    TVS_SHOWSELALWAYS |
                    WS_CHILD | WS_VISIBLE;


    BOOL bCreated = m_TreeCtrl.Create( dwStyle,
                                       CRect(0, 0, 100, 100),
                                       this,
                                       101);

    ASSERT(bCreated);

    //
    // Add a client edge
    //
    m_TreeCtrl.ModifyStyleEx( 0, WS_EX_CLIENTEDGE, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
    m_TreeCtrl.ShowWindow(SW_SHOW);

    return 0;
}

void CXTreeCtrl::OnSize(UINT nType, int cx, int cy)
{
    COleControl::OnSize(nType, cx, cy);
    //
    // Resize the tree contol to cover the entire window
    //
    if( m_TreeCtrl.GetSafeHwnd() != NULL )
    {
        m_TreeCtrl.SetWindowPos( NULL,0, 0, cx, cy, SWP_NOZORDER|SWP_NOACTIVATE );
    }
}

long CXTreeCtrl::AddItem(long hParent, short nNodeType, LPCTSTR szLabel, short nImageIndex, BOOL bHasChildren)
{
    HTREEITEM hti,parentItem;

    //
    // Resolve the parent item
    //
    if (hParent == -1)
    {
        parentItem = TVI_ROOT;
    }
    else
    {
        parentItem = (HTREEITEM)hParent;
    }

    //
    // Add the new item to the tree control
    //
    hti = m_TreeCtrl.AddItem(parentItem, nNodeType, szLabel, nImageIndex, bHasChildren);
    Invalidate();

    //
    // Return the handle to the new item
    return (long)hti;
}

LRESULT CXTreeCtrl::OnDblClickTreeItem(WPARAM,LPARAM)
{
    //
    // Retrieve information about the item cliked on
    //
    HTREEITEM item = m_TreeCtrl.GetSelectedItem();
    int Type;
    CString Path;
    m_TreeCtrl.GetSelectedTreeItem(Path,Type);

    //
    // Fire the event to the container
    //
    FireLBDblClick((long)item,Type,Path);
    return 0;
}

LRESULT CXTreeCtrl::OnTreeExpanding(WPARAM,LPARAM)
{
    //
    // Retrieve information about the expanding item
    //
    HTREEITEM item = m_TreeCtrl.GetItemToExpand();
    int Type =  m_TreeCtrl.GetItemType(item);
    CString Path = m_TreeCtrl.GetItemPath(item);

    //
    // Fire the event to the container
    //
    FireXTreeExpanding((long)item,Type,Path);
    return 0;
}


void CXTreeCtrl::AddImageToList(OLE_HANDLE hBitmap)
{
    //
    // Convert handle to a bitmap image and add to the image list
    //
    CBitmap * pb=new CBitmap;
    pb->Attach((HGDIOBJ)hBitmap);
    m_TreeCtrl.AddImageToList(pb);
    delete pb;
}

void CXTreeCtrl::DeleteAllItems()
{
    m_TreeCtrl.DeleteAllItems();
}

BSTR CXTreeCtrl::GetItemPath(long hTreeItem)
{
    CString strResult;
    strResult = m_TreeCtrl.GetItemPath((HTREEITEM)hTreeItem);
    return strResult.AllocSysString();
}

short CXTreeCtrl::GetItemType(long hTreeItem)
{
    return m_TreeCtrl.GetItemType((HTREEITEM)hTreeItem);
}

long CXTreeCtrl::GetSelectedItem()
{
    return (long)m_TreeCtrl.GetSelectedItem();
}

void CXTreeCtrl::OnLButtonDown(UINT nFlags, CPoint point)
{
    UINT uFlags;
    HTREEITEM htItem = m_TreeCtrl.HitTest(point, &uFlags);
    if ((htItem != NULL) && (uFlags & TVHT_ONITEM))
        m_TreeCtrl.Select(htItem, TVGN_DROPHILITE);
    COleControl::OnLButtonDown(nFlags, point);
}


void CXTreeCtrl::SetItemImage(long hTreeItem, short nImageIndex)
{
    m_TreeCtrl.SetItemImage((HTREEITEM)hTreeItem, nImageIndex, nImageIndex);
}



void CXTreeCtrl::OnContextMenu(CWnd* pWnd, CPoint point)
{
    m_TreeCtrl.OnContextMenu (point);
}

LRESULT CXTreeCtrl::OnContextMenuClick (WPARAM MessageID ,LPARAM)
{
    CString Path;
    int Type;
    CString Entry;
    m_TreeCtrl.GetMenuItemClicked(Path, Type, Entry);

    //Don't perform action if there is no project selected
    if(strcmp(Path,"No Project Selected"))
    {
        FireXTreeContextClick(Path, Type, Entry);
    }
    return 0;
}

LRESULT CXTreeCtrl::OnItemProperties(WPARAM type, LPARAM lParam)
{
    LPCTSTR path = (LPCTSTR)lParam;
    //Don't try to show the properties if there is no project selected
    if(strcmp(path,"No Project Selected"))
    {
        FireXTreeItemProperties(path, type);
    }
    return 0;
}

short CXTreeCtrl::SetChildren(long hTreeItem, BOOL bHasChildren)
{
    TV_ITEM tvi;
    tvi.mask = TVIF_CHILDREN | TVIF_HANDLE;
    tvi.hItem = (HTREEITEM) hTreeItem;
    tvi.cChildren = (bHasChildren) ? 1 : 0;
    m_TreeCtrl.SetItem(&tvi);
    return 0;
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Method called by host to define a tree item type
// @parm    short | nNodeType | Item type to define
// @parm    LPCTSTR | szMenuItems | Context menu entries
// @parm    BOOL | bHasProperties | Has properties menu item
// @parm    BOOL | bDraggable | Item is draggable
void CXTreeCtrl::DefineNodeType(short nNodeType, LPCTSTR szMenuItems, BOOL bHasProperties, BOOL bDraggable)
{
    m_TreeCtrl.DefineType(nNodeType, szMenuItems, bHasProperties ,bDraggable);
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Method called by host to enable/disable a menu item for a given type
// @parm    long | nNodeType | Item type to modify
// @parm    LPCTSTR | szLabel | Selected tree item
// @parm    BOOL | bEnabled | Enabled/Disabled
void CXTreeCtrl::EnableMenuItem(long nNodeType, LPCTSTR szLabel, BOOL bEnabled)
{
    m_TreeCtrl.EnableMenuItem(nNodeType, szLabel, bEnabled);
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Fires an event back to the host when a context menu has been requested
// @parm    int | ID | Not Used
// @parm    long | hTreeItem | Selected tree item
LPARAM CXTreeCtrl::OnShowContext(WPARAM ID, LPARAM hTreeItem)
{
    CString path = m_TreeCtrl.GetItemPath((HTREEITEM)hTreeItem);
    int nodeType = m_TreeCtrl.GetItemType((HTREEITEM)hTreeItem);
    FireXShowContext(nodeType, path);
    return 0;
}

void CXTreeCtrl::RefreshTree()
{
    m_TreeCtrl.RefreshTree();
}

void CXTreeCtrl::EnableMenus(BOOL bEnabled)
{
    m_TreeCtrl.EnableMenus(bEnabled);
}

LRESULT CXTreeCtrl::OnItemDelete(WPARAM type, LPARAM lParam)
{
    LPCTSTR path = (LPCTSTR)lParam;
    //Don't try to show the properties if there is no project selected
    if(strcmp(path,"No Project Selected"))
    {
        FireXTreeItemDelete(path, type);
    }
    return 0;
}


BOOL CXTreeCtrl::OnEraseBkgnd(CDC* pDC)
{
    return true;
    return COleControl::OnEraseBkgnd(pDC);
}

void CXTreeCtrl::OnPaint()
{
    CPaintDC dc(this); // device context for painting


    // Do not call COleControl::OnPaint() for painting messages
}


LRESULT CXTreeCtrl::OnShowHelp(WPARAM , LPARAM )
{
    FireXShowHelp();
    return 0;
}
