/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// @doc     Document
// @module  CXTreeCtrl | Main Control class which handles methods and fires events
//
////////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_XTREECTL_H__A8692644_DDE1_11D2_BCFB_00608CB45114__INCLUDED_)
#define AFX_XTREECTL_H__A8692644_DDE1_11D2_BCFB_00608CB45114__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// XTreeCtl.h : Declaration of the CXTreeCtrl ActiveX Control class.
#include "CYRTreeCtrl.h"
/////////////////////////////////////////////////////////////////////////////
// CXTreeCtrl : See XTreeCtl.cpp for implementation.

// @class Control class
// @base public | COleControl
class CXTreeCtrl : public COleControl
{
    DECLARE_DYNCREATE(CXTreeCtrl)

    // @access Public Members
public:
    // @cmember Constructor
    CXTreeCtrl();
    // @cmember Tree Control Dialog
    CYRTreeCtrl m_TreeCtrl;

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CXTreeCtrl)
public:
    virtual void OnDraw(CDC* pdc, const CRect& rcBounds, const CRect& rcInvalid);
    virtual void DoPropExchange(CPropExchange* pPX);
    virtual void OnResetState();
    //}}AFX_VIRTUAL

    // Implementation
    // @access Protected Members
protected:
    // @cmember Destructor
    ~CXTreeCtrl();

    DECLARE_OLECREATE_EX(CXTreeCtrl)    // Class factory and guid
    DECLARE_OLETYPELIB(CXTreeCtrl)      // GetTypeInfo
    DECLARE_PROPPAGEIDS(CXTreeCtrl)     // Property page IDs
    DECLARE_OLECTLTYPE(CXTreeCtrl)      // Type name and misc status

    // Message maps
    //{{AFX_MSG(CXTreeCtrl)
    afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
    afx_msg void OnSize(UINT nType, int cx, int cy);
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnContextMenu (CWnd* pWnd, CPoint point);
    afx_msg BOOL OnEraseBkgnd(CDC* pDC);
    afx_msg void OnPaint();
    //}}AFX_MSG
    afx_msg LRESULT OnContextMenuClick (WPARAM,LPARAM);
    afx_msg LRESULT OnItemProperties(WPARAM,LPARAM);
    afx_msg LRESULT OnShowContext(WPARAM,LPARAM);
    afx_msg LRESULT OnItemDelete(WPARAM type, LPARAM lParam);
    afx_msg LRESULT OnShowHelp(WPARAM , LPARAM );

    // @cmember Handles a double-click event on a tree item
    afx_msg LRESULT OnDblClickTreeItem(WPARAM,LPARAM);

    // @cmember Message Handler for when a tree item is expanded
    afx_msg LRESULT OnTreeExpanding(WPARAM,LPARAM);

    DECLARE_MESSAGE_MAP()



    // Dispatch maps
    //{{AFX_DISPATCH(CXTreeCtrl)
    afx_msg long AddItem(long hParent, short nNodeType, LPCTSTR szLabel, short nImageIndex, BOOL bHasChildren);
    afx_msg void AddImageToList(OLE_HANDLE hBitmap);
    afx_msg void DeleteAllItems();
    afx_msg BSTR GetItemPath(long hTreeItem);
    afx_msg short GetItemType(long hTreeItem);
    afx_msg long GetSelectedItem();
    afx_msg void SetItemImage(long hTreeItem, short nImageIndex);
    afx_msg short SetChildren(long hTreeItem, BOOL bHasChildren);
    afx_msg void DefineNodeType(short nNodeType, LPCTSTR szMenuItems, BOOL bHasProperties, BOOL bDraggable);
    afx_msg void EnableMenuItem(long nNodeType, LPCTSTR szLabel, BOOL bEnabled);
    afx_msg void RefreshTree();
    afx_msg void EnableMenus(BOOL bEnabled);
    //}}AFX_DISPATCH
    DECLARE_DISPATCH_MAP()

    // @cmember Show About Box
    afx_msg void AboutBox();

    // Event maps
    //{{AFX_EVENT(CXTreeCtrl)
    void FireLBDblClick(long hTreeItem, short nNodeType, LPCTSTR szPath)
    {
        FireEvent(eventidLBDblClick,EVENT_PARAM(VTS_I4  VTS_I2  VTS_BSTR), hTreeItem, nNodeType, szPath);
    }
    void FireXTreeExpanding(long hTreeItem, short nNodeType, LPCTSTR szPath)
    {
        FireEvent(eventidXTreeExpanding,EVENT_PARAM(VTS_I4  VTS_I2  VTS_BSTR), hTreeItem, nNodeType, szPath);
    }
    void FireXTreeContextClick(LPCTSTR strPath, short nType, LPCTSTR strEntry)
    {
        FireEvent(eventidXTreeContextClick,EVENT_PARAM(VTS_BSTR  VTS_I2  VTS_BSTR), strPath, nType, strEntry);
    }
    void FireXTreeItemProperties(LPCTSTR strPath, short nType)
    {
        FireEvent(eventidXTreeItemProperties,EVENT_PARAM(VTS_BSTR  VTS_I2), strPath, nType);
    }
    void FireXShowContext(short nNodeType, LPCTSTR strPath)
    {
        FireEvent(eventidXShowContext,EVENT_PARAM(VTS_I2  VTS_BSTR), nNodeType, strPath);
    }
    void FireXTreeItemDelete(LPCTSTR strPath, short nType)
    {
        FireEvent(eventidXTreeItemDelete,EVENT_PARAM(VTS_BSTR  VTS_I2), strPath, nType);
    }
    void FireXShowHelp()
    {
        FireEvent(eventidXShowHelp,EVENT_PARAM(VTS_NONE));
    }
    //}}AFX_EVENT
    DECLARE_EVENT_MAP()

    // Dispatch and event IDs
    // @access Public Members
public:
    // @cmember Previously Selected Tree Item
    HTREEITEM m_pOldSel;
    enum {
        //{{AFX_DISP_ID(CXTreeCtrl)
        dispidAddItem = 1L,
        dispidAddImageToList = 2L,
        dispidDeleteAllItems = 3L,
        dispidGetItemPath = 4L,
        dispidGetItemType = 5L,
        dispidGetSelectedItem = 6L,
        dispidSetItemImage = 7L,
        dispidSetChildren = 8L,
        dispidDefineNodeType = 9L,
        dispidEnableMenuItem = 10L,
        dispidRefreshTree = 11L,
        dispidEnableMenus = 12L,
        eventidLBDblClick = 1L,
        eventidXTreeExpanding = 2L,
        eventidXTreeContextClick = 3L,
        eventidXTreeItemProperties = 4L,
        eventidXShowContext = 5L,
        eventidXTreeItemDelete = 6L,
        eventidXShowHelp = 7L,
        //}}AFX_DISP_ID
    };
private:
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_XTREECTL_H__A8692644_DDE1_11D2_BCFB_00608CB45114__INCLUDED)
