/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CORBAClient
// @module  CORBAClient | This module contains the implementation of the
//          CORBAClient class.  This class is responsible for hiding low-level
//          CORBA functionality and wrapping it up into an easy to use class.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CORBA/CORBAClient.h>
#include <CORBA/CORBAHelper.inl>
using namespace CORBA;

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor creates an instance of a CORBAClient object.  It
//          creates the ORB and BOA.
// @parm    The number of command line arguments.
// @parm    The array of command line arguments.
//
// @xref    <c CORBAGeneral>
//
// @ex      How to create a CORBAClient object using this constructor: |
//          int main( int argc, char** argv )
//          {
//              // Create CORBAClient object
//              CORBAClient* pClient = new CORBAClient( argc, argv );
//
//              ...
//
//              delete pClient;
//
//              return 1;
//          }
CORBAClient::CORBAClient( int argc, char** argv )
        :CORBAGeneral( argc, argv )
{}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor creates an instance of a CORBAClient object.  It
//          does not construct the ORB and BOA.  Instead they are passed as
//          parameters.
// @parm    The current ORB object.
// @parm    The current BOA object.
//
// @xref    <c CORBAGeneral>
//
// @ex      How to create a CORBAClient object using this constructor: |
//          int main( int argc, char** argv )
//          {
//              // Initialise ORB and BOA
//              CORBA::ORB_ptr orb = CORBA::ORB_Init( argc, argv, "omniORB2" );
//              CORBA::BOA_ptr boa = orb->BOA_Init( argc, argv, "omniORB2_BOA" );
//
//              // Create CORBAClient object
//              CORBAClient* pClient = new CORBAClient( orb, boa );
//
//              ...
//
//              delete pClient;
//
//              return 1;
//          }

CORBAClient::CORBAClient( ORB_ptr orb, BOA_ptr boa )
        :CORBAGeneral( orb, boa )
{}


