/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CORBAServer
///////////////////////////////////////////////////////////////////////////////

#ifndef _CORBAServer_h_
#define _CORBAServer_h_

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CORBA/CORBAGeneral.h>

///////////////////////////////////////////////////////////////////////////////
// @class   The CORBAServer class is intended to be used whenever an object
//          is required to be a CORBA server.  It simplifies the connections
//          and registration of objects.
// @base    public | CORBAGeneral
class CYR_EXPORT CORBAServer : public CORBAGeneral
{
    // @access Public Members
public:
    // Constructors/Destructors
    CORBAServer( int argc, char** argv );     // @cmember Constructor.
    CORBAServer( CORBA::ORB_ptr orb, CORBA::BOA_ptr boa );  // @cmember Constructor.
    virtual ~CORBAServer();  // @cmember Destructor

    // @cmember Register an object with NameService.
    void RegisterObject( CORBA::Object_ptr obj, const char* cls, const char* name );
    // @cmember Deregister an object with NameService
    void DeregisterObject( const char* cls, const char* name );
    // @cmember Inform BOA that an object is ready for use.
    void ReadyObject( CORBA::Object_ptr obj );
};

#endif
