/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrImpactLib
// @module  Specific Folder Wrappers |This module contains several simple,
//          specific Folder types that have been derived from the base wrapper
//          class.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrImpactLib/CECImpactFolders.h>

///////////////////////////////////////////////////////////////////////////////
// Constants
const char* CECSourceFolder::FOLDER_NAME = "Source";
const char* CECTestFolder::FOLDER_NAME = "Tests";
const char* CECObjectCodeFolder::FOLDER_NAME = "ObjectCode";
const char* CECDataFolder::FOLDER_NAME = "Data";
const char* CECNodeFolder::FOLDER_NAME = "Nodes";

///////////////////////////////////////////////////////////////////////////////
// The following macro expands to fully decalre the methods for simple folder
// types.
#define SPECIFIC_FOLDER( CLASS_X )                          \
CLASS_X::CLASS_X( CECOrbFolder* parent )                    \
: CECOrbFolder( parent, CLASS_X::FOLDER_NAME )      \
{                                                           \
}                                                           \

SPECIFIC_FOLDER( CECSourceFolder );
SPECIFIC_FOLDER( CECTestFolder );
SPECIFIC_FOLDER( CECObjectCodeFolder );
SPECIFIC_FOLDER( CECDataFolder );
SPECIFIC_FOLDER( CECNodeFolder );

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor retrieves a specific Folder from the OpenSTARep
//          database.
// @parm    Pointer to the parent Folder.
// @parm    Name of the Folder to retrieve.
CECTestStore::CECTestStore( CECTestFolder* parent, const char* name )
        : CECOrbFolder( parent, name )
{}

