/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifndef __MEMORY_VIEW__
#define __MEMORY_VIEW__

//
//  class CTLibrary : Should be used to view memory in ascii format.
//  ---------------
//


//==================================================================< Pragmas >
#ifdef  WIN32
#  pragma warning(disable : 4710)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __EXCEPTION__
#  include <exception.hpp>
#endif  // __EXCEPTION__

#ifndef __MTSAFE__
#  include <mtsafe.hpp>
#endif  // __MTSAFE__



//===================================================================< Macros >
#define DLLSPEC_MEMVIEW DLLSPEC (CYRTOOLS)


//====================================================================< Class >

///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// Class CTMemViewException //
///////////////////////////////////////////////////////////////////////////////
// Class used to keep memory view classes exceptions.
class DLLSPEC_MEMVIEW CTMemViewException : public CTException
{
    //-----------------------------------------------------------------------------
public:                     // Constructor(s) and destructor.
    CTMemViewException (const string& clMsg, const string& clFilename, short sLineno, long lNumber = -1L)
: CTException (clMsg, clFilename, sLineno, lNumber)
    { }


public:                     // Operators.
    void                    operator= (const CTMemViewException& exception)
    {
        CTException::operator= (exception) ;
    }

    //-----------------------------------------------------------------------------
}

; // CTMemViewException

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// End of Class CTMemViewException //
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////// Class CTMemView //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_MEMVIEW CTMemView : virtual public CTMTSafe
{
    //-----------------------------------------------------------------------------
public:             // Constructor(s) and destructor.
    CTMemView (const void* pBuffer, const long lBufLen, string label = "", int iWidth = 16, string sepAscii = " ", char cDefChar = ' ', string endOfLine = "") ;
    ~CTMemView () ;

public:             // Get methodes.
    const void*     getBinary (void) const
    {
        return pBinary_ ;
    } ;
    const long      getBinaryLen (void) const
    {
        return lBinaryLen_ ;
    } ;
    string&     getAscii (void)
    {
        return ascii_ ;
    } ;
    string      getAscii (void) const
    {
        return ascii_ ;
    } ;

    //-----------------------------------------------------------------------------
private:            // Members.
    void*           pBinary_ ;
    long            lBinaryLen_ ;
    string      ascii_ ;

    //-----------------------------------------------------------------------------
}

; // CTMemView

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////// End of Class CTMemView //
///////////////////////////////////////////////////////////////////////////////

#endif  // __MEMORY_VIEW__
