/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrWrapLib
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECOrbBorrowThread_h
#define _CECOrbBorrowThread_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrWrapLib/CECOrbServer.h>

///////////////////////////////////////////////////////////////////////////////
// RogueWave includes
#include <map>
using std::map;
///////////////////////////////////////////////////////////////////////////////
// Standard includes
#include <functional>

///////////////////////////////////////////////////////////////////////////////
// Typedefs
typedef map<string,CECOrbBorrow*,std::less<string> > BorrowArray;

///////////////////////////////////////////////////////////////////////////////
// @class   CECOrbBorrowThread | This class is responsible for maintaingin all
//          underlying CORBA objects within the repository wrappers.  It
//          creates a separate thread that will ensure all objects outlive the
//          reaper process until such time as they are deregistered.
class CECOrbBorrowThread
{
    // @access Public Members
public:
    CECOrbBorrowThread(CECOrbServer* server);           // @cmember Constructor
    virtual ~CECOrbBorrowThread();  // @cmember Destructor

    void Preserve( CECOrbBorrow* item ); // @cmember Preserve borrowed items
    void Dispose( CECOrbBorrow* item );  // @cmember Dispose of borrowed items

    // @access Protected Members
protected:
    BorrowArray m_array;        // @cmember Array of borrwed items
    bool m_running;             // @cmember Determines if thread is running or not
    void Borrower();            // @cmember Borrowing method
    CECOrbServer* m_server;
};

#endif

