/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc CyrFFApi
///////////////////////////////////////////////////////////////////////////////

#ifndef _CECFFProject_h
#define _CECFFProject_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrffApi/CECFFServer.h>

///////////////////////////////////////////////////////////////////////////////
// @class   CECFFProject | This class is a wrapper around the COM interface
//          to OpenSTARep's Project objects.  It provides a basic set of methods
//          for creation and retrieval of Projects.  Specific Project
//          instances, such as an Impact Project, are derived from this class
//          and are expected to initialize themselves correctly.
// @base    public | CECFFBean
class CYR_EXPORT CECFFProject : public CECFFBean
{
    // @access Public Members
public:
    // @cmember Project Retrieval constructor
    CECFFProject( CECFFServer* server, const char* name );
    // @cmember Project creation constructor
    CECFFProject( CECFFServer* server, const char* name, const char* desc );
    virtual ~CECFFProject(); // @cmember Destructor

    string GetName() const;         // @cmember Returns Project name
    string GetDescription() const;  // @cmember Returns Project description

    void Remove();

    CECFFServer* GetParentServer();

    // @access Protected Members
protected:
    bool RetrieveProject( CECFFServer* parent, const char* name );

    CECFFServer* m_parent;
    string m_name;
    string m_desc;
};

#endif
