/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrFFApi
// @module  Server Implementation | This module contains the implementation of
//          the CECFFServer class.  This class is simply a C++ wrapper around the
//          OpenSTARep COM IStServer interface.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <CyrffApi/CECFFServer.h>
#include <solutions.h>
#include <iostream.h>

CECFFBean::CECFFBean()
{}

CECFFBean::~CECFFBean()
{}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This constructor creates an instance of an IStServer interface.
//          This interface is then used to reference any other methods within
//          the OpenSTARep IStServer interface.  The constructor also connects to
//          the database and logs a specified user on to that database.  This
//          user is then the only user that can log on to the database via this
//          particular IStServer interface.
// @parm    The user name of the user requesting access.
// @parm    The specified user's password.

CECFFServer::CECFFServer( const char* user, const char* password )
{
    // Error information
    const char* where = "CECFFServer::CECFFServer";


    long status = CYR_SERVER_CONNECT_FAILED;

    // Retrieve settings from registry
    HKEY hRegKey;
    DWORD length;
    unsigned char* buf = NULL;

    // Get handle to registry entry
    long error = RegOpenKeyEx( HKEY_CURRENT_USER, SOLUTIONS_SERVER_KEY, 0,
                               KEY_QUERY_VALUE, &hRegKey );
    if ( error != ERROR_SUCCESS )
        throw CyrRepositoryException( "Cannot read registry settings", where, status );

    // Read server address values from registry
    RegQueryValueEx( hRegKey, "ServerPort", NULL, NULL, NULL, &length );
    buf = new unsigned char[length];
    RegQueryValueEx( hRegKey, "ServerPort", NULL, NULL, buf, &length );
    m_port = atol( (char*)buf );
    delete [] buf;

    RegQueryValueEx( hRegKey, "ServerAddress", NULL, NULL, NULL, &length );
    buf = new unsigned char[length];
    RegQueryValueEx( hRegKey, "ServerAddress", NULL, NULL, buf, &length );
    m_address=(char*)buf;
    delete [] buf;

    // Close registry entry
    RegCloseKey( hRegKey );
    m_username=user;
    m_password=password;

}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The destructor destroys the underlying COM object as part of its
//          tidying up.  It also disconnects from the OpenSTARep database.
CECFFServer::~CECFFServer()
{
    // Error information
    const char* where = "CECFFServer::~CECFFServer";
    long status = CYR_SERVER_CONNECT_FAILED;
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns the TCP/IP port number of the OpenSTARep database
//          that the underlying IStServer interface references.
// @rdesc   The TCP/IP port number.
long CECFFServer::GetPort() const
{
    return m_port;
}
string CECFFServer::GetAddress() const
{
    return m_address;
}
string CECFFServer::GetUserName() const
{
    return m_username;
}
string CECFFServer::GetPassword() const
{
    return m_password;
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method retrieves a list of Project names currently available
//          through the underlying COM interface.  This list is separated by
//          a single '\|' character and the list is terminated by two '\|'
//          characters.
// @devnote This list is only a snap shot and there is no guarantee that two
//          consecutive requests will return either the same list or the same
//          ordering within the list.
// @rdesc   The bar separated list.
// @parm    Returns the number of elements in the list.
string CECFFServer::GetProjects( long& count ) const
{
    return "";
}
