/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     CyrDiagnosticProvider
// @module  CyrDiagnosticProvider | This module contains the implementation of the
//          CyrDiagnosticProvider class.  This class is an actual Provider, used to
//          create objects for the purpose of logging diagnostics
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
//
#include "Factory/CyrDiagnosticProv.h"
#include "Factory/Diagnostics_I.h"

using namespace CORBA;

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The constructor simply registers the objects that this Provider
//          can create.
// @parm    The name of this Provider object.
// @parm    The name of the Factory object which owns this Provider.
// @parm    The current ORB instance.
//
CyrDiagnosticProvider::CyrDiagnosticProvider( const char* name, const char* owner,
        CORBA::ORB_ptr orb )
        :ProviderImpl( name, owner, orb )
{
    // Register CORBA objects that this Provider can create
    Register( Diagnostics::CLASS_NAME, name, PROVIDER_EXECUTOR );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method creates a new insance of a CORBA object based on the
//          passed parameter list.
// @rdesc   A pointer to the newly created object or a nil reference.
// @parm    Reference to an array of constructor parameters.
CORBA::Object_ptr CyrDiagnosticProvider::New( ParamList_ptr params )
{
    CORBA::String_var cls  = params->GetValue( Factory::PARAM_CLASS );
    CORBA::String_var name = params->GetValue( Factory::PARAM_NAME );
    CORBA::String_var pd   = params->GetValue( Factory::PARAM_PERIOD );
    long period = atol( pd );

    try
    {
        // Determine if object already exists
        CORBA::Object_ptr obj = m_pServer->GetObjectFromClass( cls, name );
        if ( !CORBA::is_nil( obj ) )
        {
            Borrow( cls, name, period );
            return obj;
        }
    }
    catch( CyrCORBAException )
    {
        // If we catch an exception here it is because the class does not
        // exist.  Therefore we should continue and create a new instance.
    }

    // Object does not exist therefore create a new one.

    if ( strcmp( cls, Diagnostics::CLASS_NAME ) == 0 )
    {
        // Create new diagnostic object
        DiagnosticImpl* dp = new DiagnosticsImpl( m_pServer, name );

        // Store it so that it can be freed later
        Store( dp, cls, name, period );

        // Return reference
        return dp->_this();
    }

    // Return a nil object as we haven't found one we can create
    return Object::_nil();
}

