/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     Registry
// @module  Windows Registry Interface Provider | This module contains the
//          implementation of the RegistryProvider class.  This Provider allows
//          the getting and setting of Windows registry keys. Obviously  this
//          Provider should only be instantiated on a Windows platform.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Factory/RegistryProvider.h>
#include <Objects/Registry_i.h>

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The constructor simply registers the single object type that this
//          Provider can create and creates a single instance of that object.
// @parm    The name of this Provider object.
// @parm    The name of the Factory object that owns this Provider.
// @parm    The current ORB instance.
RegistryProvider::RegistryProvider( CORBAServer* pServer, const char* name,
                                    const char* owner )
        :ProviderImpl( pServer, name, owner )
{
    // Register CORBA objects that this Provider can create
    Register( Registry::CLASS_NAME, name, PROVIDER_DATA );

    // Create an instance of the registry object
    m_registry = new RegistryImpl();
    m_pServer->ReadyObject( m_registry );
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   The registry destructor simply disposes of the internal Registry
//          class.
void RegistryProvider::Shutdown()
{
    // Dispose of class
    if ( !CORBA::is_nil( m_registry ))
        m_registry->_dispose();

    ProviderImpl::Shutdown();
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   This method returns a reference to the internal CORBA class.
// @rdesc   A pointer to the internal object or a nil reference.
// @parm    Reference to an array of constructor parameters.
CORBA::Object_ptr RegistryProvider::New( ParamList_ptr params )
{
    // Get default parameters out of list
    CORBA::String_var cls = params->GetValue( Factory::PARAM_CLASS );

    // Just check we have been asked for the correct class
    if ( strcmp( cls, Registry::CLASS_NAME ) == 0 )
    {
        // Return reference to class
        return m_registry->_this();
    }

    return CORBA::Object::_nil();
}
