/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc Logging
///////////////////////////////////////////////////////////////////////////////

#ifndef _CyrLog_i_h
#define _CyrLog_i_h

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Logging/CyrLog.h>
#include <CORBA/CORBAServer.h>
#include <CECInetInfo.h>
#include <ctime.hpp>
///////////////////////////////////////////////////////////////////////////////
#include <map>
#include <list>
using std::map;
using std::list;
///////////////////////////////////////////////////////////////////////////////
// Typedefs
typedef map<string,long,std::less<string> > TraceLevelMap;
typedef list<string> TraceList;

#define CYR_DAEMON  "CyranoDaemon"
#define CYR_VDK002  "CyranoVDK002"
#define CYR_FACTORY "CyranoFactory"
#define CYR_LOG     "CyranoLog"
#define CYR_EVENTS  "CyranoEvents"
#define CYR_OTA     "CyranoOTA"

///////////////////////////////////////////////////////////////////////////////
// Constants
// @const int | MIN_FLUSH | The minimum number of entries before flushing begins.
// @const int | NUM_FLUSH | The maximum number of entries to flush each time.
const int MIN_FLUSH = 100;
const int NUM_FLUSH = 20;

///////////////////////////////////////////////////////////////////////////////
// @class   CyrLogImpl |This class represents the implementation of the CORBA
//          class CyrLog.  This is the basic interface for Cyrano logging and
//          simply contains all generic functionality.  To output to files,
//          etc.  derive from this class and override the FlushEntries method,
//          the constructor and the destructor.  This will allow the entries to
//          be written to a location of your choice.  Currently the entries are
//          simply lost.
// @base    public | _sk_CyrLog
class CyrLogImpl : public virtual _sk_CyrLog
{
    // @access Public Members
public:
    CyrLogImpl( CORBAServer* pServer, long level );  // @cmember Constructor
    virtual ~CyrLogImpl();          // @cmember Destructor

    // @access IDL generated methods

    // @cmember Adds a single trace message to the list
    void AddTrace( const char* category, long level, const char* msg );
    void SetLevel( const char* category, long level );  // @cmember Sets current trace level
    long GetLevel( const char* category );              // @cmember Gets current trace level
    char* GetFirstTrace();           // @cmember Gets first trace entry
    char* GetNextTrace();            // @cmember Gets next trace entry

    TraceCat* GetTraceCategories();  // @cmember Gets list of trace levels

    // @access Protected Members
protected:
    string m_node;  // @cmember The current node name
    string m_user;  // @cmember The current user name
    string m_exec;  // @cmember The current executor name

    long m_defaultLevel;        // @cmember The default level for new categories
    TraceLevelMap m_catLevels;  // @cmember List of all current trace levels

    // @cmember Flushes entries from list.
    virtual void FlushEntries();
    void ClearArray();

    TraceList::iterator m_index;          // @cmember Iteration marker
    TraceList m_traceList;      // @cmember Trace list

    CORBAServer* m_pServer;     // @cmember Pointer to CORBA helper functionality
}
;

#endif
