/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

///////////////////////////////////////////////////////////////////////////////
// @doc     Registry
// @module  Windows Registry Class | This module contains the implementation of
//          the Registry class.  This class allows the getting and setting of
//          Windows registry keys. Obviously this class will only work on a
//          Windows platform.
//
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Cyrano includes
#include <Objects/Registry_i.h>
#include <Solutions.h>
#include <string>

using std::string;

RegistryImpl::RegistryImpl()
{
    // Open default registry key
    RegOpenKeyEx( HKEY_CURRENT_USER, SOLUTIONS_KEY, 0, KEY_QUERY_VALUE, &m_hRegKey );
}

RegistryImpl::~RegistryImpl()
{
    // Close default key
    if ( m_hRegKey != NULL )
        RegCloseKey( m_hRegKey );
}

long RegistryImpl::GetLongFromKey( const char* key )
{
    long ret = 0;
    DWORD length = sizeof(long);
    GetSubKeyValue( key, (unsigned char*)&ret, length );
    return ret;
}

char* RegistryImpl::GetStringFromKey( const char* key )
{
    // Get registry entry
    DWORD length = 256;
    char* buf = new char[length];
    GetSubKeyValue( key, (unsigned char*)buf, length );

    // Return CORBA string
    string ret = buf;
    delete [] buf;
    return CORBA::string_dup( ret.c_str() );
}

void RegistryImpl::GetSubKeyValue( const char* key, unsigned char* buf, DWORD& length )
{
    // Last entry in string is actual value.  The rest is purely hierarchy.
    // First we need to open the correct key.
    string regEntry = key;
    size_t index = regEntry.find_last_of( '\\' );
    regEntry.resize( index );

    HKEY hKey;
    long error = RegOpenKeyEx( m_hRegKey, regEntry.c_str(), NULL, KEY_QUERY_VALUE, &hKey );
    if ( error == ERROR_SUCCESS )
    {
        // Get value of registry entry
        regEntry = key;
        regEntry = regEntry.erase( 0, index );
        RegQueryValueEx( hKey, regEntry.c_str(), NULL, NULL, (unsigned char*)buf, &length );

        // Close open sub key
        RegCloseKey( hKey );
    }
}

void RegistryImpl::SetLongFromKey( const char* key, long value )
{
    // Last entry in string is actual value.  The rest is purely hierarchy.
    // First we need to open the correct key.
    string regEntry = key;
    size_t index = regEntry.find_last_of( '\\' );
    regEntry.resize( index );

    HKEY hKey;
    DWORD disp;
    long error = RegCreateKeyEx( m_hRegKey, regEntry.c_str(), NULL, NULL,
                                 REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
                                 NULL, &hKey, &disp );
    if ( error == ERROR_SUCCESS )
    {
        // Set value of registry entry
        regEntry = key;
        regEntry = regEntry.erase( 0, index );
        RegSetValueEx( hKey, regEntry.c_str(), 0, REG_DWORD, (unsigned char*)&value, sizeof(long) );

        // Close open sub key
        RegCloseKey( hKey );
    }
}

void RegistryImpl::SetStringFromKey( const char* key, const char* value )
{}

