/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Server.h: interface for the CYRServer class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CYRSERVER_H__E31B985E_5801_11D4_9481_00C04F89E392__INCLUDED_)
#define AFX_CYRSERVER_H__E31B985E_5801_11D4_9481_00C04F89E392__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CECComServer;
class CYRProject;

#include "CYRFileManagement.h"

class CYRFILEMANAGEMENT_EXPORTS CYRServer
{
public:
    CYRServer();
    virtual ~CYRServer();

    typedef enum{enRootFolder,enCaptureFolder,enDataFolder,enDependenciesFolder,enNodesFolder,
                 enObjectCodeFolder,enParametersFolder,enSourceFolder,enTestsFolder,enNumFolderTypes}ProjectFolder;

    //Server functions
    const CString& GetUserName()
    {
        return m_strUserName;
    }
    const CString& GetPassword()
    {
        return m_strPassword;
    }
    const CString& GetHostName()
    {
        return m_strHostName;
    }

    //Source control functions
    BOOL CheckOutFile(ProjectFolder nFolder,const CString& strSubDir,const CString& strFile);
    BOOL CheckInFile(ProjectFolder nFolder,const CString& strSubDir,const CString& strFile,const CString& strDescription=_T(""));

    CString GetFolderPath(ProjectFolder nFolder);
    CString GetFolderName(ProjectFolder nFolder);
    BOOL GetFileList(ProjectFolder nFolder,const CString& strSubDir, CStringArray& arstrFiles);
    BOOL GetFolderList(ProjectFolder nFolder,const CString& strSubDir,CStringArray& arstrFiles);
    BOOL DeleteFile(ProjectFolder nFolder,const CString& strSubDir,const CString& strFile);
    BOOL DeleteTest(const CString& strTest);
    BOOL DeleteFolder(ProjectFolder nFolder,const CString& strSubDir);

    //Project functions
    BOOL OpenProject(const CString& strProject,BOOL bRefresh=true);
    BOOL IsThereAnOpenProject()
    {
        return (m_pOpenProject != NULL);
    }
    BOOL DeleteProject(const CString& strProject);
    BOOL GetProjectList(CStringArray& arstrProjectList);
    void CloseProject();
    CString GetOpenProjectName();
    CString GetFullOpenProjectName();
    BOOL CreateProject(const CString& strProject,const CString& strDescription);

    int GetProjectCount();

    //Misc
    const CString& GetLastError()
    {
        return m_strLastError;
    }
    BOOL DoesFileExist(ProjectFolder nFolder,const CString& strSubDir,const CString& strFile);

protected:
    void GetProjectListFromDisk(CStringArray& arstrProjectList);
    static void DeleteDirectory(const CString& strDir);

    CYRProject *m_pOpenProject;
    BOOL m_bUsingRepository;

    CString m_strLastError;
    int m_nProjectCount;

    CString m_strUserName;
    CString m_strPassword;
    CString m_strHostName;
    CString m_strWorkingDirectory;
    friend CYRProject;
};

#endif // !defined(AFX_CYRSERVER_H__E31B985E_5801_11D4_9481_00C04F89E392__INCLUDED_)
