/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//========================================================================
//
// CYRService.h
// ------------
//
//    This is the include file for the CYRService class
//
//=========================================================================

#ifndef _CYRSERVICE_H_
#define _CYRSERVICE_H_

#define SERVICE_CONTROL_USER 128

class CYRService
{
public:
    CYRService(const char* szServiceName);
    virtual ~CYRService();

    // Command line parser
    BOOL ParseCommandLine(int argc, char* argv[]);

    // Check to see if installed
    BOOL IsInstalled();

    // Install the service
    BOOL Install();

    // Change an existing configuration
    BOOL ChangeConfig(const char* startup);

    // Uninstall service
    BOOL Uninstall();

    // Start the service
    BOOL StartService();

    // Set a new status for the service
    void SetStatus(DWORD dwState);

    // Initialize the service
    BOOL Initialize();

    // RUN the service code
    virtual void Run();

    // Initialisation code
    virtual BOOL OnInit();

    // Stop the service
    virtual void OnStop();

    // Interrogate the service
    virtual void OnInterrogate();

    // Pause the service
    virtual void OnPause();

    // Continue the service
    virtual void OnContinue();

    // Shutdown
    virtual void OnShutdown();

    // User intervention
    virtual BOOL OnUserControl(DWORD dwOpcode);

    // static member functions
    static void WINAPI ServiceMain(DWORD dwArgc, LPTSTR* lpszArgv);
    static void WINAPI Handler(DWORD dwOpcode);

    // data members
    char m_szServiceName[64];
    int m_iMajorVersion;
    int m_iMinorVersion;
    SERVICE_STATUS_HANDLE m_hServiceStatus;
    SERVICE_STATUS m_Status;
    BOOL m_bIsRunning;

    // static data
    static CYRService* m_pThis; // nasty hack to get object ptr

private:
    HANDLE m_hEventSource;

};

#endif // _CYRSERVICE_H_
