/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//========================================================================
//
// CYRWDT001.cpp
// -------------
//
//    This is the implementation file for the CYRWDT001 class
//
//=========================================================================

#include "CYRServiceApp.h"
#include "CYRWDT001.h"

//
// CYRWDT001
// =========
//
// Declare service as a named service - <CYRANO Names Service>
CYRWDT001::CYRWDT001()
        :CYRService("CYRANO Names Service")
{
    m_iStartParam   = 0;
    m_iIncParam     = 1;
    m_iState        = m_iStartParam;

    m_hOmniNames    = NULL;
    m_OmniNamesPid  = 0;
}

//
// OnInit
// ======
//
// This routine is always called first when the service is run
BOOL CYRWDT001::OnInit()
{
    // Read the registry parameters
    // Try opening the registry key:
    // HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\<AppName>\Parameters
    HKEY hkey = NULL;
    char szKey[1024];

    strcpy(szKey, "SYSTEM\\CurrentControlSet\\Services\\");
    strcat(szKey, m_szServiceName);
    strcat(szKey, "\\Parameters");

    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                     szKey,
                     0,
                     KEY_QUERY_VALUE,
                     &hkey) == ERROR_SUCCESS)
    {
        // Yes we are installed
        DWORD dwType = 0;
        DWORD dwSize = sizeof(m_iStartParam);

        RegQueryValueEx(hkey,
                        "Start",
                        NULL,
                        &dwType,
                        (BYTE*)&m_iStartParam,
                        &dwSize);

        dwSize = sizeof(m_iIncParam);

        RegQueryValueEx(hkey,
                        "Inc",
                        NULL,
                        &dwType,
                        (BYTE*)&m_iIncParam,
                        &dwSize);

        RegCloseKey(hkey);
    }

    // Set the initial state
    m_iState = m_iStartParam;

    return TRUE;
}

//
// Run
// ===
//
// The main routine called when the service is to be executed
void CYRWDT001::Run()
{
    m_bIsRunning = FALSE;

    if (GetCyranoHome())
    {
        // Try starting Omninames
        if (StartOmniNames())
        {
            m_iState += m_iIncParam;
            m_bIsRunning = TRUE;
        }
    }

    BOOL bFallOver = FALSE;

    while (m_bIsRunning)
    {
        if (m_hOmniNames != 0)
        {
            // And wait for it to fallover
            if (WaitForSingleObject(m_hOmniNames, 0) == WAIT_OBJECT_0)
                bFallOver = TRUE;
        }
        else
        {
            bFallOver = TRUE;
        }

        // Keep checking every five seconds
        ::Sleep(5000);
    }
}

//
// OnUserControl
// =============
//
// Process user control requests
BOOL CYRWDT001::OnUserControl(DWORD dwOpcode)
{
    switch (dwOpcode)
    {
    case SERVICE_CONTROL_USER + 0:
        {
            // Save the current status in the registry
            SaveStatus();
            return TRUE;
        }
    default:
        {
            break;
        }
    }
    return FALSE; // say not handled
}

//
// SaveStatus
// ==========
//
// Save the current status in the registry

void CYRWDT001::SaveStatus()
{
    // Try opening the registry key:
    // HKEY_LOCAL_MACHINE\SYSTEM\CurrentControlSet\Services\<AppName>\...
    HKEY hkey = NULL;
    char szKey[1024];

    strcpy(szKey, "SYSTEM\\CurrentControlSet\\Services\\");
    strcat(szKey, m_szServiceName);
    strcat(szKey, "\\Status");

    DWORD dwDisp;
    DWORD dwErr;

    dwErr = RegCreateKeyEx( HKEY_LOCAL_MACHINE,
                            szKey,
                            0,
                            "",
                            REG_OPTION_NON_VOLATILE,
                            KEY_WRITE,
                            NULL,
                            &hkey,
                            &dwDisp);

    if (dwErr != ERROR_SUCCESS)
    {
        return;
    }

    // Set the registry values
    RegSetValueEx(hkey,
                  "Current",
                  0,
                  REG_DWORD,
                  (BYTE*)&m_iState,
                  sizeof(m_iState));

    // Finished with key
    RegCloseKey(hkey);
}

//
// StartOmniNames
// ==============
//
// Start the OmniNames names service if present
//
BOOL CYRWDT001::StartOmniNames ()
{
    //
    // Get the execuable name for omninames. If this file does not exist we will not start
    // up the server.
    (void) strcpy(m_chImageName, m_chCyranoHome);
    (void) strcat(m_chImageName, "\\Server\\OmniOrb\\omninames.exe");

    //
    // Tidy up the temporary work files which should be in the OmniNames Log directory
    if (!DeleteLogs())
        return FALSE;

    //
    // Read in the OmniNames Port Number from the registry
    GetOmniPortNo();

    //
    //    ...and run it.
    return StartProcess ();
}

//
// DeleteLogs
// ==========
//
// Delete the existing Omninames logs
BOOL CYRWDT001::DeleteLogs()
{
    char chOmniLogDir[256];
    if (GetEnvironmentVariable ("OMNINAMES_LOGDIR" ,chOmniLogDir ,sizeof (chOmniLogDir)) <= 0)
    {
        return FALSE;
    }
    //
    //    Get the machine name as the log files use this.
    //
    char ComputerName[MAX_COMPUTERNAME_LENGTH + 1];
    DWORD ComputerNameLength = MAX_COMPUTERNAME_LENGTH + 1;
    GetComputerName (ComputerName, &ComputerNameLength);

    //
    //    ...and format the names of the log files and delete them if they exist.
    //
    char chFileName[255];
    (void) strcpy (chFileName, chOmniLogDir);
    (void) strcat (chFileName, "\\omninames-");
    (void) strcat (chFileName, ComputerName);

    char chBakFile[255];
    (void) strcpy (chBakFile, chFileName);
    (void) strcat (chBakFile, ".bak");

    char chLogFile[255];
    (void) strcpy (chLogFile, chFileName);
    (void) strcat (chLogFile, ".log");

    try
    {
        ::DeleteFile( chLogFile );
        ::DeleteFile( chBakFile );
    }
    catch (...)
    {}


    return TRUE;
}

//
// StartProcess
// ============
//
// Start up the specified process and returning status of call
BOOL CYRWDT001::StartProcess ()
{
    char    chCommandLine[512];

    //
    // Format the command line
    (void) strcpy (chCommandLine, m_chImageName);

    (void) strcat (chCommandLine, " -start ");
    (void) strcat (chCommandLine, m_chOmniPortNo);

    char ErrorBuffer[255];
    STARTUPINFO StartInfo = {sizeof(StartInfo)};

    ZeroMemory(&StartInfo,sizeof(STARTUPINFO));
    StartInfo.cb = sizeof(STARTUPINFO);
    StartInfo.dwFlags = STARTF_USESTDHANDLES|STARTF_USESHOWWINDOW;
    StartInfo.hStdOutput = NULL;
    StartInfo.hStdInput  = NULL;
    StartInfo.hStdError  = NULL;
    StartInfo.wShowWindow = SW_HIDE;

    char chCommonDir[255];
    (void) strcpy (chCommonDir, m_chCyranoHome);
    (void) strcat (chCommonDir, "\\Common");

    ::SetCurrentDirectory(chCommonDir);

    // Create the process to execute the command
    PROCESS_INFORMATION pi;

    if (!::CreateProcess(NULL,
                         chCommandLine,       // command line
                         NULL,                // process security attributes
                         NULL,                // primary thread security attributes
                         TRUE,                // handles are inherited
                         CREATE_NEW_CONSOLE,  // creation flags
                         NULL,                // use parent's environment
                         chCommonDir,         // Current directory
                         &StartInfo,          // STARTUPINFO pointer
                         &pi))                // receives PROCESS_INFORMATION
    {
        DebugBreak();
        (void) strcpy (ErrorBuffer, "Error invoking the image ");
        (void) strcpy (ErrorBuffer, chCommandLine);

        LPTSTR lpMsgBuf;

        FormatMessage (FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                       NULL,
                       GetLastError(),
                       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                       (LPTSTR) &lpMsgBuf,
                       0,
                       NULL);
        //
        // Free the buffer.
        LocalFree( lpMsgBuf );
        return FALSE;
    }

    //
    // ...and save the process details.

    m_hOmniNames = pi.hProcess;
    m_OmniNamesPid = pi.dwProcessId;

    return TRUE;
}

//
// OnStop
// ======
//
// Called when the service control manager wants to stop the service
//
void CYRWDT001::OnStop()
{
    if (m_hOmniNames != 0)
    {
        ::TerminateProcess (m_hOmniNames ,0);
        m_hOmniNames = NULL;
        m_OmniNamesPid = 0;
    }
}


//
// GetOmniPortNo
// =============
//
// Obtain the OmniNames Port Number
//
void CYRWDT001::GetOmniPortNo()
{
    // From HKEY_LOCAL_MACHINE\SOFTWARE\ORL\OMNIORB\2.0
    HKEY    hRegKey = NULL;
    char    szKeyValueData[256];
    DWORD   dwKeyLen = sizeof(szKeyValueData);

    //
    // First lets open the key
    if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, "SOFTWARE\\ORL\\OMNIORB\\2.0", 0, KEY_ALL_ACCESS, &hRegKey) == ERROR_SUCCESS)
    {
        //
        // And query the Requested Field
        if (RegQueryValueEx(hRegKey, "ORBInitialPort", 0, NULL, (unsigned char *) &szKeyValueData, &dwKeyLen) == ERROR_SUCCESS)
        {
            (void) strcpy (m_chOmniPortNo, szKeyValueData);
        }

        // then close the key again
        RegCloseKey(hRegKey);
    }
}

//
// GetCyranoHome
// =============
//
// Obtain the Cyrano Home Environmental Variable
//
BOOL CYRWDT001::GetCyranoHome()
{
    if (GetEnvironmentVariable ("CYRANO_HOME" ,m_chCyranoHome ,sizeof (m_chCyranoHome)) <= 0)
    {
        return FALSE;
    }
    return TRUE;
}
