/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SCLStructure.cpp: implementation of the CSCLStructure class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <IECache.h>
#include "SCLStructure.h"

//----------------------------------------------------------------------------

DB_URL_FIELDS::DB_URL_FIELDS()
{
    m_bPrimary      = FALSE;
    m_bHTMLURL      = FALSE;

    m_lConId        = -1;
    m_lBeginLine    = -1;
    m_lEndGetLine   = -1;
    m_lEndLine      = -1;
}

//----------------------------------------------------------------------------

void DB_URL_FIELDS::SetBeginLine(long _lBeginLine)
{
    m_lBeginLine  = _lBeginLine;
    m_lEndGetLine = _lBeginLine;
}

//----------------------------------------------------------------------------

void DB_URL_FIELDS::SetEndGetLine(long _lEndGetLine)
{
    m_lEndGetLine = _lEndGetLine;
}

//----------------------------------------------------------------------------

void DB_URL_FIELDS::SetEndLine(long _lEndLine)
{
    m_lEndLine = _lEndLine;
}

//----------------------------------------------------------------------------

void DB_URL_FIELDS::ToURL()
{
    int nPos;

    m_strRealURL = m_strURL;

    nPos = m_strURL.Find(' ');
    if( nPos >= 0 )
        m_strURL= m_strURL.Left( nPos ) ;


}

//----------------------------------------------------------------------------

void DB_URL_FIELDS::SetURL(char *_pURL)
{
    m_strURL += DeleteQuote(_pURL);
}

//----------------------------------------------------------------------------

void DB_URL_FIELDS::SetArgURL(char *_pURL)
{
    m_strArgURL += DeleteQuote(_pURL);
}


//----------------------------------------------------------------------------

void DB_URL_FIELDS::SetConId(long _lConId)
{
    m_lConId = _lConId;
}

//----------------------------------------------------------------------------

void DB_URL_FIELDS::SetPrimary(BOOL _bPrimary)
{
    m_bPrimary = _bPrimary;
}

//----------------------------------------------------------------------------

void DB_URL_FIELDS::SetHTMLURL(BOOL _bHTMLURL)
{
    m_bHTMLURL = _bHTMLURL;
}


//----------------------------------------------------------------------------

void DB_URL_FIELDS::SetHTMLURL(CString _strDataFile)
{
    long    lRequestID = 1 ;//$$$

    CIECache cache( _strDataFile ) ;
    if( !cache.Init() )
        return;

    m_bHTMLURL = cache.IsHTMLURL(m_strURL,m_lConId,lRequestID);
    m_bPrimary = cache.IsPrimaryURL(m_strURL,m_lConId,lRequestID);
}

//----------------------------------------------------------------------------

CString DB_URL_FIELDS::DeleteQuote(CString _strURL)
{
    CString strTempURL;

    strTempURL = _strURL;

    int nPos = strTempURL.Find('"' );
    if( nPos >= 0 )
    {
        strTempURL = strTempURL.Right( strTempURL.GetLength() - nPos - 1 ) ;
        nPos = strTempURL.ReverseFind('"' );
        if( nPos >= 0 )
        {
            strTempURL = strTempURL.Left( nPos ) ;
        }
    }
    return strTempURL;
}


//----------------------------------------------------------------------------

CSCLStructure::CSCLStructure()
        :    m_pCurrentURL(NULL)
{
    m_lSCLLineNo       = -1;
    m_lEnvironmentLine = -1;
    m_lEnvironmentLine = -1;
    m_lDefinitionLine  = -1;
    m_lCodeLine        = -1;
    m_lExitLine        = -1;

}

//----------------------------------------------------------------------------

CSCLStructure::~CSCLStructure()
{
    Reset();
}

//---------------------------------------------------------------------------

void CSCLStructure::Reset()
{
    int i;
    for ( i = 0; i < m_aData.GetSize(); i++)
        delete m_aData.GetAt(i);

    m_aData.RemoveAll();

    for ( i = 0; i < m_aVarData.GetSize(); i++)
        delete m_aVarData.GetAt(i);

    m_aVarData.RemoveAll();

}

//----------------------------------------------------------------------------

void CSCLStructure::SetLastError(long _lSCLLineNo, const CString& _strSCLText)
{
    m_lSCLLineNo = _lSCLLineNo;
    m_strSCLText = _strSCLText;
}

//----------------------------------------------------------------------------

void CSCLStructure::SetEnvironmentLine( long _lEnvironmentLine)
{
    m_lEnvironmentLine = _lEnvironmentLine;
}

//----------------------------------------------------------------------------

void CSCLStructure::SetDefinitionLine ( long _lDefinitionLine )
{
    m_lDefinitionLine = _lDefinitionLine;
}

//----------------------------------------------------------------------------

void CSCLStructure::SetCodeLine       ( long _lCodeLine       )
{
    m_lCodeLine = _lCodeLine;
}

//----------------------------------------------------------------------------

void CSCLStructure::SetExitLine       ( long _lExitLine       )
{
    m_lExitLine = _lExitLine;
}

//----------------------------------------------------------------------------

long CSCLStructure::GetEnvironmentLine()
{
    return m_lEnvironmentLine;
}

//----------------------------------------------------------------------------

long CSCLStructure::GetDefinitionLine ()
{
    return m_lDefinitionLine;
}

//----------------------------------------------------------------------------

long CSCLStructure::GetCodeLine       ()
{
    return m_lCodeLine;
}

//----------------------------------------------------------------------------

long CSCLStructure::GetExitLine       ()
{
    return m_lExitLine;
}

//----------------------------------------------------------------------------

void CSCLStructure::AllocURL()
{
    m_pCurrentURL = new DB_URL_FIELDS();
    m_aData.Add(m_pCurrentURL);

}


//----------------------------------------------------------------------------

void CSCLStructure::ToURL()
{
    ASSERT( m_pCurrentURL != NULL );

    m_pCurrentURL->ToURL();
}

//----------------------------------------------------------------------------

void CSCLStructure::SetURL(char *_pURL)
{
    ASSERT( m_pCurrentURL != NULL );

    m_pCurrentURL->SetURL(_pURL);
}

//----------------------------------------------------------------------------

void CSCLStructure::SetArgURL(char *_pURL)
{
    ASSERT( m_pCurrentURL != NULL );

    m_pCurrentURL->SetArgURL(_pURL);
}

//----------------------------------------------------------------------------

void CSCLStructure::SetConId(long _lConId)
{
    ASSERT( m_pCurrentURL != NULL );

    m_pCurrentURL->SetConId(_lConId);
}

//----------------------------------------------------------------------------

void CSCLStructure::SetBeginLine(long _lBeginLine)
{
    ASSERT( m_pCurrentURL != NULL );

    m_pCurrentURL->SetBeginLine(_lBeginLine);
}

//----------------------------------------------------------------------------

void CSCLStructure::SetEndGetLine(long _lEndGetLine)
{
    ASSERT( m_pCurrentURL != NULL );

    m_pCurrentURL->SetEndGetLine(_lEndGetLine);
}

//----------------------------------------------------------------------------

void CSCLStructure::SetEndLine(long _lEndLine)
{
    ASSERT( m_pCurrentURL != NULL );

    m_pCurrentURL->SetEndLine(_lEndLine);
}

//----------------------------------------------------------------------------

void CSCLStructure::SetPrimary(BOOL _bPrimary)
{
    ASSERT( m_pCurrentURL != NULL );

    m_pCurrentURL->SetPrimary(_bPrimary);
}

//----------------------------------------------------------------------------

void CSCLStructure::SetHTMLURL(BOOL _bHTMLURL)
{
    ASSERT( m_pCurrentURL != NULL );

    m_pCurrentURL->SetHTMLURL(_bHTMLURL);
}

//----------------------------------------------------------------------------

void CSCLStructure::SetHTMLURL(CString _strDataFile)
{
    CString strServerHeader;
    CString strClientHeader;
    long    lRequestID = 1 ;//$$$

    CIECache cache( _strDataFile ) ;
    if( !cache.Init() )
        return;

    for (int nRow = 0; nRow < m_aData.GetSize(); nRow++)
    {
        pDB_URL_FIELDS pdbFields =  m_aData.GetAt(nRow) ;
        pdbFields->SetHTMLURL( cache.IsHTMLURL(pdbFields->m_strURL,
                                               pdbFields->m_lConId,
                                               lRequestID ));
        pdbFields->SetPrimary( cache.IsPrimaryURL(pdbFields->m_strURL,
                               pdbFields->m_lConId,
                               lRequestID ));
    }
}


//----------------------------------------------------------------------------

BOOL CSCLStructure::IsHTMLURL(long _lLine)
{
    BOOL bIsHTMLURL = FALSE;

    for (int nRow = 0; nRow < m_aData.GetSize(); nRow++)
    {
        pDB_URL_FIELDS pdbFields =  m_aData.GetAt(nRow) ;
        if ( _lLine < pdbFields->m_lBeginLine || _lLine > pdbFields->m_lEndLine )
            continue;

        if (! pdbFields->m_bHTMLURL )
            continue;

        bIsHTMLURL = TRUE;
        break;
    }

    return bIsHTMLURL;
}


//----------------------------------------------------------------------------

BOOL CSCLStructure::GetHTMLInfo(long _lLine, long& _lConID, CString& _strURL)
{
    BOOL bHTMLInfo = FALSE;

    for (int nRow = 0; nRow < m_aData.GetSize(); nRow++)
    {
        pDB_URL_FIELDS pdbFields =  m_aData.GetAt(nRow) ;
        if ( _lLine < pdbFields->m_lBeginLine || _lLine > pdbFields->m_lEndLine )
            continue;

        if (! pdbFields->m_bHTMLURL )
            continue;

        _lConID = pdbFields->m_lConId;
        _strURL = pdbFields->m_strURL;

        bHTMLInfo = TRUE;
        break;
    }

    return bHTMLInfo;

}

//----------------------------------------------------------------------------

pDB_URL_FIELDS CSCLStructure::GetURLInfo(CString& _strURL, long _lConID)
{
    for (int nRow = 0; nRow < m_aData.GetSize(); nRow++)
    {
        pDB_URL_FIELDS pdbFields =  m_aData.GetAt(nRow) ;
        if ( _lConID != pdbFields->m_lConId || _strURL != pdbFields->m_strURL )
            continue;

        return pdbFields;
    }

    ASSERT(FALSE);
    return NULL;
}

//----------------------------------------------------------------------------

long CSCLStructure::GetInsertLine(long _lConID, CString _strURL)
{
    long lLine;

    lLine   = m_lCodeLine + 1;

    for (int nRow = 0; nRow < m_aData.GetSize(); nRow++)
    {
        pDB_URL_FIELDS pdbFields =  m_aData.GetAt(nRow) ;
        if ( _lConID != pdbFields->m_lConId || _strURL != pdbFields->m_strURL )
            continue;

        lLine = pdbFields->m_lEndLine + 1;
        break;
    }


    return lLine;
}


//----------------------------------------------------------------------------

long CSCLStructure::GetLineEndBlock(long _lConID, CString _strURL)
{
    long lLine;

    lLine   = -1;

    for (int nRow = 0; nRow < m_aData.GetSize(); nRow++)
    {
        pDB_URL_FIELDS pdbFields =  m_aData.GetAt(nRow) ;
        if ( _lConID != pdbFields->m_lConId && _strURL != pdbFields->m_strURL )
            continue;

        lLine = pdbFields->m_lEndLine + 1;

        for (int nRowSec = nRow+1; nRowSec < m_aData.GetSize(); nRowSec++)
        {
            pDB_URL_FIELDS pdbSecFields =  m_aData.GetAt(nRowSec) ;
            if ( !pdbSecFields->m_bPrimary )
            {
                lLine = pdbSecFields->m_lEndLine + 1;
                continue;
            }
            else
                break;
        }

        break;
    }


    ASSERT (lLine != -1);
    return lLine;
}


//----------------------------------------------------------------------------

void CSCLStructure::AllocVariable()
{
    m_pCurrentVar = new DB_VAR_FIELDS();
    m_aVarData.Add(m_pCurrentVar);

}

//----------------------------------------------------------------------------

void CSCLStructure::SetVarName(char *_pVarName)
{
    ASSERT( m_pCurrentVar != NULL );

    m_pCurrentVar->m_strName = _pVarName;
}

//----------------------------------------------------------------------------

void CSCLStructure::SetVarBeginLine(long _lBeginLine)
{
    ASSERT( m_pCurrentVar != NULL );

    m_pCurrentVar->m_lBeginLine = _lBeginLine;
}

//----------------------------------------------------------------------------

void CSCLStructure::SetVarEndLine(long _lEndLine)
{
    ASSERT( m_pCurrentVar != NULL );

    m_pCurrentVar->m_lEndLine = _lEndLine;
}

//----------------------------------------------------------------------------

BOOL CSCLStructure::IsVariableLine(long _lLine, long* _plBeginLine, long* _plEndLine)
{
    ASSERT( m_pCurrentVar != NULL );

    BOOL bRet = FALSE;
    for (int nRow = 0; nRow < m_aVarData.GetSize(); nRow++)
    {
        pDB_VAR_FIELDS pdbFields =  m_aVarData.GetAt(nRow) ;
        if ( (_lLine >= pdbFields->m_lBeginLine) && (_lLine <= pdbFields->m_lEndLine) )
        {
            bRet = TRUE;
            *_plBeginLine = pdbFields->m_lBeginLine;
            *_plEndLine   = pdbFields->m_lEndLine;
            break;
        }
    }

    return bRet;
}



