/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SCLStructure.h: interface for the CSCLStructure class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SCLSTRUCTURE_H__BF0C5FB3_2CA6_11D4_9336_00104B4B2995__INCLUDED_)
#define AFX_SCLSTRUCTURE_H__BF0C5FB3_2CA6_11D4_9336_00104B4B2995__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class DB_URL_FIELDS
{
public:
    CString m_strURL;
    CString m_strRealURL;
    CString m_strArgURL;
    long    m_lConId;
    long    m_lBeginLine;
    long    m_lEndGetLine;
    long    m_lEndLine;

    BOOL    m_bPrimary;
    BOOL    m_bHTMLURL;

public:
    DB_URL_FIELDS();

    void SetURL(char *_pURL);
    void SetArgURL(char *_pURL);
    void ToURL();

    void SetConId(long _lConId);
    void SetBeginLine(long _lBeginLine);
    void SetEndGetLine(long _lEndGetLine);
    void SetEndLine(long _lEndLine);

    void SetPrimary(BOOL _bPrimary);
    void SetHTMLURL(BOOL _bHTMLURL);
    void SetHTMLURL(CString _strDataFile);

    CString DeleteQuote(CString _strURL);

};

typedef DB_URL_FIELDS*  pDB_URL_FIELDS;


typedef struct
{
    CString m_strName;
    long    m_lBeginLine;
    long    m_lEndLine;

}
DB_VAR_FIELDS, *pDB_VAR_FIELDS;

class CSCLStructure
{
private :
    pDB_URL_FIELDS  m_pCurrentURL;
    pDB_VAR_FIELDS  m_pCurrentVar;

    long m_lEnvironmentLine ;
    long m_lDefinitionLine  ;
    long m_lCodeLine        ;
    long m_lExitLine        ;

public:
    long     m_lSCLLineNo;
    CString  m_strSCLText;

protected:

    CString DeleteQuote(CString _strURL);

public:
    CTypedPtrArray <CPtrArray, pDB_URL_FIELDS> m_aData ;
    CTypedPtrArray <CPtrArray, pDB_VAR_FIELDS> m_aVarData ;

public:
    CSCLStructure();
    virtual ~CSCLStructure();

    void Reset();
    void SetLastError(long _lSCLLineNo, const CString& _strSCLText);

    void SetEnvironmentLine( long _lEnvironmentLine);
    void SetDefinitionLine ( long _lDefinitionLine );
    void SetCodeLine       ( long _lCodeLine       );
    void SetExitLine       ( long _lExitLine       );

    long GetEnvironmentLine();
    long GetDefinitionLine ();
    long GetCodeLine       ();
    long GetExitLine       ();

    void AllocURL();
    void SetURL(char *_pURL);
    void SetArgURL(char *_pURL);
    void ToURL();

    void SetConId(long _lConId);
    void SetBeginLine(long _lBeginLine);
    void SetEndGetLine(long _lEndGetLine);
    void SetEndLine(long _lEndLine);

    void SetPrimary(BOOL _bPrimary);
    void SetHTMLURL(BOOL _bHTMLURL);
    void SetHTMLURL(CString _strDataFile);

    BOOL IsHTMLURL(long _lLine);
    BOOL GetHTMLInfo(long _lLine, long& _lConID, CString& _strURL);
    pDB_URL_FIELDS GetURLInfo(CString& _strURL, long _lConID);

    long GetInsertLine(long _lConID, CString _strURL);
    long GetLineEndBlock(long _lConID, CString _strURL);

    void AllocVariable();
    void SetVarName(char *_pVarName);
    void SetVarBeginLine(long _lBeginLine);
    void SetVarEndLine(long _lEndLine);
    BOOL IsVariableLine(long _lLine, long* _plBeginLine, long* _plEndLine);

};

#endif // !defined(AFX_SCLSTRUCTURE_H__BF0C5FB3_2CA6_11D4_9336_00104B4B2995__INCLUDED_)
