/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


#include "stdafx.h"
#include "ConstantesLib.h"
#include "VarStructures.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------
//                              CVarAutoGen
//---------------------------------------------------------------------------

CVarAutoGen::CVarAutoGen()
{
    m_lLowerLimit = 0 ;
    m_lUpperLimit = 0 ;

    m_lStepValue = 1 ;
}

//---------------------------------------------------------------------------

CVarAutoGen::~CVarAutoGen()
{
}

//---------------------------------------------------------------------------
//                              CVarDBParam
//---------------------------------------------------------------------------


CVarDBParam::CVarDBParam()
{
    m_lReqType      = 0 ;        // Table or Request
    m_lReqSelType   = 0 ;     // AllRows or NumberMax
    m_lMaxRowNumber = 0 ;
}

//---------------------------------------------------------------------------

CVarDBParam::~CVarDBParam()
{
}

//---------------------------------------------------------------------------


BOOL CVarDBParam::Connect()
{
    return m_VarDBSession.Connect(m_strDSN, m_strUserLogin, m_strUserPwd) ;
}

//---------------------------------------------------------------------------

void CVarDBParam::Disconnect()
{
    m_VarDBSession.Disconnect() ;
}

//---------------------------------------------------------------------------

CString CVarDBParam::GetRequestSource()
{
    CString strReqSource ;

    if(m_lReqType == VAR_DBSOURCE_TABLE)
    {
        strReqSource.Format(IDS_REQ_SOURCE,
                            m_strColumnName,
                            m_strTableOwner,
                            m_strTableName) ;
    }
    else
    {
        strReqSource = m_strReqSource ;
    }

    return strReqSource ;
}

//---------------------------------------------------------------------------
//                              CVarDBSession
//---------------------------------------------------------------------------

CVarDBSession::CVarDBSession()
{
    m_pstrDSN       = NULL ;
    m_pstrUserLogin = NULL ;
    m_pstrUserPwd   = NULL ;
}

//---------------------------------------------------------------------------

CVarDBSession::~CVarDBSession()
{
}

//---------------------------------------------------------------------------


BOOL CVarDBSession::Connect(CString _strDSN, CString  _strUserLogin, CString _strUserPwd )
{
    CString strConnection ;

    Disconnect() ;

    strConnection.Format(IDS_FMT_CONNECTION, _strUserLogin, _strUserPwd) ;

    TRY
    {
        m_bConnected = m_Database.Open( _strDSN, FALSE, FALSE, strConnection) ;
    }

    CATCH(CDBException, e)
    {
        CString strInvalidData ;
        CString strError ;
        //-----
        strError.LoadString(IDS_ERROR_LIB) ;
        strInvalidData.LoadString(IDS_INVALID_LOGIN_PASSWORD) ;
        AfxGetMainWnd()->MessageBox(strInvalidData, strError) ;
        m_bConnected = FALSE ;
    }
    END_CATCH

    return m_bConnected ;
}

//---------------------------------------------------------------------------

void CVarDBSession::Disconnect()
{
    if ( m_bConnected && m_Database.IsOpen() )
    {
        m_Database.Close() ;
    }

    m_bConnected = FALSE ;

}

//---------------------------------------------------------------------------

BOOL CVarDBSession::IsConnected()
{
    return m_bConnected ;
}

//---------------------------------------------------------------------------
//                              CVarFile
//---------------------------------------------------------------------------

CVarFile::CVarFile()
{
}

//---------------------------------------------------------------------------


CVarFile::~CVarFile()
{
}

//----------------------------------------------------------------------------


void CVarFile::SetName(CString _strValue)
{
    m_strFilePath = DeleteQuote(_strValue);
}
//----------------------------------------------------------------------------

CString CVarFile::DeleteQuote(CString _strValue)
{
    CString strTempValue;

    strTempValue = _strValue;

    int nPos = strTempValue.Find('"' );
    if( nPos >= 0 )
    {
        strTempValue = strTempValue.Right( strTempValue.GetLength() - nPos - 1 ) ;
        nPos = strTempValue.ReverseFind('"' );
        if( nPos >= 0 )
        {
            strTempValue = strTempValue.Left( nPos ) ;
        }
    }
    return strTempValue;
}

//---------------------------------------------------------------------------
//                              CVarGeneral
//---------------------------------------------------------------------------

CVarGeneral::CVarGeneral()
{
    m_strVarName     = "" ;
    m_lSourceInfo    = VAR_SOURCE_LIST ;
    m_lVarOrder      = VAR_ORDER_SEQUENTIAL ;
    m_lVarScope      = VAR_SCOPE_LOCAL ;
    m_lVarType       = VAR_DATATYPE_CHAR ;
}

//---------------------------------------------------------------------------

CVarGeneral::~CVarGeneral()
{
}

//---------------------------------------------------------------------------
//                              CVarInfo
//---------------------------------------------------------------------------



CVarInfo::CVarInfo()
{
}

//---------------------------------------------------------------------------


CVarInfo::~CVarInfo()
{
}

//---------------------------------------------------------------------------
//                              CVarValueList
//---------------------------------------------------------------------------


CVarValueList::CVarValueList()
{
    m_bGenerateFile = FALSE ;
}

//---------------------------------------------------------------------------

CVarValueList::~CVarValueList()
{
    /*$$$ for (int i = 0; i < m_Values.GetSize(); i++)
         delete m_Values.GetAt(i);*/

    m_Values.RemoveAll() ;
}

//----------------------------------------------------------------------------

void CVarValueList::Add(CString _strValue)
{
    m_Values.Add(DeleteQuote(_strValue));
}

//----------------------------------------------------------------------------

CString CVarValueList::DeleteQuote(CString _strValue)
{
    CString strTempValue;

    strTempValue = _strValue;

    int nPos = strTempValue.Find('"' );
    if( nPos >= 0 )
    {
        strTempValue = strTempValue.Right( strTempValue.GetLength() - nPos - 1 ) ;
        nPos = strTempValue.ReverseFind('"' );
        if( nPos >= 0 )
        {
            strTempValue = strTempValue.Left( nPos ) ;
        }
    }
    return strTempValue;
}


//---------------------------------------------------------------------------

CVarResponse::CVarResponse()
{}

//---------------------------------------------------------------------------


CVarResponse::~CVarResponse()
{}

