/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// AbstractTree.h: interface for the CAbstractTree class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ABSTRACTTREE_H__5DFC9339_844B_11D2_ABF7_00105A483E3C__INCLUDED_)
#define AFX_ABSTRACTTREE_H__5DFC9339_844B_11D2_ABF7_00105A483E3C__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "MenuParam.h"
#include "IDynMenu32.h"


class CMenuParam;
//
class CAbstractTree : public IDynMenu32Client
{

    struct TItemInfo
    {
        DWORD   m_dwID ;
        DWORD   m_dwParentID ;
        CString m_strText ;
        int     m_nImage ;
        int     m_nSelectedImage ;
    } ;

    WORD            m_wRecurCount ;

    CMenuParam      m_ContextMenu ;

    WORD            m_wStatus ;

    BOOL            m_bDestroyInfoForDisplay ;

protected :
    enum Status
    {
        status_no_status_bar_update = 0x0008 ,
        status_post_popup_menu      = 0x0010 ,
        status_no_selchange_timer   = 0x0020 ,
    };


public:
    struct TInsertInfo
    {
        // All inserted items are recorded here :
        CMap< DWORD, DWORD &, LPVOID, LPVOID > m_MapIDToItem ;
        // Items without a parent are awaiting insertion in :
        CArray< TItemInfo , TItemInfo & >        m_Orphans ;
    }
    * m_pInsertInfo ;

    BOOL NewInsertInfo() ;

    //store items
    BOOL StoreItem( DWORD dwID , HTREEITEM hItem ) ;
    //store  Items without a parent are awaiting insertion in
    BOOL StoreOrphan( TItemInfo & info ) ;

public:
    CAbstractTree( BOOL bDestroyInfoForDisplay = TRUE );
    virtual ~CAbstractTree();

    // Classes that inherit from 'CTreeAbstract' MUST implement :
    virtual CTreeCtrl & GetTreeCtrl() = 0 ;

    //Insert item in the tree specified by GetTreeCtrl()
    BOOL InsertItem( HTREEITEM & hItem ,
                     LPCTSTR     lpszItem ,
                     int         nImage ,
                     int         nSelectedImage ,
                     DWORD       dwID ,
                     DWORD       dwParentID = 0L,
                     HTREEITEM   hAfter = TVI_LAST  ) ;

    BOOL InsertItemBefore( HTREEITEM & hItem ,
                           HTREEITEM   hBeforeItem ,
                           LPCTSTR     lpszItem ,
                           int         nImage ,
                           int         nSelectedImage ,
                           DWORD       dwID ,
                           DWORD       dwParentID =0L ) ;

    BOOL InsertItemAfter( HTREEITEM & hItem ,
                          HTREEITEM   hAfter ,
                          LPCTSTR     lpszItem ,
                          int         nImage ,
                          int         nSelectedImage ,
                          DWORD       dwID ,
                          DWORD       dwParentID =0L ) ;


    // Insert orphan items whose parents are in 'm_pInsertInfo->m_MapIDToItem'
    // and delete those items from 'm_pInsertInfo->m_Orphans'
    // if bFinalize = TRUE it means every items have been inserted
    // if not ASSERT appear.
    BOOL LayOut( BOOL bFinalize = TRUE ) ;


    //Expand and Collapse Function

    virtual BOOL CanExpandAll() const ;
    virtual BOOL CanCollapseAll() const ;
    virtual BOOL ExpandAll() ;
    virtual BOOL CollapseAll() ;

    virtual BOOL CanExpandBranch( HTREEITEM hItem = (HTREEITEM)NULL ) const ;
    virtual BOOL CanCollapseBranch( HTREEITEM hItem = (HTREEITEM)NULL ) const ;

    virtual BOOL ExpandBranch( HTREEITEM hItem = (HTREEITEM)NULL ) ;
    virtual BOOL CollapseBranch( HTREEITEM hItem = (HTREEITEM)NULL ) ;


    //Menu Fonctions

public :
    BOOL GetStatus( Status status ) const ;

protected:
    void SetStatus( Status status , BOOL bSet = TRUE ) ;

public :
    BOOL GetPostPopupMenu() const ;
    void SetPostPopupMenu( BOOL bActivatePostMode ) ;

    // Contextual menus management :
    virtual BOOL DoPopupMenu( CPoint clientPoint ) ; // Class-specific menu (if implemented).
    virtual BOOL CanDoPopupMenu() const ;            // Class-specific condition for callig the above.
    virtual void DoOwnerContextMenu( CMenu * pSubMenu )
    {}
    ;

    // Implementation of the IDynMenu32Client interface : it is the window
    // which receives messages
    virtual HRESULT GetClientWindow( /*[out]*/ CWnd * * ppWnd ) = 0 ;
    virtual HRESULT GetCmdMsgTargetWindow( /*[out]*/ CWnd * * ppTargetWnd ) = 0 ;

    // Called by the 'IDynMenu32' right before openig the menu.
    // Gives the client a chance of modifying the menu.
    virtual HRESULT OnPreMenuPopup( /*[in]*/ CMenu * pMenu ) ;
};

#endif // !defined(AFX_ABSTRACTTREE_H__5DFC9339_844B_11D2_ABF7_00105A483E3C__INCLUDED_)
