#ifndef _DATASTORE_H_
#define _DATASTORE_H_
/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


//---------------------------------------------------------------------------
// 'CStringStore' is designed for saving on string allocations. It's not meant
// to be a clever string storage management system, strings are just added one
// after the other in a buffer grown as needed.
// String addresses are to be stored by the client.
//
// Hence the class will be used for storing, probably all at once, a big batch of
// strings that are going to be destroyed all at the same time.
// N.B : there is NO RemoveString() !

//////////////////////////////////////////////////////////////////////
// CStringStore
//////////////////////////////////////////////////////////////////////
class CStringStore
{
private:
    struct TBuffer
    {
        TCHAR   * m_pzBuffer ;
        TBuffer * m_pNextBuffer ;
        int       m_nSize ;
        int       m_nRemains ;

        TBuffer( int nGrowBy ) ;
        ~TBuffer() ;
    };

    TBuffer * m_pBuffer ;
    TBuffer * m_pCurrent ;
    int       m_nGrowBy ;

    TBuffer *     NewBuffer( int nGrowBy = -1 ) ;

    const TCHAR * Store( const TCHAR * pczString , TCHAR cSep = (TCHAR)0 ) ;

public:
    CStringStore( int nGrowBy = -1 ) ;
    ~CStringStore() ;

    const TCHAR * AddString( const TCHAR * pczString , TCHAR cSep = (TCHAR)0 ) ;
    // Calls the above copying return value into 'pczAssign'. If != NULL returns TRUE.
    BOOL          AddStringAssign( const TCHAR * & pczAssign , const TCHAR * pczString , TCHAR cSep = (TCHAR)0 ) ;

    void Purge() ;
} ;

//////////////////////////////////////////////////////////////////////
// CDataStore
//////////////////////////////////////////////////////////////////////
class CDataStore : public CStringStore
{
public:
    CDataStore( int nStructGrowBy = -1 ,      // Struct storage increment
                int nTextGrowBy   = -1 )
    {}

    ; // Text (CStringStore) storage increment
    virtual ~CDataStore()
    {}
    ;
} ;

//////////////////////////////////////////////////////////////////////
// CGenDataStoreBase
//////////////////////////////////////////////////////////////////////
class CGenDataStoreBase : public CStringStore
{
public:
CGenDataStoreBase( int nGrowBy = 32 ) : CStringStore( nGrowBy )
    {}

} ;


#endif // _DATASTORE_H_
