/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ExtendedTreeCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "CyrRes.h"
#include "ExtendedTreeCtrl.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


//////////////////////////////////////////////////////////////////////
// CExtendedTreeCtrl
//////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------

IMPLEMENT_DYNAMIC(CExtendedTreeCtrl,CTreeCtrl)

//--------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CExtendedTreeCtrl, CTreeCtrl)
//{{AFX_MSG_MAP(CExtendedTreeCtrl)
ON_WM_RBUTTONDOWN()
//
ON_COMMAND(ID_EXTGRID_EXPAND_ALL, _OnExpandAll)
ON_UPDATE_COMMAND_UI(ID_EXTGRID_EXPAND_ALL, _OnUpdateExpandAll)
//
ON_COMMAND(ID_EXTGRID_COLLAPSE_ALL, _OnCollapseAll)
ON_UPDATE_COMMAND_UI(ID_EXTGRID_COLLAPSE_ALL, _OnUpdateCollapseAll)
//
ON_COMMAND(ID_EXTGRID_EXPAND_BRANCH, _OnExpand)
ON_UPDATE_COMMAND_UI(ID_EXTGRID_EXPAND_BRANCH, _OnUpdateExpand)
//
ON_COMMAND(ID_EXTGRID_COLLAPSE_BRANCH, _OnCollapse)
ON_UPDATE_COMMAND_UI(ID_EXTGRID_COLLAPSE_BRANCH, _OnUpdateCollapse)
//
ON_WM_INITMENUPOPUP()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//--------------------------------------------------------------------

CExtendedTreeCtrl::CExtendedTreeCtrl(BOOL bDestroyInfoForDisplay /*= TRUE*/ ) :
        CAbstractTree( bDestroyInfoForDisplay )
{}

//--------------------------------------------------------------------


CExtendedTreeCtrl::~CExtendedTreeCtrl()
{}

//--------------------------------------------------------------------

// Get the tree control for abstract tree

CTreeCtrl & CExtendedTreeCtrl::GetTreeCtrl()
{
    ASSERT( this != NULL ) ;
    return *this ;
}

//--------------------------------------------------------------------

//Get the current item. If no current item, it returns the root

HTREEITEM CExtendedTreeCtrl::GetAtLeastTheRoot( CPoint * pHitPoint ) const
{
    UINT      nFlags = 0 ;
    HTREEITEM hItem  = pHitPoint != NULL ? HitTest( *pHitPoint , &nFlags ) : (HTREEITEM)NULL ;

    if ( hItem == (HTREEITEM)NULL || ! ( nFlags & TVHT_ONITEM|TVHT_ONITEMBUTTON ) )
    {
        if ( ( hItem = GetSelectedItem() ) == (HTREEITEM)NULL )
        {
            hItem = GetRootItem() ;
        }
    }

    return hItem ;
}

//--------------------------------------------------------------------

// Insert items in the tree when you know the order
HTREEITEM CExtendedTreeCtrl::AddItem( int         nDepth ,
                                      int       * pnLastDepth ,
                                      HTREEITEM * phLastItem ,
                                      LPCTSTR     lpszItem  /*=LPSTR_TEXTCALLBACK*/ ,
                                      void      * pItemData /*=NULL*/ ,
                                      int         nImage         /*=I_IMAGECALLBACK*/ ,
                                      int         nSelectedImage /*=I_IMAGECALLBACK*/ )
{
    if ( pnLastDepth == NULL || phLastItem == NULL )
    {
        ASSERT(FALSE) ;
        return (HTREEITEM)NULL ;
    }

    HTREEITEM hParent      = TVI_ROOT ;
    HTREEITEM hInsertAfter = TVI_LAST ;

    if ( GetCount() == 0 ) // Virgin state
    {
        ASSERT( nDepth == 1 ) ; // Only the root is acceptable.
        // Keep initial values.
    }

    else
    {
        ASSERT( phLastItem != NULL ) ;
        //the new item is at the same level in the tree than the other item
        if ( nDepth == *pnLastDepth )
        {
            // Siblings are to share the same parent.
            hParent = GetParentItem( *phLastItem ) ;
            ASSERT( nDepth == 1 || hParent != (HTREEITEM)NULL ) ;
        }
        // the new item is the child of the last item
        else if ( nDepth > *pnLastDepth )
        {
            ASSERT( nDepth - *pnLastDepth == 1 ) ; // Cannot increase by more than one to the right !
            hParent = *phLastItem ;
        }
        else
        {
            // the item is one of the ancestor of the last item
            ASSERT( nDepth > 0 ) ;
            hParent = *phLastItem ;
            for ( int nDiff = *pnLastDepth - nDepth + 1 ; nDiff > 0 ; nDiff-- )
            {
                hParent = GetParentItem( hParent ) ;
            }
        }
    }

    // insert item
    HTREEITEM hNewItem = CTreeCtrl::InsertItem( lpszItem , nImage , nSelectedImage , hParent , hInsertAfter ) ;

    if ( hNewItem != (HTREEITEM)NULL )
    {
        *phLastItem = hNewItem ;

        if ( pItemData != NULL )
        {
            //store data
            if ( SetItemData( hNewItem , reinterpret_cast<DWORD>(pItemData) ) == FALSE )
            {
                DeleteItem( hNewItem ) ;     // It's got to be all
                hNewItem = (HTREEITEM)NULL ; // or nothing
            }

        }

        if ( hNewItem != (HTREEITEM)NULL ) // Still OK.
            *pnLastDepth = nDepth ;

    }

    return hNewItem ;
}

//--------------------------------------------------------------------

//create tree control
BOOL CExtendedTreeCtrl::Create( CWnd * pParent , UINT nStaticPatternID , UINT nTreeCtrlID )
{
    ASSERT( pParent != NULL && pParent->IsKindOf(RUNTIME_CLASS(CWnd)) ) ;
    ASSERT( nStaticPatternID != 0 ) ;
    ASSERT( nTreeCtrlID != 0 ) ;

    CWnd * pStatic = pParent->GetDlgItem(nStaticPatternID) ;

    ASSERT( pStatic != NULL && pStatic->IsKindOf(RUNTIME_CLASS(CWnd)) ) ;

    CRect rc ;
    pStatic->GetWindowRect( &rc ) ;
    pParent->ScreenToClient( &rc ) ;

    pStatic->ShowWindow( SW_HIDE ) ;


    return CTreeCtrl::Create(  WS_BORDER|WS_CHILD|WS_VISIBLE|TVS_LINESATROOT
                               |TVS_HASLINES|TVS_HASBUTTONS|TVS_SHOWSELALWAYS|TVS_DISABLEDRAGDROP,
                               rc,
                               pParent ,
                               nTreeCtrlID ) ;
}

// --------------------------------------------------------------------------

//get the window responsible for the menu
HRESULT CExtendedTreeCtrl::GetClientWindow( /*[out]*/ CWnd * * ppWnd )
{
    if ( ppWnd != NULL )
    {
        *ppWnd = this ;
        return S_OK ;
    }

    return E_INVALIDARG ;
}

// --------------------------------------------------------------------------

//Get the window responsible of the messages of the menu
HRESULT CExtendedTreeCtrl::GetCmdMsgTargetWindow( /*[out]*/ CWnd * * ppTargetWnd )
{
    if ( ppTargetWnd != NULL )
    {
        *ppTargetWnd = this ;
        return S_OK ;
    }

    return E_INVALIDARG ;
}

//--------------------------------------------------------------------


void CExtendedTreeCtrl::OnRButtonDown(UINT nFlags, CPoint point)
{
    UINT        nItemFlags      = 0 ;
    HTREEITEM   hItem           = HitTest( point , &nItemFlags ) ;
    BOOL        bAncestorCalled = FALSE ;
    BOOL        bPostPopupMenu  = GetPostPopupMenu() ;

    if ( hItem != (HTREEITEM)NULL && ( nItemFlags & TVHT_ONITEM ) != 0 )
    {
        if ( bPostPopupMenu )
            SetStatus( CAbstractTree::status_no_selchange_timer , TRUE ) ;

        CTreeCtrl::OnRButtonDown(nFlags, point) ;
        bAncestorCalled = true ;

        //select item
        Select( hItem , TVGN_CARET ) ;

        if ( bPostPopupMenu )
            SetStatus( CAbstractTree::status_no_selchange_timer , FALSE ) ;
    }

    //Display popup menu
    if ( CanDoPopupMenu() )
    {
        if ( bPostPopupMenu == FALSE )
        {
            ClientToScreen( &point ) ;
            DoPopupMenu( point ) ;
        }
    }
    else if ( bAncestorCalled == FALSE )
    {
        CTreeCtrl::OnRButtonDown(nFlags, point);
    }


}

//--------------------------------------------------------------------

BOOL CExtendedTreeCtrl::DoPopupMenu( CPoint clientPoint )
{
    return SUCCEEDED( CyrLibDoDefaultPopupMenu( this ,
                      &clientPoint ,
                      CDynMenu32DefaultImpl::mi_expand_group ) ) ;
}

//--------------------------------------------------------------------

void CExtendedTreeCtrl::_OnExpandAll()
{
    ExpandAll() ;
}

//--------------------------------------------------------------------

void CExtendedTreeCtrl::_OnUpdateExpandAll(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CanExpandAll() ) ;
}

//--------------------------------------------------------------------

void CExtendedTreeCtrl::_OnCollapseAll()
{
    CollapseAll() ;
}

//--------------------------------------------------------------------

void CExtendedTreeCtrl::_OnUpdateCollapseAll(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CanCollapseAll() ) ;
}

//--------------------------------------------------------------------

void CExtendedTreeCtrl::_OnExpand()
{
    ExpandBranch() ;
}

//--------------------------------------------------------------------

void CExtendedTreeCtrl::_OnUpdateExpand(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CanExpandBranch() ) ;
}

//--------------------------------------------------------------------

void CExtendedTreeCtrl::_OnCollapse()
{
    CollapseBranch() ;
}

//--------------------------------------------------------------------

void CExtendedTreeCtrl::_OnUpdateCollapse(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CanCollapseBranch() ) ;
}

//--------------------------------------------------------------------

//InitMenu popup
void CExtendedTreeCtrl::OnInitMenuPopup(CMenu* pMenu, UINT, BOOL bSysMenu)
{
    ::CyrLibOnInitMenuPopup( this , pMenu, bSysMenu ) ;
}

// --------------------------------------------------------------------------

//init menu popup if desired
HRESULT CExtendedTreeCtrl::OnPreMenuPopup( /*[in]*/ CMenu * pMenu )
{
    if ( pMenu != NULL )
    {
        ASSERT( ::IsMenu( pMenu->m_hMenu ) ) ;
        DoOwnerContextMenu( pMenu ) ;
        return S_OK ;
    }

    return E_INVALIDARG ;
}

// --------------------------------------------------------------------------

// if you want to have your own menu popup pSubmenu is a pointer
// of the menu tree. You have to overload this function

void CExtendedTreeCtrl::DoOwnerContextMenu( CMenu * pSubMenu )
{
}

// --------------------------------------------------------------------------

//insert items in the tree. You don't need to know the order
// the item are identified by their id and their parent's id
// if the parent is the root. Its parent 's id = 0

BOOL CExtendedTreeCtrl::InsertItem( HTREEITEM & hItem ,
                                    LPCTSTR     lpszItem ,
                                    int         nImage ,
                                    int         nSelectedImage ,
                                    DWORD       dwID ,
                                    DWORD       dwParentID )
{
    return CAbstractTree::InsertItem( hItem , // handle of the new item added
                                      lpszItem , // new item text
                                      nImage ,   // bitmap
                                      nSelectedImage , //selected bitmap
                                      dwID ,            //id associated with the item
                                      dwParentID ) ; // id associated with the parent item
}

// --------------------------------------------------------------------------

//insert items in the tree before hBeforeItem. You don't need to know the order
// the item are identified by their id and their parent's id
// if the parent is the root. Its parent 's id = 0

BOOL CExtendedTreeCtrl::InsertItemBefore( HTREEITEM & hItem ,
        HTREEITEM   hBeforeItem ,
        LPCTSTR     lpszItem ,
        int         nImage ,
        int         nSelectedImage ,
        DWORD       dwID ,
        DWORD       dwParentID )
{
    return CAbstractTree::InsertItemBefore( hItem ,
                                            hBeforeItem ,
                                            lpszItem ,
                                            nImage ,
                                            nSelectedImage ,
                                            dwID ,
                                            dwParentID ) ;
}



// --------------------------------------------------------------------------

//insert items in the tree before hBeforeItem. You don't need to know the order
// the item are identified by their id and their parent's id
// if the parent is the root. Its parent 's id = 0
BOOL CExtendedTreeCtrl::InsertItemAfter( HTREEITEM & hItem ,
        HTREEITEM   hAfter ,
        LPCTSTR     lpszItem ,
        int         nImage ,
        int         nSelectedImage ,
        DWORD       dwID ,
        DWORD       dwParentID )


{
    return CAbstractTree::InsertItemAfter( hItem ,
                                           hAfter ,
                                           lpszItem ,
                                           nImage ,
                                           nSelectedImage ,
                                           dwID ,
                                           dwParentID ) ;
}


