/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// FilterItem.h: interface for the CFilterItem class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FILTERITEM_H__2919F056_E32C_11D1_9188_00104B4B2995__INCLUDED_)
#define AFX_FILTERITEM_H__2919F056_E32C_11D1_9188_00104B4B2995__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//////////////////////////////////////////////////////////////////////
// CGridDataStoreInterface
//////////////////////////////////////////////////////////////////////
class CFilterItem : public CObject
{
public:

    enum Op
    {
        opNone = 0x0000    ,
        opEq   = 0x0001    ,
        opGr   = 0x0002    ,
        opLr   = 0x0004    ,
        opGrEq = opEq|opGr ,
        opLrEq = opEq|opLr ,
        opDiff = opGr|opLr ,
        // Regular expressions
        opRegExpr = 0x0010 ,
        opCase    = 0x0020
    };

    enum LogOp
    {
        lopNone = 0x00 ,
        lopAnd  = 0x01 ,
        lopOr   = 0x02 ,
        lopNot  = 0x10 ,
    } ;

private:
    CFilterItem * m_pLogItem ; // CFilterItem associated to this through logical operator
    BYTE          m_cLogOp ;   // logical operator

    CString m_strLoVal ; // Floor
    CString m_strHiVal ; // Ceiling
    Op      m_LoOp ;
    Op      m_HiOp ;

    DWORD   m_dwUserData ;

public:
    CFilterItem() ;
    ~CFilterItem() ;

    CFilterItem & operator = ( const CFilterItem & source ) ;

    BOOL SetRange( LPCTSTR pczVal    , Op op     , LPCTSTR pczHiVal = NULL       , Op op2 = opNone ) ;
    BOOL GetRange( LPCTSTR * p_pczVal, Op * p_op , LPCTSTR * p_pczHiVal = NULL   , Op * p_op2 = NULL ) const ;

    void  SetUserData( DWORD dwUserData ) ;
    DWORD GetUserData() const ;

    CFilterItem & And( CFilterItem * pLogItem ) ;
    CFilterItem & Or(  CFilterItem * pLogItem ) ;
    CFilterItem & Not( CFilterItem * pLogItem ) ;

    BOOL          GetLogOp( LogOp lop = lopNone ) const ; // Default means "Is there a logical operator ?"
    CFilterItem * GetLogItem( LogOp * plop = NULL ) const ;


    void SetLogItem( CFilterItem * pLogItem , LogOp lop ) ;
} ;

#endif // !defined(AFX_FILTERITEM_H__2919F056_E32C_11D1_9188_00104B4B2995__INCLUDED_)
