/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CG: This file was added by the Splash Screen component.
// Splash.cpp : implementation file
//

#include "stdafx.h"  // e. g. stdafx.h
#include "resource.h"  // e.g. resource.h

#include "Splash.h"  // e.g. splash.h

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
//   Splash Screen class

//---------------------------------------------------------------------------

BOOL CSplashWnd::c_bShowSplashWnd;
CSplashWnd* CSplashWnd::c_pSplashWnd;
CSplashWnd::CSplashWnd():
        m_nTime( 0 )
{}

//---------------------------------------------------------------------------


CSplashWnd::~CSplashWnd()
{
    // Clear the static window pointer.
    ASSERT(c_pSplashWnd == this);
    c_pSplashWnd = NULL;
}

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CSplashWnd, CWnd)
//{{AFX_MSG_MAP(CSplashWnd)
ON_WM_CREATE()
ON_WM_PAINT()
ON_WM_TIMER()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

//enable to show (bEnable = TRUE ) or not (bEnable = FALSE ) the splash
void CSplashWnd::EnableSplashScreen(BOOL bEnable /*= TRUE*/)
{
    c_bShowSplashWnd = bEnable;
}

//---------------------------------------------------------------------------

BOOL CSplashWnd::IsVisible()
{
    return c_pSplashWnd != NULL;
}


//---------------------------------------------------------------------------

//display the splash with the bitmap ID nID. pParentWnd = parent of the splash usually NULL

void CSplashWnd::ShowSplashScreen(CWnd* pParentWnd /*= NULL*/ , UINT nID /*= 0 */ , UINT nTime /*=0*/)
{
    if (!c_bShowSplashWnd || c_pSplashWnd != NULL)
        return;

    // Allocate a new splash screen, and create the window.
    c_pSplashWnd = new CSplashWnd;
    if (!c_pSplashWnd->Create( pParentWnd , nID , nTime ) )
        delete c_pSplashWnd;
    else
        c_pSplashWnd->UpdateWindow();
}

//---------------------------------------------------------------------------

//intercept all messages and tranfer them or not according to the location of the mouse click
BOOL CSplashWnd::PreTranslateAppMessage(MSG* pMsg)
{
    if (c_pSplashWnd == NULL)
        return FALSE;

    CRect rc ;
    //--
    c_pSplashWnd->GetWindowRect( &rc ) ;


    // If we get a keyboard or mouse message, hide the splash screen.
    if (pMsg->message == WM_NCLBUTTONDOWN ||
            pMsg->message == WM_NCRBUTTONDOWN ||
            pMsg->message == WM_NCMBUTTONDOWN)
    {
        c_pSplashWnd->HideSplashScreen();
        return FALSE;   // message handled here
    }


    if (pMsg->message == WM_KEYDOWN ||
            pMsg->message == WM_SYSKEYDOWN ||
            pMsg->message == WM_LBUTTONDOWN ||
            pMsg->message == WM_RBUTTONDOWN ||
            pMsg->message == WM_MBUTTONDOWN)
    {
        if( !rc.PtInRect( pMsg->pt ) )
        {
            c_pSplashWnd->HideSplashScreen();
            return FALSE;   // message handled here
        }

        else
        {
            c_pSplashWnd->HideSplashScreen();
            return TRUE;    // message handled here
        }

    }


    return FALSE;   // message not handled
}

//---------------------------------------------------------------------------

//create splash


BOOL CSplashWnd::Create(CWnd* pParentWnd /*= NULL*/ , UINT nID /*=0*/ , UINT nTime /*=0*/)
{
    if (!m_bitmap.LoadBitmap(nID))
        return FALSE;

    m_nTime = nTime ;
    BITMAP bm;
    m_bitmap.GetBitmap(&bm);

    return CreateEx(0,
                    AfxRegisterWndClass(0, AfxGetApp()->LoadStandardCursor(IDC_ARROW)),
                    NULL, WS_POPUP | WS_VISIBLE, 0, 0, bm.bmWidth, bm.bmHeight, pParentWnd->GetSafeHwnd(), NULL);
}

//---------------------------------------------------------------------------

//destroy splash

void CSplashWnd::HideSplashScreen()
{
    // Destroy the window, and update the mainframe.
    DestroyWindow();
    AfxGetMainWnd()->UpdateWindow();
}

//---------------------------------------------------------------------------

void CSplashWnd::PostNcDestroy()
{
    // Free the C++ class.
    delete this;
}

//---------------------------------------------------------------------------

//create the splash and initialize the time of display

int CSplashWnd::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CWnd::OnCreate(lpCreateStruct) == -1)
        return -1;

    // Center the window.
    CenterWindow();

    // Set a timer to destroy the splash screen.
    SetTimer(1, m_nTime, NULL);

    return 0;
}

//---------------------------------------------------------------------------

void CSplashWnd::OnPaint()
{
    CPaintDC dc(this);

    CDC dcImage;
    if (!dcImage.CreateCompatibleDC(&dc))
        return;

    BITMAP bm;
    m_bitmap.GetBitmap(&bm);

    // Paint the image.
    CBitmap* pOldBitmap = dcImage.SelectObject(&m_bitmap);
    dc.BitBlt(0, 0, bm.bmWidth, bm.bmHeight, &dcImage, 0, 0, SRCCOPY);
    dcImage.SelectObject(pOldBitmap);
}

//---------------------------------------------------------------------------

void CSplashWnd::OnTimer(UINT nIDEvent)
{
    // Destroy the splash screen window.
    HideSplashScreen();
}
