/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// padview.cpp : implementation file
//

#include "stdafx.h"
#include "padview.h"
#include <afxpriv.h>

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPadView

IMPLEMENT_DYNCREATE(CPadView, CEditView)

//----------------------------------------------------------------------------
//Constructor
CPadView::CPadView() :
        m_Undo(None)
{}

// --------------------------------------------------------------------------


CPadView::~CPadView()
{}

// --------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CPadView, CEditView)
//{{AFX_MSG_MAP(CPadView)
// NOTE - the ClassWizard will add and remove mapping macros here.
ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// --------------------------------------------------------------------------

void CPadView::ResetStatus()
{
    m_Undo = None ;
}

// --------------------------------------------------------------------------
// Get the text selected : strTextSel
// return the text length
int CPadView::GetTextSel( CString & strTextSel )
{
    int nBegSel ;
    int nEndSel ;
    // ----
    //Get the first and last character selected
    GetEditCtrl().GetSel( nBegSel , nEndSel ) ;

    if ( nBegSel < nEndSel )
    {
        GetWindowText( strTextSel ) ;

        //modify strTextSel
        if ( ! strTextSel.IsEmpty() )
        {
            //Get a pointer on the string to modify it
            char * pzAll = strTextSel.GetBuffer( strTextSel.GetLength() ) ;
            // ----
            memmove( pzAll , pzAll + nBegSel , nEndSel - nBegSel + 1 ) ;
            *( pzAll + ( nEndSel - nBegSel ) ) = '\0' ;
            strTextSel.ReleaseBuffer() ;
        }
    }
    else
    {
        strTextSel.Empty() ;
    }

    return strTextSel.GetLength() ;
}

/////////////////////////////////////////////////////////////////////////////
// CPadView WordWrap
// --------------------------------------------------------------------------
// return true if the text is wordwrap (ie the text is too long and
// one part of it goes to the next line )
BOOL CPadView::IsWordWrap() const
{
    return (GetStyle() & ES_AUTOHSCROLL) == 0;
}

// --------------------------------------------------------------------------
// return true if the edit is readonly
BOOL CPadView::IsReadOnly() const
{
    return (GetStyle() & ES_READONLY) != 0;
}

// --------------------------------------------------------------------------
//Set if the text should be wordwrap or not
BOOL CPadView::SetWordWrap( BOOL bWordWrap )
{
    return SetWordWrap( bWordWrap , IsReadOnly() ) ;
}

// --------------------------------------------------------------------------
//Set if the text should be wordwrap or not
BOOL CPadView::SetWordWrap( BOOL bWordWrap , BOOL bReadOnly )
{
    bWordWrap = !!bWordWrap;    // make sure ==TRUE || ==FALSE
    bReadOnly = !!bReadOnly;

    // if the same state than before -> no wrap is necessary
    if ( IsWordWrap() == bWordWrap && IsReadOnly() == bReadOnly )
    {
        return FALSE;
    }

    // preserve original control's state.
    CFont* pFont = GetFont();
    int nLen = GetBufferLength();
    //can allocate to another place than the memory allocated by the process
    char FAR* pSaveText = new far char[GetBufferLength()+1];
    GetWindowText(pSaveText, nLen+1);

    // create new edit control with appropriate style and size.
    DWORD dwStyle = dwStyleDefault & ~(ES_AUTOHSCROLL|WS_HSCROLL|WS_VISIBLE);
    if (!bWordWrap)
    {
        dwStyle |= ES_AUTOHSCROLL|WS_HSCROLL;
    }
    if (bReadOnly)
    {
        dwStyle |= ES_READONLY ;
    }

    CWnd* pParent = GetParent();
    CRect rect;
    GetWindowRect(rect);
    pParent->ScreenToClient(rect);
    CWnd* pFocus = GetFocus();

    UINT nID = GetDlgCtrlID();

    HWND hWnd =
#ifndef _WIN32
        ::CreateWindow( "edit",
                        NULL,
                        dwStyle,
                        rect.left, rect.top,
                        rect.right-rect.left,
                        rect.bottom-rect.top,
                        pParent->m_hWnd,
                        (HMENU)nID,
                        (HINSTANCE)m_segText,
                        NULL);
#else
        ::CreateWindow( "edit",
                        NULL,
                        dwStyle,
                        rect.left, rect.top,
                        rect.right-rect.left,
                        rect.bottom-rect.top,
                        pParent->m_hWnd,
                        (HMENU)nID,
                        AfxGetInstanceHandle(),
                        NULL);
#endif

    if (hWnd == NULL)
    {
        delete[] pSaveText;
        return FALSE;
    }

    // set the window text to nothing to make sure following set doesn't fail
    SetWindowText(NULL);

    // restore visual state
    ::SetWindowText(hWnd, pSaveText);
    delete[] pSaveText;
    if (pFont != NULL)
    {
        ASSERT(pFont->m_hObject != NULL);
        ::SendMessage(hWnd, WM_SETFONT, (WPARAM)pFont->m_hObject, 0);
    }

    // detach old window, attach new
    HWND hWndOld = Detach();
    ::SetWindowLong(hWndOld, GWL_WNDPROC, (LONG)*GetSuperWndProcAddr());
#ifndef _WIN32
    ::SetWindowWord(hWndOld, GWW_ID, nID+1);
#else
    ::SetWindowLong(hWndOld, GWL_ID, nID+1);
#endif
    ASSERT(m_hWnd == NULL);
    SubclassWindow(hWnd);
    ASSERT(m_hWnd == hWnd);
    UINT nTabStops = m_nTabStops;
    GetParentFrame()->SendMessage(WM_RECALCPARENT);
    GetEditCtrl().SetTabStops(nTabStops);
    GetClientRect(&rect);
    SetWindowPos(NULL, 0, 0, 0, 0,
                 SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE|SWP_NOZORDER|SWP_SHOWWINDOW);
    SetWindowPos(NULL, 0, 0, 0, 0,
                 SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE|SWP_NOZORDER|SWP_DRAWFRAME);
    SetWindowPos(NULL, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);
    UpdateWindow();

    // destroy old
    ::SetWindowPos(hWndOld, NULL, 0, 0, 0, 0,
                   SWP_HIDEWINDOW|SWP_NOREDRAW|SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE|
                   SWP_NOZORDER);
    ::DestroyWindow(hWndOld);

    // restore rest of state...
    GetEditCtrl().LimitText(nMaxSize);
    if (pFocus == this)
        SetFocus();

    // notify container that doc changed
    // GetDocument()->UpdateAllItems(NULL); // OLE

    ASSERT_VALID(this);
    return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CPadView drawing

void CPadView::OnDraw( CDC * /* pDC */ )
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CPadView message handlers
// --------------------------------------------------------------------------
// enable or not undo

void CPadView::OnUpdateEditUndo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CEditView::GetEditCtrl().CanUndo() && m_Undo != UnDone );
}

// --------------------------------------------------------------------------
// undo text
void CPadView::OnEditUndo()
{
    if ( CEditView::GetEditCtrl().CanUndo() )
    {
        if ( CEditView::GetEditCtrl().Undo() != FALSE )
        {
            CEditView::GetEditCtrl().SetModify( FALSE ) ;
            m_Undo = UnDone ;
        }
    }
}

// --------------------------------------------------------------------------
// enable or not redo
void CPadView::OnUpdateEditRedo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( CEditView::GetEditCtrl().CanUndo() && m_Undo == UnDone );
}

// --------------------------------------------------------------------------
//redo text
void CPadView::OnEditRedo()
{
    if ( CEditView::GetEditCtrl().CanUndo() )
    {
        if ( CEditView::GetEditCtrl().Undo() != FALSE )
        {
            CEditView::GetEditCtrl().SetModify( FALSE ) ;
            m_Undo = ReDone ;
        }
    }
}

/////////////////////////////////////////////////////////////////////////////
// CMarkupPadView

IMPLEMENT_DYNCREATE(CMarkupPadView, CPadView)

// --------------------------------------------------------------------------

CMarkupPadView::CMarkupPadView()
{}

// --------------------------------------------------------------------------


CMarkupPadView::~CMarkupPadView()
{}

// --------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CMarkupPadView, CPadView)
//{{AFX_MSG_MAP(CMarkupPadView)
// NOTE - the ClassWizard will add and remove mapping macros here.
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// --------------------------------------------------------------------------
// return TRUE if pczBegDelim and pczEndDelim belongs to the text
BOOL CMarkupPadView::HasDelimitedToken( const char * pczBegDelim , const char * pczEndDelim )
{
    int  nSave1 ;
    int  nSave2 ;
    BOOL bRet = FALSE ;
    // ----
    // Get the current selection
    GetEditCtrl().GetSel( nSave1 , nSave2 ) ;

    //search pczBegDelim backward and forward. If it exists search pczEndDelim forward
    if ( FindText( pczBegDelim , TRUE , TRUE ) != FALSE || FindText( pczBegDelim , FALSE , TRUE ) )
    {
        bRet = FindText( pczEndDelim , TRUE , TRUE ) ;
        //Reselect the initial selected text (because findText select the text found)
        GetEditCtrl().SetSel( nSave1 , nSave2 ) ;
    }
    return bRet ;
}

// --------------------------------------------------------------------------

// select text between pczFom and pczTo
BOOL CMarkupPadView::SelectBetween( const char * pczFom , const char * pczTo )
{
    //Search text pczFom
    if ( FindText( pczFom , TRUE , TRUE ) != FALSE || FindText( pczFom , FALSE , TRUE ) )
    {
        int nStartBeg ;
        int nStartEnd ;
        // ----
        //Get the first character of pczFom : nStartBeg
        //and the last character of pczFom : nStartEnd
        GetEditCtrl().GetSel( nStartBeg , nStartEnd ) ;

        //Search pczTo
        if ( FindText( pczTo , TRUE , TRUE ) != FALSE )
        {
            int nStopBeg ;
            int nStopEnd ;
            // ----
            //Get the first character of pczFom : nStopBeg
            //and the last character of pczFom : nStopEnd
            GetEditCtrl().GetSel( nStopBeg , nStopEnd ) ;
            GetEditCtrl().SetSel( nStartBeg , nStopEnd + 1 ) ;
            return TRUE ;
        }
    }
    return FALSE ;
}

/////////////////////////////////////////////////////////////////////////////
// CMarkupPadView drawing
// --------------------------------------------------------------------------

void CMarkupPadView::OnDraw( CDC * /* pDC */ )
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CMarkupPadView message handlers
// --------------------------------------------------------------------------

