/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// DOMStructure.cpp: implementation of the CDOMStructure class.
//
//////////////////////////////////////////////////////////////////////

#include "DOMStructure.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
//#define new DEBUG_NEW
#endif

#define STR_ROOT        "ROOT(0)"

//--------------------------------------------------------------------

CNode::CNode()
{
    m_pczAddress    = NULL ;
    m_pParent       = NULL ;
    m_nType         = NodeRoot ;
}

//--------------------------------------------------------------------

CNode::~CNode()
{
    if( m_pczAddress )
        delete (char*) m_pczAddress ;
}

//--------------------------------------------------------------------

void CNode::SetAddress( char* pczAddress )
{
    if( m_pczAddress )
    {
        delete (char*) m_pczAddress ;
        m_pczAddress    = NULL ;
    }

    m_pczAddress = new char[strlen(pczAddress ) + 1] ;
    strcpy( m_pczAddress , pczAddress ) ;
}

//--------------------------------------------------------------------

CText::CText()
{
    m_pczText = NULL ;
    m_nType    = NodeText ;
}

//--------------------------------------------------------------------

CText::~CText()
{
    if( m_pczText )
        delete (char*) m_pczText  ;
}

//--------------------------------------------------------------------

void CText::SetText( char* pczText )
{
    if( m_pczText )
    {
        delete (char*) m_pczText ;
        m_pczText = NULL ;
    }

    m_pczText = new char[ strlen( pczText ) + 1 ] ;
    strcpy( m_pczText , pczText ) ;
}

//--------------------------------------------------------------------

CAttribute::CAttribute()
{
    m_pczName  = NULL ;
    m_pczValue = NULL ;
    m_nType    = NodeAttribute ;
}

//--------------------------------------------------------------------

CAttribute::~CAttribute()
{
    if( m_pczName )
        delete (char*) m_pczName ;
    if( m_pczValue )
        delete (char*) m_pczValue ;
}

//--------------------------------------------------------------------

void CAttribute::SetName( char* pczName )
{
    if( m_pczName )
    {
        delete (char*) m_pczName ;
        m_pczName  = NULL ;
    }
    m_pczName = new char[ strlen( pczName ) + 1 ] ;
    strcpy( m_pczName , pczName ) ;
}

//--------------------------------------------------------------------

void CAttribute::SetValue( char* pczValue )
{
    if( m_pczValue )
    {
        delete (char*) m_pczValue ;
        m_pczValue = NULL ;
    }
    m_pczValue = new char[ strlen( pczValue ) + 1 ] ;
    strcpy( m_pczValue , pczValue ) ;
}


//--------------------------------------------------------------------

CAttributeList::CAttributeList()
{
    m_pAttribute = NULL ;
    m_pNextAttributeList = NULL ;
}

//--------------------------------------------------------------------

CAttributeList::~CAttributeList()
{}

//--------------------------------------------------------------------


CTextList::CTextList()
{
    m_pText = NULL ;
    m_pNextTextList = NULL ;
}

//--------------------------------------------------------------------

CTextList::~CTextList()
{}

//--------------------------------------------------------------------


CTag::CTag(HTMLType eType)
{
    Init(eType);
}



//--------------------------------------------------------------------

CTag::CTag(HTMLType eType , char* pczName)
{
    Init(eType);
    SetName( pczName );
}


//--------------------------------------------------------------------

CTag::~CTag()
{
    if( m_pczName )
        delete (char*) m_pczName ;
}

//--------------------------------------------------------------------

CTag::Init(HTMLType eType)
{
    m_pParent          = NULL ;
    m_nType            = eType ;
    m_nLastChildIndex  = 0 ;
    m_pczName          = NULL ;

    m_pTagList         = new CTagList() ;
    m_pAttributesList  = new CAttributeList() ;
    m_pTextList        = new CTextList() ;
}

//--------------------------------------------------------------------

void CTag::SetName( char* pczName )
{
    if( m_pczName )
    {
        delete (char*) m_pczName ;
        m_pczName          = NULL ;
    }
    m_pczName = new char[ strlen( pczName ) + 1 ] ;
    strcpy( m_pczName , pczName ) ;
}

//--------------------------------------------------------------------

CTagList::CTagList()
{
    m_pTag      = NULL ;
    m_pNextTagList  = NULL ;
}

//--------------------------------------------------------------------

CTagList::~CTagList()
{}


//--------------------------------------------------------------------


CDOMStructure::CDOMStructure()
{
    Init() ;
}

//--------------------------------------------------------------------

CDOMStructure::~CDOMStructure()
{
    Reset() ;
}

//--------------------------------------------------------------------

void CDOMStructure::Reset()
{
    if( m_pCurrentParent )
    {
        while( m_pCurrentParent->m_nType != NodeRoot )
            m_pCurrentParent = (CTag*) m_pCurrentParent->m_pParent ;
    }

    Reset( m_pCurrentParent->m_pTagList );
    Reset( m_pCurrentParent->m_pAttributesList );
    Reset( m_pCurrentParent->m_pTextList ) ;
    delete m_pCurrentParent ;
}

//--------------------------------------------------------------------

void CDOMStructure::Reset( CTextList* pTextList )
{
    if( !pTextList )
        return ;

    if( pTextList->m_pNextTextList )
        Reset( pTextList->m_pNextTextList ) ;
    if( pTextList->m_pText )
        delete pTextList->m_pText ;

    delete pTextList ;
}

//--------------------------------------------------------------------

void CDOMStructure::Reset( CAttributeList* pAttributeList )
{
    if( !pAttributeList )
        return ;

    if( pAttributeList->m_pNextAttributeList )
        Reset( pAttributeList->m_pNextAttributeList ) ;
    if( pAttributeList->m_pAttribute )
        delete pAttributeList->m_pAttribute;

    delete pAttributeList ;
}

//--------------------------------------------------------------------

void CDOMStructure::Reset( CTagList* pTagList )
{
    if( !pTagList )
        return ;

    if( pTagList->m_pNextTagList )
        Reset( pTagList->m_pNextTagList ) ;
    if( pTagList->m_pTag )
    {
        Reset( pTagList->m_pTag->m_pTagList ) ;
        Reset( pTagList->m_pTag->m_pAttributesList );
        Reset( pTagList->m_pTag->m_pTextList ) ;
        delete pTagList->m_pTag;
    }

    delete pTagList ;
}

//--------------------------------------------------------------------

void CDOMStructure::Init()
{
    m_pCurrentParent = new CTag(NodeRoot , STR_ROOT) ; //Root node is a tag.
    //---
    m_pCurrentParent->SetAddress( STR_ROOT ) ;

}

//--------------------------------------------------------------------

void CDOMStructure::SetTagAddress( int nIndex , CTag* pTag )
{
    char pczBuffer[10] ;
    char* pczAddress = NULL ;
    //----
    sprintf( pczBuffer,"(%d)", nIndex );
    pczAddress = new char[ strlen(m_pCurrentParent->m_pczAddress) + strlen(pTag->m_pczName) + strlen( pczBuffer) + 2];
    strcpy( pczAddress , m_pCurrentParent->m_pczAddress ) ;
    strcat( pczAddress , "\\");
    strcat( pczAddress , pTag->m_pczName ) ;
    strcat( pczAddress , pczBuffer ) ;
    pTag->SetAddress( pczAddress ) ;
    if( pczAddress )
        delete (char*) pczAddress ;
}

//--------------------------------------------------------------------

void CDOMStructure::SetAttributeAddress( int nIndex , CAttribute* pAttribute )
{
    char pczBuffer[10] ;
    char* pczAddress = NULL ;
    //----
    sprintf( pczBuffer,"(%d)", nIndex );
    pczAddress = new char[ strlen(m_pCurrentParent->m_pczAddress) + strlen(pAttribute->m_pczName) + strlen( pczBuffer) + 2];
    strcpy( pczAddress , m_pCurrentParent->m_pczAddress ) ;
    strcat( pczAddress , "\\");
    strcat( pczAddress , pAttribute->m_pczName ) ;
    strcat( pczAddress , pczBuffer ) ;
    pAttribute->SetAddress( pczAddress ) ;
    if( pczAddress )
        delete (char*) pczAddress ;
}

//--------------------------------------------------------------------

void CDOMStructure::SetTextAddress( int nIndex , CText* pText )
{
    char pczBuffer[10] ;
    char* pczAddress = NULL ;
    //----
    sprintf( pczBuffer,"Text(%d)", nIndex );
    pczAddress = new char[ strlen(m_pCurrentParent->m_pczAddress)  + strlen( pczBuffer) + 2];
    strcpy( pczAddress , m_pCurrentParent->m_pczAddress ) ;
    strcat( pczAddress , "\\");
    strcat( pczAddress , pczBuffer ) ;
    pText->SetAddress( pczAddress ) ;
    if( pczAddress )
        delete (char*) pczAddress ;
}

//--------------------------------------------------------------------

void CDOMStructure::InitParentTagList( CTag* pTag )
{
    //init child tag list for parent
    if( m_pCurrentParent->m_pTagList->m_pTag == NULL )
    {
        m_pCurrentParent->m_pTagList->m_pTag = pTag ;
        m_pCurrentParent->m_pTagList->m_pNextTagList = new CTagList() ;
    }
    else
    {
        CTagList *pTemp ;
        //----
        pTemp = m_pCurrentParent->m_pTagList->m_pNextTagList ;

        while( pTemp && pTemp->m_pTag )
        {
            pTemp = pTemp->m_pNextTagList ;
        }

        pTemp->m_pTag = pTag ;
        pTemp->m_pNextTagList = new CTagList() ;
    }

}

//--------------------------------------------------------------------

void CDOMStructure::InitParentAttributeList( CAttribute* pAttribute )
{
    //init child tag list for parent
    if( m_pCurrentParent->m_pAttributesList->m_pAttribute == NULL )
    {
        m_pCurrentParent->m_pAttributesList->m_pAttribute = pAttribute ;
        m_pCurrentParent->m_pAttributesList->m_pNextAttributeList = new CAttributeList() ;
    }
    else
    {
        CAttributeList *pTemp ;
        //----
        pTemp = m_pCurrentParent->m_pAttributesList->m_pNextAttributeList ;

        while( pTemp && pTemp->m_pAttribute )
        {
            pTemp = pTemp->m_pNextAttributeList ;
        }

        pTemp->m_pAttribute = pAttribute ;
        pTemp->m_pNextAttributeList = new CAttributeList() ;
    }

}

//--------------------------------------------------------------------

void CDOMStructure::InitParentTextList( CText* pText )
{
    //init child tag list for parent
    if( m_pCurrentParent->m_pTextList->m_pText == NULL )
    {
        m_pCurrentParent->m_pTextList->m_pText = pText ;
        m_pCurrentParent->m_pTextList->m_pNextTextList = new CTextList() ;
    }
    else
    {
        CTextList *pTemp ;
        //----
        pTemp = m_pCurrentParent->m_pTextList->m_pNextTextList ;

        while( pTemp && pTemp->m_pText )
        {
            pTemp = pTemp->m_pNextTextList ;
        }

        pTemp->m_pText = pText ;
        pTemp->m_pNextTextList = new CTextList() ;
    }

}

//--------------------------------------------------------------------

bool CDOMStructure::InsertTagChild( char* pczName )
{
    CTag* pTag = new CTag(NodeTag) ;
    //----
    //1 - Name
    if( pczName )
        pTag->SetName( pczName) ;

    //3 - Parent
    pTag->m_pParent = m_pCurrentParent ;

    //4 - Index
    m_pCurrentParent->m_nLastChildIndex ++ ;

    //5 - Address
    SetTagAddress( m_pCurrentParent->m_nLastChildIndex , pTag );


    //6 - Init Parent Child list
    InitParentTagList( pTag ) ;

    //7 - Init New Parent
    m_pCurrentParent = pTag ;


    return true ;

}

//--------------------------------------------------------------------

bool CDOMStructure::InsertAttributeChild( char* pczName , char* pczValue )
{
    CAttribute* pAttribute = new CAttribute() ;
    //----
    //1 - Name
    if( pczName )
        pAttribute->SetName( pczName ) ;

    //3 - Value
    if( pczValue )
        pAttribute->SetValue( pczValue ) ;

    //4 - Parent
    pAttribute->m_pParent = m_pCurrentParent ;

    //5 - Init Parent Child Attribute list
    InitParentAttributeList( pAttribute ) ;

    //6 - Index
    m_pCurrentParent->m_nLastChildIndex ++ ;

    //Address
    SetAttributeAddress( m_pCurrentParent->m_nLastChildIndex , pAttribute ) ;


    return true ;

}

//--------------------------------------------------------------------

bool CDOMStructure::InsertTextChild( char* pczText )
{
    CText* pText = new CText() ;
    //----

    //2 - Value
    if( pczText )
        pText->SetText( pczText) ;

    //3 - Parent
    pText->m_pParent = m_pCurrentParent ;

    //4 - Init Parent Child Text list
    InitParentTextList( pText );

    //5 - Index
    m_pCurrentParent->m_nLastChildIndex ++ ;

    //6 - Address
    SetTextAddress( m_pCurrentParent->m_nLastChildIndex , pText ) ;

    return true ;

}

//--------------------------------------------------------------------

bool CDOMStructure::EndTag( char* pczName )
{
    CTag *pParent = m_pCurrentParent ;
    //---
    if( stricmp( pczName , "\0" ) == 0 )
    {
        m_pCurrentParent = (CTag*) m_pCurrentParent->m_pParent ;
        return true ;
    }
    else
    {
        while( pParent->m_nType != NodeRoot )
        {
            if( stricmp( pParent->m_pczName , pczName ) == 0 )
            {
                m_pCurrentParent = (CTag*) pParent->m_pParent ;
                return true ;
            }
            else
            {
                pParent = (CTag*) pParent->m_pParent ;
            }

        }
        return false ;
    }
}

//--------------------------------------------------------------------

CTag* CDOMStructure::GetRootItem()
{
    CTag* pTag =  m_pCurrentParent ;
    while( pTag->m_nType != NodeRoot )
        pTag = (CTag*) pTag->m_pParent ;

    return pTag ;
}

//--------------------------------------------------------------------

bool CDOMStructure::GetValue( const char* pczAddress , char*& pczValue )
{
    CTag* pTag = GetRootItem() ;
    //---
    if( pTag == NULL )
        return false ;

    return GetTagValue( pTag->m_pTagList , pczAddress , pczValue ) ;
}

//--------------------------------------------------------------------

bool CDOMStructure::GetTagValue( CTagList* pTagList , const char* pczAddress , char*& pczValue )
{
    if( !pTagList )
        return false ;

    CTag* pTag = pTagList->m_pTag ;
    //---
    if( !pTag )
        return false ;

    //
    if( stricmp( pTag->m_pczAddress , pczAddress ) == 0 )
    {
        pczValue = new char[strlen(pTag->m_pczName) + 1];
        strcpy(pczValue , pTag->m_pczName ) ;
        return true ;
    }

    //
    if( GetAttributeValue( pTag->m_pAttributesList , pczAddress , pczValue  ) )
        return true ;
    if( GetTextValue( pTag->m_pTextList , pczAddress , pczValue  ) )
        return true ;
    if( GetTagValue( pTag->m_pTagList , pczAddress , pczValue ) )
        return true ;

    return GetTagValue( pTagList->m_pNextTagList , pczAddress , pczValue ) ;
}

//--------------------------------------------------------------------

bool CDOMStructure::GetAttributeValue( CAttributeList* pAttributeList , const char* pczAddress , char*& pczValue )
{
    if( !pAttributeList )
        return false ;

    CAttribute* pAttribute = pAttributeList->m_pAttribute ;
    if( pAttribute )
    {
        if( stricmp( pAttribute->m_pczAddress , pczAddress ) == 0 )
        {
            pczValue = new char[strlen(pAttribute->m_pczValue) + 1];
            int iLen = strlen(pAttribute->m_pczValue) ;
            if (pAttribute->m_pczValue[0] == '"' && pAttribute->m_pczValue[iLen - 1] == '"')
            {
                strncpy( pczValue , &pAttribute->m_pczValue[1], iLen - 2 ) ;
                pczValue[iLen - 2] = 0 ;
            }
            else
                strcpy( pczValue , pAttribute->m_pczValue ) ;
            return true ;
        }
        else
            return GetAttributeValue( pAttributeList->m_pNextAttributeList , pczAddress , pczValue ) ;
    }
    else
        return GetAttributeValue( pAttributeList->m_pNextAttributeList , pczAddress , pczValue ) ;

}

//--------------------------------------------------------------------

bool CDOMStructure::GetTextValue( CTextList* pTextList , const char* pczAddress , char*& pczValue )
{
    if( !pTextList )
        return false ;
    else
    {
        CText* pText = pTextList->m_pText ;
        if( pText )
        {
            if( stricmp( pText->m_pczAddress , pczAddress ) == 0 )
            {
                pczValue = new char[strlen(pText->m_pczText) + 1];
                int iLen = strlen(pText->m_pczText) ;
                if (pText->m_pczText[0] == '"' && pText->m_pczText[iLen - 1] == '"')
                {
                    strncpy( pczValue , &pText->m_pczText[1], iLen - 2 ) ;
                    pczValue[iLen - 2] = 0 ;
                }
                else
                    strcpy( pczValue , pText->m_pczText ) ;
                return true ;
            }
            else
                return GetTextValue( pTextList->m_pNextTextList , pczAddress , pczValue ) ;
        }
        else
            return GetTextValue( pTextList->m_pNextTextList , pczAddress , pczValue ) ;
    }

}

