/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

//==================================================================< Pragmas >


//=================================================================< Includes >
#include <fstream>
using std::ifstream;
using std::istream;
#include <iostream>
using std::cin;

#ifndef __DEFINITION__
#  include "definition.h"
#endif  // __DEFINITION__

#ifndef __EXCEPTION__
#  include <exception.hpp>
#endif  // __EXCEPTION__

#ifndef __MUTEX__
#  include <mutex.hpp>
#endif  // __MUTEX__

#ifndef __SOCKET__
#  include <socket.hpp>
#endif  // __SOCKET__

#ifndef __GATEWAY_CONSOLE_COMMAND__
#  define DLLSPEC_GWCONSCMD
#  include <gwconscmd.hpp>
#endif  // __GATEWAY_CONSOLE_COMMAND__


//====================================================================< Class >
///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////// Class CGConsoleException //
///////////////////////////////////////////////////////////////////////////////
// Class used to keep console exception.
class CGConsoleException : public CTException
{
    //-----------------------------------------------------------------------------
public:                     // Constructor(s) and destructor.
    CGConsoleException (const string& clMsg, const string& clFilename, short sLineno, long lNumber = -1L)
: CTException (clMsg, clFilename, sLineno, lNumber)
    { }


public:                     // Operators.
    void                    operator= (const CGConsoleException& exception)
    {
        CTException::operator= (exception) ;
    }

    //-----------------------------------------------------------------------------
}

; // CGConsoleException

///////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////// End of Class CGConsoleException //
///////////////////////////////////////////////////////////////////////////////

//=========================================================< Global Variables >
string              gatewayName ;


//=========================================================< Global Functions >
void parseCommand (CTSocket& socket, istream& in) ;

bool help (CTSocket&, vector<string>& command)
{
    if ( (command.front () != "help") && (command.front () != "?") )
    {
        throw CGConsoleException (string ("help: ") + command.front () + " is not help command.", __FILE__, __LINE__) ;
    }
    if (command.size () > 2)
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("help: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }
    if (command.size () == 1)
    {
        cout << "who           listen" << endl ;
        cout << "kill          quit,exit" << endl ;
        cout << "shutdown      trace" << endl ;
        cout << "run           rem,#" << endl ;
        cout << "capture" << endl ;
        cout << "help,?" << endl ;
        cout << "type help <command> for more help on a command." << endl ;
    }
    else
    {
        for (int i=0; i<command[1].size(); i++)
            command[1][i] = tolower(command[1][i]);
        if (command[1] == "who")
        {
            cout << "list the connections." << endl ;
        }
        else if (command[1] == "listen")
        {
            cout << "listen for administration [at <port>] [using <library>] :" << endl ;
            cout << "create a new administration listen task." << endl ;
            cout << "  <port>      : port to listen on." << endl ;
            cout << "  <library>   : library to use to parse the commands." << endl ;
            cout << "listen [for [default] <host:port>] [at <port>] [using <library>] :" << endl ;
            cout << "create a new gateway listen task." << endl ;
            cout << "  default     : <host:port> is used if clients can not specify the tcp address server to connect to." << endl ;
            cout << "  <host:port> : tcp address server to connect to." << endl ;
            cout << "  <port>      : port to listen on." << endl ;
            cout << "  <library>   : library to use to parse the packets." << endl ;
        }
        else if (command[1] == "kill")
        {
            cout << "kill [-r] <task id> :" << endl ;
            cout << "kill a task." << endl ;
            cout << "  -r        : kill the specified task and its children." << endl ;
            cout << "  <task id> : task to kill." << endl ;
        }
        else if (command[1] == "shutdown")
        {
            cout << "shutdown [with no wait] :" << endl ;
            cout << "shutdown the gateway." << endl ;
            cout << "  with no wait : shutdown imediately." << endl ;
        }
        else if ( (command[1] == "quit") || (command[1] == "exit") )
        {
            cout << "terminate this task." << endl ;
        }
        else if (command[1] == "trace")
        {
            cout << "trace <on|off> :" << endl ;
            cout << "activate or desactivate trace mode." << endl ;
        }
        else if (command[1] == "run")
        {
            cout << "run <command file> :" << endl ;
            cout << "execute commands from the file." << endl ;
        }
        else if ( (command[1] == "rem") || (command[1] == "#") )
        {
            cout << "rem (or #) <string> :" << endl ;
            cout << "comments." << endl ;
        }
        else if ( command[1] == "capture" )
        {
            cout << "capture <Oracle server name> record in <\"tof file name\"> [only/multi] [tof file title]" << endl ;
            cout << "capture Oracle statements and save them in the specified tof file." << endl ;
            cout << "  <Oracle server name> : Oracle target server name." << endl ;
            cout << "  <\"tof file name\">    : tof file name (in brackets)." << endl ;
            cout << "  [only]               : One tof for all the connection(s)." << endl ;
            cout << "  [tof file title]     : Tof file title." << endl ;
            cout << "capture <Oracle server name> stop|pause|resume" << endl ;
            cout << "  stop   : stop the capture." << endl;
            cout << "  pause  : pause the capture." << endl;
            cout << "  resume : resume the capture." << endl;
            cout << "capture <Oracle server name> comment <comment string>" << endl ;
        }
        else if ( (command[1] == "help") || (command[1] == "?") )
        {
            cout << "help (or ?) [<command>] : display help for." << endl ;
        }
        else
        {
            throw CGConsoleException (string ("help: ") + command[1] + " is not a valid command.", __FILE__, __LINE__) ;
        }
    }

    return true ;
} // help

bool who (CTSocket& socket, vector<string>& command)
{
    if (command.front () != "who")
    {
        throw CGConsoleException (string ("who: ") + command.front () + " is not a who command.", __FILE__, __LINE__) ;
    }
    if (command.size () != 1)
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("who: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }

    socket << string ("CONSOLE:WHO") ;

    cout << "-------------------------------------------------------------------------------" << endl ;
    cout << "  Id  + PId  + Type    + Client Address     + Server Address     + Library     " << endl ;
    cout << "-------------------------------------------------------------------------------" << endl ;

    CGWhoList   whoList ;

    socket >> whoList ;
    for (size_t i = 0 ; i < whoList.size () ; ++ i)
    {
        string  id ;
        string  parentId ;
        string  type ;
        string  clientAddress ;
        string  serverAddress ;
        string  library ;

        char szBuf[6] ;
        sprintf (szBuf, "%4hu", (unsigned short) whoList[i].getTaskId () ) ;
        id = szBuf ;

        if (whoList[i].getParentTaskId () )
        {
            sprintf (szBuf, "%4hu", (unsigned short) whoList[i].getParentTaskId () ) ;
            parentId = szBuf ;
        }
        else
        {
            parentId = "    " ;
        }

        type = (char*) ( (whoList[i].getType () == 0) ? "Control" : ( (whoList[i].getType () == 1) || (whoList[i].getType () == 3) ) ? "Listen " : "Client ") ;

        if ( (whoList[i].getType () != 1) && (whoList[i].getType () != 3) )
        {
            clientAddress = whoList[i].getClientAddress () ;
        }
        else
        {
            clientAddress = whoList[i].getGatewayAddress () ;
        }
        serverAddress = whoList[i].getTargetAddress () ;
        library = whoList[i].getLibrary () ;

        while (clientAddress.size () < 18)  clientAddress += ' ' ;
        if (clientAddress.size () > 18)     clientAddress = clientAddress.substr (0, 18) ;

        while (serverAddress.size () < 18)  serverAddress += ' ' ;
        if (serverAddress.size () > 18)     serverAddress = serverAddress.substr  (0, 18) ;

        while (library.size () < 12)            library += ' ' ;
        if (library.size () > 12)               library = library.substr  (0, 12) ;

        cout << " " << id << "   " << parentId << "   " << type << "   " << clientAddress << "   " << serverAddress << "   " << library << endl ;
    }

    cout << "-------------------------------------------------------------------------------" << endl ;
    cout << " Number of jobs: " << whoList.size () << "." << endl ;

    string  result ;
    socket >> result ;

    if (result != "OK")
    {
        cout << "error receveing jobs list." << endl ;
    }

    return true ;
} // who

bool trace (CTSocket& socket, vector<string>& command)
{
    if (command.front () != "trace")
    {
        throw CGConsoleException (string ("trace: ") + command.front () + " is not a trace command.", __FILE__, __LINE__) ;
    }
    if (command.size () != 2)
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("trace: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }
    for (int i=0; i<command[1].size(); i++)
        command[1][i] = tolower(command[1][i]);
    if ( (command[1] != "on") && (command[1] != "off") )
    {
        throw CGConsoleException (string ("trace: ") + command[1] + " is not an invalid trace argument.", __FILE__, __LINE__) ;
    }

    socket << string ("CONSOLE:TRACE") ;

    CGTrace     trace (command[1] == "on") ;
    string  result ;

    socket << trace ;
    socket >> result ;

    if (result == "OK")
    {
        cout << "trace " << command[1] << " is ok." << endl ;
    }
    else
    {
        cout << "trace " << command[1] << " failed." << endl ;
    }

    return true ;
} // trace

bool listen (CTSocket& socket, vector<string>& command)
{
    if (command.front () != "listen")
    {
        throw CGConsoleException (string ("listen: ") + command.front () + " is not a listen command.", __FILE__, __LINE__) ;
    }
    if ( (command.size () < 3) && (command.size () > 8) )
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("listen: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }

    socket << string ("CONSOLE:LISTEN") ;

    bool        bAdministration = false ;
    string  targetAddress ;
    string  listenPort ;
    string  library ;
    bool        bTargetAddressAutoDetection = false ;

    for (unsigned i = 1 ; i < command.size () ; i += 2)
    {
        for (int j=0; j<command[i].size(); j++)
            command[i][j] = tolower(command[i][j]);
        if (command[i] == "for")
        {
            if (command[i + 1] == "administration")
            {
                bAdministration = true ;
                ++ i ;
            }
            else if (command[i + 1] == "default")
            {
                bTargetAddressAutoDetection = true ;
                targetAddress = command[i + 2] ;
                ++ i ;
            }
            else
            {
                targetAddress = command[i + 1] ;
            }
        }
        else if (command[i] == "at")
        {
            listenPort = command[i + 1] ;
        }
        else if (command[i] == "using")
        {
            library = command[i + 1] ;
        }
        else
        {
            throw CGConsoleException (string ("listen: ") + command[i] + " is not a valid listen argument.", __FILE__, __LINE__) ;
        }
    }

    CGListen    listen (bAdministration, targetAddress, listenPort, library, bTargetAddressAutoDetection) ;
    string  result ;

    socket << listen ;
    socket >> result ;

    if (result == "OK")
    {
        cout << "new listening task created." << endl ;
    }
    else
    {
        cout << "creation of listening task failed." << endl ;
    }

    return true ;
} // listen

bool kill (CTSocket& socket, vector<string>& command)
{
    if (command.front () != "kill")
    {
        throw CGConsoleException (string ("kill: ") + command.front () + " is not a kill command.", __FILE__, __LINE__) ;
    }
    if ( (command.size () != 2) && (command.size () != 3) )
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("kill: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }

    socket << string ("CONSOLE:KILL") ;

    bool            bChildrenAlso = false ;
    long            lTaskId = 0 ;

    for (unsigned i = 1 ; i < command.size () ; ++ i)
    {
        for (int j=0; j<command[i].size(); j++)
            command[i][j] = tolower(command[i][j]);
        if (command[i] == "-r")
        {
            bChildrenAlso = true ;
        }
        else
        {
            sscanf (command[i].data (), "%ld", &lTaskId) ;
        }
    }

    CGKill      kill (lTaskId, bChildrenAlso) ;
    string  result ;

    socket << kill ;
    socket >> result ;

    if (result == "OK")
    {
        cout << "task killed." << endl ;
    }
    else
    {
        cout << "killing task failed." << endl ;
    }

    return true ;
} // kill

bool shutdown (CTSocket& socket, vector<string>& command)
{
    if (command.front () != "shutdown")
    {
        throw CGConsoleException (string ("shutdown: ") + command.front () + " is not a shutdown command.", __FILE__, __LINE__) ;
    }
    if ( (command.size () != 1) && (command.size () != 4) )
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("shutdown: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }

    socket << string ("CONSOLE:SHUTDOWN") ;

    bool    bWithNoWait = false ;

    if (command.size () == 4)
    {
        if ( (command[1] == "with") && (command[2] == "no") && (command[3] == "wait") )
        {
            bWithNoWait = true ;
        }
        else
        {
            throw CGConsoleException (string ("shutdown: ") + command[1] + " " + command[2] + " " + command[3] + " is a bad argument.", __FILE__, __LINE__) ;
        }
    }

    CGShutdown  shutdown (bWithNoWait) ;
    string  result ;

    socket << shutdown ;
    socket >> result ;

    if (result == "OK")
    {
        cout << "shutdown is ok." << endl ;
    }
    else
    {
        cout << "shutdown failed." << endl ;
    }

    return !bWithNoWait ;
} // shutdown

bool rem (CTSocket&, vector<string>& command)
{
    if ( (command.front () != "rem") && (command.front () != "#") )
    {
        throw CGConsoleException (string ("rem: ") + command.front () + " is not a comment command.", __FILE__, __LINE__) ;
    }

    return true ;
} // rem

bool run (CTSocket& socket, vector<string>& command)
{
    if (command.front () != "run")
    {
        throw CGConsoleException (string ("run: ") + command.front () + " is not run command.", __FILE__, __LINE__) ;
    }
    if (command.size () != 2)
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("run: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }
    ifstream    commandFile (command[1].c_str(), ios::in) ;
    if (!commandFile)
    {
        throw CGConsoleException (string ("run: ") + command[1] + " is a bad file name.", __FILE__, __LINE__) ;
    }
    cout << "execute commands from " << command[1] << " ..." << endl ;
    parseCommand (socket, commandFile) ;
    cout << "ok." << endl ;

    return true ;
} // run

bool quit (CTSocket& socket, vector<string>& command)
{
    if ( (command.front () != "quit") && (command.front () != "exit") )
    {
        throw CGConsoleException (string ("quit: ") + command.front () + " is not a quit command.", __FILE__, __LINE__) ;
    }
    if (command.size () != 1)
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("quit: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }

    socket << string ("CONSOLE:GOODBYE") ;

    return false ;
} // quit


bool capture (CTSocket& socket, vector<string>& command)
{
    if (command.front () != "capture")
    {
        throw CGConsoleException (string ("capture: ") + command.front () + " is not a capture command.", __FILE__, __LINE__) ;
    }
    if (command.size () < 3)
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("capture: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }

    socket << string ("SELECT:CAPTURE") ;

    socket << command[1] ; // server name
    socket << command[2] ; // RECORD, STOP, PAUSE, RESUME

    // RECORD command
    if (command[2] == "record")
    {
        if (command[3] != "in" || command[4][0] != '\"')
        {
            throw CGConsoleException (string ("capture: ") + command.front () + " is not a capture command.", __FILE__, __LINE__) ;
        }

        // Tof File Name
        string tofFile = "";
        unsigned i = 4;
        if (command[4][command[4].size() - 1] == '\"')
            tofFile = command[4].substr(1, command[4].size() - 2);
        else
        {
            tofFile = command[4].substr(1, command[4].size() - 1);
            i = 5;
            while (i <= command.size() - 1 && command[i][command[i].size() - 1] != '\"')
            {
                tofFile += " ";
                tofFile += command[i];
                i++;
            }
            tofFile += " ";
            tofFile += command[i].substr(0, command[i].size() - 1);
        }
        socket << tofFile; // tof File name

        i ++;
        if (i <= command.size() - 1)
        {
            unsigned j = i;
            if (command[j] == "only")
            {
                socket << string("only");    // One tof
                j++;
            }
            else
                socket << string("multi");  // multi tof

            string message = " ";
            for (i = j; i < command.size(); i++)
            {
                message += command[i];
                message += " ";
            }
            socket << message; // message
        }

        else
        {
            socket << string("multi");
            socket << string(" ");
        }
    }
    else if (command[2] == "comment")
    {
        string comment = " ";
        for (unsigned i = 3; i < command.size(); i++)
        {
            comment += command[i];
            comment += " ";
        }
        socket << comment; // message
    }


    string result = "";
    socket >> result;
    if (result == "OK")
        cout << "ok." << endl ;
    else
        cout << "failed." << endl;

    return true ;
} // capture

bool webconnection(CTSocket& socket, vector<string>& command)
{
    if (command.front () != "webconnection")
    {
        throw CGConsoleException (string ("webconnection: ") + command.front () + " is not a webconnection command.", __FILE__, __LINE__) ;
    }
    if (command.size () < 3)
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("webconnection: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }

    socket << string ("SELECT:WEBCONNECTION");

    socket << command[1] ; // proxy parameters
    socket << command[2] ; // exceptions
    socket << command[3] ; // no_proxy_for_local or proxy_for_local

    string result = "";
    socket >> result;
    if (result == "OK")
        cout << "ok." << endl ;
    else
        cout << "failed." << endl;

    return true ;
} // webconnections


bool setDocumentsPath(CTSocket& socket, vector<string>& command)
{
    if (command.front () != "setdocumentspath")
    {
        throw CGConsoleException (string ("setdocumentspath: ") + command.front () + " is not a webconnection command.", __FILE__, __LINE__) ;
    }
    if (command.size () < 2)
    {
        char szBuf[6] ;
        sprintf (szBuf, "%u", (unsigned int) command.size () ) ;
        throw CGConsoleException (string ("setDocumentsPath: ") + szBuf + " is a bad number of arguments.", __FILE__, __LINE__) ;
    }

    socket << string ("SELECT:SETDOCUMENTSPATH");
    socket << command[1] ; // path

    string  result ;
    socket >> result ;

    return true ;
} // webconnections


void parseCommand (CTSocket& socket, istream& in)
{
    string  cmdLine ;
    bool        bContinue = true ;
    char zBuf[1024];

    while (bContinue)
    {
        try
        {
            if (in.eof () )
            {
                bContinue = false ;
                continue ;
            }

            cout << gatewayName << "> " << std::flush ;

            if (!in)
            {
                throw CTException (string ("parseCommand: ") + " error on input stream.", __FILE__, __LINE__, (long) in.rdstate () ) ;
            }
            in.getline(zBuf, sizeof(zBuf));
            cmdLine = zBuf;
            if (cmdLine.empty () )
            {
                cout << endl ;
                continue ;
            }
            if (in != std::cin)
            {
                cout << cmdLine << endl ;
            }

            vector<string>  command ;

            while (!cmdLine.empty () )
            {
                if ( (cmdLine [0] == ' ') || (cmdLine [0] == '\t') )
                {
                    cmdLine.erase (0, 1) ;
                    continue ;
                }
                size_t  iIndex = cmdLine.find_first_of (" \t") ;
                if (iIndex == string::npos)
                {
                    iIndex = cmdLine.size () ;
                }
                command.push_back (cmdLine.substr(0, iIndex) ) ;
                cmdLine.erase (0, iIndex) ;
            }

            for (int j=0; j<command.front ().size(); j++)
                command.front ()[j] = tolower(command.front ()[j]);

            if ( (command.front () == "help") || (command.front () == "?") )
            {
                bContinue = help (socket, command) ;
            }
            else if (command.front () == "who")
            {
                bContinue = who (socket, command) ;
            }
            else if (command.front () == "trace")
            {
                bContinue = trace (socket, command) ;
            }
            else if (command.front () == "listen")
            {
                bContinue = listen (socket, command) ;
            }
            else if (command.front () == "kill")
            {
                bContinue = kill (socket, command) ;
            }
            else if (command.front () == "shutdown")
            {
                bContinue = shutdown (socket, command) ;
            }
            else if ( (command.front () == "rem") || (command.front () == "#") )
            {
                bContinue = rem (socket, command) ;
            }
            else if (command.front () == "run")
            {
                bContinue = run (socket, command) ;
            }
            else if ( (command.front () == "quit") || (command.front () == "exit") )
            {
                bContinue = quit (socket, command) ;
            }
            else if (command.front () == "capture")
            {
                bContinue = capture (socket, command) ;
            }
            else if (command.front () == "webconnection")
            {
                bContinue = webconnection(socket, command) ;
            }
            else if (command.front () == "setdocumentspath")
            {
                bContinue = setDocumentsPath(socket, command) ;
            }
            else
            {
                cout << command.front () << " is not a valid command." << endl ;
                cout << "Type ? or help for the list of the valids commands." << endl ;
            }
        }
        catch (CGConsoleException& e)
        {
            cerr << e << endl ;
        }
    }
} // parseCommand

int main (int iArgc, char* zArgv[])
{
    int iRet = 0 ;

    try
    {
        if (iArgc < 3)
        {
            throw CTException ("not enough arguments.\nUsage : gwconsole hostname port [Command DLL]", __FILE__, __LINE__) ;
        }

        port_t  port = 0 ;
        sscanf (zArgv[2], "%hu", &port) ;
        if (port == 0)
        {
            throw CTException (string (zArgv[2]) + " is not a good port number\nUsage : gwconsole hostname port", __FILE__, __LINE__) ;
        }

        gatewayName = string (zArgv[1]) + ":" + zArgv[2] ;

        cout << "Connecting to <" << gatewayName << "> ..." << endl ;
        CTSocket    socket (zArgv[1], port) ;

        string useDll = "CONSOLE:USE ";
        if (iArgc == 4)
            useDll += string(zArgv[3]);
        else
            useDll += "gworacle";

        socket << useDll ;

        string  cmdResult ;
        socket >> cmdResult ;

        if (cmdResult == "FAIL")
        {
            throw CTException (string ("use command failed"), __FILE__, __LINE__) ;
        }

        parseCommand (socket, cin) ;

        cout << "bye !" << endl ;
    }
    catch (CTException& e)
    {
        cerr << e << endl ;
        iRet = (int) e.getNumber () ;
    }

    return iRet ;
} // main
