/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef __CONTEXTCLASS__
#define __CONTEXTCLASS__

#ifdef  WIN32
#pragma warning(disable : 4251 4512 4514 4516 4275)
#endif  // WIN32


//=================================================================< Includes >
#ifndef __DEFINITION__
#include  "definition.h"
#endif  // __DEFINITION__

#include  <fstream>
#include  <cstdlib>
#include <string>
#include <list>
using std::string;
using std::list;
using std::ofstream;

#if ( defined(WIN32) || defined(HPUX) )
#include    <sys/timeb.h>
#endif

#include    <time.h>


#  ifdef    WIN32
#    pragma warning(disable : 4100 4512 4516)
#  endif    // WIN32
#  ifdef    WIN32
#    pragma warning(default : 4100 4512)
#  endif    // WIN32

#include  <exception.hpp>
#include  <cbinary.hpp>
#include <mutex.hpp>


#ifdef BUILDDLLGWHTTP
#  define DLLSPEC_GWHTTP __EXPORT__
#else
#  define DLLSPEC_GWHTTP __IMPORT__
#endif  // BUILDDLLGWHTTP

#define CYRANO_SSL_URL "http://{"
#define CYRANO_SSL_STR "{"

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CContext //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWHTTP CContext
{
public:
    CContext();
    ~CContext();

    void                    setConnectId(unsigned int uiConnectId)
    {
        uiConnectId_ = uiConnectId ;
    }
    void                    setServerName(string serverName)
    {
        serverName_ = serverName;
    }
    void                    setPort(int iPort)
    {
        iPort_ = iPort ;
    }
    void                    setTofStream (ofstream* tofStream)
    {
        tofStream_ = tofStream ;
    }
    void                    setMethod(string method)
    {
        method_ = method;
    }
    void                    setRequestUrl(string requestUrl) ;
    void                    setHttpVersion(string httpVersion)
    {
        httpVersion_ = httpVersion;
    }
    void                    setThroughProxy(bool bThroughProxy)
    {
        bThroughProxy_ = bThroughProxy ;
    }
    void                    setCookieId(int iCookieId)
    {
        iCookieId_ = iCookieId ;
    }
    void                    incCookieId()
    {
        iCookieId_++;
    }
    void                    set100continue(bool b100continue)
    {
        b100continue_ = b100continue;
    }

    unsigned int            getConnectId(void) const
    {
        return uiConnectId_ ;
    }
    const string    &       getServerName(void) const
    {
        return serverName_;
    }
    int                     getPort(void) const
    {
        return iPort_ ;
    }
    ofstream*               getTofStream (void) const
    {
        return tofStream_ ;
    }
    const string    &       getMethod(void) const
    {
        return method_;
    }
    const string    &       getRequestUrl(void) const
    {
        return requestUrl_;
    }
    const string    &       getHttpVersion(void) const
    {
        return httpVersion_;
    }
    bool                getThroughProxy(void) const
    {
        return bThroughProxy_ ;
    }
    int                     getCookieId(void) const
    {
        return iCookieId_ ;
    }
    bool                is100continue() const
    {
        return b100continue_ ;
    }

    bool                operator == (const CContext& context) const ;
    bool                operator < (const CContext& context) const ;

private:
    unsigned int            uiConnectId_ ;
    string              serverName_;
    int                     iPort_ ;
    bool                bThroughProxy_ ;
    string              requestUrl_ ;
    string              httpVersion_ ;
    string              method_ ;
    int                     iCookieId_ ;
    bool                b100continue_ ;

    ofstream*               tofStream_ ;        // Descripteur du fichier tof
}
;

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////// Class CContextList //
///////////////////////////////////////////////////////////////////////////////
typedef list<CContext*> CContextStlList ;
class DLLSPEC_GWHTTP CContextList  : public CTMonitor<CTMutex>
{
    //-----------------------------------------------------------------------------
public:                     // Constructor(s) and destructor.
    CContextList () ;
    ~CContextList () ;

    //-----------------------------------------------------------------------------
public:                     // Control methodes.
    void                    add (CContext* context) ;
    void                    remove (CContext* context) ;
    CContext*               find (CContext* context) ;
    unsigned int            entries (void)
    {
        return list_.size ();
    }
    void                    reset () ;
    //-----------------------------------------------------------------------------
public:                     // Get* methodes.
    const CContextStlList&  getList (void) const
    {
        return list_ ;
    }


    //-----------------------------------------------------------------------------
private:                    // Members.
    CContextStlList list_ ;

    //-----------------------------------------------------------------------------
}

; // CContextList

class DLLSPEC_GWHTTP CHttpException  : public CTException
{
    //-----------------------------------------------------------------------------
public:                     // Constructor(s) and destructor.
    CHttpException (const string& clMsg, const string& clFilename, short sLineno, int lNumber = -1L)
: CTException (clMsg, clFilename, sLineno, lNumber)
    {  }

    //-----------------------------------------------------------------------------
}

; // HttpException

#endif
