/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifdef  WIN32
#pragma warning(disable : 4100 4201 4214 4209 4115 4512 4514 4516 4705 4710)
#endif

#ifndef __CSTOF__
#include  "cstof.hpp"
#endif

#include <list.h>
using std::list;

CTTime              CTofReference::timePreviousWrite_ ;
extern string   gListeDocuments ;
extern int          giSpid ;

CTofReference & CTofReference::operator = (const CTofReference &)
{
    return *this;
}

void CTofReference::addContext(CContext *context)
{
    contextList_.add(context);
}

void CTofReference::removeContext(CContext *context)
{
    contextList_.remove(context);
}

bool CTofReference::analyse(const string & conId, const string & tofFileName, const string & strMode, const string & strCommand, const string & strMessage)
{
    enumSelectStatus status;
    bool oneTof = FALSE;

    if (strCommand == "record")
        status = ESL_RECORD;
    else if (strCommand == "stop")
        status = ESL_STOP;
    else if (strCommand == "pause")
        status = ESL_PAUSE;
    else if (strCommand == "resume")
        status = ESL_RESUME;
    else if (strCommand == "comment")
        status = ESL_COMMENT;

    if (strMode == "only")
        oneTof = TRUE;

    return analyse(tofFileName, oneTof, status, strMessage);
}

bool CTofReference::analyse (const string & tofFileName, bool oneTof, enumSelectStatus selectStatus, const string & message)
{
    CTLockGuard <CTMutex> guard(tofMutex_);
    CContext    *context    =   NULL;
    CContextStlList  liste = contextList_.getList();

    CContextStlList :: iterator ilistedeb=liste.begin();
    size_t lastSlash = string::npos;

    switch(selectStatus)
    {
        //
        // RECORD Action
        //
    case ESL_RECORD :
        gListeDocuments = "" ;
        // The precedent action was record
        if (selectStatus_ == ESL_RECORD)
            break;
        selectStatus_ = ESL_RECORD;
        tofFileName_ = tofFileName;
        bOutOfOrder_ = false ;
        /* tof Timer Name from tof File Name */
        lastSlash = tofFileName_.find_last_of("\\");
        if (lastSlash != string::npos)
        {
            tofTimerName_ = tofFileName_.substr(lastSlash + 1, tofFileName_.length() - lastSlash - 5);
        }
        else
        {
            tofTimerName_ = tofFileName_.substr(0, tofFileName_.length() - 4);
        }

        // We want one TOF file for all connections
        oneTof_ = oneTof;

        // Tof Title
        tofTitle_ = message;

        // Create the common TOF file
        if (oneTof_)
            createTofFile(tofFileName_);

        //
        // If OneTof == TRUE, AssociateTofFile does not create files,
        // else, it creates one file per connecion.
        //

        while (ilistedeb != liste.end())
        {
            if ((*ilistedeb)->getConnectId != 0)
                associateTofFile(*ilistedeb);
            ilistedeb++;
        }
        break;
        //
        // STOP Action
        //
    case ESL_STOP :
        // the precedent action was stop
        if (selectStatus_ == ESL_STOP)
            break;
        CGWHttp::Concatenate() ;

        selectStatus_ = ESL_STOP;

        tofFileName_ = "";
        tofTitle_ = "";
        spid_ = 0;
        timePreviousWrite_.reset() ;
        //
        // Before, there were one TOF far all connections,
        // We close this file.
        //
        if (oneTof_)
        {
            closeTofFile();
            oneTof_ = FALSE;

            while (ilistedeb != liste.end())
            {
                (*ilistedeb)->setTofStream(NULL);
                ilistedeb++;
            }


        }
        //
        // Before, there were one TOF per connection,
        // We close all these files.
        //
        else
        {
            while (ilistedeb != liste.end())
            {
                if ((*ilistedeb)->getConnectId != 0)
                    closeTofFile(*ilistedeb);
                ilistedeb++;
            }
        }
        break;
        //
        // PAUSE Action
        //
    case ESL_PAUSE :
        // the precedent action was pause
        if (selectStatus_ == ESL_PAUSE)
            break;
        selectStatus_ = ESL_PAUSE;
        break;
        //
        // RESUME Action
        //
    case ESL_RESUME :
        // the precedent action was pause
        if (selectStatus_ == ESL_RECORD)
            break;
        selectStatus_ = ESL_RECORD;
        break;
        //
        // COMMENT Action
        //
    case ESL_COMMENT :
        if (message == "")
            break;
        if (oneTof_)
        {
            writeCommentInTofFile(NULL, message);
            break;
        }
        while (ilistedeb != liste.end())
        {
            writeCommentInTofFile((*ilistedeb), message);
            ilistedeb++;
        }
        break;
    }

    return(TRUE);
}


bool CTofReference::createTofFile(string & tofFileName)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    time_t          ltime;
    struct tm       *stm;
    char            Msg[255];
    char            day[12][4] = {  "jan","feb","mar","apr","may","jun",
                                    "jul","aug","sep","oct","nov","dec"};

    tofStream_.open(tofFileName.data(), ios::out | ios::binary);

    if (tofStream_.fail() != 0)
    {
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        string message = "";
        message += "cannot open ";
        message += tofFileName;
        message += "\r\n";
        message += zBuf;
        throw message;
    }

    tofStream_ << "tof.5\r\n" \
    "dbtype 1\r\n" \
    "dbversion 10\r\n";

    time(&ltime);
    stm = localtime(&ltime);

    sprintf(Msg,    "date %02.2d-%s-%d %02.2d:%02.2d:%02.2d\r\n",
            stm->tm_mday, day[stm->tm_mon], stm->tm_year + 1900,
            stm->tm_hour, stm->tm_min, stm->tm_sec);
    if (serverName_ != "FILEGate" )
        tofStream_ << Msg;

    if (tofTitle_ != "")
        tofStream_ << "name " << tofTitle_ << "\r\n";

    tofStream_ << "\r\n";

    tofStream_ << "STimer\r\n" << "name T_" << tofTimerName_ << "\r\n\r" << endl;

    return TRUE;
}

bool CTofReference::createTofFile(CContext *context, string & tofFileName, string & tofTimerName)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    time_t          ltime;
    struct tm       *stm;
    char            Msg[255];
    char            day[12][4] = {  "jan","feb","mar","apr","may","jun",
                                    "jul","aug","sep","oct","nov","dec"};

    ofstream * f = context->getTofStream();
    f = new ofstream;
    f->open(tofFileName.data(), ios::out | ios::binary);
    context->setTofStream(f);
    if (f->fail() != 0)
    {
        char zBuf[255] ;
        FormatMessage (FORMAT_MESSAGE_FROM_SYSTEM, NULL, GetLastError (), MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) zBuf, sizeof (zBuf), NULL) ;
        string message = "";
        message += "cannot open ";
        message += tofFileName;
        message += "\r\n";
        message += zBuf;
        throw message;
    }

    *f << "tof.5\r\n" \
    "dbtype 1\r\n" \
    "dbversion 10\r\n";

    time(&ltime);
    stm = localtime(&ltime);

    sprintf(Msg,    "date %02.2d-%s-%d %02.2d:%02.2d:%02.2d\r\n",
            stm->tm_mday, day[stm->tm_mon], stm->tm_year + 1900,
            stm->tm_hour, stm->tm_min, stm->tm_sec);
    if (serverName_ != "FILEGate" )
        *f << Msg;

    if (tofTitle_ != "")
        *f << "name " << tofTitle_ << "\r\n";

    *f << "\r\n";

    *f << "STimer\r\n" << "name T_" << tofTimerName << "\r\n\r" << endl;

    return TRUE;
}

void CTofReference::associateTofFile(CContext *context)
{
    CTLockGuard <CTMutex > guard(tofMutex_);
    string tofFileName;
    // One TOF file for all connections
    if (oneTof_)
    {
        context->setTofStream(&tofStream_);
    }
    // One TOF file per connection
    else
    {
        if (selectStatus_ != ESL_STOP)
        {
            //
            // The first file has the specified TOF file name
            // others have the name : FileName + spid number
            //
            spid_ ++;

            string tofTimerName = tofTimerName_;

            if (spid_ == 1)
                tofFileName = tofFileName_;
            else
            {
                char bufTmp[10];
                char fileNameTmp[256];

                memset(fileNameTmp, 0, sizeof(fileNameTmp));
                sprintf(bufTmp, "%d", context->getConnectId());
                strncpy(fileNameTmp, tofFileName_.data(), tofFileName_.length() - 4);
                tofFileName = fileNameTmp;
                tofFileName += bufTmp;
                tofFileName += ".tof";
                tofTimerName += bufTmp;
            }
            createTofFile(context, tofFileName, tofTimerName);
        }
    }
}

void CTofReference::closeTofFile(CContext *context)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (!oneTof_)
    {
        ofstream * f = context->getTofStream();
        if (f != NULL)
        {
            *f << "\r\nSynchronize\r\n\r\n" ;
            if (context->getConnectId() == 1)
                *f << "\r\nETimer\r\n" << "name T_" << tofTimerName_ << "\r\n\r" << endl;
            else
                *f << "\r\nETimer\r\n" << "name T_" << tofTimerName_ << context->getConnectId() << "\r\n\r" << endl;

            if (f->is_open() != 0)
                f->close();
            delete f;
            f = NULL;
        }
        context->setTofStream(NULL);
    }
    else
    {
        tofStream_ << "\r\nSynchronize\r\n\r\n" ;
        tofStream_ << "\r\nETimer\r\n" << "name T_" << tofTimerName_ << "\r" << endl;
        stringStlList::iterator iter=varList_.begin();
        tofStream_ << "\r\nTimdef\r\n" \
        << "name T_" << tofTimerName_ << "\r" << endl ;


        string varName ;

        while(iter != varList_.end())
        {
            tofStream_ << "\r\nVardef\r\n" \
            << "name " << (*iter) << "\r\n" \
            << "option 4225\r\n" \
            << "size 1024\r" << endl;

            iter++;
        }
        varList_.clear();

        if (context == NULL)
            tofStream_.close();
    }
}

void CTofReference::writeCommentInTofFile(CContext *context, const string & message)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (selectStatus_ == ESL_RECORD)
    {
        if (oneTof_)
            tofStream_ << "\r\nComment " << SEPARATOR << "\r\n" << message << " " << SEPARATOR << "\r" << endl ;
        else
        {
            ofstream * f = context->getTofStream();
            if (f == NULL)
                return;
            *f << "\r\nComment " << SEPARATOR << "\r\n" << message << " " << SEPARATOR << "\r" << endl ;
        }
    }
}

string CTofReference::modifyStringAsScl(string input)
{
    int iIndex = string::npos ;


    string str = input ;
    string output = "";
    while ((iIndex = CGWHttp::find_in_string(str,"~",false,0)) != string::npos)
    {
        output += str.substr(0, iIndex) ;
        output += "~~" ;
        str = str.substr(iIndex + 1, str.length() - iIndex - 1) ;
    }
    output += str ;

    str = output ;
    output = "" ;
    while ((iIndex = CGWHttp::find_in_string(str,"^", false,0)) != string::npos)
    {
        output += str.substr(0, iIndex) ;
        output += "~^" ;
        str = str.substr(iIndex + 1, str.length() - iIndex - 1) ;
    }
    output += str ;

    str = output ;
    output = "" ;
    while ((iIndex = CGWHttp::find_in_string(str,"\r\n",false,0)) != string::npos)
    {
        output += str.substr(0, iIndex) ;
        output += "^J" ;
        str = str.substr(iIndex + 2, str.length() - iIndex - 2) ;
    }
    output += str ;

    str = output ;
    output = "" ;
    while ((iIndex = CGWHttp::find_in_string(str,"\"",false,0)) != string::npos)
    {
        output += str.substr(0, iIndex) ;
        output += "~<22>" ;
        str = str.substr(iIndex + 1, str.length() - iIndex - 1) ;
    }
    output += str ;

    str = output ;
    output = "" ;
    char zBuf[10] ;
    for (iIndex=0; iIndex<str.length(); iIndex++)
    {
        if ((unsigned char)str[(size_t)iIndex] >= (unsigned char)0x7F)
        {
            if ((unsigned char)str[(size_t)iIndex] != (unsigned char)0xFF)
                sprintf(zBuf, "~<%2.2x>", (unsigned char)str[(size_t)iIndex]) ;
            else
                sprintf(zBuf, "~<0FF>");
            output += zBuf ;
        }
        else
            output += str[(size_t)iIndex] ;
    }

    return output ;
}

void CTofReference::writeConnectInTofFile(CContext *context)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (selectStatus_ == ESL_RECORD)
    {
        ofstream * f = context->getTofStream();
        if (f == NULL)
            return;

        try
        {
            CTTime timeWrite;
            timeWrite.now();
            CTTime idleTime ;

            if (serverName_ != "FILEGate" && !timePreviousWrite_.isNull() )
                idleTime = timeWrite - timePreviousWrite_;
            if (idleTime != 0)
            {
                *f << "Wait\r\n" ;
                *f << "tdelay " << (unsigned long)idleTime << "\r\n\r" << endl ;
            }

            *f << "Connect\r\n" \
            "server \"" << context->getServerName() << ":" << context->getPort() << "\"\r\n" \
            "conid " << context->getConnectId() << "\r\n\r" << endl;
        }
        catch(...)
        {
            perror("");
        }
    }
    timePreviousWrite_.now() ;
    eObjectType_ = EOT_CONNECT ;
}

void CTofReference::writeDisconnectInTofFile(CContext *context, bool bFullDocument)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (selectStatus_ == ESL_RECORD)
    {
        ofstream * f = context->getTofStream();
        if (f == NULL)
            return;
        *f << "Disconnect\r\n" ;
        *f << "conid " << context->getConnectId() << "\r\n" ;
        if (bFullDocument == FALSE)
            *f << "options 1\r\n";
        else
            *f << "options 0\r\n";

        *f << "\r" << endl ;
    }
    eObjectType_ = EOT_DISCONNECT ;
}


void CTofReference::writeRequestInTofFile(CContext *context, const string & header, char * pBody, int iBodyLen)
{
    try
    {
        CTLockGuard <CTMutex > guard(tofMutex_);

        if (selectStatus_ == ESL_RECORD)
        {
            ofstream * f = context->getTofStream();
            if (f == NULL)
                return;
            CTTime timeWrite;
            timeWrite.now();
            CTTime idleTime ;

            if (serverName_ != "FILEGate" && !timePreviousWrite_.isNull() )
                idleTime = timeWrite - timePreviousWrite_;

            if (idleTime != 0)
            {
                *f << "Wait\r\n" ;
                *f << "tdelay " << (unsigned long)idleTime << "\r" << endl ;
            }

            *f << "\r\nRequest\r\n";
            *f << "conid " << context->getConnectId() << "\r\n" ;
            if (CGWHttp::find_in_string(context->getMethod(),"GET", false, 0) == 0)
                *f << "type 0\r\n" ;
            else if (CGWHttp::find_in_string(context->getMethod(),"HEAD",false,0) == 0)
                *f << "type 1\r\n" ;
            else if (CGWHttp::find_in_string(context->getMethod(),"POST",false,0) == 0)
                *f << "type 2\r\n" ;
            string url = modifyStringAsScl(context->getRequestUrl()) ;
            *f << "uri \"" << url << " " << context->getHttpVersion() << "\"\r\n" ;
            string head = modifyStringAsScl(header) ;
            *f << "header " << SEPARATOR << "\r\n";
            *f << "\"" << head << "\"" << SEPARATOR << "\r\n" ;
            if (iBodyLen > 0)
            {
                char * pBodyTmp = new char[iBodyLen + 1] ;
                if (pBodyTmp == NULL)
                    pBodyTmp = pBody ;
                else
                {
                    memset(pBodyTmp, 0, iBodyLen + 1) ;
                    memcpy(pBodyTmp, pBody, iBodyLen) ;
                }
                // Test to see if the body is binary or ascii
                bool bAscii = TRUE ;
                for (int i=0; i<iBodyLen; i++)
                {
                    if (pBodyTmp[i] > 0x7f ||
                            (pBodyTmp[i] < 0x20 && pBodyTmp[i] != 0x0D && pBodyTmp[i] != 0x0A)
                       )
                    {
                        bAscii = FALSE ;
                        break ;
                    }
                }

                *f << "body " << SEPARATOR << "\r\n" ;

                // ASCII parameters
                string bod = modifyStringAsScl(pBodyTmp) ;
                if (bAscii)
                    *f << "\"" << bod << "\"" << SEPARATOR << "\r\n" ;
                // BINARY parameters
                else
                {
                    // Short binary parameters
                    if (iBodyLen < 255)
                    {
                        *f << "\"0x" ;
                        for (int i=0; i<iBodyLen; i++)
                        {
                            char pBuf[10] ;
                            sprintf(pBuf, "%02.2x", (unsigned char)pBodyTmp[i]) ;
                            *f << pBuf ;
                        }
                        *f << "\"" << SEPARATOR << "\r\n" ;
                    }
                    // Long binary parameters
                    else
                        *f << "\"binary(" << iBodyLen << ")\"" << SEPARATOR << "\r\n" ;
                }
                delete [] pBodyTmp ;
            }
            *f << "\r" << endl ;
        }
        timePreviousWrite_.now() ;
    }
    catch (...)
    {
        perror("Error in write request in tof file");
    }
    eObjectType_ = EOT_REQUEST ;
}

void CTofReference::writeLoadResponseHeader(CContext * context, string & header, string & variable_prefix)
{
    try
    {
        CTLockGuard <CTMutex > guard(tofMutex_);

        if (selectStatus_ == ESL_RECORD)
        {
            ofstream * f = context->getTofStream();
            if (f == NULL)
                return;

            char bufConid[20] ;
            sprintf(bufConid, "%d_%d", context->getConnectId(), context->getCookieId()) ;
            string vName = variable_prefix ;
            vName += "_";
            vName += bufConid ;
            context->incCookieId();

            *f << "\r\nLload\r\n";
            *f << "conid " << context->getConnectId() << "\r\n" ;
            *f << "data 0\r\n" ;
            *f << "identifier \"" << header << "\"\r\n";
            *f << "into " << vName << "\r\n" ;
            *f << "\r" << endl ;
            varList_.push_back(vName) ;
        }
    }
    catch (...)
    {
        perror("Error in write LoadResponseHeader in tof file");
    }

}

void CTofReference::writeSetvarInTofFile(CContext * context)
{
    CTLockGuard <CTMutex > guard(tofMutex_);

    if (selectStatus_ == ESL_RECORD)
    {}

    eObjectType_ = EOT_SETVAR ;
}


