/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#ifndef __GWHTTPCLASS__
#define __GWHTTPCLASS__

#include  "definition.h"
#include <windows.h>
#include <tchar.h>
#include <Wininet.h>
#include  <memview.hpp>
#include <string>
using std::string;

#ifdef BUILDDLLGWHTTP
#  define DLLSPEC_GWHTTP __EXPORT__
#else
#  define DLLSPEC_GWHTTP __IMPORT__
#endif  // BUILDDLLGWHTTP

#ifndef __GATEWAY_PARSE__
#define DLLSPEC_GWPARSE DLLSPEC_GWHTTP
#  include "gwparse.hpp"
#endif  // __GATEWAY_PARSE__

#if ( defined(WIN32) || defined(HPUX) )
#include    <sys/timeb.h>
#endif
#include    <time.h>

#  include <ctime.hpp>

#include "context.hpp"
#include "targetEvaluation.hpp"


///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Class CGWHttp //
///////////////////////////////////////////////////////////////////////////////
class DLLSPEC_GWHTTP CGWHttp : public CGWParse
{
    //-----------------------------------------------------------------------------
public:                         // Virtual methodes.
    const bool                  getPreconnectionMode (void) const
    {
        return FALSE;
    }
    bool                        isFullDocument (void) const
    {
        return bFullServerDocument_ ;
    } ;
    long                        parseBuffer (const int iDataFrom, bool& bUrgent, const void* pBuf, long& lBufLen, const long lOffset, const long lMaxLen, long& lMoreData) ;

    string                      parseHTMLandRemoveSSL(string html, bool & bModified);
    void                        parseHTTPandRemoveSSL(char * packet, int& iHeaderSize, bool bSsl = FALSE) ;
    string                      replaceCyranoSslFromHeader(string & header) ;
    bool                        sendSSL(string & request, char * body ,int bodySize) ;
    void                        sendNormal(const string & host, int iPort, char * packet, int iSize) ;
    void                        saveDocumentInMemory(char *packet, long lBufLen, bool bDelete = TRUE) ;
    void                        saveDocumentAsFile() ;
    void                        removeHostFromUrl(char * packet, int & iHeaderSize, int iBodySize) ;
    void                        removeClientHeaders(char * packet, int & iHeaderSize, int iBodySize) ;
    void                        replaceProxyClientHeaders(char * packet, int & iHeaderSize, int iBodySize) ;
    void                        removeServerCacheHeaders(char * packet, int & iHeaderSize, int iBodySize) ;
    void                        getServerHeaderInformations(string strPacket) ;
    void                        getClientHeaderInformations(string header) ;
    char *                      extractDocumentFromChunkedDatas(char * pBody, int & iDocumentLength) ;
    char *                      generateChunkedDatas(char * pBuf, int iBufLength, int & iLen) ;
    string                      extractClientHeadersForSSL(char * pHeader) ;
    string                      removeClientHeadersForSSL(char * pHeader) ;
    void                        writeLoadResponseHeader(string strHeader) ;
    bool                        isLastChunk() ;
    void                        targetEvaluate(string & strConnect, int & iPort, bool & bProxy) ;
    static void                 Concatenate(void) ;
    string                      modifystringAsScl(string input) ;
    void                        addCookies(char *& packet, int & iHeaderSize);

    //-----------------------------------------------------------------------------
public:                         // Constructor(s) and destructor.
    CGWHttp (const void* pConnectionId, const string& taskName, bool& bTraceOn, CTOLogFileList& logFileList, CTSocket*& pClient, CTSocket*& pTarget) ;
    ~CGWHttp ();

public:                         // Operators.
    CGWHttp&                    operator = (const CGWHttp&) ;
    // Find text in string, return index if found or -1 if not.

    static int find_in_string(string lookin, string lookfor, bool matchcase, unsigned startpos);


    //-----------------------------------------------------------------------------
public:                     // Members.
    static CTMutex      myMutex_;
private :
    CContext * pContext_ ;          // Connection context
    int iSpid_ ;                    // Connection Id
    int iClientBodySize_ ;          // Client Body size for POST requests
    char * ClientPreviousPacket_ ;  // buffer to save previous client packet
    long lClientPreviousPacketLen_ ;// previous client packet length
    char * ServerPreviousPacket_ ;  // buffer to save previous server packet
    long lServerPreviousPacketLen_ ;// previous server packet length
    char endDocument_[20] ;         // save the end of previous packet html doc
    long lEndDocumentLen_ ;         // end previous document length (keep in case of https url cut)
    char * savedDocument_ ;         // save document in memory to put it in a file
    long lSavedDocumentLen_ ;       // save document length
    bool bServerSendHeader_ ;       // Recevied server header ?
    CTSocket * pClient_;            // client socket connection
    CTSocket * pTarget_;            // web server socket connection
    int iDataFrom_ ;                // FROM_CLIENT or FROM_TARGET
    string clientHeader_ ;          // save client HTTP header
    string serverHeader_ ;          // save server HTTP header
    int iServerHeaderSize_ ;        // server HTTP header size
    int iServerContentLength_ ;     // server HTTP "content-length" header
    int iServerIndexTextHtml_ ;     // index of server HTTP "IndexTexHtml" header value
    int iServerIndexChunked_ ;      // index of server HTTP "IndexTexHtml" header value
    int iServerIndexSetCookie_ ;    // index of server HTTP "Set-Cookie" header value
    int iServerIndexLocation_ ;     // index of server HTTP "Location" header value
    bool bServerConnectionClose_ ;  // WEB server will close the connection
    int iIndexSslURL_ ;             // Ssl url index in HTTP header
    bool bClientKeepAlive_ ;        // Client request contains a Keep-Alive header

    char statusCode_[10] ;          // HTTP status code
    string previousHost_ ;          // previous target server connection.
    int iPreviousPort_ ;            // previous target port connection.
    string previousProtocolType_ ;  // previous protocol (http or https)
    bool bFullServerDocument_ ;

    int iServerIndexJavaScript_ ;   // index of server HTTP "javascript" header value

    //-----------------------------------------------------------------------------
}

; // CGWHttp


#endif
