/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECAboutPlusDlg.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "CECAboutPlusDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "htmlhelp.h"
#include "Help\AboutBoxAdv.inl"

/////////////////////////////////////////////////////////////////////////////
// CECAboutPlusDlg dialog


CECAboutPlusDlg::CECAboutPlusDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECAboutPlusDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECAboutPlusDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

}


CECAboutPlusDlg::~CECAboutPlusDlg ()
{
    m_FileNamesArray.RemoveAll();
    m_FilePathArray.RemoveAll();
    m_FileVersionArray.RemoveAll();
}


void CECAboutPlusDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECAboutPlusDlg)
    // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECAboutPlusDlg, CDialog)
//{{AFX_MSG_MAP(CECAboutPlusDlg)
ON_NOTIFY(LVN_COLUMNCLICK, IDC_FILELIST, OnColumnClick)
ON_BN_CLICKED(IDC_EXPORT, OnExport)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECAboutPlusDlg message handlers

BOOL CECAboutPlusDlg::OnInitDialog()
{
    CDialog::OnInitDialog();
    CWaitCursor WC;


    //
    //    Translate CYRANO_HOME as this will be the root directory for all images
    //
    char CyranoHome[256];
    if (GetEnvironmentVariable ("CYRANO_HOME" ,CyranoHome ,sizeof (CyranoHome)) <= 0)
    {
        AfxMessageBox ("The CYRANO_HOME Environment variable has not been defined\nPlease re-install the product.");
        return -1;
    }

    //
    //    ...and ensure that it has a trailing slash
    //
    int Length = strlen (CyranoHome);
    if (CyranoHome[Length-1] != '\\')
        (void) strcat (CyranoHome ,"\\");

    m_CyranoHome = CyranoHome;

    CWnd* pWnd = GetDlgItem(IDC_CYRANO_HOME);
    ASSERT (pWnd != NULL);
    pWnd->SetWindowText(m_CyranoHome);

    //
    //    Now the initialization of the product versions list box.
    //
    //    Setup the columns for the list control
    //
    SetupListBox();

    //
    //    Populate the dialog with the version information from the executables in the product
    //    installation directory.
    //
    PopulateListBox();


    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


//
//    SetupListBox
//    ============
//
//    Setup the titles for the list box.
//

void CECAboutPlusDlg::SetupListBox()
{

    //
    //    Subclass the list box with a CECListBox
    //
    CListCtrl* pWnd = (CListCtrl*) GetDlgItem (IDC_FILELIST);
    ASSERT (pWnd != NULL);
    m_ListBox.SubclassWindow (pWnd->m_hWnd);

    //
    //    Initailize the list box titles.
    //
    m_ListBox.InsertColumn(0,"Filename",LVCFMT_LEFT,90,0);
    m_ListBox.InsertColumn(1,"Version",LVCFMT_LEFT,98,1);
    m_ListBox.InsertColumn(2,"File Path",LVCFMT_LEFT,150, 2);
}

//
//    PopulateListBox
//    ===============
//
//    Populate the listbox with the product version information from the various executables and
//    DLL's in the installation directory.
//
void CECAboutPlusDlg::PopulateListBox()
{

    //
    // List EXE files
    //
    ListFiles (m_CyranoHome, "exe");

    // List DLL files

    ListFiles (m_CyranoHome, "dll");

    //
    // Insert the retrieved strings into the list control
    AddToListCtrl();
}

//
//    ListFiles
//    =========
//
//    This routine lists the version numbers and adds the details to the list box.
//
void CECAboutPlusDlg::ListFiles (CString& FilePath, CString strExt)
{
    VS_FIXEDFILEINFO *fixedfileinfo;
    DWORD           size;
    DWORD           handle;
    void            *info;
    UINT            versionlen;
    BOOL            FoundFile = TRUE;
    char            FileVersionString[20];
    long            FileIndex = 0;


    CFileFind finder;
    CString fullExt = "."+strExt;
    int nExtLen = fullExt.GetLength();

    BOOL bWorking = finder.FindFile(FilePath+"*.*");
    while (bWorking)
    {
        bWorking = finder.FindNextFile();
        CString FullFileName = finder.GetFilePath();
        CString FileName = finder.GetFileName();

        // If the file is a directory make a recursive call to list the files
        // in the found directory.
        if (finder.IsDirectory() && FileName != "." && FileName != "..")
        {
            if (FullFileName.Right(1) != "\\")
                FullFileName+="\\";

            ListFiles(FullFileName, strExt);
            continue;
        }


        if (FileName.Right(nExtLen) != fullExt)
            continue;
        //
        //    Get the size of the version information structure for this file so that we can allocate
        //    a buffer large enough to hold it.
        //
        size = ::GetFileVersionInfoSize (FullFileName.GetBuffer(0), &handle);
        info = malloc(size);                // Allocate buffer for it

        //    Recover the file version information structure at this time.
        //
        if (::GetFileVersionInfo(FullFileName.GetBuffer(0), handle, size, info))
        {

            //
            //    Query for the base version information.
            //
            if (::VerQueryValue(info, "\\", (void**) &fixedfileinfo, &versionlen))
            {

                //
                //    Format the File version string.
                //
                sprintf (FileVersionString, "%d.%d.%d.%d"
                         ,fixedfileinfo->dwFileVersionMS/0x10000
                         ,fixedfileinfo->dwFileVersionMS%0x10000
                         ,fixedfileinfo->dwFileVersionLS/0x10000
                         ,fixedfileinfo->dwFileVersionLS%0x10000);

                //
                //    And add the data to the list control to the arrays to be inserted later
                //
                m_FileNamesArray.Add(finder.GetFileName());
                m_FilePathArray.Add(FilePath);
                m_FileVersionArray.Add (FileVersionString);

                FileIndex++;
            }
        }

        //
        //    Free the allocated information structure
        //
        free (info);

    }
} // CECProductVersionDlg::ListFiles

//
// OnColumnClick
// =============
//
// Message handler for whence the user clicks on the column.
void CECAboutPlusDlg::OnColumnClick(NMHDR* pNMHDR, LRESULT* pResult)
{
    //
    // Wait cursor
    CWaitCursor wait;

    //
    // Retrieve column that was clicked from the notification
    NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

    //
    // Retrieve which column was clicked
    m_nColumnClicked = pNMListView->iSubItem;
    if ( m_nColumnClicked == -1 )
        return;

    //
    // If this was the same column as previously sorted then switch the type of
    // sort
    if (m_nColumnClicked != m_nLastSortedColumn)
    {
        //
        // Different column, select to sort it in ascending fashion
        m_bIsAscendingSort = TRUE;
        //
        // Save column for future
        m_nLastSortedColumn = m_nColumnClicked;
    } // if sorting on a different column than previous
    else
    {
        //
        // Same column, just switch the sorting
        m_bIsAscendingSort = (!m_bIsAscendingSort);
    } // else clause of - if sorting on a different column than previous

    //
    //    Sort the list of items. The sort is dependant on which item we have clicked on
    //    and the type of item which we are displaying.
    //
    typedef int (CALLBACK *SORTCOMPAREPROC) (LPARAM, LPARAM, LPARAM);
    SORTCOMPAREPROC pfnSortCompareProc = (SORTCOMPAREPROC) MakeProcInstance((FARPROC) SortedListCompareProc,
                                         AfxGetInstanceHandle());

    ListView_SortItems (pNMListView->hdr.hwndFrom,
                        pfnSortCompareProc,
                        (LPARAM) (this));

    *pResult = 0;
} // CECProductVersionDlg::OnColumnClick


//
//    SortedListCompareProc
//    =====================
//
//    This routine is called as a sort routine so that the list control items may be sorted
//    according to the column heading selected.
//
//    Note that the list control may contain both integer and character data nad may be sorted
//    either is ascending or descending order.
//
int CALLBACK EXPORT CECAboutPlusDlg::SortedListCompareProc (LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
    int iResult;
    //
    // Convert lParam1/lParam2 to strings
    int nItemInd1 = (int) lParam1;
    int nItemInd2 = (int) lParam2;

    CECAboutPlusDlg* pDlg = (CECAboutPlusDlg*) lParamSort;
    //
    // Sort the thing
    iResult = pDlg->SortItems( nItemInd1, nItemInd2);
    return iResult;
    //  iResult = (pDlg->m_FileNamesArray[ nItemInd1 ] > pDlg->m_FileNamesArray[ nItemInd2 ]);
    //  return (iResult);
}



//
// SortItems
// =========
//
// Private.
//
// This member function is called from the callback function which is called to sort two items.
//
// Since the callback function cannot access private member variables, and the idea of making
// such variables public for the sole purpose that your callback function can access it is not
// very appealing, the callback member function invokes this member function which CAN access
// private member variables, and make the necessary comparisons.
//
// Member variables protected via this scheme (ie which otherwise would have to be public, if
// access to them was granted from the callback member function) are:
//          m_FileNamesArray
//          m_FileVersionArray
//          m_nColumnClicked

int CECAboutPlusDlg::SortItems(int nItem1, int nItem2)
{
    //
    CStringArray* pWhichArray;
    if ( m_nColumnClicked == 0)
    {
        pWhichArray = &m_FileNamesArray;
    }

    else if ( m_nColumnClicked == 1)
    {
        pWhichArray = &m_FileVersionArray;
    }

    else if ( m_nColumnClicked == 2)
    {
        pWhichArray = &m_FilePathArray;
    }

    else
    {
        pWhichArray = NULL;
        VERIFY (FALSE);
    }


    //
    // Ensure the indices are valid for this array
    //
    if ((nItem1 > pWhichArray->GetSize() ) ||
            ( nItem2 > pWhichArray->GetSize() ))
        ASSERT( FALSE );

    //
    // Make the comparison
    int nCmpRes;
    CString str1 = pWhichArray->GetAt(nItem1);
    str1.MakeUpper();
    CString str2 = pWhichArray->GetAt(nItem2);
    str2.MakeUpper();
    if ( str1 > str2 )
        nCmpRes = 1;
    else if ( str1 < str2 )
        nCmpRes = -1;
    else
        nCmpRes = 0;

    //
    // Reverse sort order if not sorting in ascending order
    if (!m_bIsAscendingSort)
    {
        if (nCmpRes ==  1) nCmpRes = -1;
        if (nCmpRes == -1) nCmpRes =  1;
    } // if not sorting in ascending order


    return nCmpRes;
}

//
// AddToListCtrl
// =============
//
// Private
//
// Fill the list control with the list of files/versions obtained.
//
void CECAboutPlusDlg::AddToListCtrl()
{
    CString         TempString;

    //
    //    Now insert the retrieved info into the list control
    //
    for (int Loop = 0; Loop < m_FileNamesArray.GetSize(); Loop++)
    {
        int nItem = m_ListBox.InsertItem(Loop, m_FileNamesArray.GetAt(Loop));
        m_ListBox.SetItemText(nItem,1, m_FileVersionArray.GetAt(Loop));
        m_ListBox.SetItemText(nItem,2, m_FilePathArray.GetAt(Loop));
    } // loop for all entries in the array
} // CECProductVersionDlg::AddToListCtrl


//
//    OnExport
//    ========
//
//    Export the contents of this list as a comma-separated data file.
//

void CECAboutPlusDlg::OnExport()
{
    static char BASED_CODE szFilter[] = "CSV File (*.csv)|*.csv||";

    CString InitialFileName = "FileVersions.csv";
    CFileDialog* pExportDialog = new CFileDialog (FALSE
                                 ,".csv"
                                 ,InitialFileName
                                 ,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT
                                 ,szFilter);
    if (pExportDialog->DoModal() != IDOK) return;

    //
    //    Loop through the list box and add each to
    //
    FILE* pExportFile = fopen (pExportDialog->GetPathName().GetBuffer(0), "wb");
    if (pExportFile == NULL)
    {
        AfxMessageBox ("Failed to open the Export file");
        return;
    }

    CString ExportLine;

    for (int Loop = 0; Loop < m_FileNamesArray.GetSize(); Loop++)
    {
        ExportLine.Format ("%s,%s,%s\n"
                           ,m_FileNamesArray.GetAt(Loop)
                           ,m_FileVersionArray.GetAt(Loop)
                           ,m_FilePathArray.GetAt(Loop));

        (void) fwrite (ExportLine.GetBuffer(0), sizeof(char), ExportLine.GetLength(), pExportFile);
    }
    (void) fclose (pExportFile);
}

BOOL CECAboutPlusDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
