/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// ChildFrm.cpp : implementation of the CChildFrame class
//

#include "stdafx.h"
#include "Modeller.h"
#include "DOMView.h"
#include "DOMTreeView.h"
#include "OutputCheckView.h"
#include "OutputReplayView.h"
#include "OutputTraceView.h"
#include "ClientHeaderView.h"
#include "ServerHeaderView.h"
#include "SCLCaptureScriptView.h"
#include "ModellerHTMLView.h"
#include "ModellerDoc.h"
#include "Constantes.h"
#include "ChildFrm.h"

extern CModellerApp    theApp ;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define SPLIT_SEPARATE 12

extern CSCLStructure*  pSCLStructure ;

/////////////////////////////////////////////////////////////////////////////
// CChildFrame
//--------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CChildFrame, CMDIChildWnd)

//--------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CChildFrame, CMDIChildWnd)
//{{AFX_MSG_MAP(CChildFrame)
ON_WM_CREATE()
ON_WM_DESTROY()
ON_WM_CLOSE()
//}}AFX_MSG_MAP
ON_CBN_SELCHANGE(ID_PAGE_COMBO,OnSelChangePageCombo)
END_MESSAGE_MAP()

//--------------------------------------------------------------------------

CChildFrame::CChildFrame()
        :   m_pDoc(NULL),
        m_pChildWndInfo(NULL),
        m_bFirstTimeActivate(TRUE)
{
}

//--------------------------------------------------------------------------


CChildFrame::~CChildFrame()
{}

//--------------------------------------------------------------------------


BOOL CChildFrame::PreCreateWindow(CREATESTRUCT& cs)
{
    // TODO: Modify the Window class or styles here by modifying
    //  the CREATESTRUCT cs

    if( !CMDIChildWnd::PreCreateWindow(cs) )
        return FALSE;

    return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CChildFrame diagnostics
//--------------------------------------------------------------------------

#ifdef _DEBUG
void CChildFrame::AssertValid() const
{
    CMDIChildWnd::AssertValid();
}

//--------------------------------------------------------------------------

void CChildFrame::Dump(CDumpContext& dc) const
{
    CMDIChildWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CChildFrame message handlers
//--------------------------------------------------------------------------

BOOL CChildFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
{
    m_pDoc = static_cast<CModellerDoc *>(pContext->m_pCurrentDoc);

    if (!m_wndFilterBar.Create(this,IDD_FILTER_DLG,WS_VISIBLE,1))
    {
        TRACE0("Failed to create filter  bar\n");
        return -1;      // fail to create
    }


    if (!m_wndReBar.Create(this) ||
            !m_wndReBar.AddBar(&m_wndFilterBar))
    {
        TRACE0("Failed to create rebar\n");
        return -1;      // fail to create
    }
    //m_wndReBar.SetBarStyle(m_wndReBar.GetBarStyle() & ~CBRS_HIDE_INPLACE);


    //----

    CRect   cr;
    GetClientRect(&cr);
    CSize SplitSize(50,100);

    // 1 Splitter
    if ( !m_Splitter.CreateStatic(this,2,1) )
        return FALSE;

    // 1.1
    m_SplitterScript.CreateStatic(&m_Splitter,1,2);

    // 1.1.1 script
    pContext->m_pNewViewClass   = RUNTIME_CLASS(CSCLCaptureScriptView);
    m_SplitterScript.CreateView(0,0,pContext->m_pNewViewClass,SplitSize, pContext);

    // 1.1.2 Variable
    m_TabWndURL.Create(CBCGTabWnd::STYLE_3D,&m_SplitterScript, m_SplitterScript.IdFromRowCol(0,1));
    m_TabWndURL.SetImageList (IDB_PROJECT, 16, RGB (0,128,128));

    m_TabWndURL.AddTab(RUNTIME_CLASS(CModellerHTMLView) , _T("HTML")            , pContext,-1,0);
    m_TabWndURL.AddTab(RUNTIME_CLASS(CDOMView)          , _T("Structure")       , pContext,-1,0);
    m_TabWndURL.AddTab(RUNTIME_CLASS(CDomTreeView)      , _T("DOM")             , pContext,-1,0);
    m_TabWndURL.AddTab(RUNTIME_CLASS(CServerHeaderView) , _T("Server Header")   , pContext,-1,0);
    m_TabWndURL.AddTab(RUNTIME_CLASS(CClientHeaderView) , _T("Client Header")   , pContext,-1,0);

    m_TabWndURL.SetActiveTab(0);

    // 1.2 output
    m_TabWndOutput.Create(CBCGTabWnd::STYLE_FLAT, &m_Splitter,  m_Splitter.IdFromRowCol(1,0));
    m_TabWndOutput.AddTab(RUNTIME_CLASS(COutputCheckView), _T("Check"), pContext);

    m_TabWndOutput.AddTab(RUNTIME_CLASS(COutputReplayView), _T("Replay"), pContext);

    m_TabWndOutput.AddTab(RUNTIME_CLASS(COutputTraceView), _T("Trace"), pContext);

    m_TabWndOutput.SetActiveTab(0);

    m_SplitterScript.RecalcLayout();
    m_Splitter.RecalcLayout();
    RecalcLayout();

    return TRUE;
}

//--------------------------------------------------------------------------

int CChildFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    ReadInfo();

    if (CMDIChildWnd::OnCreate(lpCreateStruct) == -1)
        return -1;

    return 0;
}


//----------------------------------------------------------------------------

void CChildFrame::ReadInfo()
{
    ASSERT ( m_pChildWndInfo == NULL );

    m_pChildWndInfo = theApp.GetSerialization()->GetObject( ID_SERIALIZE_CHILDWND );
    if ( m_pChildWndInfo == NULL )
    {
        GetWindowPlacement( &(m_Info.m_wp) );
        m_pChildWndInfo = theApp.GetSerialization()->AddObject( ID_SERIALIZE_CHILDWND );


        m_pChildWndInfo->ResetEnum();
        m_pChildWndInfo->SetInt ( m_Info.m_wp.length                    );
        m_pChildWndInfo->SetInt ( m_Info.m_wp.flags                     );
        m_pChildWndInfo->SetInt ( m_Info.m_wp.showCmd                   );
        m_pChildWndInfo->SetLong( m_Info.m_wp.ptMinPosition.x           );
        m_pChildWndInfo->SetLong( m_Info.m_wp.ptMinPosition.y           );
        m_pChildWndInfo->SetLong( m_Info.m_wp.ptMaxPosition.x           );
        m_pChildWndInfo->SetLong( m_Info.m_wp.ptMaxPosition.y           );
        m_pChildWndInfo->SetLong( m_Info.m_wp.rcNormalPosition.bottom   );
        m_pChildWndInfo->SetLong( m_Info.m_wp.rcNormalPosition.left     );
        m_pChildWndInfo->SetLong( m_Info.m_wp.rcNormalPosition.right    );
        m_pChildWndInfo->SetLong( m_Info.m_wp.rcNormalPosition.top      );
        m_pChildWndInfo->SetInt ( m_Info.m_iLeftScript                  );
        m_pChildWndInfo->SetInt ( m_Info.m_iRightScript                 );
        m_pChildWndInfo->SetInt ( m_Info.m_iHeightScript                );
        m_pChildWndInfo->SetInt ( m_Info.m_iHeightScriptOutput          );


    }
    else
    {
        m_pChildWndInfo->ResetEnum();
        m_Info.m_wp.length                  = m_pChildWndInfo->GetInt();
        m_Info.m_wp.flags                   = m_pChildWndInfo->GetInt();
        m_Info.m_wp.showCmd                 = m_pChildWndInfo->GetInt();
        m_Info.m_wp.ptMinPosition.x         = m_pChildWndInfo->GetLong();
        m_Info.m_wp.ptMinPosition.y         = m_pChildWndInfo->GetLong();
        m_Info.m_wp.ptMaxPosition.x         = m_pChildWndInfo->GetLong();
        m_Info.m_wp.ptMaxPosition.y         = m_pChildWndInfo->GetLong();
        m_Info.m_wp.rcNormalPosition.bottom = m_pChildWndInfo->GetLong();
        m_Info.m_wp.rcNormalPosition.left   = m_pChildWndInfo->GetLong();
        m_Info.m_wp.rcNormalPosition.right  = m_pChildWndInfo->GetLong();
        m_Info.m_wp.rcNormalPosition.top    = m_pChildWndInfo->GetLong();
        m_Info.m_iLeftScript                = m_pChildWndInfo->GetInt() - SPLIT_SEPARATE;
        m_Info.m_iRightScript               = m_pChildWndInfo->GetInt() + SPLIT_SEPARATE;
        m_Info.m_iHeightScript              = m_pChildWndInfo->GetInt() - SPLIT_SEPARATE;
        m_Info.m_iHeightScriptOutput        = m_pChildWndInfo->GetInt() + SPLIT_SEPARATE;
    }

}

//----------------------------------------------------------------------------

void CChildFrame::SaveInfo()
{
    int iTemp;

    ASSERT ( m_pChildWndInfo != NULL );

    GetWindowPlacement( &(m_Info.m_wp) );

    m_SplitterScript.GetColumnInfo( 0, m_Info.m_iLeftScript  , iTemp  );
    m_SplitterScript.GetColumnInfo( 1, m_Info.m_iRightScript , iTemp );
    m_Splitter.GetRowInfo   ( 0, m_Info.m_iHeightScript, iTemp );
    m_Splitter.GetRowInfo   ( 1, m_Info.m_iHeightScriptOutput, iTemp );

    m_pChildWndInfo->ResetEnum();
    m_pChildWndInfo->SetInt ( m_Info.m_wp.length                    );
    m_pChildWndInfo->SetInt ( m_Info.m_wp.flags                     );
    m_pChildWndInfo->SetInt ( m_Info.m_wp.showCmd                   );
    m_pChildWndInfo->SetLong( m_Info.m_wp.ptMinPosition.x           );
    m_pChildWndInfo->SetLong( m_Info.m_wp.ptMinPosition.y           );
    m_pChildWndInfo->SetLong( m_Info.m_wp.ptMaxPosition.x           );
    m_pChildWndInfo->SetLong( m_Info.m_wp.ptMaxPosition.y           );
    m_pChildWndInfo->SetLong( m_Info.m_wp.rcNormalPosition.bottom   );
    m_pChildWndInfo->SetLong( m_Info.m_wp.rcNormalPosition.left     );
    m_pChildWndInfo->SetLong( m_Info.m_wp.rcNormalPosition.right    );
    m_pChildWndInfo->SetLong( m_Info.m_wp.rcNormalPosition.top      );
    m_pChildWndInfo->SetInt ( m_Info.m_iLeftScript                  );
    m_pChildWndInfo->SetInt ( m_Info.m_iRightScript                 );
    m_pChildWndInfo->SetInt ( m_Info.m_iHeightScript                );
    m_pChildWndInfo->SetInt ( m_Info.m_iHeightScriptOutput          );
}


//--------------------------------------------------------------------------

void CChildFrame::ActivateFrame(int nCmdShow)
{
    CMDIChildWnd::ActivateFrame(nCmdShow);
    if ( m_bFirstTimeActivate )
    {
        m_bFirstTimeActivate = FALSE;
        SetWindowPlacement( &(m_Info.m_wp) );

        m_Splitter.SetRowInfo(0,m_Info.m_iHeightScript,0);
        m_Splitter.SetRowInfo(1,m_Info.m_iHeightScriptOutput,0);
        m_Splitter.SetRowsSize(m_Info.m_iHeightScript, m_Info.m_iHeightScriptOutput);
        m_Splitter.RecalcLayout();

        m_SplitterScript.SetColumnInfo(0,m_Info.m_iLeftScript,0);
        m_SplitterScript.SetColumnInfo(1,m_Info.m_iRightScript,0);
        m_SplitterScript.RecalcLayout();
        m_Splitter.SetActivePane(0,0);
        m_SplitterScript.SetActivePane(0,0);
    }

}

//--------------------------------------------------------------------------

void CChildFrame::OnDestroy()
{
    SaveInfo();

    CMDIChildWnd::OnDestroy();

}

//--------------------------------------------------------------------------

void CChildFrame::ShowCheckWnd()
{
    m_TabWndOutput.SetActiveTab(0);
    m_TabWndOutput.UpdateWindow();
}

//--------------------------------------------------------------------------

void CChildFrame::ShowReplayWnd()
{
    m_TabWndOutput.SetActiveTab(1);
    m_TabWndOutput.UpdateWindow();
}

//--------------------------------------------------------------------------

void CChildFrame::RefreshURL()
{
    CComboBox* pCombo = (CComboBox*)m_wndFilterBar.GetDlgItem(ID_PAGE_COMBO);
    if ( pCombo == NULL )
        return;

    pCombo->SetRedraw(FALSE);

    pCombo->ResetContent();

    long nConId;
    CString strURL;
    for(int nNdx=0;nNdx<pSCLStructure->m_aData.GetSize();nNdx++)
    {
        if(pSCLStructure->m_aData[nNdx]->m_bPrimary)
        {
            nConId = pSCLStructure->m_aData[nNdx]->m_lConId;
            strURL = pSCLStructure->m_aData[nNdx]->m_strURL;

            int nItem = pCombo->AddString( strURL );
            pCombo->SetItemData( nItem, nConId );
        }
    }

    pCombo->SetCurSel(0);

    pCombo->SetRedraw();
}

//--------------------------------------------------------------------------

void CChildFrame::OnSelChangePageCombo()
{
    CComboBox* pCombo = (CComboBox*)m_wndFilterBar.GetDlgItem(ID_PAGE_COMBO);
    if ( pCombo == NULL )
        return;

    int nCurSel = pCombo->GetCurSel();
    if(nCurSel != CB_ERR)
    {
        CString strURL ;
        int nConId = (int)pCombo->GetItemData(nCurSel);
        CSCLCaptureScriptView *pView = (CSCLCaptureScriptView*)m_SplitterScript.GetPane(0,0);

        pCombo->GetLBText( nCurSel, strURL );

        m_pDoc->Parse();

        pDB_URL_FIELDS pdbFields;
        pdbFields = pSCLStructure->GetURLInfo( strURL, nConId );

        if(pdbFields)
        {
            pView->GetEditCtrl()->GoToLine( pdbFields->m_lBeginLine-1 );
            pView->GetEditCtrl()->SelectLine(pdbFields->m_lBeginLine-1,TRUE);
            pView->GetEditCtrl()->SetFocus();
        }
        else
            AfxMessageBox("Could not find URL in the script.");
    }
}
