/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// DOMView.cpp : implementation file
//

#include "stdafx.h"
#include "Modeller.h"
#include "ModellerDoc.h"
#include "Constantes.h"
#include "VarAddrNameDlg.h"
#include "DOMView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



//----------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CDOMView, CExtendedTreeView)
//{{AFX_MSG_MAP(CDomTreeView)
ON_UPDATE_COMMAND_UI(IDM_ADDRESS, OnUpdateAddressing)
ON_COMMAND(IDM_ADDRESS, OnAddressing)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//----------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CDOMView, CExtendedTreeView)

//----------------------------------------------------------------------------

CDOMView::CDOMView()
{}

//----------------------------------------------------------------------------


CDOMView::~CDOMView()
{}


/////////////////////////////////////////////////////////////////////////////
// CDOMView drawing
//----------------------------------------------------------------------------


void CDOMView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CDOMView diagnostics
//----------------------------------------------------------------------------


#ifdef _DEBUG
void CDOMView::AssertValid() const
{
    CExtendedTreeView::AssertValid();
}

//----------------------------------------------------------------------------

void CDOMView::Dump(CDumpContext& dc) const
{
    CExtendedTreeView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDOMView message handlers
//----------------------------------------------------------------------------

void CDOMView::OnInitialUpdate()
{
    CExtendedTreeView::OnInitialUpdate();
    //-----
    InitTreeCtrlDisplay() ;
}

//----------------------------------------------------------------------------

void CDOMView::InitTreeCtrlDisplay()
{
    CBitmap bmp;
    //----


    m_ImageList.Create( 16, 16, ILC_COLOR, 1, 1 );

    VERIFY(bmp.LoadBitmap(IDB_DOM));
    m_nidDOM = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();


    VERIFY(bmp.LoadBitmap(IDB_LINK));
    m_nidLink = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_SCRIPT));
    m_nidScript = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_FRAME));
    m_nidFrame = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_IMAGE));
    m_nidImage = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();


    VERIFY(bmp.LoadBitmap(IDB_INPUT));
    m_nidInput = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_SELECT));
    m_nidCombo = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_TABLE));
    m_nidTable = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_CHECKBOX));
    m_nidChkBox = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    VERIFY(bmp.LoadBitmap(IDB_RADIO));
    m_nidRadio = m_ImageList.Add(&bmp, RGB(0,0,0));
    bmp.DeleteObject();

    GetTreeCtrl().SetImageList(&m_ImageList , TVSIL_NORMAL);
}

//----------------------------------------------------------------------------

void CDOMView::FillTreeCtrl()
{
    HTREEITEM hItem ;
    //----
    InsertItem( hItem , "Link"  ,m_nidDOM, m_nidDOM, ID_LINK , ID_ROOT );
    InsertItem( hItem , "Script",m_nidDOM, m_nidDOM, ID_SCRIPT , ID_ROOT );
    InsertItem( hItem , "Frame" ,m_nidDOM, m_nidDOM, ID_FRAME , ID_ROOT );
    InsertItem( hItem , "Image" ,m_nidDOM, m_nidDOM, ID_IMAGE , ID_ROOT );
    InsertItem( hItem , "Input" ,m_nidDOM, m_nidDOM, ID_INPUT , ID_ROOT );
    InsertItem( hItem , "Checkbox" ,m_nidDOM, m_nidDOM, ID_CHKBOX , ID_ROOT );
    InsertItem( hItem , "Radio Button" ,m_nidDOM, m_nidDOM, ID_RADIO , ID_ROOT );
    InsertItem( hItem , "Combo/Listbox" ,m_nidDOM, m_nidDOM, ID_SELECT , ID_ROOT );
    //InsertItem( hItem , "Table" ,m_nidDOM, m_nidDOM, ID_TABLE , ID_ROOT );
    //InsertItem( hItem , "Other" ,m_nidDOM, m_nidDOM, ID_OTHER , ID_ROOT );
}

//----------------------------------------------------------------------------


void CDOMView::ShowDow()
{
    CTag* pTag = static_cast<CModellerDoc*> (GetDocument() )->GetDomStructure()->GetRootItem() ;
    InsertTag( pTag->m_pTagList ) ;
}

//----------------------------------------------------------------------------

void CDOMView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
    if( lHint == ID_DISPLAY_DOM_URL )
    {
        GetTreeCtrl().DeleteAllItems() ;
        FillTreeCtrl()  ;
        ShowDow();
        LayOut();
        ExpandAll();
    }
    else
        CExtendedTreeView::OnUpdate(pSender, lHint, pHint);

}

//----------------------------------------------------------------------------

void CDOMView::InsertTag( CTagList *pTagList )
{
    CTag* pTag = pTagList->m_pTag ;
    if( pTag )
    {
        InsertTagInTree( pTag ) ;
        InsertTag( pTag->m_pTagList ) ;
        InsertTag( pTagList->m_pNextTagList ) ;
    }
}

//----------------------------------------------------------------------------

void CDOMView::InsertTagInTree( CTag* pTag)
{
    if( stricmp( pTag->m_pczName , STR_LINK ) == 0 )
        InsertLink( pTag ) ;
    else if( stricmp( pTag->m_pczName , STR_SCRIPT) == 0 )
        InsertScript( pTag ) ;
    else if( stricmp( pTag->m_pczName , STR_FRAME ) == 0 )
        InsertFrame( pTag ) ;
    else if( stricmp( pTag->m_pczName , STR_IMAGE ) == 0 )
        InsertImage( pTag );
    else if( stricmp( pTag->m_pczName , STR_INPUT ) == 0 )
        InsertInput( pTag ) ;
    else if( stricmp( pTag->m_pczName , STR_SELECT ) == 0 )
        InsertSelect( pTag) ;
    else if( stricmp( pTag->m_pczName , STR_TABLE ) == 0 )
        InsertTable( pTag ) ;
}

//----------------------------------------------------------------------------

void CDOMView::InsertLink( CTag* pTag)
{
    CAttributeList* pAttributeList = pTag->m_pAttributesList ;
    CAttribute *pAttribute = pAttributeList->m_pAttribute ;
    //---

    while( pAttribute && stricmp( pAttribute->m_pczName , STR_HREF ) != 0 && pAttributeList )
    {
        pAttributeList = pAttributeList->m_pNextAttributeList ;
        pAttribute = pAttributeList->m_pAttribute ;
    }

    HTREEITEM hItem ;
    //---
    if( pAttribute )
        InsertItem( hItem , pAttribute->m_pczValue ,m_nidLink, m_nidLink, (DWORD) pAttribute->m_pczAddress , ID_LINK );
}

//----------------------------------------------------------------------------

void CDOMView::InsertScript( CTag* pTag)
{
    CString strText;

    CTextList* pTextList = pTag->m_pTextList ;
    CText *pText = pTextList->m_pText ;

    //---
    while( pText  )
    {
        strText += pText->m_pczText;
        pTextList = pTextList->m_pNextTextList ;
        if( pTextList )
            pText = pTextList->m_pText ;
        else
            pText = NULL ;
    }

    HTREEITEM hItem ;
    InsertItem( hItem , strText ,m_nidScript, m_nidScript, (DWORD) pTag->m_pczAddress , ID_SCRIPT );
}

//----------------------------------------------------------------------------

void CDOMView::InsertFrame( CTag* pTag)
{
    CAttributeList* pAttributeList = pTag->m_pAttributesList ;
    CAttribute *pAttribute = pAttributeList->m_pAttribute ;
    //---

    while( pAttribute && stricmp( pAttribute->m_pczName , STR_SRC ) != 0 && pAttributeList )
    {
        pAttributeList = pAttributeList->m_pNextAttributeList ;
        pAttribute = pAttributeList->m_pAttribute ;
    }

    HTREEITEM hItem ;
    //---
    if( pAttribute )
        InsertItem( hItem , pAttribute->m_pczValue ,m_nidFrame, m_nidFrame, (DWORD) pAttribute->m_pczAddress , ID_FRAME );
}

//----------------------------------------------------------------------------

void CDOMView::InsertImage( CTag* pTag)
{
    CAttributeList* pAttributeList = pTag->m_pAttributesList ;
    CAttribute *pAttribute = pAttributeList->m_pAttribute ;
    //---

    while( pAttribute && stricmp( pAttribute->m_pczName , STR_SRC ) != 0 && pAttributeList )
    {
        pAttributeList = pAttributeList->m_pNextAttributeList ;
        pAttribute = pAttributeList->m_pAttribute ;
    }

    HTREEITEM hItem ;
    //---
    if( pAttribute )
        InsertItem( hItem , pAttribute->m_pczValue ,m_nidImage, m_nidImage, (DWORD) pAttribute->m_pczAddress , ID_IMAGE );
}

//----------------------------------------------------------------------------

void CDOMView::InsertInput( CTag* pTag)
{
    CAttributeList* pAttributeList = pTag->m_pAttributesList ;
    CAttribute *pAttribute = pAttributeList->m_pAttribute ;
    //---

    while( pAttribute && stricmp( pAttribute->m_pczName , STR_TYPE ) != 0 && pAttributeList )
    {
        pAttributeList = pAttributeList->m_pNextAttributeList ;
        pAttribute = pAttributeList->m_pAttribute ;
    }

    if( pAttribute )
    {
        if( stricmp( pAttribute->m_pczValue  , STR_TYPE_TEXT ) == 0 ||
                stricmp( pAttribute->m_pczValue  , STR_TYPE_TEXT2 ) == 0 ||
                stricmp( pAttribute->m_pczValue  , STR_TYPE_PWD ) == 0   ||
                stricmp( pAttribute->m_pczValue  , STR_TYPE_PWD2 ) == 0 )
            InsertInputText( pTag ) ;
        else
            if( stricmp( pAttribute->m_pczValue  , STR_TYPE_CHECKBOX ) == 0 ||
                    stricmp( pAttribute->m_pczValue  , STR_TYPE_CHECKBOX2 ) == 0 )
                InsertInputCheckbox( pTag ) ;
            else
                if( stricmp( pAttribute->m_pczValue  , STR_TYPE_RADIO ) == 0 ||
                        stricmp( pAttribute->m_pczValue  , STR_TYPE_RADIO2 ) == 0 )
                    InsertInputRadio( pTag ) ;
    }
}

//----------------------------------------------------------------------------

void CDOMView::InsertSelect( CTag* pTag)
{
    CAttributeList* pAttributeList = pTag->m_pAttributesList ;
    CAttribute *pAttribute = pAttributeList->m_pAttribute ;
    //---

    while( pAttribute && stricmp( pAttribute->m_pczName , STR_NAME ) != 0 && pAttributeList )
    {
        pAttributeList = pAttributeList->m_pNextAttributeList ;
        pAttribute = pAttributeList->m_pAttribute ;
    }

    HTREEITEM hItem ;
    //---
    if( pAttribute )
        InsertItem( hItem , pAttribute->m_pczValue ,m_nidCombo, m_nidCombo, (DWORD) pAttribute->m_pczAddress  , ID_SELECT );
}

//----------------------------------------------------------------------------

void CDOMView::InsertTable( CTag* pTag)
{}

//----------------------------------------------------------------------------


void CDOMView::InsertInputText( CTag* pTag)
{
    CAttributeList* pAttributeList = pTag->m_pAttributesList ;
    CAttribute *pAttribute = pAttributeList->m_pAttribute ;
    //---

    while( pAttribute && stricmp( pAttribute->m_pczName , STR_NAME ) != 0 && pAttributeList )
    {
        pAttributeList = pAttributeList->m_pNextAttributeList ;
        pAttribute = pAttributeList->m_pAttribute ;
    }

    HTREEITEM hItem ;
    //---
    if( pAttribute )
        InsertItem( hItem , pAttribute->m_pczValue ,m_nidInput, m_nidInput, (DWORD) pAttribute->m_pczAddress  , ID_INPUT );
}

//----------------------------------------------------------------------------

void CDOMView::InsertInputCheckbox( CTag* pTag)
{
    CAttributeList* pAttributeList = pTag->m_pAttributesList ;
    CAttribute *pAttribute = pAttributeList->m_pAttribute ;
    //---

    while( pAttribute && stricmp( pAttribute->m_pczName , STR_NAME ) != 0 && pAttributeList )
    {
        pAttributeList = pAttributeList->m_pNextAttributeList ;
        pAttribute = pAttributeList->m_pAttribute ;
    }

    HTREEITEM hItem ;
    //---
    if( pAttribute )
        InsertItem( hItem , pAttribute->m_pczValue ,m_nidChkBox, m_nidChkBox, (DWORD) pAttribute->m_pczAddress  , ID_CHKBOX );
}

//----------------------------------------------------------------------------

void CDOMView::InsertInputRadio( CTag* pTag)
{
    CAttributeList* pAttributeList = pTag->m_pAttributesList ;
    CAttribute *pAttribute = pAttributeList->m_pAttribute ;
    //---

    while( pAttribute && stricmp( pAttribute->m_pczName , STR_NAME ) != 0 && pAttributeList )
    {
        pAttributeList = pAttributeList->m_pNextAttributeList ;
        pAttribute = pAttributeList->m_pAttribute ;
    }

    HTREEITEM hItem ;
    //---
    if( pAttribute )
        InsertItem( hItem , pAttribute->m_pczValue ,m_nidRadio, m_nidRadio, (DWORD) pAttribute->m_pczAddress  , ID_RADIO );
}

//----------------------------------------------------------------------------

void CDOMView::DoOwnerContextMenu( CMenu * pSubMenu )
{
    pSubMenu->AppendMenu( MF_SEPARATOR ) ;
    pSubMenu->AppendMenu( MF_STRING , IDM_ADDRESS , _T("Address" ) ) ;

}

//----------------------------------------------------------------------------

void CDOMView::OnUpdateAddressing(CCmdUI* pCmdUI)
{
    CTreeCtrl & pTreeCtrl = GetTreeCtrl() ;
    HTREEITEM hItem         = (HTREEITEM)NULL;
    HTREEITEM hParentItem   = (HTREEITEM)NULL;
    //---
    hItem       = pTreeCtrl.GetSelectedItem() ;
    if ( hItem != (HTREEITEM)NULL )
    {
        hParentItem = pTreeCtrl.GetParentItem(hItem) ;
    }

    pCmdUI->Enable( (hItem != (HTREEITEM)NULL) && (hParentItem != (HTREEITEM)NULL) ) ;
}

//----------------------------------------------------------------------------

void CDOMView::OnAddressing()
{
    HTREEITEM       hItem ;
    CVarResponse    VarResponse ;
    CModellerDoc *  pDoc ;
    //---
    VarResponse.m_ResponseType = RT_BODY ;

    CVarAddrNameDlg Dlg( &VarResponse ) ;
    if( Dlg.DoModal() == IDCANCEL )
        return;

    hItem = GetTreeCtrl().GetSelectedItem() ;
    VarResponse.m_strIdentifier = (char*) ( GetTreeCtrl().GetItemData( hItem ) ) ;

    pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->Adressing( &VarResponse ) ;
}


