/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// DomTreeView.cpp : implementation of the CDomTreeView class
//

#include "stdafx.h"
#include "Modeller.h"
#include "ModellerDoc.h"
#include "Constantes.h"
#include "VarAddrNameDlg.h"
#include "DomTreeView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif




/////////////////////////////////////////////////////////////////////////////
// CDomTreeView
//----------------------------------------------------------------------------

const UINT    WM_FINDREPLACE = RegisterWindowMessage( FINDMSGSTRING );

IMPLEMENT_DYNCREATE(CDomTreeView, CExtendedTreeView)

//----------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CDomTreeView, CExtendedTreeView)
//{{AFX_MSG_MAP(CDomTreeView)
ON_UPDATE_COMMAND_UI(IDM_ADDRESS, OnUpdateAddressing)
ON_COMMAND(IDM_ADDRESS, OnAddressing)
ON_UPDATE_COMMAND_UI(IDM_SEARCH, OnUpdateSearch)
ON_COMMAND(IDM_SEARCH, OnInitSearch)
ON_REGISTERED_MESSAGE( WM_FINDREPLACE, OnDoSearch )
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDomTreeView construction/destruction
//----------------------------------------------------------------------------

CDomTreeView::CDomTreeView():
        m_nidTag(0),
        m_nidAttribute(0),
        m_nidText(0),
        m_pFindDlg( NULL )
{
    // TODO: add construction code here

}

//----------------------------------------------------------------------------


CDomTreeView::~CDomTreeView()
{
    if( m_pFindDlg != NULL )
    {
        delete m_pFindDlg ;
    }
}

//----------------------------------------------------------------------------

BOOL CDomTreeView::PreCreateWindow(CREATESTRUCT& cs)
{
    // TODO: Modify the Window class or styles here by modifying
    //  the CREATESTRUCT cs

    return CExtendedTreeView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CDomTreeView drawing
//----------------------------------------------------------------------------

void CDomTreeView::OnDraw(CDC* pDC)
{
    // TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// CDomTreeView diagnostics
//----------------------------------------------------------------------------


#ifdef _DEBUG
void CDomTreeView::AssertValid() const
{
    CExtendedTreeView::AssertValid();
}

//----------------------------------------------------------------------------

void CDomTreeView::Dump(CDumpContext& dc) const
{
    CExtendedTreeView::Dump(dc);
}

//----------------------------------------------------------------------------

#endif //_DEBUG


//----------------------------------------------------------------------------

void CDomTreeView::InitTreeCtrlDisplay()
{
    CBitmap bmpTag;
    CBitmap bmpAttribute;
    CBitmap bmpText;
    //----


    m_ImageList.Create( 16, 16, ILC_COLOR, 1, 1 );

    VERIFY(bmpTag.LoadBitmap(IDB_TAG));
    m_nidTag = m_ImageList.Add(&bmpTag, RGB(0,0,0));
    bmpTag.DeleteObject();


    VERIFY(bmpAttribute.LoadBitmap(IDB_ATTRIBUTE));
    m_nidAttribute = m_ImageList.Add(&bmpAttribute, RGB(0,0,0));
    bmpAttribute.DeleteObject();

    VERIFY(bmpText.LoadBitmap(IDB_TEXT));
    m_nidText = m_ImageList.Add(&bmpText, RGB(0,0,0));
    bmpText.DeleteObject();

    GetTreeCtrl().SetImageList(&m_ImageList , TVSIL_NORMAL);
}

//----------------------------------------------------------------------------

void CDomTreeView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
    if( lHint == ID_DISPLAY_DOM_URL )
    {
        GetTreeCtrl().DeleteAllItems() ;
        ShowDom();
        LayOut();
        ExpandAll();
    }
    else
        CExtendedTreeView::OnUpdate(pSender, lHint, pHint);
}

//----------------------------------------------------------------------------

void CDomTreeView::ShowDom()
{
    CTag* pTag = static_cast<CModellerDoc*> (GetDocument() )->GetDomStructure()->GetRootItem() ;
    InsertTagInTree( pTag) ;
    InsertTag( pTag->m_pTagList ) ;
}

//----------------------------------------------------------------------------

void CDomTreeView::InsertTag( CTagList *pTagList )
{
    CTag* pTag = pTagList->m_pTag ;
    if( pTag )
    {
        InsertTagInTree( pTag ) ;
        InsertTag( pTag->m_pTagList ) ;
        InsertTag( pTagList->m_pNextTagList ) ;
    }
}

//----------------------------------------------------------------------------

void CDomTreeView::InsertTagInTree( CTag* pTag)
{
    HTREEITEM hItem ;
    //---
    if( pTag )
    {
        DWORD dwParent = 0L;
        //---
        if( pTag->m_pParent )
            dwParent = (DWORD) (pTag->m_pParent->m_pczAddress) ;
        InsertItem( hItem , pTag->m_pczName ,m_nidTag, m_nidTag, (DWORD) pTag->m_pczAddress , dwParent );
        InsertAttribute( pTag ) ;
        InsertText( pTag) ;
    }
}

//----------------------------------------------------------------------------

void CDomTreeView::InsertAttribute( CTag* pTag)
{
    CAttributeList* pAttributeList = pTag->m_pAttributesList ;
    CAttribute *pAttribute = pAttributeList->m_pAttribute ;
    HTREEITEM hItem ;
    //---
    while( pAttribute  )
    {
        CString strLabel ;
        //---
        strLabel = pAttribute->m_pczName ;
        strLabel += " :" ;
        strLabel += pAttribute->m_pczValue ;

        InsertItem( hItem , strLabel , m_nidAttribute, m_nidAttribute, (DWORD) pAttribute->m_pczAddress , (DWORD) pTag->m_pczAddress );
        pAttributeList = pAttributeList->m_pNextAttributeList ;
        if( pAttributeList )
            pAttribute = pAttributeList->m_pAttribute ;
        else
            pAttribute = NULL ;
    }
}

//----------------------------------------------------------------------------

void CDomTreeView::InsertText( CTag* pTag)
{
    CTextList* pTextList = pTag->m_pTextList ;
    CText *pText = pTextList->m_pText ;
    HTREEITEM hItem ;
    //---
    while( pText  )
    {
        InsertItem( hItem , pText->m_pczText , m_nidText, m_nidText, (DWORD) pText->m_pczAddress , (DWORD) pTag->m_pczAddress );
        pTextList = pTextList->m_pNextTextList ;
        if( pTextList )
            pText = pTextList->m_pText ;
        else
            pText = NULL ;
    }
}

//----------------------------------------------------------------------------

void CDomTreeView::OnInitialUpdate()
{
    CExtendedTreeView::OnInitialUpdate();

    InitTreeCtrlDisplay() ;
}

//----------------------------------------------------------------------------

void CDomTreeView::DoOwnerContextMenu( CMenu * pSubMenu )
{
    pSubMenu->AppendMenu( MF_SEPARATOR ) ;
    pSubMenu->AppendMenu( MF_STRING , IDM_ADDRESS , _T("Address" ) ) ;
    pSubMenu->AppendMenu( MF_STRING , IDM_SEARCH , _T("Search" ) ) ;
}

//----------------------------------------------------------------------------

void CDomTreeView::OnUpdateAddressing(CCmdUI* pCmdUI)
{
    CTreeCtrl & pTreeCtrl = GetTreeCtrl() ;
    HTREEITEM hItem         = (HTREEITEM)NULL;
    HTREEITEM hParentItem   = (HTREEITEM)NULL;
    //---
    hItem       = pTreeCtrl.GetSelectedItem() ;
    if ( hItem != (HTREEITEM)NULL )
    {
        hParentItem = pTreeCtrl.GetParentItem(hItem) ;
    }

    pCmdUI->Enable( (hItem != (HTREEITEM)NULL) && (hParentItem != (HTREEITEM)NULL) ) ;
}

//----------------------------------------------------------------------------

void CDomTreeView::OnAddressing()
{
    HTREEITEM       hItem ;
    CVarResponse    VarResponse ;
    CModellerDoc *  pDoc ;
    //---
    VarResponse.m_ResponseType = RT_BODY ;

    CVarAddrNameDlg Dlg( &VarResponse ) ;
    if( Dlg.DoModal() == IDCANCEL )
        return;

    hItem = GetTreeCtrl().GetSelectedItem() ;
    VarResponse.m_strIdentifier = (char*) ( GetTreeCtrl().GetItemData( hItem ) ) ;

    pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->Adressing( &VarResponse ) ;
}

//----------------------------------------------------------------------------

void CDomTreeView::OnUpdateSearch(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( TRUE ) ;
}

//----------------------------------------------------------------------------

void CDomTreeView::OnInitSearch()
{
    m_pFindDlg = new CFindReplaceDialog();  // Must be created on the heap

    m_pFindDlg->m_fr.lStructSize    = sizeof(FINDREPLACE);
    m_pFindDlg->m_fr.hwndOwner      = this->m_hWnd;
    m_pFindDlg->Create( TRUE, "", "", FR_DOWN, this );
}

//----------------------------------------------------------------------------

LRESULT CDomTreeView::OnDoSearch( WPARAM w , LPARAM pText )
{
    HTREEITEM       hFoundItem ;
    //---
    if( !m_pFindDlg->IsTerminating() )
    {
        hFoundItem      = FindItem( m_pFindDlg->GetFindString(),
                                    m_pFindDlg->MatchCase(),
                                    m_pFindDlg->SearchDown(),
                                    m_pFindDlg->MatchWholeWord() ) ;

        if( hFoundItem != NULL )
        {
            GetTreeCtrl().SelectItem(   hFoundItem ) ;
        }
    }
    else
    {
        m_pFindDlg = NULL ;
    }

    return 0 ;
}

//----------------------------------------------------------------------------
// FindItem         - Finds an item that contains the search string
// Returns          - Handle to the item or NULL
// str              - String to search for
// bCaseSensitive   - Should the search be case sensitive
// bDownDir         - Search direction - TRUE for down
// bWholeWord       - True if search should match whole words
// hItem            - Item to start searching from. NULL for
//                  - currently selected item
HTREEITEM CDomTreeView::FindItem(   CString &str,
                                    BOOL bCaseSensitive /*= FALSE*/,
                                    BOOL bDownDir       /*= TRUE*/,
                                    BOOL bWholeWord     /*= FALSE*/,
                                    HTREEITEM hItem     /*= NULL*/)
{
    int lenSearchStr = str.GetLength();
    if( lenSearchStr == 0 ) return NULL;

    HTREEITEM htiSel = hItem ? hItem : GetTreeCtrl().GetSelectedItem();
    HTREEITEM htiCur = bDownDir ? GetNextItem( htiSel ) : GetPrevItem( htiSel );
    CString sSearch = str;

    if( htiCur == NULL && htiSel != NULL) // make sure it ends if we
        // started with no selection
    {
        if( bDownDir )
            htiCur = GetTreeCtrl().GetRootItem();
        else
            htiCur = GetLastItem( NULL );
    }

    if( !bCaseSensitive )
        sSearch.MakeLower();

    while( htiCur && htiCur != htiSel )
    {
        CString sItemText = GetTreeCtrl().GetItemText( htiCur );
        if( !bCaseSensitive )
            sItemText.MakeLower();

        int n;
        while( (n = sItemText.Find( sSearch )) != -1 )
        {
            // Search string found
            if( bWholeWord )
            {
                // Check preceding char
                if( n != 0 )
                {
                    if( isalpha(sItemText[n-1]) ||
                            sItemText[n-1] == '_' )
                    {
                        // Not whole word
                        sItemText = sItemText.Right(
                                        sItemText.GetLength() - n -
                                        lenSearchStr );
                        continue;
                    }
                }

                // Check succeeding char
                if( sItemText.GetLength() > n + lenSearchStr
                        && ( isalpha(sItemText[n+lenSearchStr]) ||
                             sItemText[n+lenSearchStr] == '_' ) )
                {
                    // Not whole word
                    sItemText = sItemText.Right( sItemText.GetLength()
                                                 - n - sSearch.GetLength() );
                    continue;
                }
            }

            if( IsFindValid( htiCur ) )
                return htiCur;
            else break;
        }


        htiCur = bDownDir ? GetNextItem( htiCur ) : GetPrevItem( htiCur );
        if( htiCur == NULL && htiSel != NULL)   // make sure it ends if we
            // started with no selection
        {
            if( bDownDir )
                htiCur = GetTreeCtrl().GetRootItem();
            else
                htiCur = GetLastItem( NULL );
        }
    }
    return NULL;
}

//----------------------------------------------------------------------------

// IsFindValid  - Virtual function used by FindItem to allow this
//        function to filter the result of FindItem
// Returns  - True if item matches the criteria
// Arg      - Handle of the item
BOOL CDomTreeView::IsFindValid( HTREEITEM )
{
    return TRUE;
}

//----------------------------------------------------------------------------
// GetNextItem  - Get next item as if outline was completely expanded
// Returns      - The item immediately below the reference item
// hItem        - The reference item
//----------------------------------------------------------------------------
HTREEITEM CDomTreeView::GetNextItem(HTREEITEM hItem)
{
    HTREEITEM hti = NULL;

    if ( GetTreeCtrl().ItemHasChildren(hItem))
        hti = GetTreeCtrl().GetChildItem(hItem);

    if (hti == NULL)
    {
        while ((hti = GetTreeCtrl().GetNextSiblingItem(hItem)) == NULL)
        {
            if ((hItem = GetTreeCtrl().GetParentItem(hItem)) == NULL)
                return NULL;
        }
    }
    return hti;
}

//----------------------------------------------------------------------------
// GetNextItem  - Get previous item as if outline was completely expanded
// Returns              - The item immediately above the reference item
// hItem                - The reference item
//----------------------------------------------------------------------------
HTREEITEM CDomTreeView::GetPrevItem( HTREEITEM hItem )
{
    HTREEITEM       hti;

    hti = GetTreeCtrl().GetPrevSiblingItem(hItem);
    if( hti == NULL )
        hti = GetTreeCtrl().GetParentItem(hItem);
    else
        hti = GetLastItem(hti);
    return hti;
}

//----------------------------------------------------------------------------
// GetLastItem  - Gets last item in the branch
// Returns      - Last item
// hItem        - Node identifying the branch. NULL will
//                return the last item in outine
//----------------------------------------------------------------------------
HTREEITEM CDomTreeView::GetLastItem( HTREEITEM hItem )
{
    // Last child of the last child of the last child ...
    HTREEITEM htiNext;

    if( hItem == NULL )
    {
        // Get the last item at the top level
        htiNext = GetTreeCtrl().GetRootItem();
        while( htiNext )
        {
            hItem = htiNext;
            htiNext = GetTreeCtrl().GetNextSiblingItem( htiNext );
        }
    }

    while( GetTreeCtrl().ItemHasChildren( hItem ) )
    {
        htiNext = GetTreeCtrl().GetChildItem( hItem );
        while( htiNext )
        {
            hItem = htiNext;
            htiNext = GetTreeCtrl().GetNextSiblingItem( htiNext );
        }
    }

    return hItem;
}





