/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// IfServerResponseTestPPage.cpp : implementation file
//

#include "stdafx.h"
#include "IfServerResponseTestPPage.h"
#include <SCLStructure.h>
#include "htmlhelp.h"
#include "Help\IfServerResponseTestPPagee.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CIfServerResponseTestPPage property page

BEGIN_MESSAGE_MAP(CIfServerResponseTestPPage, CExtendedPropertyPage)
//{{AFX_MSG_MAP(CIfServerResponseTestPPage)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//--------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CIfServerResponseTestPPage, CExtendedPropertyPage)

//--------------------------------------------------------------------------

CIfServerResponseTestPPage::CIfServerResponseTestPPage(CIfServerResponseTest* _pServerResponseTest,CSCLStructure *_pSCLStructure) :
        CExtendedPropertyPage(CIfServerResponseTestPPage::IDD)
{
    ASSERT(_pServerResponseTest != NULL);
    m_pServerResponseTest = _pServerResponseTest;
    m_pSCLStructure = _pSCLStructure;
    //{{AFX_DATA_INIT(CIfServerResponseTestPPage)
    m_nResponseTime = 0;
    //}}AFX_DATA_INIT
}

//--------------------------------------------------------------------------


CIfServerResponseTestPPage::~CIfServerResponseTestPPage()
{}

//--------------------------------------------------------------------------


void CIfServerResponseTestPPage::DoDataExchange(CDataExchange* pDX)
{
    CExtendedPropertyPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CIfServerResponseTestPPage)
    DDX_Control(pDX, IDC_IF_SR_URL_COMBO, m_ctlURLCombo);
    DDX_Control(pDX, IDC_IF_SRTIME_SPIN, m_ctlResponseTimeSpin);
    DDX_Text(pDX, IDC_IF_SRTIME_EDIT, m_nResponseTime);
    //}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
// CIfServerResponseTestPPage message handlers
//--------------------------------------------------------------------------


BOOL CIfServerResponseTestPPage::OnInitDialog()
{
    CExtendedPropertyPage::OnInitDialog();

    m_ctlResponseTimeSpin.SetRange32(0,0x7FFFFFFF);

    Values2Ctrl();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//--------------------------------------------------------------------------


void CIfServerResponseTestPPage::Ctrl2Values()
{
    UpdateData();

    CString& strSelectedURL = m_pServerResponseTest->m_strServerResponseURL;
    strSelectedURL.Empty();

    int nCurSel = m_ctlURLCombo.GetCurSel();
    if(nCurSel!=CB_ERR)
    {
        m_ctlURLCombo.GetLBText(nCurSel,strSelectedURL);
        m_pServerResponseTest->m_nServerResponseURLConId = m_ctlURLCombo.GetItemData(nCurSel);
    }

    m_pServerResponseTest->m_nResponseTime = m_nResponseTime;
}

//--------------------------------------------------------------------------

void CIfServerResponseTestPPage::Values2Ctrl()
{
    CString& strSelectedURL = m_pServerResponseTest->m_strServerResponseURL;

    m_ctlURLCombo.SetRedraw(FALSE);
    m_ctlURLCombo.ResetContent();

    long nConId;
    CString strURL;
    for(int nNdx=0;nNdx<m_pSCLStructure->m_aData.GetSize();nNdx++)
    {
        if(m_pSCLStructure->m_aData[nNdx]->m_bHTMLURL &&
                m_pSCLStructure->m_aData[nNdx]->m_bPrimary)
        {
            nConId = m_pSCLStructure->m_aData[nNdx]->m_lConId;
            strURL = m_pSCLStructure->m_aData[nNdx]->m_strURL;
            if(m_ctlURLCombo.FindStringExact(-1,strURL)== CB_ERR)
            {
                int nItem = m_ctlURLCombo.AddString(strURL);
                m_ctlURLCombo.SetItemData(nItem,nConId);
            }
        }
    }

    if(m_ctlURLCombo.SelectString(-1,strSelectedURL) == CB_ERR)
        m_ctlURLCombo.SetCurSel(0);

    m_ctlURLCombo.SetRedraw();

    m_nResponseTime = m_pServerResponseTest->m_nResponseTime;
    m_ctlResponseTimeSpin.SetPos(m_nResponseTime);

    UpdateData(FALSE);
}

//--------------------------------------------------------------------------

LRESULT CIfServerResponseTestPPage::OnWizardBack()
{
    return IDD_IF_VALUE_TYPE_PPAGE;
}

//--------------------------------------------------------------------------

LRESULT CIfServerResponseTestPPage::OnWizardNext()
{
    return -1;
}

//--------------------------------------------------------------------------

HBRUSH CIfServerResponseTestPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CExtendedPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem(IDC_DESCRIPTION)->m_hWnd)
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    // TODO: Return a different brush if the default is not desired
    return hbr;
}

BOOL CIfServerResponseTestPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
