/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// IfTextMatchPPage.cpp : implementation file
//

#include "stdafx.h"
#include "IfTextMatchPPage.h"
#include <SCLStructure.h>
#include "htmlhelp.h"
#include "Help\IfTextMatchPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CIfTextMatchPPage property page

BEGIN_MESSAGE_MAP(CIfTextMatchPPage, CExtendedPropertyPage)
//{{AFX_MSG_MAP(CIfTextMatchPPage)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//--------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CIfTextMatchPPage, CExtendedPropertyPage)

//--------------------------------------------------------------------------

CIfTextMatchPPage::CIfTextMatchPPage(CIfTextMatch* _pTextMatch,CSCLStructure *_pSCLStructure) :
        CExtendedPropertyPage(CIfTextMatchPPage::IDD)
{
    ASSERT(_pTextMatch!=NULL);
    m_pTextMatch = _pTextMatch;
    m_pSCLStructure = _pSCLStructure;
    //{{AFX_DATA_INIT(CIfTextMatchPPage)
    m_strTextToMatch = _T("");
    //}}AFX_DATA_INIT
}

//--------------------------------------------------------------------------


CIfTextMatchPPage::~CIfTextMatchPPage()
{}

//--------------------------------------------------------------------------


void CIfTextMatchPPage::DoDataExchange(CDataExchange* pDX)
{
    CExtendedPropertyPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CIfTextMatchPPage)
    DDX_Control(pDX, IDC_IF_TEXT_MATCH_URL_COMBO, m_ctlTextMatchURL);
    DDX_Text(pDX, IDC_IF_TEXT_EDIT, m_strTextToMatch);
    //}}AFX_DATA_MAP
}

//--------------------------------------------------------------------------

/////////////////////////////////////////////////////////////////////////////
// CIfTextMatchPPage message handlers
//--------------------------------------------------------------------------


BOOL CIfTextMatchPPage::OnInitDialog()
{
    CExtendedPropertyPage::OnInitDialog();

    Values2Ctrl();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//--------------------------------------------------------------------------


void CIfTextMatchPPage::Ctrl2Values()
{
    UpdateData();

    CString& strSelectedURL = m_pTextMatch->m_strTextMatchURL;
    strSelectedURL.Empty();

    int nCurSel = m_ctlTextMatchURL.GetCurSel();
    if(nCurSel!=CB_ERR)
    {
        m_ctlTextMatchURL.GetLBText(nCurSel,strSelectedURL);
        m_pTextMatch->m_nTextMatchURLConId = m_ctlTextMatchURL.GetItemData(nCurSel);
    }

    m_pTextMatch->m_strTextToMatch = m_strTextToMatch;
}

//--------------------------------------------------------------------------

void CIfTextMatchPPage::Values2Ctrl()
{
    CString& strSelectedURL = m_pTextMatch->m_strTextMatchURL;

    m_ctlTextMatchURL.SetRedraw(FALSE);
    m_ctlTextMatchURL.ResetContent();

    long nConId;
    CString strURL;
    for(int nNdx=0;nNdx<m_pSCLStructure->m_aData.GetSize();nNdx++)
    {
        if(m_pSCLStructure->m_aData[nNdx]->m_bHTMLURL &&
                m_pSCLStructure->m_aData[nNdx]->m_bPrimary)
        {
            nConId = m_pSCLStructure->m_aData[nNdx]->m_lConId;
            strURL = m_pSCLStructure->m_aData[nNdx]->m_strURL;
            if(m_ctlTextMatchURL.FindStringExact(-1,strURL)== CB_ERR)
            {
                int nItem = m_ctlTextMatchURL.AddString(strURL);
                m_ctlTextMatchURL.SetItemData(nItem,nConId);
            }
        }
    }

    if(m_ctlTextMatchURL.SelectString(-1,strSelectedURL) == CB_ERR)
        m_ctlTextMatchURL.SetCurSel(0);

    m_ctlTextMatchURL.SetRedraw();

    m_strTextToMatch = m_pTextMatch->m_strTextToMatch;

    UpdateData(FALSE);
}

//--------------------------------------------------------------------------

LRESULT CIfTextMatchPPage::OnWizardBack()
{
    return IDD_IF_VALUE_TYPE_PPAGE;
}

LRESULT CIfTextMatchPPage::OnWizardNext()
{
    return -1;
}

//--------------------------------------------------------------------------

HBRUSH CIfTextMatchPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CExtendedPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem(IDC_DESCRIPTION)->m_hWnd)
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    // TODO: Return a different brush if the default is not desired
    return hbr;
}

BOOL CIfTextMatchPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
