/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// IfValueTypePPage.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "IfValueTypePPage.h"
#include "htmlhelp.h"
#include "Help\IfValueTypePPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CIfValueTypePPage property page
//--------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CIfValueTypePPage, CExtendedPropertyPage)
//{{AFX_MSG_MAP(CIfValueTypePPage)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//--------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CIfValueTypePPage, CExtendedPropertyPage)

//--------------------------------------------------------------------------

CIfValueTypePPage::CIfValueTypePPage(CIfValueType* _pValueType) : CExtendedPropertyPage(CIfValueTypePPage::IDD)
{
    ASSERT(_pValueType!=NULL);
    m_pValueType = _pValueType;
    //{{AFX_DATA_INIT(CIfValueTypePPage)
    m_nIfValueType = -1;
    //}}AFX_DATA_INIT
}

//--------------------------------------------------------------------------


CIfValueTypePPage::~CIfValueTypePPage()
{}

//--------------------------------------------------------------------------


void CIfValueTypePPage::DoDataExchange(CDataExchange* pDX)
{
    CExtendedPropertyPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CIfValueTypePPage)
    DDX_Radio(pDX, IDC_IF_BOOLEAN_RADIO, m_nIfValueType);
    //}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
// CIfValueTypePPage message handlers
//--------------------------------------------------------------------------


BOOL CIfValueTypePPage::OnInitDialog()
{
    CExtendedPropertyPage::OnInitDialog();

    Values2Ctrl();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//--------------------------------------------------------------------------


void CIfValueTypePPage::Ctrl2Values()
{
    UpdateData();

    m_pValueType->m_nIfValueType = (CIfValueType::IfValueTypes)m_nIfValueType;
}

//--------------------------------------------------------------------------

void CIfValueTypePPage::Values2Ctrl()
{
    m_nIfValueType = m_pValueType->m_nIfValueType;

    UpdateData(FALSE);
}

//--------------------------------------------------------------------------

LRESULT CIfValueTypePPage::OnWizardNext()
{
    UpdateData();

    Ctrl2Values();

    switch(m_nIfValueType)
    {
    case CIfValueType::enBooleanExpression:
        return IDD_IF_BOOLEAN_EXP_PPAGE;
        break;
    case CIfValueType::enServerResponseTest:
        return IDD_IF_SRTEST_PPAGE;
        break;
    case CIfValueType::enTextMatching:
        return IDD_IF_TEXT_MATCH_PPAGE;
        break;
    }

    return CExtendedPropertyPage::OnWizardNext();
}

//--------------------------------------------------------------------------

HBRUSH CIfValueTypePPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CExtendedPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem(IDC_DESCRIPTION)->m_hWnd)
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    // TODO: Return a different brush if the default is not desired
    return hbr;
}

BOOL CIfValueTypePPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
