/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"

#include "MainFrm.h"
#include "BasedDocument.h"
#include "htmlhelp.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include <ConstantesLib.h>

#include "Constantes.h"

#include "EnumModellerData.h"
extern CModellerApp theApp ;

/////////////////////////////////////////////////////////////////////////////
// CMainFrame
//--------------------------------------------------------------------------

IMPLEMENT_DYNAMIC(CMainFrame, CBCGMDIFrameWnd)

//--------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CMainFrame, CBCGMDIFrameWnd)
//{{AFX_MSG_MAP(CMainFrame)
ON_WM_CREATE()
ON_MESSAGE( WM_COPYDATA , OnGetAppArgs )
ON_WM_DESTROY()
ON_COMMAND(ID_HELP_CONTENTS, OnHelpContents)
ON_COMMAND(ID_HELP_INDEX, OnHelpIndex)
ON_COMMAND(ID_HELP_SEARCH, OnHelpSearch)
ON_WM_CLOSE()
ON_COMMAND(ID_HELP_SCLREFERENCE, OnHelpSclreference)
//}}AFX_MSG_MAP
ON_MESSAGE(WM_CARET_POS_CHANGED, OnCaretPosChanged)
END_MESSAGE_MAP()

//--------------------------------------------------------------------------
static UINT indicators[] =
    {
        ID_SEPARATOR,           // status line indicator
        ID_INDICATOR_CURSOR,
        ID_INDICATOR_CAPS,
        ID_INDICATOR_NUM,
        ID_INDICATOR_SCRL,
    };

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction
//--------------------------------------------------------------------------

CMainFrame::CMainFrame()
        :   m_pMainWndInfo(NULL),
        m_MainToolBar(),
        m_CaptureToolBar(),
        m_VariableToolBar()
{
    ///    cSpawn.LoadToolBarResource(IDR_MAINFRAME);
    //    cSpawn.AddToolBarResource(IDR_CAPTURE);
    //  cSpawn.AddToolBarResource(IDR_VARIABLE);

}

//--------------------------------------------------------------------------


CMainFrame::~CMainFrame()
{}

//--------------------------------------------------------------------------


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CBCGMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
        return -1;

    m_MainToolBar.CreateEx(this,TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
                           | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

    m_CaptureToolBar.CreateEx(this,TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
                              | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

    m_VariableToolBar.CreateEx(this,TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
                               | CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

    m_MainToolBar.LoadToolBar(IDR_MAINFRAME);
    m_CaptureToolBar.LoadToolBar(IDR_CAPTURE);
    m_VariableToolBar.LoadToolBar(IDR_VARIABLE);

    m_MainToolBar.EnableDocking(CBRS_ALIGN_ANY);
    m_CaptureToolBar.EnableDocking(CBRS_ALIGN_ANY);
    m_VariableToolBar.EnableDocking(CBRS_ALIGN_ANY);
    EnableDocking(CBRS_ALIGN_ANY);

    DockControlBar(&m_MainToolBar);
    DockControlBar(&m_CaptureToolBar);
    DockControlBar(&m_VariableToolBar);

    DockControlBarLeftOf(&m_CaptureToolBar,&m_MainToolBar);
    DockControlBarLeftOf(&m_VariableToolBar,&m_CaptureToolBar);



    if (!m_wndStatusBar.Create(this) ||
            !m_wndStatusBar.SetIndicators(indicators,
                                          sizeof(indicators)/sizeof(UINT)))
    {
        TRACE0("Failed to create status bar\n");
        return -1;      // fail to create
    }




    ReadInfo();

    return 0;
}

//----------------------------------------------------------------------------

void CMainFrame::OnDestroy()
{
    SaveInfo();

    CBCGMDIFrameWnd::OnDestroy();
}

//--------------------------------------------------------------------------

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
    cs.lpszClass = theApp.GetClassName() ;

    if( !CBCGMDIFrameWnd::PreCreateWindow(cs) )
        return FALSE;
    // TODO: Modify the Window class or styles here by modifying
    //  the CREATESTRUCT cs

    return TRUE;
}

//--------------------------------------------------------------------------

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
    CBCGMDIFrameWnd::AssertValid();
}

//--------------------------------------------------------------------------

void CMainFrame::Dump(CDumpContext& dc) const
{
    CBCGMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG



//--------------------------------------------------------------------------

LRESULT CMainFrame::OnGetAppArgs( WPARAM wFlags , LPARAM pParam )
{
    COPYDATASTRUCT* pDataStruct = (COPYDATASTRUCT* ) pParam ;
    TCHAR* pczBuffer = (TCHAR*) (pDataStruct->lpData) ;

    theApp.ApplyProjectInfo( pczBuffer ) ;

    return TRUE ;
}


//----------------------------------------------------------------------------

void CMainFrame::ReadInfo()
{
    ASSERT ( m_pMainWndInfo == NULL );

    m_pMainWndInfo = theApp.GetSerialization()->GetObject( ID_SERIALIZE_MAINWND );
    if ( m_pMainWndInfo == NULL )
    {
        m_pMainWndInfo = theApp.GetSerialization()->AddObject( ID_SERIALIZE_MAINWND );

    }
    else
    {
        WINDOWPLACEMENT wp;

        m_pMainWndInfo->ResetEnum();
        wp.length                   = m_pMainWndInfo->GetInt();
        wp.flags                    = m_pMainWndInfo->GetInt();
        wp.showCmd                  = m_pMainWndInfo->GetInt();
        wp.ptMinPosition.x          = m_pMainWndInfo->GetLong();
        wp.ptMinPosition.y          = m_pMainWndInfo->GetLong();
        wp.ptMaxPosition.x          = m_pMainWndInfo->GetLong();
        wp.ptMaxPosition.y          = m_pMainWndInfo->GetLong();
        wp.rcNormalPosition.bottom  = m_pMainWndInfo->GetLong();
        wp.rcNormalPosition.left    = m_pMainWndInfo->GetLong();
        wp.rcNormalPosition.right   = m_pMainWndInfo->GetLong();
        wp.rcNormalPosition.top     = m_pMainWndInfo->GetLong();

        SetWindowPlacement(&wp);
    }

}

//----------------------------------------------------------------------------

void CMainFrame::SaveInfo()
{
    ASSERT ( m_pMainWndInfo != NULL );

    WINDOWPLACEMENT wp;
    GetWindowPlacement(&wp);

    m_pMainWndInfo->ResetEnum();
    m_pMainWndInfo->SetInt ( wp.length                  );
    m_pMainWndInfo->SetInt ( wp.flags                   );
    m_pMainWndInfo->SetInt ( wp.showCmd                 );
    m_pMainWndInfo->SetLong( wp.ptMinPosition.x         );
    m_pMainWndInfo->SetLong( wp.ptMinPosition.y         );
    m_pMainWndInfo->SetLong( wp.ptMaxPosition.x         );
    m_pMainWndInfo->SetLong( wp.ptMaxPosition.y         );
    m_pMainWndInfo->SetLong( wp.rcNormalPosition.bottom );
    m_pMainWndInfo->SetLong( wp.rcNormalPosition.left   );
    m_pMainWndInfo->SetLong( wp.rcNormalPosition.right  );
    m_pMainWndInfo->SetLong( wp.rcNormalPosition.top    );


}

LRESULT CMainFrame::OnCaretPosChanged(WPARAM,LPARAM lParam)
{
    int nLine = LOWORD(lParam);
    int nCol = HIWORD(lParam);

    int nIndex = m_wndStatusBar.CommandToIndex(ID_INDICATOR_CURSOR);
    if(nIndex == -1)
        return 0;

    if(nLine == (WORD)-1)
    {
        m_wndStatusBar.SetPaneInfo(nIndex,ID_INDICATOR_CURSOR,SBPS_NORMAL,50);
        m_wndStatusBar.SetPaneText(nIndex,"");
    }
    else
    {
        CString strText;
        strText.Format("Ln %d,Col %d",nLine,nCol);
        m_wndStatusBar.SetPaneText(nIndex,strText);

        CClientDC dc(&m_wndStatusBar);
        int nWidth = dc.GetTextExtent(strText).cx;
        m_wndStatusBar.SetPaneInfo(nIndex,ID_INDICATOR_CURSOR,SBPS_NORMAL,nWidth);
    }

    return 0;
}

void CMainFrame::OnHelpContents()
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp;

    HtmlHelp(NULL, contextHelp, HH_DISPLAY_TOC, 0);
}

void CMainFrame::OnHelpIndex()
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp;

    HtmlHelp(NULL, contextHelp, HH_DISPLAY_INDEX, 0);
}

void CMainFrame::OnHelpSearch()
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp;

    HH_FTS_QUERY q ;
    q.cbStruct         = sizeof(HH_FTS_QUERY) ;
    q.fUniCodeStrings  = FALSE ;
    q.pszSearchQuery   = "";
    q.iProximity       = HH_FTS_DEFAULT_PROXIMITY ;
    q.fStemmedSearch   = FALSE ;
    q.fTitleOnly       = FALSE ;
    q.fExecute         = FALSE ;
    q.pszWindow        = NULL ;
    HtmlHelp(NULL, contextHelp, HH_DISPLAY_SEARCH, (DWORD)&q);
}

void CMainFrame::OnHelpSclreference()
{
    CString contextHelp;
    contextHelp.LoadString(IDS_SCL_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp;

    HtmlHelp(NULL, contextHelp, HH_DISPLAY_TOC, 0);
}
