/* Copyright (C) 2000 Cyrano SA

This file is part of OpenSTA

OpenSTA is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free
Software Foundation; either version 2, or (at your option) any later
version.

OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along
with OpenSTA; see the file COPYING.  If not, write to the Free Software
Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

For more information about the GNU Public license see:
http://www.gnu.org/copyleft/gpl.html

OpenSTA is at:
http://www.opensta.org/
http://opensta.sourceforge.net/
*/

// ModellerDoc.cpp : implementation of the CModellerDoc class
//
#include "stdafx.h"
#include <IECache.h>
#include <CSerialization.h>
#include <ConstantesLib.h>
#include <phParse.hpp>
#include <SCLParser.h>
#include "CyrExecuteLocalCommand.h"
#include "Modeller.h"
#include "Constantes.h"
#include "SCLCommentsDlg.h"
#include "resource.h"
#include "VarPSheet.h"
#include "DelPSheet.h"
#include "SyncPSheet.h"
#include "LoopPSheet.h"
#include "IfPSheet.h"
#include "library.hpp"
#include "SCLGenerator.h"
#include "VIPSheet.h"
#include "URLListDlg.h"

#include "VariousObject.h"
#include "ModellerDoc.h"
#include "ChildFrm.h"
#include "CaptureSaveDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//#pragma warning( disable : 4786 )

extern CModellerApp     theApp ;
extern CSCLStructure*   pSCLStructure ;
extern CVarInfo*        pVarInfo;
extern pDB_URL_FIELDS   pURLInfo;


typedef int  (*fnTELibMain) (HWND hwind, char *TofFile, char *OdlDir);
typedef void (*fnTELibStop)(int);

/////////////////////////////////////////////////////////////////////////////
// CModellerDoc
//--------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CModellerDoc, CBasedDocument)

//--------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CModellerDoc, CBasedDocument)
//{{AFX_MSG_MAP(CModellerDoc)
ON_COMMAND(ID_CAPTURE_CHECK, OnCaptureCheck)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_CHECK, OnUpdateCaptureCheck)
ON_COMMAND(ID_CAPTURE_PAUSE, OnCapturePause)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_PAUSE, OnUpdateCapturePause)
ON_COMMAND(ID_CAPTURE_RECORD, OnCaptureRecord)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_RECORD, OnUpdateCaptureRecord)
ON_COMMAND(ID_CAPTURE_REPLAY, OnCaptureReplay)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_REPLAY, OnUpdateCaptureReplay)
ON_COMMAND(ID_CAPTURE_STOP, OnCaptureStop)
ON_UPDATE_COMMAND_UI(ID_CAPTURE_STOP, OnUpdateCaptureStop)
ON_COMMAND(ID_SCL_COMMENT, OnSclComment)
ON_UPDATE_COMMAND_UI(ID_SCL_COMMENT, OnUpdateSclComment)
ON_COMMAND(ID_FILE_SAVE, OnFileSave)
ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
ON_UPDATE_COMMAND_UI(IDM_VARIABLE_CREATE, OnUpdateVariableCreate)
ON_COMMAND(IDM_VARIABLE_CREATE, OnVariableCreate)
ON_UPDATE_COMMAND_UI(IDM_VARIABLE_MODIFY, OnUpdateVariableModify)
ON_COMMAND(IDM_VARIABLE_MODIFY, OnVariableModify)
ON_COMMAND(IDM_DELAY, OnDelay)
ON_COMMAND(IDM_SYNCHRONIZE, OnSynchronize)
ON_COMMAND(IDM_CONDITION, OnCondition)
ON_COMMAND(IDM_LOOP, OnLoop)
ON_COMMAND(IDM_INIT_VARIABLE, OnInitVariable)
ON_UPDATE_COMMAND_UI(ID_VIEW_REFRESH, OnUpdateViewRefresh)
ON_COMMAND(ID_VIEW_REFRESH, OnViewRefresh)
ON_COMMAND(ID_FILE_SAVE_AS, OnFileSaveAs)
ON_UPDATE_COMMAND_UI(ID_FILE_SAVE_AS, OnUpdateFileSaveAs)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()



/////////////////////////////////////////////////////////////////////////////
// CModellerDoc construction/destruction
//--------------------------------------------------------------------------

CModellerDoc::CModellerDoc()
        :CBasedDocument(),
        m_pCaptureView(NULL),
        m_Gateway(this),
        m_bRelease(TRUE),
        m_bPause( FALSE ) ,
        m_bLocalRecord( FALSE ) ,
        m_bLocalReplay( FALSE ) ,
        m_hOutputReplayWnd(0),
        m_pReplayLibrary(NULL)
{}

//--------------------------------------------------------------------------


CModellerDoc::~CModellerDoc()
{
    if (m_pReplayLibrary != NULL)
        delete m_pReplayLibrary;

}


//--------------------------------------------------------------------------

void CModellerDoc::SetCaptureView(CSCLCaptureScriptView* _pCaptureView)
{
    m_pCaptureView = _pCaptureView;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::Parse(BOOL _bForceParse)
{
    CString strScript ;
    strScript = m_pCaptureView->m_Editor.GetWindowText() ;

    return Parse(strScript, _bForceParse);
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::Parse(CString _strScript, BOOL _bForceParse)
{
    BOOL bRet;
    BOOL bModified = m_pCaptureView->m_Editor.IsModified();

    pSCLStructure = &m_SCLStructure;

    if ( !bModified && !_bForceParse)
        return TRUE;

    pSCLStructure->Reset();

    SCLParser SclParser( (HANDLE)AfxGetInstanceHandle() ) ;
    SclParser.Reset() ;
    bRet = SclParser.ParseString( _strScript);

    CString strDataFile ;
    strDataFile = GetSCLDocInfo()->GetWorkingFolder();
    strDataFile += GetSCLDocInfo()->GetALLFileName();

    m_SCLStructure.SetHTMLURL(strDataFile);

    return bRet;

}

//--------------------------------------------------------------------------

BOOL CModellerDoc::ParseVariable(CString _strScript, CVarInfo *_pVarInfo)
{
    BOOL bRet;
    SCLParser SclParser( (HANDLE)AfxGetInstanceHandle() ) ;
    SclParser.Reset() ;

    pVarInfo = _pVarInfo;

    bRet = SclParser.ParseString( _strScript);
    ASSERT(bRet);

    pVarInfo = NULL;
    return bRet;

}


//--------------------------------------------------------------------------

BOOL CModellerDoc::ParseGet(long _lLine, pDB_URL_FIELDS  _pURLInfo)
{
    //
    CString strText;
    long lEndLine;

    //lEndLine = m_pCaptureView->m_Editor.GetEdit()->m_iLinesInFile;
    lEndLine = _lLine+50;

    strText = m_pCaptureView->m_Editor.GetString(_lLine,lEndLine);

    return ParseGet(strText, _pURLInfo);
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::ParseGet(CString _strScript, pDB_URL_FIELDS _pURLInfo)
{
    BOOL bRet;
    SCLParser SclParser( (HANDLE)AfxGetInstanceHandle() ) ;
    SclParser.Reset() ;

    pURLInfo = _pURLInfo;

    pSCLStructure = NULL;

    bRet = SclParser.ParseString( _strScript);
    //ASSERT(bRet);

    pURLInfo = NULL;
    return bRet;

}

//--------------------------------------------------------------------------
void CModellerDoc::OnUpdateGoToURL(long _lLine, CCmdUI* pCmdUI )
{
    CString strDataFile;
    std::list<URLKey>   URLList ;


    strDataFile = GetSCLDocInfo()->GetFullALLFileName() ;

    CIECache cache( strDataFile ) ;
    if( !cache.Init() )
        return;

    DB_URL_FIELDS  URLInfo;
    ParseGet(_lLine, &URLInfo);
    URLInfo.SetHTMLURL(strDataFile);

    if ( !URLInfo.m_strURL.IsEmpty() )
        cache.QueryHTMLURL( URLInfo.m_strURL, URLInfo.m_lConId, URLList ) ;

    pCmdUI->Enable((URLInfo.m_bHTMLURL) || (URLList.size()>0));
}

//--------------------------------------------------------------------------

void CModellerDoc::GoToURL(long _lLine)
{
    //----
    //Init Cache
    CString             strDataFile;
    CString             strBody ;
    DWORD               dwLength ;
    std::list<URLKey>   URLList ;
    long                lSize ;

    strDataFile = GetSCLDocInfo()->GetFullALLFileName() ;

    theApp.SetGlobalOffLine( TRUE ) ;

    CIECache cache( strDataFile ) ;
    if( !cache.Init() )
        return;

    long lRequestID = 1 ;


    DB_URL_FIELDS  URLInfo;
    ParseGet(_lLine,&URLInfo);

    URLInfo.SetHTMLURL(strDataFile);

    m_strLastURL    = URLInfo.m_strURL;
    m_lConId        = URLInfo.m_lConId;

    cache.QueryHTMLURL( m_strLastURL, m_lConId, URLList ) ;

    lSize   = URLList.size() ;
    if( lSize == 0 )
    {
        CString strTitle ;
        CString strMsg ;
        //---
        strTitle.LoadString( IDS_WARNING ) ;
        strMsg.LoadString( IDS_NOMATCH ) ;

        AfxGetMainWnd()->MessageBox( strMsg, strTitle , MB_OK ) ;
    }
    else if( lSize == 1 )
    {
        URLKey  key ;
        //---
        key             = URLList.front() ;
        m_strLastURL    = key.m_strURL ;
        m_lConId        = key.m_dwConnId ;
    }
    else
    {
        URLKey          key ;
        CURLListDlg     urllistdlg( URLList, key ) ;
        //---
        if( urllistdlg.DoModal() == IDCANCEL )
            return ;

        m_strLastURL    = key.m_strURL ;
        m_lConId        = key.m_dwConnId ;
    }

    //HTML View
    cache.CacheAllURLsFromCaptureFile();
    CStringObject objURL(m_strLastURL);
    UpdateAllViews( NULL , ID_DISPLAY_PRIMARY_URL , &objURL );

    //ClientHeaderView
    LPCSTR pcClientValue = cache.GetClientHeader(m_strLastURL, m_lConId, &dwLength , lRequestID);
    CString strClientHeader( pcClientValue, dwLength );
    CStringObject objClient( strClientHeader ) ;
    UpdateAllViews( NULL , ID_DISPLAY_CLIENTHEADER_URL , &objClient );

    //ServerHeaderView
    LPCSTR pcServerValue = cache.GetServerHeader(m_strLastURL, m_lConId, &dwLength , lRequestID);
    CString strServerHeader( pcServerValue, dwLength );
    CStringObject objServer( strServerHeader ) ;
    UpdateAllViews( NULL , ID_DISPLAY_SERVERHEADER_URL , &objServer );

    //DomView
    strBody = (char*) ( cache.GetBody(m_strLastURL , m_lConId , &dwLength , lRequestID ) );
    m_DomStructure.Reset();
    m_DomStructure.Init();

    PhLexer<unsigned char*> lex(&m_DomStructure, (UCHAR*)((LPCTSTR)strBody));
    PhParser<unsigned char*> Parser(&m_DomStructure ,lex ) ;
    //---
    if( !Parser.Parse( ) )
    {
        //        AfxGetMainWnd()->MessageBox( "Parsing not managed !", "Warning" , MB_OK ) ;
    }


    UpdateAllViews( NULL , ID_DISPLAY_DOM_URL , NULL );

}


//--------------------------------------------------------------------------

void CModellerDoc::SetOutputReplayWnd(HWND _hOutputReplayWnd)
{
    m_hOutputReplayWnd = _hOutputReplayWnd;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::OnNewDocument()
{
    if (!CBasedDocument::OnNewDocument())
        return FALSE;

    return TRUE;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
    if (!CBasedDocument::OnOpenDocument(lpszPathName))
        return FALSE;

    return TRUE;
}

//-------------------------------------------------------------------------

BOOL CModellerDoc::OnSaveDocument(LPCTSTR lpszPathName)
{
    return CBasedDocument::OnSaveDocument(lpszPathName);
}

//--------------------------------------------------------------------------

void CModellerDoc::Serialize(CArchive& ar)
{
    if (ar.IsStoring())
    {}

    else
    {}

}

/////////////////////////////////////////////////////////////////////////////
// CModellerDoc diagnostics
//--------------------------------------------------------------------------

#ifdef _DEBUG
void CModellerDoc::AssertValid() const
{
    CBasedDocument::AssertValid();
}

//--------------------------------------------------------------------------

void CModellerDoc::Dump(CDumpContext& dc) const
{
    CBasedDocument::Dump(dc);
}
#endif //_DEBUG



//--------------------------------------------------------------------------

void CModellerDoc::OnCaptureCheck()
{
    CWaitCursor wc;

    if(m_pCaptureView->m_Editor.IsModified())
    {
        CString strScript= m_pCaptureView->m_Editor.GetWindowText() ;

        CFile file ;
        CString strSCLFile = GetSCLDocInfo()->GetFullSCLFileName() ;
        if( file.Open( strSCLFile , CFile::modeCreate|CFile::modeWrite|CFile::shareExclusive ) )
        {
            file.Write( strScript , strScript.GetLength() ) ;
            file.Close() ;
        }
    }

    POSITION pos = GetFirstViewPosition();
    CChildFrame *pFrame = static_cast<CChildFrame *>(GetNextView(pos)->GetParentFrame());
    pFrame->ShowCheckWnd();

    //1) chemin court du compilateur
    TCHAR  szModellerPath[_MAX_PATH];
    TCHAR  szShortSCLCompilerPath[_MAX_PATH];
    char szdrive[_MAX_DRIVE];
    char szdir[_MAX_DIR];

    CString strSCLCompilerPath;

    // ----
    VERIFY(::GetModuleFileName(AfxGetApp()->m_hInstance, szModellerPath, _MAX_PATH));

    _splitpath( szModellerPath, szdrive, szdir, NULL, NULL );

    strcpy(szModellerPath,szdrive);
    strcat(szModellerPath,szdir);

    strSCLCompilerPath = GetExtendedProfileString(HKEY_CURRENT_USER, theApp.GetRegistryPath(),
                         STR_SCL_COMPILER_ENTRY, szModellerPath);

    GetShortPathName( (LPCSTR)strSCLCompilerPath,
                      szShortSCLCompilerPath,
                      _MAX_PATH);


    //2) chemin du fichier SCL
    CString strFullSCLFileName ;
    CString strFullTOFFileName ;

    strFullSCLFileName = GetSCLDocInfo()->GetFullSCLFileName() ;
    strFullTOFFileName = GetSCLDocInfo()->GetFullTOFFileName() ;

    //3) formatage de la commande
    CString strCmd;
    CString strSCLExecute;


    strCmd.LoadString( IDS_SCL_CHECKER ) ;
    strSCLExecute.Format( strCmd , szShortSCLCompilerPath, strFullTOFFileName, strFullSCLFileName) ;

    //Execution
    CyrExecuteLocalCommand Checker(GetSCLDocInfo()->GetSCLPath()) ;

    UpdateAllViews( NULL , ID_SCL_CHECKER , &CStringObject("Compiling...") )  ;

    CString strOutput;
    if( Checker.ExecuteCommand  ((CHAR*)((LPCTSTR)strSCLExecute) , strOutput) == 0 )
    {
        if(strOutput.IsEmpty())
            strOutput = "Compilation Successful";
        CStringObject objOutput((CString)strOutput);
        UpdateAllViews( NULL , ID_SCL_CHECKER , &objOutput ) ;
    }

    //copy  dans le repertoire project
    CString strExistingTofFileName;
    CString strNewTofFileName;

    strExistingTofFileName  =  szShortSCLCompilerPath;
    strExistingTofFileName +=  GetSCLDocInfo()->GetTOFFileName();

    strNewTofFileName =  GetSCLDocInfo()->GetFullTOFFileName();

    CopyFile(strExistingTofFileName, strNewTofFileName, FALSE);
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCaptureCheck(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( !theApp.m_bReplay && !theApp.m_bRecording ) ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnCapturePause()
{
    m_Gateway.PauseRecord() ;
    m_bPause = !m_bPause ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCapturePause(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( theApp.m_bRecording && m_bLocalRecord) ;
    pCmdUI->SetCheck(m_bPause ? 1 : 0);
}

//--------------------------------------------------------------------------

void CModellerDoc::OnCaptureRecord()
{
    GetSCLDocInfo()->SetBrowserID(theApp.m_nBrowserID);
    if(!m_Gateway.GetDefaultParameters(&theApp.m_GatewayData.m_ProxyInfo.m_LocalProxy))
        return;

    CWaitCursor wc;

    CString strScript;

    if(GetSCLDocInfo()->GetSCLFileName() == UNTITLED_SCL_DOCUMENT)
    {
        CCaptureSaveDlg dlg(CCaptureSaveDlg::enScriptName);
        if(dlg.DoModal()!=IDOK)
            return;

        GetSCLDocInfo()->SetFileInfo(dlg.m_strScriptName,GetSCLDocInfo()->GetBrowserID());
    }

    //
    if(m_pCaptureView->m_Editor.IsModified())
    {
        int nResult = AfxMessageBox(IDS_SAVE_MODIFIED_SCRIPT,MB_YESNOCANCEL);
        if(nResult == IDCANCEL)
            return;
        else
            if(nResult == IDYES)
            {
                DoSave(NULL);
            }
    }

    strScript.Empty();
    Parse(strScript);
    m_pCaptureView->SetText(strScript,TRUE) ;
    m_pCaptureView->ShowPrimaryURL();

    //

    //  UpdateAllViews( NULL, ID_DEFAULT_GATEWAY_PARAMETER, NULL ) ;

    theApp.SetGlobalOffLine( FALSE ) ;
    theApp.m_bRecording = m_Gateway.Record(theApp.m_bConsole,theApp.m_bTrace) ;
    m_bLocalRecord      = theApp.m_bRecording;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCaptureRecord(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( !theApp.m_bRecording && !theApp.m_bReplay ) ;
}


//--------------------------------------------------------------------------

void CModellerDoc::OnCaptureReplay()
{
    CWaitCursor wc;
    //---
    if(m_pCaptureView->m_Editor.IsModified())
    {
        CString strScript= m_pCaptureView->m_Editor.GetWindowText() ;

        CFile file ;
        CString strSCLFile = GetSCLDocInfo()->GetFullSCLFileName() ;
        if( file.Open( strSCLFile , CFile::modeCreate|CFile::modeWrite|CFile::shareExclusive ) )
        {
            file.Write( strScript , strScript.GetLength() ) ;
            file.Close() ;
        }
    }

    POSITION pos = GetFirstViewPosition();
    CChildFrame *pFrame = static_cast<CChildFrame *>(GetNextView(pos)->GetParentFrame());
    pFrame->ShowReplayWnd();

    //clear replay output window
    UpdateAllViews( NULL , ID_REPLAY_SCL ,&CStringObject("")) ;

    //1) chemin du fichier SCL
    CString strFullTofFileName ;
    CString strFullSclFileName ;

    strFullTofFileName = GetSCLDocInfo()->GetFullTOFFileName();
    strFullSclFileName = GetSCLDocInfo()->GetFullSCLFileName();

    if(!IsTofUptodate(strFullTofFileName,strFullSclFileName))
    {
        //Do the compile
        OnCaptureCheck();

        //Check if the tof is uptodate, this will verify that the compile succeeded
        if(!IsTofUptodate(strFullTofFileName,strFullSclFileName))
            return;   //Compile failed

        //Select the replay window again
        pFrame->ShowReplayWnd();
    }

    {
        TCHAR  szModellerPath[_MAX_PATH];
        TCHAR  szShortModellerPath[_MAX_PATH];
        char szdrive[_MAX_DRIVE];
        char szdir[_MAX_DIR];


        // ----
        VERIFY(::GetModuleFileName(AfxGetApp()->m_hInstance, szModellerPath, _MAX_PATH));

        _splitpath( szModellerPath, szdrive, szdir, NULL, NULL );

        strcpy(szModellerPath,szdrive);
        strcat(szModellerPath,szdir);

        GetShortPathName( szModellerPath,
                          szShortModellerPath,
                          _MAX_PATH);

        theApp.m_bReplay = TRUE;
        m_bLocalReplay   = TRUE;
        try
        {
            if ( m_pReplayLibrary == NULL )
            {
                m_pReplayLibrary = new CTLibrary("TEHttpLib");
                CTProcedure TELibMainProc(*m_pReplayLibrary, "TELibMain") ;
                fnTELibMain TELibMain = (fnTELibMain) TELibMainProc.getAddress();
                TELibMain(m_hOutputReplayWnd, (CHAR*)((LPCTSTR)strFullTofFileName), szShortModellerPath);
            }
        }
        catch(CTException &e)
        {
            delete m_pReplayLibrary;
            m_pReplayLibrary = NULL;
        }

    }
}

//--------------------------------------------------------------------------

void CModellerDoc::StopReplay(int _iMode)
{

    try
    {
        theApp.m_bReplay = FALSE;
        m_bLocalReplay   = FALSE;

        CTProcedure TELibStopProc(*m_pReplayLibrary, "TELibStop") ;
        fnTELibStop TELibStop = (fnTELibStop) TELibStopProc.getAddress();
        TELibStop(_iMode);

        if ((_iMode == 0)&&(m_pReplayLibrary!=NULL))
        {
            delete m_pReplayLibrary;
            m_pReplayLibrary = NULL;
        }

    }
    catch(CTException &)
    {
        try
        {
            if(m_pReplayLibrary!=NULL) delete m_pReplayLibrary;
        }
        catch(...)
        {
        }

        m_pReplayLibrary = NULL;
    }
}


//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCaptureReplay(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(  !theApp.m_bRecording && !theApp.m_bReplay ) ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnCaptureStop()
{
    if ( theApp.m_bRecording)
    {
        CWaitCursor wc;
        //---
        theApp.m_bRecording = FALSE;
        m_bLocalRecord      = FALSE;
        if( m_Gateway.StopRecord() )
        {
            CString strScript;
            //----
            m_Gateway.GetSCLRecordedScript(strScript) ;

            Parse(strScript);
            m_pCaptureView->SetText(strScript,TRUE) ;
            m_pCaptureView->ShowPrimaryURL();
        }
    }
    else
        StopReplay(1);

    m_bPause = false;

}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateCaptureStop(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( (theApp.m_bRecording && m_bLocalRecord ) || ( theApp.m_bReplay && m_bLocalReplay)) ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateViewRefresh(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//--------------------------------------------------------------------------

void CModellerDoc::OnViewRefresh()
{
    Parse(TRUE);
    m_pCaptureView->ShowPrimaryURL();
}

//--------------------------------------------------------------------

void CModellerDoc::OnUpdateVariableCreate(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE &&  !theApp.m_bRecording && !theApp.m_bReplay);
}

//--------------------------------------------------------------------

void CModellerDoc::OnVariableCreate()
{
    CVarInfo    VarInfo ;
    CString     strLineToInsert ;
    long        lineToInsert ;

    VarInfo.m_VarGeneral.m_strVarName = theApp.m_strVariablePrefix;
    CVarPSheet WizardDlg( &VarInfo, TRUE, TRUE, GetSCLStructure(), GetSCLDocInfo()->GetFullALLFileName()  ) ;

    if( WizardDlg.DoModal() == IDCANCEL )
        return ;

    Parse();

    CString strFileNameInScript ;
    CString strFVRFileFullPath ;

    if( VarInfo.m_VarGeneral.m_lSourceInfo == VAR_SOURCE_FILE )
    {// FILE
        CString strValue ;
        int     nPos ;

        //
        nPos     = VarInfo.m_VarFile.m_strFilePath.ReverseFind( '\\' ) ;
        if( nPos >= 0 )
        {
            strValue = VarInfo.m_VarFile.m_strFilePath.Mid( nPos + 1 ) ;
        }

        nPos     = strValue.ReverseFind( '.' );
        if( nPos >= 0 )
        {
            strValue = strValue.Left( nPos ) ;
        }

        strFileNameInScript = strValue ;
        strFVRFileFullPath  = GetSCLDocInfo()->GetFVRPath() + strValue + STR_FVR_EXTENSION;

        CopyFile( VarInfo.m_VarFile.m_strFilePath, strFVRFileFullPath, FALSE ) ;

        strValue = VarInfo.m_VarFile.m_strFilePath ;


        strLineToInsert = CSCLGenerator::GenVarFileDecl( VarInfo.m_VarGeneral.m_strVarName,
                          strFileNameInScript,
                          VarInfo.m_VarGeneral.m_lVarType,
                          VarInfo.m_VarGeneral.m_lVarScope ) ;

    }
    else if (VarInfo.m_VarValueList.m_bGenerateFile)
    {//GENERATE IN FILE
        try
        {
            strFVRFileFullPath  = GetSCLDocInfo()->GetFVRPath() + VarInfo.m_VarGeneral.m_strVarName + STR_FVR_EXTENSION;
            strFVRFileFullPath.MakeUpper() ;

            CStdioFile FVRFile( strFVRFileFullPath, CFile::modeCreate
                                | CFile::modeWrite | CFile::typeText ) ;

            for( int i=0; i < VarInfo.m_VarValueList.m_Values.GetSize(); i++ )
                FVRFile.WriteString( VarInfo.m_VarValueList.m_Values.GetAt(i) + "\n" ) ;

            FVRFile.Close() ;
            strFileNameInScript = VarInfo.m_VarGeneral.m_strVarName ;
        }
        catch( CFileException e )
        {}


        strLineToInsert = CSCLGenerator::GenVarFileDecl( VarInfo.m_VarGeneral.m_strVarName,
                          strFileNameInScript,
                          VarInfo.m_VarGeneral.m_lVarType,
                          VarInfo.m_VarGeneral.m_lVarScope ) ;


    }
    else
    {// VALUE LIST
        strLineToInsert = CSCLGenerator::GenVarValListDecl(  VarInfo.m_VarGeneral.m_strVarName,
                          &(VarInfo.m_VarValueList.m_Values),
                          VarInfo.m_VarGeneral.m_lVarOrder,
                          VarInfo.m_VarGeneral.m_lVarType,
                          VarInfo.m_VarGeneral.m_lVarScope ) ;
    }


    lineToInsert    = m_SCLStructure.GetCodeLine() ;
    m_pCaptureView->m_Editor.InsertString( lineToInsert, strLineToInsert ) ;

}

//--------------------------------------------------------------------

void CModellerDoc::OnUpdateVariableModify(CCmdUI* pCmdUI)
{
    long lCurrentLine;
    long lBeginLine;
    long lEndLine;
    BOOL bVarLine;

    lCurrentLine= m_pCaptureView->m_Editor.GetCurrentLine();
    bVarLine = m_SCLStructure.IsVariableLine(lCurrentLine, &lBeginLine, &lEndLine);
    pCmdUI->Enable(bVarLine &&  !theApp.m_bRecording && !theApp.m_bReplay);
}

//--------------------------------------------------------------------

void CModellerDoc::OnVariableModify()
{
    long    lCurrentLine;
    long    lBeginLine;
    long    lEndLine;
    CString strText;
    BOOL    bVarLine;
    CString     strLineToInsert ;
    long        lineToInsert ;
    //---

    pVarInfo = NULL;
    Parse();

    lCurrentLine= m_pCaptureView->m_Editor.GetCurrentLine();
    bVarLine = m_SCLStructure.IsVariableLine(lCurrentLine, &lBeginLine, &lEndLine);

    if (!bVarLine)
        return;

    strText = m_pCaptureView->m_Editor.GetString(lBeginLine,lEndLine);

    CVarInfo VarInfo;
    ParseVariable(strText, &VarInfo);

    VarInfo.m_VarFile.m_strFVRPath = GetSCLDocInfo()->GetFVRPath() ;

    CString &strFile = VarInfo.m_VarFile.m_strFilePath;
    if(strFile.Right(strlen(STR_FVR_EXTENSION)).CompareNoCase(STR_FVR_EXTENSION)!=0)
        strFile += STR_FVR_EXTENSION;


    CVarPSheet WizardDlg( &VarInfo, FALSE, VarInfo.m_VarGeneral.m_lSourceInfo == VAR_SOURCE_FILE ) ;

    if( WizardDlg.DoModal() == IDCANCEL )
        return ;

    m_pCaptureView->m_Editor.DeleteLines( lBeginLine, lEndLine );

    Parse();

    if( VarInfo.m_VarGeneral.m_lSourceInfo == VAR_SOURCE_FILE )
    {
        //If has FVR extension remove it
        int nPos = VarInfo.m_VarFile.m_strFilePath.ReverseFind('.');
        if(nPos != -1)
            VarInfo.m_VarFile.m_strFilePath = VarInfo.m_VarFile.m_strFilePath.Left(nPos);

        strLineToInsert = CSCLGenerator::GenVarFileDecl( VarInfo.m_VarGeneral.m_strVarName,
                          VarInfo.m_VarFile.m_strFilePath,
                          VarInfo.m_VarGeneral.m_lVarType,
                          VarInfo.m_VarGeneral.m_lVarScope ) ;
    }
    else
    {
        strLineToInsert = CSCLGenerator::GenVarValListDecl(  VarInfo.m_VarGeneral.m_strVarName,
                          &(VarInfo.m_VarValueList.m_Values),
                          VarInfo.m_VarGeneral.m_lVarOrder,
                          VarInfo.m_VarGeneral.m_lVarType,
                          VarInfo.m_VarGeneral.m_lVarScope ) ;
    }

    lineToInsert    = lBeginLine;
    m_pCaptureView->m_Editor.InsertString( lineToInsert, strLineToInsert ) ;
}

//--------------------------------------------------------------------------

void CModellerDoc::GetGatewayDefaultInfomation( _TPreferencesProxyData *pData )
{
    m_Gateway.GetDefaultParameters(  pData ) ;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::IsCaptureLocal()
{
    return theApp.IsCaptureLocal();
}


//--------------------------------------------------------------------------

void CModellerDoc::OnSclComment()
{
    CString strComments ;
    //----
    CSCLCommentsDlg dlg( &strComments ) ;
    //---
    int nRet = dlg.DoModal() ;
    if( nRet == IDOK )
        m_Gateway.InsertComments( strComments ) ;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateSclComment(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( IsRecording() && !m_bPause && m_bLocalRecord) ;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::IsRecording()
{
    return theApp.m_bRecording ;
}

//--------------------------------------------------------------------------

void CModellerDoc::GetGatewayInfomation( _TPreferencesGatewayData *pData )
{
    m_pCaptureView->GetGatewayInfomation( pData ) ;
}


//--------------------------------------------------------------------------

BOOL CModellerDoc::RetrieveFromRepository()
{
    CWaitCursor wc;

    CString strProjectDir = GetSCLDocInfo()->GetProjectPath() + theApp.m_RepositoryAPI.GetOpenProjectName() + "\\";
    CString strSourcesDir = strProjectDir + STR_SOURCE + "\\";
    CString strWorkingPath = GetSCLDocInfo()->GetWorkingFolder();

    CopyFile(strSourcesDir + STR_REPOSITORY_GLOBALVAR,strWorkingPath + STR_REPOSITORY_GLOBALVAR,false);
    CopyFile(strSourcesDir + STR_REPOSITORY_INCLUDE,strWorkingPath + STR_REPOSITORY_INCLUDE,false);

    if(GetSCLDocInfo()->GetSCLFileName() == UNTITLED_SCL_DOCUMENT)
        return true;

    theApp.m_RepositoryAPI.RetrieveProjectCaptureFiles(GetSCLDocInfo()->GetSCLFileName());

    //---
    CFile file ;
    //----
    CString strSCLFile = strSourcesDir + GetSCLDocInfo()->GetSCLFileName();

    if( file.Open( strSCLFile, CFile::modeRead|CFile::shareDenyWrite) )
    {
        //check browser
        CObjScript obscript;

        int nCount = file.GetLength() ;
        TCHAR *pczBuffer = new TCHAR[nCount + 1 ] ;
        file.Read( pczBuffer , nCount ) ;
        pczBuffer[nCount] = '\0' ;
        if( pczBuffer )
        {
            obscript.m_strScript = pczBuffer ;
            delete pczBuffer ;
        }
        file.Close() ;
        //----
        obscript.m_bModify = FALSE;

        InitBrowserFromScript( &obscript) ;

        //Copy capture to working folder
        CopyFile(strSCLFile,strWorkingPath + GetSCLDocInfo()->GetSCLFileName(),false);

        CString strAllFile = strProjectDir + STR_CAPTURE + "\\" + GetSCLDocInfo()->GetSCLFileName();
        strAllFile += "\\" + GetSCLDocInfo()->GetSCLFileName();
        int nPos = strAllFile.ReverseFind('.');
        strAllFile.Delete(nPos,strlen(STR_SCL_EXTENSION));
        strAllFile += STR_ALL_EXTENSION;

        CopyFile(strAllFile,strWorkingPath + GetSCLDocInfo()->GetALLFileName(),false);

        BOOL bRet = Parse(obscript.m_strScript);
        m_pCaptureView->SetText(obscript.m_strScript, obscript.m_bModify) ;
    }

    CString strDataPath = strProjectDir + STR_REPOSITORY_DATA +"\\";

    CFileFind FileFinder;
    BOOL bWorking = FileFinder.FindFile(strDataPath + "*" + STR_FVR_EXTENSION);
    while(bWorking)
    {
        bWorking = FileFinder.FindNextFile();
        CopyFile(FileFinder.GetFilePath(),strWorkingPath + FileFinder.GetFileName(),FALSE);
    }

    return TRUE;
}

//--------------------------------------------------------------------------

void CModellerDoc::InitBrowserFromScript( CObjScript *pObjScript )
{
    CWaitCursor wc;

    long lID;

    lID = ID_BROWSER_NONE;

    //Browser Type
    if( pObjScript->m_strScript.Find( STR_BROWSER_IE4 ) >= 0 )
        lID = ID_BROWSER_IE4 ;
    else
    {
        if( pObjScript->m_strScript.Find( STR_BROWSER_IE5 ) >= 0 )
            lID = ID_BROWSER_IE5 ;
        else
        {
            if( pObjScript->m_strScript.Find( STR_BROWSER_NS47 ) >= 0 )
                lID = ID_BROWSER_NS47 ;
            else
            {
                SetDefaultBrowser(pObjScript) ;
                return ;
            }
        }
    }

    GetSCLDocInfo()->SetBrowserID(lID) ;
}


//--------------------------------------------------------------------------

void CModellerDoc::SetDefaultBrowser(CObjScript *pObjScript)
{
    CString strText ;
    CString strWarning ;
    //---
    strWarning.LoadString( IDS_WARNING ) ;
    strText.LoadString( IDS_NO_BROWSER_SELECTED ) ;
    AfxGetMainWnd()->MessageBox( strText , strWarning , MB_OK ) ;
    //---
    CFile File ;
    CString strSCLFile ;
    //---

    strSCLFile = GetSCLDocInfo()->GetFullSCLFileName() ;


    if( File.Open( strSCLFile , CFile::shareExclusive|CFile::modeReadWrite) )
    {
        TCHAR *pczBrowser = NULL ;
        int nCount = File.GetLength() ;
        TCHAR *pczBufferRead = new TCHAR[nCount + 1] ;
        //----
        File.SeekToBegin( );
        File.Read( pczBufferRead , nCount ) ;
        pczBufferRead[nCount] = '\0' ;

        //Write Browser : default IE5
        pczBrowser = new TCHAR[ strlen( STR_BROWSER_IE5 ) + 3] ;
        strcpy( pczBrowser , STR_BROWSER_IE5 ) ;
        strcat( pczBrowser , _T("\r\n") ) ;

        if( pczBrowser )
            nCount += strlen( pczBrowser ) ;

        //write new Buffer
        TCHAR *pczBufferWrite = new TCHAR[nCount + 1] ;
        //---
        if( pczBrowser )
            strcpy( pczBufferWrite , pczBrowser ) ;
        if( pczBufferRead )
            strcat( pczBufferWrite , pczBufferRead ) ;

        File.SeekToBegin( );
        File.Write( pczBufferWrite ,nCount ) ;

        File.Close() ;

        pObjScript->m_strScript = pczBufferWrite ;
        pObjScript->m_bModify = TRUE;
        GetSCLDocInfo()->SetBrowserID(ID_BROWSER_IE5) ;

        if(pczBufferWrite )
            delete pczBufferWrite ;
        if(pczBufferRead )
            delete pczBufferRead ;
        if(pczBrowser )
            delete pczBrowser ;

    }
}

//--------------------------------------------------------------------------

void CModellerDoc::OnFileSave()
{
    CWaitCursor wc;

    if(GetSCLDocInfo()->GetSCLFileName() == UNTITLED_SCL_DOCUMENT)
    {
        OnFileSaveAs();
        return;
    }

    m_bRelease = FALSE;
    CBasedDocument::OnFileSave();
    m_bRelease = TRUE;
}

//--------------------------------------------------------------------------

void CModellerDoc::OnUpdateFileSave(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( !IsRecording() ) ;
}

//--------------------------------------------------------------------------

BOOL CModellerDoc::DoSave(LPCTSTR lpszPathName, BOOL bReplace)
{
    if(GetSCLDocInfo()->GetSCLFileName() == UNTITLED_SCL_DOCUMENT)
    {
        OnFileSaveAs();
        return true;
    }

    return SaveToRepository();
}


//--------------------------------------------------------------------------

BOOL CModellerDoc::MoveFilesToRepositoryStorage()
{
    CWaitCursor wc;

    BOOL bResult = TRUE;

    //create the Repository directories if do not exist
    SECURITY_ATTRIBUTES SecurityAttributes ;
    SecurityAttributes.nLength = sizeof( SECURITY_ATTRIBUTES ) ;
    SecurityAttributes.lpSecurityDescriptor = NULL ;

    CString strSCLFile = GetSCLDocInfo()->GetSCLFileName();
    CString strAllFile = GetSCLDocInfo()->GetALLFileName();

    CString strProjectName = theApp.m_RepositoryAPI.GetOpenProjectName();
    CString strAppProjectPath = GetSCLDocInfo()->GetProjectPath( ) ;

    //Project
    CString strProjectPath = strAppProjectPath ;
    strProjectPath += strProjectName ;
    CreateDirectory(  strProjectPath  , &SecurityAttributes );

    //Source
    CString strSourcePath = strProjectPath;
    strSourcePath += "\\" ;
    strSourcePath += STR_SOURCE ;
    ;
    CreateDirectory(  strSourcePath  , &SecurityAttributes );

    //Capture
    CString strCapturePath = strProjectPath;
    strCapturePath += "\\" ;
    strCapturePath += STR_CAPTURE ;
    CreateDirectory(  strCapturePath  , &SecurityAttributes );

    //Data
    CString strDataPath = strProjectPath;
    strDataPath += "\\" ;
    strDataPath += STR_REPOSITORY_DATA ;
    strDataPath += "\\";
    CreateDirectory(  strDataPath  , &SecurityAttributes );

    //SCL
    CString strSCLPath = strProjectPath;
    strSCLPath += "\\" ;
    strSCLPath += STR_CAPTURE ;
    strSCLPath += "\\" ;
    strSCLPath += strSCLFile ;
    CreateDirectory(  strSCLPath  , &SecurityAttributes );

    //Copy files
    CString strWorkingPath = GetSCLDocInfo()->GetWorkingFolder();

    // 1- SCL File
    CString strSCLFromFullPath = strWorkingPath  ;
    strSCLFromFullPath += strSCLFile ;
    CString strSCLToFullPath = strSourcePath ;
    strSCLToFullPath += "\\" ;
    strSCLToFullPath += strSCLFile ;
    bResult &= CopyFile(  strSCLFromFullPath , strSCLToFullPath, FALSE ) ;

    // 2- All File
    CString strAllFromFullPath = strWorkingPath ;
    strAllFromFullPath += strAllFile ;
    CString strAllToFullPath = strSCLPath ;
    strAllToFullPath += "\\" ;
    strAllToFullPath += strAllFile ;
    bResult &= CopyFile(  strAllFromFullPath , strAllToFullPath, FALSE);

    //3 - FVR Files
    CFileFind FileFinder;
    BOOL bWorking = FileFinder.FindFile(strWorkingPath + "*" + STR_FVR_EXTENSION);
    while(bWorking)
    {
        bWorking = FileFinder.FindNextFile();
        bResult &= CopyFile(FileFinder.GetFilePath(),strDataPath + FileFinder.GetFileName(),FALSE);
    }

    return bResult ;
}

//--------------------------------------------------------------------

BOOL CModellerDoc::SaveToRepository()
{
    CWaitCursor wc;
    CString strScript ;

    strScript = m_pCaptureView->m_Editor.GetWindowText() ;
    m_pCaptureView->m_Editor.SetModified( FALSE) ;

    Parse();

    //Save new script int scl file
    CFile file ;
    //---

    CString strSCLFile = GetSCLDocInfo()->GetSCLFileName() ;
    CString strFullPath = GetSCLDocInfo()->GetWorkingFolder() + strSCLFile ;
    if( file.Open( strFullPath , CFile::modeCreate|CFile::modeWrite|CFile::shareExclusive ) )
    {
        file.Write( strScript , strScript.GetLength() ) ;
        file.Close() ;
    }

    CString strProjectName = theApp.m_RepositoryAPI.GetOpenProjectName();
    GetSCLDocInfo()->SetProjectName(strProjectName);

    MoveFilesToRepositoryStorage();

    theApp.GetRepositoryAPI()->AddProjectCaptureFiles(strSCLFile) ;

    return TRUE;
}

//--------------------------------------------------------------------

void CModellerDoc::SetErrorLine( long lRow )
{
    UpdateAllViews( NULL , ID_CHECKER_ERROR , &CLongObject(lRow) ) ;
}

//--------------------------------------------------------------------

BOOL CModellerDoc::SaveModified()
{
    if(IsRecording())
    {
        AfxMessageBox("You cannot close the Modeller while a capture is in progress.Select the stop capture command before closing the Modeller.",MB_OK);
        return false;
    }

    CWaitCursor wc;

    SetModifiedFlag(m_pCaptureView->m_Editor.IsModified());

    if(IsModified())
    {
        int nResult = AfxMessageBox("You have made changes to this script\nDo you want to save them now?",MB_YESNOCANCEL);
        if(nResult == IDCANCEL)
            return 0;
        else
            if(nResult == IDNO)
                return 1;
            else
            {
                DoSave(NULL);
                return 1;
            }
    }
    else
        return CBasedDocument::SaveModified();
}


//--------------------------------------------------------------------

void CModellerDoc::OnCloseDocument()
{
    theApp.GetRepositoryAPI()->SaveProjectCaptureFiles( GetSCLDocInfo()->GetSCLFileName() , true  ) ;

    DeleteWorkingDirectory();
    theApp.UpdateWindowText();

    CBasedDocument::OnCloseDocument();
}

//--------------------------------------------------------------------

BOOL CModellerDoc::IsTofUptodate(const CString& strTofFileName,const CString& strSclFileName)
{
    //Check if a tof file is more recent than its corresponding SCL file
    CFileStatus fsSCLFile,fsTOFFile;

    //Get the tof files status
    if(!CFile::GetStatus(strTofFileName,fsTOFFile))
    {
        //The tof file does not exist so its not uptodate!
        return false;
    }

    //Get the scl files status
    if(!CFile::GetStatus(strSclFileName,fsSCLFile))
    {
        //Ooops the scl file does not exist, What do we do now?
        ASSERT(FALSE);
    }

    //Check if the scl file is more recent than the TOF
    //return false if it is, true otherwise
    return (fsTOFFile.m_mtime >= fsSCLFile.m_mtime);
}

//--------------------------------------------------------------------

void CModellerDoc::OnDelay()
{
    CDelInfo    DelInfo ;

    CDelPSheet  WizardDlg(FALSE, &DelInfo);
    if(WizardDlg.DoModal() == IDCANCEL)
        return;

    CString     strLineToInsert ;
    //---
    Parse();

    strLineToInsert = CSCLGenerator::GenDelayStat( &DelInfo ) ;
    m_pCaptureView->m_Editor.InsertString( m_pCaptureView->m_Editor.GetCurrentLine(), strLineToInsert ) ;
}

//--------------------------------------------------------------------

void CModellerDoc::OnSynchronize()
{
    Parse();

    CSyncInfo SyncInfo;
    //---
    SyncInfo.m_SyncEvent.m_lCurrentLine = m_pCaptureView->m_Editor.GetCurrentLine() ;
    CSyncPSheet WizardDlg(FALSE, &SyncInfo,&m_SCLStructure);
    if(WizardDlg.DoModal() == IDCANCEL)
        return;

    CString     strLineToInsert ;
    long        lineToInsert ;

    strLineToInsert = CSCLGenerator::GenSynchronizeStat();
    lineToInsert    = m_SCLStructure.GetLineEndBlock( SyncInfo.m_SyncEvent.m_nSyncEventURLConId, SyncInfo.m_SyncEvent.m_strSyncEventURL ) ;
    m_pCaptureView->m_Editor.InsertString( lineToInsert, strLineToInsert ) ;
}

//--------------------------------------------------------------------

void CModellerDoc::OnCondition()
{
    Parse();

    CIfInfo IfInfo;

    CIfPSheet WizardDlg(TRUE,&IfInfo,&m_SCLStructure);
    if(WizardDlg.DoModal() == IDCANCEL)
        return;

    CString     strLineToInsert ;
    long        lineToInsert ;

    strLineToInsert = CSCLGenerator::GenIfStat( &IfInfo ) ;

    switch( IfInfo.m_ValueType.m_nIfValueType )
    {
    case CIfValueType::enBooleanExpression :
        lineToInsert = m_pCaptureView->m_Editor.GetCurrentLine() ;
        break ;
    case CIfValueType::enServerResponseTest :
        lineToInsert = m_SCLStructure.GetLineEndBlock(
                           IfInfo.m_ServerResponseTest.m_nServerResponseURLConId,
                           IfInfo.m_ServerResponseTest.m_strServerResponseURL ) ;
        break ;
    case CIfValueType::enTextMatching :
        lineToInsert = m_SCLStructure.GetInsertLine(
                           IfInfo.m_TextMatch.m_nTextMatchURLConId,
                           IfInfo.m_TextMatch.m_strTextMatchURL ) ;
        break ;
    }

    m_pCaptureView->m_Editor.InsertString( lineToInsert, strLineToInsert ) ;
}

//--------------------------------------------------------------------

void CModellerDoc::OnLoop()
{
    CLoopInfo LoopInfo;

    CLoopPSheet WizardDlg(FALSE, &LoopInfo);
    if(WizardDlg.DoModal() == IDCANCEL)
        return;

    CString     strLineToInsert ;
    //---
    Parse();

    strLineToInsert = CSCLGenerator::GenLoopStat( &LoopInfo) ;
    m_pCaptureView->m_Editor.InsertString( m_pCaptureView->m_Editor.GetCurrentLine(), strLineToInsert ) ;
}

//--------------------------------------------------------------------

void CModellerDoc::OnInitVariable()
{
    CVIInfo VIInfo ;

    Parse() ;

    VIInfo.m_SyncEvent.m_lCurrentLine = m_pCaptureView->m_Editor.GetCurrentLine() ;
    CVIPSheet WizardDlg(TRUE,&VIInfo,&m_SCLStructure);
    if(WizardDlg.DoModal() == IDCANCEL)
        return;

    CString     strLineToInsert ;
    //---

    m_pCaptureView->m_Editor.DeleteLines( VIInfo.m_SyncEvent.m_lGETBeginLine, VIInfo.m_SyncEvent.m_lGETEndLine ) ;

    strLineToInsert = CSCLGenerator::GenURLVariablisation( &VIInfo ) ;
    m_pCaptureView->m_Editor.ClearSelection() ;
    m_pCaptureView->m_Editor.InsertString( VIInfo.m_SyncEvent.m_lGETBeginLine, strLineToInsert ) ;
}

//--------------------------------------------------------------------

void CModellerDoc::Adressing( CVarResponse * _pVarResponse )
{
    CString     strLineToInsert ;
    long        lineToInsert ;

    Parse();

    _pVarResponse->m_lConId = m_lConId;

    //statement
    strLineToInsert = CSCLGenerator::GenVarResponseInfoStat( _pVarResponse ) ;
    lineToInsert    = m_SCLStructure.GetInsertLine( m_lConId, m_strLastURL ) ;
    m_pCaptureView->m_Editor.InsertString( lineToInsert, strLineToInsert ) ;

    //declaration
    strLineToInsert = CSCLGenerator::GenVarResponseInfoDecl( _pVarResponse->m_strVarName ) ;
    lineToInsert    = m_SCLStructure.GetCodeLine() - 1 ;
    m_pCaptureView->m_Editor.InsertString( lineToInsert, strLineToInsert ) ;


}

//--------------------------------------------------------------------

void CModellerDoc::OnFileSaveAs()
{
    CCaptureSaveDlg dlg;
    if(dlg.DoModal() == IDOK)
    {
        CWaitCursor wc;

        CString strProject = theApp.m_RepositoryAPI.GetOpenProjectName();
        CString strSCLFile = dlg.m_strScriptName;

        //Get the location of the existing all file and fvr files
        CString strWorkingFolder = GetSCLDocInfo()->GetWorkingFolder();

        CString strOldSclFile = strWorkingFolder + GetSCLDocInfo()->GetSCLFileName();
        CString strNewSclFile = strWorkingFolder + strSCLFile;
        CFile::Rename(strOldSclFile,strNewSclFile);

        CString strOldAllFile = strWorkingFolder + GetSCLDocInfo()->GetALLFileName();
        CString strAllFile = strSCLFile;
        strAllFile.Replace(STR_SCL_EXTENSION,STR_ALL_EXTENSION);
        CString strNewAllFile = strWorkingFolder + strAllFile;
        CFile::Rename(strOldAllFile,strNewAllFile);

        //Setup new file information
        GetSCLDocInfo()->SetFileInfo(strSCLFile) ;
        SetTitle(strSCLFile);

        //This will create all directories we need
        MoveFilesToRepositoryStorage();

        SaveToRepository();
    }
}

void CModellerDoc::OnUpdateFileSaveAs(CCmdUI* pCmdUI)
{
    pCmdUI->Enable( !IsRecording() ) ;
}

void CModellerDoc::DeleteWorkingDirectory()
{
    CString strWorkingDir = GetSCLDocInfo()->GetWorkingFolder();

    CFileFind *pFileFind = new CFileFind;
    if(pFileFind->FindFile(strWorkingDir + "*"))
    {
        BOOL bQuit = false;
        do
        {
            bQuit = !pFileFind->FindNextFile();

            if(pFileFind->IsDots())
                continue;

            if(pFileFind->IsDirectory())
                continue;

            DeleteFile(pFileFind->GetFilePath());

        }
        while(!bQuit);
    }
    delete pFileFind;
    pFileFind = NULL;

    RemoveDirectory(strWorkingDir);
}
