/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SCLCaptureScriptView.cpp : implementation file
//

#include "stdafx.h"
#include <ConstantesLib.h>
#include "Modeller.h"
#include "SCLCaptureScriptView.h"
#include "Constantes.h"
#include "ModellerDoc.h"
#include "VariousObject.h"
#include "ChildFrm.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CModellerApp theApp ;

#define ID_UPDATE_TIMER 2

/////////////////////////////////////////////////////////////////////////////
// CSCLCaptureScriptView
//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CSCLCaptureScriptView, CView)

//---------------------------------------------------------------------------

CSCLCaptureScriptView::CSCLCaptureScriptView():
        m_Editor(theApp.GetRegistryPath() )
{}

//---------------------------------------------------------------------------


CSCLCaptureScriptView::~CSCLCaptureScriptView()
{}

//---------------------------------------------------------------------------


BEGIN_MESSAGE_MAP(CSCLCaptureScriptView, CView)
//{{AFX_MSG_MAP(CSCLCaptureScriptView)
ON_COMMAND(IDM_CONDITION, OnCondition)
ON_UPDATE_COMMAND_UI(IDM_CONDITION, OnUpdateCondition)
ON_COMMAND(IDM_DELAY, OnDelay)
ON_UPDATE_COMMAND_UI(IDM_DELAY, OnUpdateDelay)
ON_COMMAND(IDM_INIT_VARIABLE, OnInitVariable)
ON_UPDATE_COMMAND_UI(IDM_INIT_VARIABLE, OnUpdateInitVariable)
ON_COMMAND(IDM_LOOP, OnLoop)
ON_UPDATE_COMMAND_UI(IDM_LOOP, OnUpdateLoop)
ON_COMMAND(IDM_SYNCHRONIZE, OnSynchronize)
ON_UPDATE_COMMAND_UI(IDM_SYNCHRONIZE, OnUpdateSynchronize)
ON_COMMAND(IDM_TOGGLEBOOKMARK, OnTogglebookmark)
ON_UPDATE_COMMAND_UI(IDM_TOGGLEBOOKMARK, OnUpdateTogglebookmark)
ON_COMMAND(IDM_GOTO_URL , OnGoToURL )
ON_UPDATE_COMMAND_UI(IDM_GOTO_URL , OnUpdateGoToURL )
ON_WM_CREATE()
ON_WM_SIZE()
ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
ON_COMMAND(ID_EDIT_CUT, OnEditCut)
ON_COMMAND(ID_EDIT_FIND, OnEditFind)
ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
ON_COMMAND(ID_EDIT_REDO, OnEditRedo)
ON_COMMAND(ID_EDIT_REPLACE, OnEditReplace)
ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCopy)
ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCut)
ON_UPDATE_COMMAND_UI(ID_EDIT_FIND, OnUpdateEditFind)
ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
ON_UPDATE_COMMAND_UI(ID_EDIT_REDO, OnUpdateEditRedo)
ON_UPDATE_COMMAND_UI(ID_EDIT_REPLACE, OnUpdateEditReplace)
ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
ON_WM_TIMER()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSCLCaptureScriptView drawing
//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnDraw(CDC* pDC)
{
    CDocument* pDoc = GetDocument();
    // TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// CSCLCaptureScriptView diagnostics
//---------------------------------------------------------------------------



#ifdef _DEBUG
void CSCLCaptureScriptView::AssertValid() const
{
    CView::AssertValid();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::Dump(CDumpContext& dc) const
{
    CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CSCLCaptureScriptView message handlers
//---------------------------------------------------------------------------

COSEditCtrl* CSCLCaptureScriptView::GetEditCtrl()
{
    return &m_Editor ;
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
    switch( lHint )
    {
    case ID_CHECKER_ERROR :
        {
            long lRow = ((CLongObject*) (pHint ))->m_lObject ;
            m_Editor.GoToLine( lRow ) ;
            m_Editor.SelectLine(TRUE) ;
            m_Editor.GoToLine( lRow ) ;
            m_Editor.SetFocus() ;
            break ;
        }
    }

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::SetText(CString _strScript, BOOL _bModify)
{
    m_Editor.SetWindowText( _strScript) ;

    m_Editor.SetModified(_bModify);

    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->Parse(_strScript,TRUE);
    ShowPrimaryURL();

    CChildFrame *pFrame = (CChildFrame *)GetParentFrame();
    pFrame->RefreshURL();

    m_Editor.Invalidate();

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::ShowPrimaryURL()
{
    int nCount;
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    ASSERT(pDoc != NULL);

    nCount = pDoc->GetSCLStructure()->m_aData.GetSize() ;

    m_Editor.ClearAllPrimaryGetMarks();
    for (int nRow = 0; nRow < nCount; nRow++)
    {
        pDB_URL_FIELDS pdbFields =  pDoc->GetSCLStructure()->m_aData.GetAt(nRow) ;
        if ( !pdbFields->m_bPrimary )
            continue;
        m_Editor.SetPrimaryGetMarkOnLine(pdbFields->m_lBeginLine, TRUE, (nCount==nRow+1));
    }
    m_Editor.Invalidate();
}




//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnInitialUpdate()
{
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    ASSERT(pDoc != NULL);

    CView::OnInitialUpdate();

    m_Editor.ShowScrollBar( TRUE );

    pDoc->SetCaptureView(this);

    SetTimer(ID_UPDATE_TIMER,3000,NULL);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView)
{
    CView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::GetGatewayInfomation( _TPreferencesGatewayData *pData )
{
    pData->m_bLocal                 = theApp.IsCaptureLocal() ;
    pData->m_lAdministrationPort    = theApp.GetAdministrationPort() ;
    pData->m_lListenPort            = theApp.GetListenPort() ;

    if( pData->m_bLocal )
    {
        pData->m_ProxyInfo.m_LocalProxy.m_bByPassProxyLocal     = theApp.GetLocalByPassProxyLocal() ;
        pData->m_ProxyInfo.m_LocalProxy.m_lAddressPort          = theApp.GetLocalAddressPort() ;
        pData->m_ProxyInfo.m_LocalProxy.m_lSecurePort           = theApp.GetLocalSecurePort() ;

        pData->m_ProxyInfo.m_LocalProxy.m_strAddressesNoProxy= theApp.GetLocalAddressesNoProxy() ;
        pData->m_ProxyInfo.m_LocalProxy.m_strProxyAddress    = theApp.GetLocalProxyAddress() ;
        pData->m_ProxyInfo.m_LocalProxy.m_strProxySecure     = theApp.GetLocalProxySecure() ;
    }
    else
    {
        pData->m_ProxyInfo.m_RemoteProxy.m_bByPassProxyLocal     = theApp.GetRemoteByPassProxyLocal() ;
        pData->m_ProxyInfo.m_RemoteProxy.m_lAddressPort          = theApp.GetRemoteAddressPort() ;
        pData->m_ProxyInfo.m_RemoteProxy.m_lSecurePort           = theApp.GetRemoteSecurePort() ;

        pData->m_ProxyInfo.m_RemoteProxy.m_strProxyAddress = theApp.GetRemoteAddressesNoProxy() ;
        pData->m_ProxyInfo.m_RemoteProxy.m_strProxyAddress = theApp.GetRemoteProxyAddress() ;
        pData->m_ProxyInfo.m_RemoteProxy.m_strProxySecure  = theApp.GetRemoteProxyAddress() ;
    }

}
//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnCondition()
{
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->OnCondition();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateCondition(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnDelay()
{
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->OnDelay();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateDelay(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}


//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnInitVariable()
{
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->OnInitVariable();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateInitVariable(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnLoop()
{
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->OnLoop();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateLoop(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnSynchronize()
{
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->OnSynchronize();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateSynchronize(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnTogglebookmark()
{
    int nCurLine;
    BOOL BookmarkSet;
    //---
    nCurLine    = m_Editor.GetCurrentLine();
    BookmarkSet = m_Editor.GetBookmark( nCurLine - 1 );
    m_Editor.SetBookmark( nCurLine - 1, !BookmarkSet );
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateTogglebookmark(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnGoToURL()
{
    long lLine = m_Editor.GetCurrentLine() - 1 ;
    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->GoToURL(lLine);

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateGoToURL( CCmdUI *pCmdUI )
{
    long lLine = m_Editor.GetCurrentLine() - 1 ;

    CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
    pDoc->OnUpdateGoToURL(lLine, pCmdUI );
}

//---------------------------------------------------------------------------

BOOL CSCLCaptureScriptView::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
    if(m_Editor.OnCmdMsg(nID,nCode,pExtra,pHandlerInfo))
        return true;

    return CView::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

//---------------------------------------------------------------------------

int CSCLCaptureScriptView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CView::OnCreate(lpCreateStruct) == -1)
        return -1;

    if ( m_Editor.Create( WS_CHILD | WS_VISIBLE | WS_VSCROLL, CRect( 0, 0, 100, 100 ), this, 100 ) == -1 )
        return -1;

    return 0;
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnSize(UINT nType, int cx, int cy)
{
    CView::OnSize(nType, cx, cy);

    if ( m_Editor.m_hWnd )
        m_Editor.SetWindowPos( NULL, -1, -1, cx, cy, SWP_NOZORDER | SWP_NOMOVE );

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditCopy()
{
    m_Editor.Copy();

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditCut()
{
    m_Editor.Cut();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditFind()
{
    m_Editor.SendMessage(CMM_EXECUTECMD, CMD_FIND, 0 );
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditPaste()
{
    m_Editor.Paste();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditRedo()
{
    m_Editor.Redo();
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditReplace()
{
    ::SendMessage( m_Editor.m_hWnd, CMM_EXECUTECMD, CMD_FINDREPLACE, 0 );
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnEditUndo()
{
    m_Editor.Undo();

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditCopy(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCopy());
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditCut(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanCut());
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditFind(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditPaste(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanPaste());

}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditRedo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanRedo());
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditReplace(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(TRUE);
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnUpdateEditUndo(CCmdUI* pCmdUI)
{
    pCmdUI->Enable(m_Editor.CanUndo());
}

//---------------------------------------------------------------------------

void CSCLCaptureScriptView::OnTimer(UINT nIDEvent)
{
    switch(nIDEvent)
    {
    case ID_UPDATE_TIMER:
        CModellerDoc* pDoc = static_cast<CModellerDoc*>(GetDocument());
        pDoc->Parse();
        ShowPrimaryURL();

        CChildFrame *pFrame = (CChildFrame *)GetParentFrame();
        pFrame->RefreshURL();
        break;
    }

    CView::OnTimer(nIDEvent);
}
