/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SCLGenerator.cpp: implementation of the CSCLGenerator class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "modeller.h"
#include "SCLGenerator.h"
#include <ConstantesLib.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//-------------------------------------------------------------------

CSCLGenerator::CSCLGenerator()
{
}

//-------------------------------------------------------------------


CSCLGenerator::~CSCLGenerator()
{
}

//-------------------------------------------------------------------


CString CSCLGenerator::GenVarFileDecl(  const CString& _strVarName,
                                        const CString& _strFileName,
                                        int _nVarType,
                                        int _nVarScope )
{
    CString strVarType ;
    CString strVarValues ;
    CString strVarScope ;
    CString strVarOrder ;
    CString strCode ;

    strVarType      = GetVarTypeString(_nVarType) ;
    strVarScope     = GetVarScopeString(_nVarScope) ;

    strCode.Format("%s %s, FILE =\"%s\", %s\n ", strVarType, _strVarName,
                   _strFileName, strVarScope ) ;

    return strCode ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GenVarValListDecl(   const CString& _strVarName,
        CStringArray * const _astrVarValues,
        int _nVarOrder,
        int _nVarType,
        int _nVarScope )
{
    CString strVarType ;
    CString strVarValues ;
    CString strVarScope ;
    CString strVarOrder ;
    CString strCode ;

    strVarType      = GetVarTypeString(_nVarType) ;
    strVarValues    = GetVarValuesString( _astrVarValues, _nVarType ) ;
    strVarScope     = GetVarScopeString(_nVarScope) ;
    strVarOrder     = GetVarOrderString(_nVarOrder) ;


    if( !strVarOrder.IsEmpty() )
    {
        strCode.Format("%s%s %s, %s, %s\n ", strVarType, _strVarName, strVarValues,
                       strVarScope, strVarOrder ) ;
    }
    else
    {
        strCode.Format("%s%s %s, %s\n ", strVarType, _strVarName, strVarValues,
                       strVarScope ) ;
    }


    return strCode ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GenVarSimpleDecl(const CString& _strVarName,
                                        int _nVarOrder,
                                        int _nVarType,
                                        int _nVarScope )
{
    CString strVarOrder ;
    CString strVarType ;
    CString strVarScope ;
    CString strCode ;

    strVarOrder     = GetVarOrderString(_nVarOrder) ;
    strVarType      = GetVarTypeString(_nVarType) ;
    strVarScope     = GetVarScopeString(_nVarScope) ;


    if( !strVarOrder.IsEmpty() )
    {
        strCode.Format("%s%s ,%s ,%s", strVarType, _strVarName, strVarScope, strVarOrder ) ;
    }
    else
    {
        strCode.Format("%s%s ,%s", strVarType, _strVarName, strVarScope ) ;
    }


    return strCode ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GenVarResponseInfoDecl(const CString& _strVarName )
{
    return GenVarSimpleDecl( _strVarName, VAR_ORDER_SEQUENTIAL, VAR_DATATYPE_CHAR, VAR_SCOPE_LOCAL);
}

//-------------------------------------------------------------------

CString CSCLGenerator::GetVarScopeString( int _nVarScope )
{
    CString strVarScope ;

    switch( _nVarScope )
    {
    case VAR_SCOPE_PRIVATE :
        strVarScope = "PRIVATE" ;
        break ;
    case VAR_SCOPE_LOCAL :
        strVarScope = "LOCAL" ;
        break ;
    case VAR_SCOPE_SCRIPT :
        strVarScope = "SCRIPT" ;
        break ;
    case VAR_SCOPE_THREAD :
        strVarScope = "THREAD" ;
        break ;
    case VAR_SCOPE_GLOBAL :
        strVarScope = "GLOBAL" ;
        break ;
    default:
        ASSERT(FALSE) ;
    }

    return strVarScope ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GetVarOrderString( int _nVarOrder )
{
    CString strVarOrder ;

    switch( _nVarOrder )
    {
    case VAR_ORDER_SEQUENTIAL :
        strVarOrder = "" ;
        break ;
    case VAR_ORDER_RANDOM :
        strVarOrder = "RANDOM" ;
        break ;
    default:
        ASSERT(FALSE) ;
    }

    return strVarOrder ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GetVarTypeString( int _nVarType )
{
    CString strVarType ;

    switch( _nVarType )
    {
    case VAR_DATATYPE_CHAR:
        strVarType = "\tCHARACTER*512\t";
        break;
    case VAR_DATATYPE_INT:
        strVarType = "\tINTEGER\t\t";
        break;
    default:
        ASSERT(FALSE) ;
    }

    return strVarType ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GetVarValuesString( CStringArray * _astrVarValues, int _nVarType )
{
    CString strValueList ;
    CString strValue ;
    CString strDisplayFirst ;
    CString strDisplayOther ;
    CString strFormat ;
    int NbValues ;

    NbValues = _astrVarValues->GetSize() ;

    if( NbValues )
    {
        if( _nVarType == VAR_DATATYPE_CHAR )
        {
            strDisplayFirst = "\"%s\"" ;
            strDisplayOther = ", \"%s\"" ;
        }
        else
        {
            strDisplayFirst = "%s" ;
            strDisplayOther = ", %s" ;
        }

        strValueList.Format( strDisplayFirst, _astrVarValues->GetAt(0) ) ;
        for(int i=1; i < NbValues; i++)
        {
            strValue.Format(strDisplayOther, _astrVarValues->GetAt(i) ) ;
            if( !(i % 3) && (i != NbValues - 1) )
            {
                strValueList += " & \n" ;
            }

            strValueList += strValue ;
        }
        strFormat =  "( %s )" ;

        strValue.Format( strFormat, strValueList ) ;
        strValueList = strValue ;
    }

    return strValueList ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GenVarResponseInfoStat( CVarResponse * _pVarResponse )
{
    CString strCode ;
    CString strResponseType ;
    CString strFormat ;
    CString strIdentifier ;
    CString strDisplayValue ;
    int nStep       = 90 ;
    int nEndStep    = 10 ;
    int nbChars ;
    int nEndChar ;
    int nLastChar ;

    switch( _pVarResponse->m_ResponseType )
    {
    case RT_HEADER :
        strResponseType = "HEADER" ;
        break ;
    case RT_BODY :
        strResponseType = "BODY" ;
        break ;
    default :
        ASSERT( FALSE ) ;
        break;
    }

    strFormat   =   " \n"
                    "\tLOAD RESPONSE_INFO %s ON %d" " & \n"
                    "\tINTO %s"                     " & \n"
                    "\t,WITH %s"                    " \n " ;

    strIdentifier   = _pVarResponse->m_strIdentifier ;
    strDisplayValue =  "\t\"" ;

    if( strIdentifier.GetLength(  ) > nStep )
    {
        nLastChar   = strIdentifier.GetLength() - 1 ;
        for( int i = 0; ( nLastChar - i ) > nStep; i += nbChars )
        {
            nEndChar = strIdentifier.Find( "\\", ( i + nStep ) ) - 1 ;

            if( ( nEndChar < 0 ) || ( nEndChar >= ( i + nStep + nEndStep ) ) )
                nbChars = nStep ;
            else
                nbChars = ( nEndChar - i ) + 1 ;

            strDisplayValue +=  strIdentifier.Mid( i, nbChars ) ;
            strDisplayValue +=  "\" &\n" ;
            strDisplayValue +=  "\t\t\"" ;
        }
        strDisplayValue +=  strIdentifier.Mid( i ) ;
        strDisplayValue +=  "\" \n" ;
    }
    else
    {
        strDisplayValue +=  strIdentifier ;
        strDisplayValue +=  "\" \n" ;
    }

    strCode.Format( strFormat, strResponseType, _pVarResponse->m_lConId,
                    _pVarResponse->m_strVarName, strDisplayValue ) ;

    return strCode ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GenDelayStat( CDelInfo * _pDelInfo )
{
    CString strCode ;

    strCode.Format(" \n\tWAIT %d \n ", _pDelInfo->m_DelDelay.m_nDelay ) ;

    return strCode ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GenSynchronizeStat()
{
    CString strCode ;

    strCode =_T("\tSYNCHRONIZE REQUESTS\n ");

    return strCode ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GenLoopStat( CLoopInfo * _pLoopInfo)
{
    CString strCode ;
    CString strFormat ;

    strFormat = " \n"
                "\tDO VAR_LOOP = 0, 2"        "\n"
                "\t\tIF ( %s ) THEN"         "\n"
                "\t\t\tSET VAR_LOOP = 2"      "\n"
                "\t\tELSE"                   "\n"
                "\t\t\tSET VAR_LOOP = 0"      "\n"
                "\t\tENDIF"                  "\n"
                "\tENDDO"                    " \n " ;

    strCode.Format(strFormat, _pLoopInfo->m_LoopExpression.m_strLoopExpression) ;

    return strCode ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GenIfStat( CIfInfo * _pIfInfo )
{
    CString strCode ;
    CString strFormat ;

    switch( _pIfInfo->m_ValueType.m_nIfValueType )
    {
    case CIfValueType::enBooleanExpression :
        strFormat =  "\n"
                     "\tIF ( %s ) THEN \n"
                     "\tENDIF \n " ;

        strCode.Format( strFormat, _pIfInfo->m_BooleanExpression.m_strBooleanExpression ) ;
        break ;
    case CIfValueType::enServerResponseTest :
        strFormat =  "\n"
                     "\tSYNCHRONIZE REQUESTS & \n"
                     "\t\t,WITH TIMEOUT %d & \n"
                     "\t\t,ON TIMEOUT GOTO ERR_LABEL \n " ;

        strCode.Format( strFormat, _pIfInfo->m_ServerResponseTest.m_nResponseTime ) ;
        break ;
    case CIfValueType::enTextMatching :
        strFormat = "\n"
                    "\tLOAD RESPONSE_INFO BODY ON %d INTO VAR_TMP"  " \n"
                    "\tSET VAR_LOOP = ~LOCATE (\"%s\", VAR_TMP)"    " \n"
                    "\tIF ( VAR_LOOP = -1 ) THEN"                   " \n"
                    "\tELSE"                                        " \n"
                    "\tENDIF"                                       " \n ";

        strCode.Format( strFormat, _pIfInfo->m_TextMatch.m_nTextMatchURLConId, _pIfInfo->m_TextMatch.m_strTextToMatch ) ;
        break ;
    default :
        ASSERT( FALSE ) ;
        break ;

    }

    return strCode ;
}

//-------------------------------------------------------------------

CString CSCLGenerator::GenURLVariablisation( CVIInfo * _pVIInfo )
{
    CString strCode ;
    CString strTmpValue ;
    CString strTabValue ;
    CString strToInsert ;
    CString strDefaultInsert ;

    int     nbArgs ;
    int     nLastArg ;
    CTypedPtrArray<CPtrArray, CVIArgInfo*> &arArgInfo = _pVIInfo->m_Variablisation.m_arArgInfo ;
    //---

    strTabValue = ( _pVIInfo->m_SyncEvent.m_bPrimary ) ? "\t" : "\t\t" ;
    strCode.Format( "%sGET URI \"%s", strTabValue, _pVIInfo->m_Variablisation.m_strBeginURL ) ;

    nbArgs      = arArgInfo.GetSize() ;
    nLastArg    = nbArgs - 1 ;
    for( int i = 0; i < nbArgs; i++ )
    {
        CVIArgInfo  * pArgInfo;
        //---
        pArgInfo    = arArgInfo.GetAt( i ) ;

        if( pArgInfo->m_bValueIsVarName )
        {
            strTmpValue.Format( "%s=\" + %s + \"", pArgInfo->m_strArgument, pArgInfo->m_strValue ) ;
        }
        else
        {
            strTmpValue.Format( "%s=%s", pArgInfo->m_strArgument, pArgInfo->m_strValue ) ;
        }

        if( i != nLastArg )
        {
            strTmpValue += "&" ;
        }

        strCode += strTmpValue ;
    }

    strTmpValue.Format( "%s \" ON %d &", _pVIInfo->m_Variablisation.m_strEndURL, _pVIInfo->m_SyncEvent.m_nSyncEventURLConId ) ;

    strCode += strTmpValue ;

    strToInsert         = " &\n" + strTabValue + "\t" ;
    strDefaultInsert    = "\" &\n" + strTabValue + "\t\"" ;

    strCode = GetDisplayString( "+", strCode, 80, strToInsert, strDefaultInsert ) ;

    return strCode ;
}

//-------------------------------------------------------------------//
//  _strSeparator       : Separator to search
//  _strCode            : String to modify
//  _nMaxLineSize       : Max Chars between two insertion
//  _strToInsert        : String to insert if a separator was found
//  _strDefaultInsert   : String to insert if a separator wasn't found
//-------------------------------------------------------------------
CString CSCLGenerator::GetDisplayString( CString _strSeparator, CString _strCode,
        long _nMaxLineSize, CString _strToInsert,
        CString _strDefaultInsert )
{
    int nRemindChars ;
    int nLastCut ;
    int nInsertSize ;
    int nCodeSize ;
    CString strResultString ;
    char    car ;
    int     nLastPlusSign ;
    int     nLastValidCar ;
    BOOL    bInQuote;

    ASSERT( _nMaxLineSize > 0 ) ;

    nLastCut = nLastValidCar = nLastPlusSign = 0 ;
    nInsertSize = _strToInsert.GetLength() ;
    nCodeSize   = _strCode.GetLength() ;

    bInQuote        = FALSE ;

    for( int i=0; i < nCodeSize; i++ )
    {
        car = _strCode[i] ;
        switch( car )
        {
        case '"':
            bInQuote = !bInQuote ;
            break ;

        case '+':
            nLastPlusSign = i ;
            break ;

        default :
            if( bInQuote )
            {
                nLastValidCar = i;
            }
        }

        if( ( i - nLastCut ) == _nMaxLineSize )
        {
            if( nLastPlusSign > nLastCut ) // Cut on plus
            {
                strResultString += _strCode.Mid( nLastCut, nLastPlusSign - nLastCut ) ;
                strResultString += _strToInsert ;
                nLastCut = nLastPlusSign ;
                i = nLastPlusSign ;
            }
            else  // Cut on last valid char
            {
                strResultString += _strCode.Mid( nLastCut, nLastValidCar - nLastCut ) ;
                strResultString += _strDefaultInsert ;
                nLastCut = nLastValidCar ;
                i = nLastValidCar ;
            }
            nRemindChars = _strCode.GetLength() - nLastCut ;

            if( nRemindChars < _nMaxLineSize )
            {
                strResultString += _strCode.Mid( nLastCut ) ;
                i = nCodeSize ; //Sortie
            }

        }
    }

    return strResultString ;
}
