/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// SyncEventPPage.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "SyncEventPPage.h"
#include <SCLStructure.h>
#include "htmlhelp.h"
#include "Help\SyncEventPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSyncEventPPage property page
//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CSyncEventPPage, CExtendedPropertyPage)
//{{AFX_MSG_MAP(CSyncEventPPage)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CSyncEventPPage, CExtendedPropertyPage)

//---------------------------------------------------------------------------

CSyncEventPPage::CSyncEventPPage(CSyncEvent * _pSyncEvent,CSCLStructure* _pSCLStructure) :
        CExtendedPropertyPage(CSyncEventPPage::IDD)
{
    ASSERT(_pSyncEvent!=NULL);
    m_pSyncEvent = _pSyncEvent;
    m_pSCLStructure = _pSCLStructure;
    //{{AFX_DATA_INIT(CSyncEventPPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
}

//---------------------------------------------------------------------------


CSyncEventPPage::~CSyncEventPPage()
{}

//---------------------------------------------------------------------------


void CSyncEventPPage::DoDataExchange(CDataExchange* pDX)
{
    CExtendedPropertyPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CSyncEventPPage)
    DDX_Control(pDX, IDC_PRIMARY_URL_COMBO, m_ctlPrimaryURL);
    //}}AFX_DATA_MAP
}

//---------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
// CSyncEventPPage message handlers
//---------------------------------------------------------------------------


BOOL CSyncEventPPage::OnInitDialog()
{
    CExtendedPropertyPage::OnInitDialog();

    Values2Ctrl();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CSyncEventPPage::Ctrl2Values()
{
    m_pSyncEvent->m_strSyncEventURL.Empty();

    int nCurSel = m_ctlPrimaryURL.GetCurSel();
    if(nCurSel!=CB_ERR)
    {
        m_ctlPrimaryURL.GetLBText(nCurSel,m_pSyncEvent->m_strSyncEventURL);
        m_pSyncEvent->m_nSyncEventURLConId = m_ctlPrimaryURL.GetItemData(nCurSel);
    }
}

//---------------------------------------------------------------------------

void CSyncEventPPage::Values2Ctrl()
{
    CString& strSelectedURL = m_pSyncEvent->m_strSyncEventURL;

    m_ctlPrimaryURL.SetRedraw(FALSE);
    m_ctlPrimaryURL.ResetContent();

    int nURLToSelect = -1 ;
    long nConId;
    CString strURL;
    for(int nNdx=0;nNdx<m_pSCLStructure->m_aData.GetSize();nNdx++)
    {
        if(m_pSCLStructure->m_aData[nNdx]->m_bHTMLURL &&
                m_pSCLStructure->m_aData[nNdx]->m_bPrimary)
        {
            nConId = m_pSCLStructure->m_aData[nNdx]->m_lConId;
            strURL = m_pSCLStructure->m_aData[nNdx]->m_strURL;
            if(m_ctlPrimaryURL.FindStringExact(-1,strURL)== CB_ERR)
            {
                int nItem = m_ctlPrimaryURL.AddString(strURL);
                m_ctlPrimaryURL.SetItemData(nItem,nConId);

                if( m_pSyncEvent->m_lCurrentLine >= m_pSCLStructure->m_aData[nNdx]->m_lBeginLine
                        && m_pSyncEvent->m_lCurrentLine <= m_pSCLStructure->m_aData[nNdx]->m_lEndGetLine )
                {
                    nURLToSelect = nItem ;
                }
            }
        }
    }

    //if(m_ctlPrimaryURL.SelectString(-1,m_pSyncEvent->m_strSyncEventURL) == CB_ERR)
    //    m_ctlPrimaryURL.SetCurSel(0);

    if( nURLToSelect >= 0 )
    {
        m_ctlPrimaryURL.SetCurSel( nURLToSelect ) ;
    }
    else
    {
        m_ctlPrimaryURL.SetCurSel( 0 ) ;
    }

    m_ctlPrimaryURL.SetRedraw();
}

//---------------------------------------------------------------------------

HBRUSH CSyncEventPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CExtendedPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem(IDC_DESCRIPTION)->m_hWnd)
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    // TODO: Return a different brush if the default is not desired
    return hbr;
}

BOOL CSyncEventPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}

