/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// URLListDlg.cpp : implementation file
//
#include "stdafx.h"
#include "URLListDlg.h"
#include "htmlhelp.h"
#include "Help\URLListDlg.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CURLListDlg, CDialog)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CURLListDlg, CDialog)
//{{AFX_MSG_MAP(CURLListDlg)
ON_WM_CTLCOLOR()
ON_NOTIFY(NM_DBLCLK, IDC_LIST_URL, OnDblclkListURL)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

CURLListDlg::CURLListDlg(   std::list<URLKey> &URLList,
                            URLKey &outKey,
                            CWnd* pParent /*=NULL*/ )
        : CDialog(CURLListDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CURLListDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pURLList  = &URLList ;
    m_poutKey   = &outKey ;

}
//---------------------------------------------------------------------------

CURLListDlg::~CURLListDlg()
{}

//----------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
// CCaptureOpenPPage message handlers
//----------------------------------------------------------------------------


BOOL CURLListDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    InitList();
    FillList();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//----------------------------------------------------------------------------



HBRUSH CURLListDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }
    return hbr;
}

//---------------------------------------------------------------------------

void CURLListDlg::FillList()
{
    int         nItem ;
    int         nIndex = 0 ;
    URLKey      key ;
    //----
    m_ListCtrl.SetRedraw(FALSE);
    m_ListCtrl.DeleteAllItems();

    nIndex  = 0 ;
    for(std::list<URLKey>::iterator it = m_pURLList->begin();
            it != m_pURLList->end(); it++ )
    {
        nIndex ++ ;
        key     = *it ;
        nItem   = m_ListCtrl.InsertItem( nIndex , key.m_strURL , m_idFile ) ;
        m_ListCtrl.SetItemData( nItem, key.m_dwConnId ) ;
    }

    m_ListCtrl.SetRedraw();
}

//---------------------------------------------------------------------------

BOOL CURLListDlg::InitList()
{
    m_ListCtrl.SubclassDlgItem( IDC_LIST_URL , this ) ;

    CBitmap bmp ;

    //Image List
    if( ! m_imlNormal.Create( 16, 16, ILC_COLOR32|ILC_MASK, 0, 1 ) )
        return FALSE;


    VERIFY(bmp.LoadBitmap( IDB_LINK ));
    m_idFile = m_imlNormal.Add( &bmp, RGB(0,128,128));
    bmp.DeleteObject();

    m_ListCtrl.SetImageList( &m_imlNormal ,LVSIL_SMALL ) ;

    return TRUE ;
}

//---------------------------------------------------------------------------

BOOL CURLListDlg::Verify( )
{
    POSITION pos = m_ListCtrl.GetFirstSelectedItemPosition( ) ;
    if( pos )
        return TRUE ;
    else
        return FALSE ;
}

//---------------------------------------------------------------------------

void CURLListDlg::DisplayErrorMessage()
{
    CString strText ;
    CString strError ;
    //---
    strText.LoadString( IDS_URL_NOSEL ) ;
    strError.LoadString( IDS_ERROR ) ;
    MessageBox( strText , strError , MB_OK ) ;
}

//---------------------------------------------------------------------------

void CURLListDlg::Ctrl2Values()
{
    POSITION pos = m_ListCtrl.GetFirstSelectedItemPosition( ) ;
    if( pos )
    {
        int     nItem ;
        URLKey  key ;
        BOOL    bFound ;
        CString strURL ;
        DWORD   dwConnId ;
        //---
        nItem       = m_ListCtrl.GetNextSelectedItem( pos ) ;
        strURL      = m_ListCtrl.GetItemText( nItem , 0 ) ;
        dwConnId    = m_ListCtrl.GetItemData( nItem ) ;

        bFound  = FALSE ;
        for(std::list<URLKey>::iterator it = m_pURLList->begin();
                it != m_pURLList->end() && !bFound; it++ )
        {
            key = *it ;
            if( key.m_strURL == strURL
                    && key.m_dwConnId == dwConnId )
            {
                bFound      = TRUE ;
                *m_poutKey  = key ;
            }
        }
    }
}

//---------------------------------------------------------------------------

void CURLListDlg::OnDblclkListURL(NMHDR* pNMHDR, LRESULT* pResult)
{
    LPNMITEMACTIVATE pNMDblClick = (LPNMITEMACTIVATE)pNMHDR;
    if(pNMDblClick->iItem != -1 )
    {
        OnOK() ;
    }

    *pResult = 0;
}

//---------------------------------------------------------------------------

void CURLListDlg::OnOK()
{
    POSITION pos = m_ListCtrl.GetFirstSelectedItemPosition( ) ;
    if( !pos )
    {
        DisplayErrorMessage() ;

        return ;
    }

    Ctrl2Values() ;

    CDialog::OnOK() ;
}

BOOL CURLListDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
