/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VISyncEventPPage.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "VISyncEventPPage.h"
#include <SCLStructure.h>
#include "htmlhelp.h"
#include "Help\VISyncEventPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVISyncEventPPage property page

//--------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVISyncEventPPage, CExtendedPropertyPage)

//--------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVISyncEventPPage, CExtendedPropertyPage)
//{{AFX_MSG_MAP(CVISyncEventPPage)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//--------------------------------------------------------------------------

CVISyncEventPPage::CVISyncEventPPage(CVISyncEvent *_pSyncEvent,CSCLStructure *_pSCLStructure) :
        CExtendedPropertyPage(CVISyncEventPPage::IDD)
{
    ASSERT(_pSyncEvent != NULL);
    m_pSyncEvent = _pSyncEvent;
    m_pSCLStructure = _pSCLStructure;
    //{{AFX_DATA_INIT(CVISyncEventPPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT
}

//--------------------------------------------------------------------------


CVISyncEventPPage::~CVISyncEventPPage()
{}

//--------------------------------------------------------------------------


void CVISyncEventPPage::DoDataExchange(CDataExchange* pDX)
{
    CExtendedPropertyPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CVISyncEventPPage)
    DDX_Control(pDX, IDC_PRIMARY_URL_COMBO, m_ctlURLCombo);
    //}}AFX_DATA_MAP
}

/////////////////////////////////////////////////////////////////////////////
// CVISyncEventPPage message handlers
//--------------------------------------------------------------------------


BOOL CVISyncEventPPage::OnInitDialog()
{
    CExtendedPropertyPage::OnInitDialog();

    Values2Ctrl();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//--------------------------------------------------------------------------


void CVISyncEventPPage::Ctrl2Values()
{
    m_pSyncEvent->m_strSyncEventURL.Empty();

    int nCurSel = m_ctlURLCombo.GetCurSel();
    if(nCurSel!=CB_ERR)
    {
        m_ctlURLCombo.GetLBText(nCurSel,m_pSyncEvent->m_strSyncEventURL);
        m_pSyncEvent->m_nSyncEventURLConId = m_ctlURLCombo.GetItemData(nCurSel);

        pDB_URL_FIELDS pdbFields;
        pdbFields = m_pSCLStructure->GetURLInfo(m_pSyncEvent->m_strSyncEventURL, m_pSyncEvent->m_nSyncEventURLConId) ;
        m_pSyncEvent->m_strSyncEventURL = pdbFields->m_strURL;
        m_pSyncEvent->m_lGETBeginLine   = pdbFields->m_lBeginLine;
        m_pSyncEvent->m_lGETEndLine     = pdbFields->m_lEndGetLine;
        m_pSyncEvent->m_bPrimary        = pdbFields->m_bPrimary;
    }
}

//--------------------------------------------------------------------------

void CVISyncEventPPage::Values2Ctrl()
{
    CString&    strSelectedURL = m_pSyncEvent->m_strSyncEventURL;
    CString     strURL;
    CSize       sz;
    CDC*        pDC = m_ctlURLCombo.GetDC();
    long        nConId;
    long        nPos ;
    int         dx = 0;
    int         nURLToSelect = -1;


    m_ctlURLCombo.SetRedraw(FALSE);
    m_ctlURLCombo.ResetContent();

    for(int nNdx=0;nNdx<m_pSCLStructure->m_aData.GetSize();nNdx++)
    {
        nConId  = m_pSCLStructure->m_aData[nNdx]->m_lConId;
        strURL  = m_pSCLStructure->m_aData[nNdx]->m_strURL;
        nPos    = strURL.Find( _T( "=" ) ) ;
        if( nPos >= 0 )
        {
            int nItem = m_ctlURLCombo.AddString( strURL );
            m_ctlURLCombo.SetItemData( nItem, nConId );

            if( m_pSyncEvent->m_lCurrentLine >= m_pSCLStructure->m_aData[nNdx]->m_lBeginLine
                    && m_pSyncEvent->m_lCurrentLine <= m_pSCLStructure->m_aData[nNdx]->m_lEndGetLine )
            {
                nURLToSelect = nItem ;
            }

            //For the scroll
            sz = pDC->GetTextExtent( strURL );
            if (sz.cx > dx)
            {
                dx = sz.cx;
            }
        }
    }

    //For the scroll
    m_ctlURLCombo.ReleaseDC(pDC);
    m_ctlURLCombo.SetHorizontalExtent(dx);  // Set the horizontal extent so every character of all strings can
    // be scrolled to.
    //-

    //if(m_ctlURLCombo.SelectString( -1, m_pSyncEvent->m_strSyncEventURL) == CB_ERR )
    //    m_ctlURLCombo.SetCurSel(0);

    if( nURLToSelect >= 0 )
    {
        m_ctlURLCombo.SetCurSel( nURLToSelect ) ;
    }
    else
    {
        m_ctlURLCombo.SetCurSel( 0 ) ;
    }

    m_ctlURLCombo.SetRedraw();
}

//--------------------------------------------------------------------------

HBRUSH CVISyncEventPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CExtendedPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd)
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }


    // TODO: Return a different brush if the default is not desired
    return hbr;
}

BOOL CVISyncEventPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
