/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VIVariablisationPPage.cpp : implementation file
//

#include "stdafx.h"
#include <SCLStructure.h>
#include "resource.h"
#include "VIVariablisationPPage.h"
#include "htmlhelp.h"
#include "Help\VIVariablisationPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define COL_ITEM_ARGUMENT   0
#define COL_ITEM_VALUE      1

extern CSCLStructure*  pSCLStructure ;
extern CStringArray*   pVariabilise ;

//---------------------------------------------------------------------------

IMPLEMENT_DYNCREATE(CVIVariablisationPPage, CExtendedPropertyPage)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVIVariablisationPPage, CExtendedPropertyPage)
//{{AFX_MSG_MAP(CVIVariablisationPPage)
ON_WM_CTLCOLOR()
ON_WM_NCACTIVATE()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------
CVIVariablisationPPage::CVIVariablisationPPage(CVIInfo * _pVIInfo)
        :   CExtendedPropertyPage(CVIVariablisationPPage::IDD)
{
    ASSERT(m_pVIInfo != NULL);
    m_pVIInfo = _pVIInfo;
    //{{AFX_DATA_INIT(CVIVariablisationPPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

}

//---------------------------------------------------------------------------


CVIVariablisationPPage::~CVIVariablisationPPage()
{
    m_arstrDefaultValues.RemoveAll() ;
}

//---------------------------------------------------------------------------

void CVIVariablisationPPage::DoDataExchange(CDataExchange* pDX)
{
    CExtendedPropertyPage::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CVIVariablisationPPage)
    //}}AFX_DATA_MAP
}

//---------------------------------------------------------------------------


BOOL CVIVariablisationPPage::OnInitDialog()
{
    CExtendedPropertyPage::OnInitDialog();

    m_wndGrid.Create(
        this,
        IDC_GRIDWND,
        IDS_GRID_HEADER_URLVAR );

    //m_wndGrid.Initialise();   //OS
    m_wndGrid.SetFocus();

    //m_wndGrid.SetLastDataRow( 1 );

    return TRUE;  // return TRUE  unless you set the focus to a control
}

//---------------------------------------------------------------------------


void CVIVariablisationPPage::Ctrl2Values()
{
    CTypedPtrArray<CPtrArray, CVIArgInfo*>& arArgInfo = m_pVIInfo->m_Variablisation.m_arArgInfo ;
    CString         strArgument ;
    CString         strValue ;
    BOOL            bValueIsVarName ;
    int             nNumArgs ;
    //---

    m_pVIInfo->m_Variablisation.Reset() ;

    nNumArgs    = m_arstrDefaultValues.GetSize() ;
    for( int nArg = 0; nArg < nNumArgs; nArg++ )
    {
        m_wndGrid.GetValueRowCol( ( nArg + 1 ), COL_ITEM_ARGUMENT, strArgument ) ; //OS
        m_wndGrid.GetValueRowCol( ( nArg + 1 ), COL_ITEM_VALUE, strValue ) ;       //OS

        bValueIsVarName             = !strValue.IsEmpty() ;

        if( strValue.IsEmpty() )
        {
            strValue    = m_arstrDefaultValues[nArg] ;
        }

        arArgInfo.Add( new CVIArgInfo(strArgument, strValue, bValueIsVarName) ) ;
    }
}

//---------------------------------------------------------------------------

void CVIVariablisationPPage::Values2Ctrl()
{
    CStringArray    arstrArguments;
    CStringArray    arstrVariables;
    CString         strURL;
    CString         strArgument;
    CString         strArgsToExtract;
    CString         strDefaultValue;
    int             nPos;
    int             nPosEqual;
    BOOL            bNoMoreArgs;
    pDB_URL_FIELDS  pURLInfo;
    //---

    m_arstrDefaultValues.RemoveAll() ;

    // Extract the args from the URL
    pURLInfo = pSCLStructure->GetURLInfo( m_pVIInfo->m_SyncEvent.m_strSyncEventURL, m_pVIInfo->m_SyncEvent.m_nSyncEventURLConId ) ;

    strURL  = pURLInfo->m_strArgURL ;

    nPos    = strURL.Find( '?' );
    if( nPos >= 0 )
    {
        int     nSpacePos;
        CString strBeginURL ;
        CString strEndURL ;
        //---
        strBeginURL = strURL.Left( nPos + 1 );

        nSpacePos = strURL.Find( _T(" ") ) ;
        ASSERT( nSpacePos > 0 );

        if( nSpacePos < ( strURL.GetLength() - 1 ) )
        {
            strEndURL           = strURL.Mid( nSpacePos );
        }
        else
        {
            strEndURL.Empty() ;
        }

        strArgsToExtract    = strURL.Mid( nPos + 1, nSpacePos - nPos - 1 ) ;

        //Update the structure
        m_pVIInfo->m_Variablisation.m_strBeginURL   = strBeginURL ;
        m_pVIInfo->m_Variablisation.m_strEndURL     = strEndURL ;
    }

    nPos        = strArgsToExtract.Find( '&' );
    if ( nPos < 0 )
    {
        nPos = strArgsToExtract.GetLength();
    }

    bNoMoreArgs = FALSE;
    while ( !bNoMoreArgs )
    {
        CString     strArgBlock ;
        //---
        strArgBlock = strArgsToExtract.Left( nPos ) ;

        nPosEqual   = strArgBlock.Find( "=" );
        ASSERT( nPosEqual > 0 );

        strArgument     = strArgBlock.Left( nPosEqual );
        strDefaultValue = strArgBlock.Mid( nPosEqual + 1 ) ;

        arstrArguments.Add( strArgument );
        m_arstrDefaultValues.Add( strDefaultValue );

        if( nPos < ( strArgsToExtract.GetLength() - 1 ) )
        {
            strArgsToExtract = strArgsToExtract.Mid( nPos + 1 );
        }
        else
        {
            strArgsToExtract.Empty();
        }

        nPos        = strArgsToExtract.Find( _T( '&' )  );
        bNoMoreArgs = strArgsToExtract.IsEmpty() ;

        if( ( nPos < 0 ) && !bNoMoreArgs )
        {
            nPos = strArgsToExtract.GetLength();
        }
    }


    // Combo string construction
    arstrVariables.RemoveAll();
    for( int nVar = 0; nVar < pSCLStructure->m_aVarData.GetSize(); nVar++ )
    {
        arstrVariables.Add( pSCLStructure->m_aVarData[nVar]->m_strName );
    }

    //Clear the grid
    m_wndGrid.DeleteAllItems(); //OS

    // Fill the argument column
    //m_wndGrid.SetLastDataRow( 0 )  ;  //OS

    m_wndGrid.SetColumnCount( 2 );   //OS
    m_wndGrid.SetRowCount( arstrArguments.GetSize() + 1 );  //OS
    m_wndGrid.SetFixedColumnCount( 0 ); //OS
    m_wndGrid.SetFixedRowCount( 1 );        //OS

    m_wndGrid.SetItemText( 0, COL_ITEM_ARGUMENT, "Argument" ); //OS
    m_wndGrid.SetItemText( 0, COL_ITEM_VALUE, "Value" ); //OS

    for( int nArg = 0; nArg < arstrArguments.GetSize(); nArg++ )
    {
        CString     strDefaultValue;
        BOOL        bFound;
        int         nVar;
        int         nLine;
        //---
        nLine = nArg + 1;
        m_wndGrid.SetItemText( nLine, COL_ITEM_ARGUMENT, arstrArguments[nArg] ); //OS
        //m_wndGrid.SetItemState( nArg, COL_ITEM_ARGUMENT, GVIS_READONLY ); //OS


        strDefaultValue.Empty() ;

        bFound = FALSE ;
        nVar = 0;
        while( nVar < pSCLStructure->m_aVarData.GetSize() && !bFound )
        {
            if( m_arstrDefaultValues.GetAt(nArg) == pSCLStructure->m_aVarData[nVar]->m_strName )
            {
                bFound = TRUE ;
                strDefaultValue = pSCLStructure->m_aVarData[nVar]->m_strName ;
            }
            nVar++ ;
        }

        // Fill the value column
        m_wndGrid.CreateListCell( nLine, COL_ITEM_VALUE, arstrVariables );    //OS
        m_wndGrid.SetItemText( nLine, COL_ITEM_VALUE, strDefaultValue ); //OS

    }
    //m_wndGrid.SetLastDataRow( arstrArguments.GetSize() )  ;   //OS

    m_wndGrid.SetRowResize( FALSE );
    m_wndGrid.SetColumnResize();
    m_wndGrid.ExpandLastColumn();

}

//---------------------------------------------------------------------------

HBRUSH CVIVariablisationPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CExtendedPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);


    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem(IDC_DESCRIPTION)->m_hWnd)
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }


    // TODO: Return a different brush if the default is not desired
    return hbr;
}

//---------------------------------------------------------------------------

BOOL CVIVariablisationPPage::OnSetActive()
{
    Values2Ctrl();

    return CExtendedPropertyPage::OnSetActive();
}

//---------------------------------------------------------------------------

BOOL CVIVariablisationPPage::OnNcActivate(BOOL bActive)
{

    return CDialog::OnNcActivate(bActive);
}

BOOL CVIVariablisationPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
