/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// VarHTMLPPage.cpp : implementation file
//

#include "stdafx.h"
#include "modeller.h"
#include "VarHTMLPPage.h"
#include "Constantes.h"
#include <phParse.hpp>
#include <IECache.h>
#include "htmlhelp.h"
#include "Help\VarHTMLPPage.inl"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CVarHTMLPPage property page
/////////////////////////////////////////////////////////////////////////////

IMPLEMENT_DYNCREATE(CVarHTMLPPage, CExtendedPropertyPage)

//---------------------------------------------------------------------------

BEGIN_MESSAGE_MAP(CVarHTMLPPage, CExtendedPropertyPage)
//{{AFX_MSG_MAP(CVarHTMLPPage)
ON_CBN_SELCHANGE(ID_VARPAGE_COMBO, OnSelchangeVarpageCombo)
ON_WM_CTLCOLOR()
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//---------------------------------------------------------------------------

CVarHTMLPPage::CVarHTMLPPage(   CVarValueList *     _pVarValueList,
                                CSCLStructure *     _pSCLStructure,
                                CString             _strALLFileName )
        : CExtendedPropertyPage(CVarHTMLPPage::IDD)
{
    //{{AFX_DATA_INIT(CVarHTMLPPage)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pVarValueList     = _pVarValueList ;
    m_pSCLStructure     = _pSCLStructure ;
    m_strALLFileName    = _strALLFileName ;
}

//---------------------------------------------------------------------------

CVarHTMLPPage::~CVarHTMLPPage()
{}

//---------------------------------------------------------------------------


BOOL CVarHTMLPPage::OnInitDialog()
{
    CExtendedPropertyPage::OnInitDialog() ;

    m_HTMLFileCombo.SubclassDlgItem( ID_VARPAGE_COMBO, this ) ;

    m_HTMLElementList.SubclassDlgItem( ID_ELT_LIST , this ) ;
    LV_COLUMN lvcS ;
    lvcS.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM ;
    lvcS.iSubItem = 0 ;
    lvcS.pszText = _T("Element") ;
    lvcS.cx = 125 ;
    lvcS.fmt = LVCFMT_LEFT ;
    m_HTMLElementList.InsertColumn(1,&lvcS) ;

    Values2Ctrl() ;

    return TRUE ;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//---------------------------------------------------------------------------


void CVarHTMLPPage::Values2Ctrl()
{
    CString     strURL ;
    int         nIndex ;

    m_HTMLFileCombo.SetRedraw(FALSE) ;
    m_HTMLFileCombo.ResetContent() ;

    for(int i=0; i<m_pSCLStructure->m_aData.GetSize(); i++)
    {
        if( m_pSCLStructure->m_aData[i]->m_bHTMLURL )
        {
            strURL  = m_pSCLStructure->m_aData[i]->m_strURL ;
            nIndex  = m_HTMLFileCombo.AddString(strURL) ;
            m_HTMLFileCombo.SetItemData( nIndex, m_pSCLStructure->m_aData[i]->m_lConId ) ;
        }
    }

    m_HTMLFileCombo.SetCurSel(0) ;
    m_HTMLFileCombo.SetRedraw() ;

    ShowElements() ;
}

//---------------------------------------------------------------------------

void CVarHTMLPPage::Ctrl2Values()
{
    CString             strHTMLFile ;
    POSITION    pos ;
    int         nItem ;
    CTag *      pTag ;

    //Get Selected Column Item
    pos = m_HTMLElementList.GetFirstSelectedItemPosition() ;

    ASSERT(pos!=NULL) ;

    nItem   = m_HTMLElementList.GetNextSelectedItem(pos) ;
    pTag    = (CTag *)(m_HTMLElementList.GetItemData(nItem)) ;

    m_pVarValueList->m_Values.RemoveAll() ;

    SearchTagOPTION( pTag->m_pTagList ) ;
}

//---------------------------------------------------------------------------

void CVarHTMLPPage::ShowElements()
{
    CString         strHTMLSource ;
    CString         strURL ;
    CTag*           pTag ;
    CIECache        cache( m_strALLFileName ) ;
    DWORD           dwLength ;
    long            lConId ;
    long            lRequestID = 1 ;

    m_HTMLElementList.DeleteAllItems();

    m_HTMLFileCombo.GetLBText( m_HTMLFileCombo.GetCurSel(), strURL ) ;
    lConId  = m_HTMLFileCombo.GetItemData( m_HTMLFileCombo.GetCurSel() ) ;

    if( cache.Init() )
    {
        strHTMLSource = (char*) ( cache.GetBody( strURL , lConId , &dwLength , lRequestID ) );
    }

    m_DOMStructure.Reset();
    m_DOMStructure.Init();
    PhLexer<unsigned char*> lex(&m_DOMStructure, (UCHAR*)((LPCTSTR)strHTMLSource));
    PhParser<unsigned char*> Parser(&m_DOMStructure ,lex ) ;

    //---
    if( !Parser.Parse( ) )
    {
        //       MessageBox( "Parsing not managed !", "Warning" , MB_OK ) ;
    }


    pTag = m_DOMStructure.GetRootItem() ;
    SearchTagSELECT( pTag->m_pTagList ) ;

}


//----------------------------------------------------------------------------

void CVarHTMLPPage::SearchTagSELECT( CTagList * _pTagList )
{
    CTag* pTag = _pTagList->m_pTag ;
    if( pTag )
    {
        if(!stricmp( pTag->m_pczName, STR_SELECT ))
        {
            InsertTagNameInList( pTag ) ;
        }

        SearchTagSELECT( pTag->m_pTagList ) ;
        SearchTagSELECT( _pTagList->m_pNextTagList ) ;
    }
}

//----------------------------------------------------------------------------

void CVarHTMLPPage::SearchTagOPTION( CTagList * _pTagList )
{
    CTag* pTag = _pTagList->m_pTag ;
    if( pTag )
    {
        if(!stricmp( pTag->m_pczName, STR_OPTION ))
        {
            InsertTagValueInValueList( pTag ) ;
        }

        SearchTagOPTION( pTag->m_pTagList ) ;
        SearchTagOPTION( _pTagList->m_pNextTagList ) ;
    }
}

//----------------------------------------------------------------------------

void CVarHTMLPPage::InsertTagNameInList(CTag * _pTag)
{
    int         nIndex ;
    CString     strAttributeValue ;
    BOOL        bAttributeExist ;

    if( _pTag )
    {
        bAttributeExist = GetAttributeValue( _pTag, STR_NAME, &strAttributeValue) ;
        ASSERT(bAttributeExist) ;
        //$$$ add the tag icon
        nIndex = m_HTMLElementList.InsertItem(0, strAttributeValue) ;
        m_HTMLElementList.SetItemData(nIndex, (DWORD)_pTag) ;
    }
}

//----------------------------------------------------------------------------

void CVarHTMLPPage::InsertTagValueInValueList(CTag * _pTag)
{
    CString     strTagText ;

    if( _pTag )
    {
        GetTagText(_pTag, &strTagText) ;
        m_pVarValueList->m_Values.Add(strTagText) ;
    }
}

//----------------------------------------------------------------------------

void CVarHTMLPPage::GetTagText( CTag* pTag, CString * _strTagText)
{
    CTextList* pTextList = pTag->m_pTextList ;
    CText *pText = pTextList->m_pText ;

    //while( pText  )
    if( pText )
    {
        *_strTagText = pText->m_pczText ;
        pTextList = pTextList->m_pNextTextList ;
        //if( pTextList )
        //    pText = pTextList->m_pText ;
        //else
        //    pText = NULL ;
    }

}

//----------------------------------------------------------------------------

BOOL CVarHTMLPPage::GetAttributeValue( CTag * _pTag, const char * _strAttributeName, CString * _strAttributeValue)
{
    CAttributeList *    pAttributeList ;
    CAttribute *        pAttribute ;
    BOOL                bAttributeFound ;

    pAttributeList  = _pTag->m_pAttributesList ;
    pAttribute      = pAttributeList->m_pAttribute ;

    //---
    for(bAttributeFound = FALSE; pAttribute && !bAttributeFound; )
    {
        //---
        if(!stricmp(pAttribute->m_pczName, _strAttributeName))
        {
            bAttributeFound = TRUE ;
            *_strAttributeValue = pAttribute->m_pczValue ;
        }

        pAttributeList = pAttributeList->m_pNextAttributeList ;
        if( pAttributeList )
            pAttribute = pAttributeList->m_pAttribute ;
        else
            pAttribute = NULL ;
    }

    return bAttributeFound ;
}

//---------------------------------------------------------------------------

LRESULT CVarHTMLPPage::OnWizardBack()
{
    return IDD_VAR_GENERAL_PPAGE ;
}

//---------------------------------------------------------------------------

LRESULT CVarHTMLPPage::OnWizardNext()
{
    if( !Verify() )
        return -1 ; //nope

    Ctrl2Values() ;

    return 0 ; // => next page in the pages array ;
}

//---------------------------------------------------------------------------


void CVarHTMLPPage::OnSelchangeVarpageCombo()
{
    ShowElements() ;
}

//---------------------------------------------------------------------------

BOOL CVarHTMLPPage::OnSetActive()
{

    return CExtendedPropertyPage::OnSetActive();
}

//---------------------------------------------------------------------------

HBRUSH CVarHTMLPPage::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
    HBRUSH hbr = CExtendedPropertyPage::OnCtlColor(pDC, pWnd, nCtlColor);

    if( nCtlColor == CTLCOLOR_STATIC && pWnd->m_hWnd == GetDlgItem( IDC_TITLE_STATIC )->m_hWnd )
    {
        pDC->SetTextColor( RGB( 0 , 0 , 255 ) ) ;
    }

    return hbr;
}

//---------------------------------------------------------------------------

BOOL CVarHTMLPPage::Verify()
{
    UINT uSelectedCount ;


    //Get Selected Column Item
    uSelectedCount = m_HTMLElementList.GetSelectedCount();

    if ( uSelectedCount < 1 )
    {
        CString     strMsg ;
        CString     strTitle ;
        //---
        strMsg.LoadString( IDS_VARHTML_NOSEL ) ;
        strTitle.LoadString( IDS_ERROR ) ;

        MessageBox( strMsg, strTitle );
        return FALSE ;
    }

    return TRUE ;
}


BOOL CVarHTMLPPage::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp;
    contextHelp.LoadString(IDS_MODELLER_HELP_FILE);
    contextHelp= getenv("CYRANO_HOME")+contextHelp+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
