/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** V - T E S T ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: CHK_LGN_CMND_CHR                */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  This module contains the CHK_LGN_CMND_CHR function, which checks     */
/*  whether the supplied character is a valid Script Language command/   */
/*  control character.                           */
/*                                       */
/*===========================================================================*/

/*
 *  PSL Macros.
 */
#include <psl_macros.h>

/*
 * Function Prototypes.
 */
#include <psl_rtndef.h>         /* PSL routines */
#include <vtest_rtndef.h>       /* V-TEST routines */

/*
 *  Structure definitions.
 */
#include <shrmsg.h>         /* PSL Messages */


/************************************************************************/
/*                                  */
/*  CHK_LGN_CMND_CHR - Check Command Character          */
/*                                  */
/*  Calling Convention:                     */
/*                                  */
/*     iret = CHK_LGN_CMND_CHR (chr_ptr)                */
/*                                  */
/*  Where:                              */
/*                                  */
/*     iret     = Unsigned Long PSL Routine Return Status:      */
/*      0      = Invalid character              */
/*      PSL_NORMAL = Success                    */
/*     chr_ptr  = Pointer to character to be checked        */
/*                                  */
/*  The routine checks whether the supplied character is a valid    */
/*  Script Language (SCL) command or control character, returning   */
/*  a status in R0.                         */
/*                                  */
/*  The valid characters are:                   */
/*                                  */
/*      ~   Default command character           */
/*      ^   Default control character           */
/*      #                           */
/*      $                           */
/*      %                           */
/*      *                           */
/*      .                           */
/*      :                           */
/*      ;                           */
/*      ?                           */
/*      ]                           */
/*      ]                           */
/*      {                           */
/*      |                           */
/*      }                           */
/*                                  */
/*  This list excludes characters that would create ambiguous   */
/*  commands, or would conflict with DCL 'control' characters   */
/*  (significant when parsing the /command_character and        */
/*  /control_character qualifiers).                 */
/*                                  */
/************************************************************************/

unsigned long CHK_LGN_CMND_CHR (char *chr_ptr)
{
    if ((*chr_ptr == '^')
            || (*chr_ptr == '~')
            || (*chr_ptr == '#')
            || (*chr_ptr == '$')
            || (*chr_ptr == '%')
            || (*chr_ptr == '*')
            || (*chr_ptr == '.')
            || (*chr_ptr == ':')
            || (*chr_ptr == ';')
            || (*chr_ptr == '?')
            || (*chr_ptr == '[')
            || (*chr_ptr == ']')
            || (*chr_ptr == '{')
            || (*chr_ptr == '|')
            || (*chr_ptr == '}'))
    {
        return PSL_NORMAL;
    }
    else
    {
        return 0;
    }
}
