/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/*===========================================================================*/
/*                                       */
/*               **** C O M M O N ****                   */
/*                                       */
/*===========================================================================*/
/*                                       */
/*          Module Name: CONFIRM_RESPONSE                */
/*          ============                         */
/*                                       */
/* Description:                                  */
/* ============                                  */
/*                                       */
/*  Routine to analyse the response to a CONFIRM prompt.             */
/*  re-written from macro.                           */
/*                                       */
/*===========================================================================*/


/*
 *   Standard ANSI header files
 */
#include <stdio.h>
#include <string.h>
#include <ctype.h>

/*
 *  PSL Macros
 */
#include <psl_macros.h>

/*  Define Function Prototypes for all routines in this module */

#include psl_common_file (psl_rtndef.h) /* PSL Function Prototypes */

/*  Structure Definitions */

#include psl_common_file (confirm_respdef.h)
/* Confirm Messages */

/*======================================================================*/
/*                                  */
/*    RESPONSE = CONFIRM_RESPONSE ()                    */
/*    ==============================                    */
/*                                  */
/*                                  */
/*    Where:                                */
/*                                  */
/*    RESPONSE = long value representing response from user.        */
/*           This will take one of the following symbolic values:   */
/*                                  */
/*          CNF_PROMPT (-1) - Invalid Input, Reprompt   */
/*          CNF_YES    ( 1) - Yes, True or 1        */
/*          CNF_NO     ( 2) - No, False, 0 or <CR>      */
/*          CNF_QUIT   ( 3) - Quit or <CTRL/Z>      */
/*          CNF_ALL    ( 4) - All               */
/*                                  */
/*======================================================================*/

long  CONFIRM_RESPONSE (void)

{
    /*
     *    Local data
     */
    char   user_input [256];
    char   *iret;          /* Status from gets */
    size_t input_length;       /* Length of user input */
    long   ret_value;      /* return value */
    unsigned long   isub;           /* Loop counter */

    /*
     *    Solicit an input from the user.
     */
    iret = gets (user_input);
    if (iret == NULL)           /* If NULL (CTRL-Z) or Error    */
    {
        return CNF_QUIT;        /* Return QUIT          */
    }
    else if (!strcmp (user_input, "\0"))/* If Carriage control      */
    {
        return CNF_NO;          /* Return NO            */
    }

    /*
     *    Convert the input string to uppercase, after validating that the input
     *    is no longer than the longest valid prompt (FALSE=5)
     */
    input_length = strlen (user_input);
    if (input_length > 5) return CNF_PROMPT;
    /* Ignore too long inputs   */
    for (isub=0; isub < input_length; isub++)
    {
        user_input [isub] = toupper (user_input [isub]);
    }

    /*
     *    Now scan the input string to find what we have specified
     */
    if ((input_length <= 3)         /* Check for "YES"  */
            &&  (!strncmp ("YES", user_input, input_length)))
    {
        ret_value = CNF_YES;
    }

    else if ((input_length <= 4)        /* Check for "TRUE" */
             &&  (!strncmp ("TRUE", user_input, input_length)))
    {
        ret_value = CNF_YES;
    }

    else if ((input_length == 1)        /* Check for "1"    */
             &&  (user_input [0] == '1'))
    {
        ret_value = CNF_YES;
    }

    else if ((input_length <= 2)        /* Check for "NO"   */
             &&  (!strncmp ("NO", user_input, input_length)))
    {
        ret_value = CNF_NO;
    }

    else if ((input_length <= 5)        /* Check for "FALSE"    */
             &&  (!strncmp ("FALSE", user_input, input_length)))
    {
        ret_value = CNF_NO;
    }

    else if ((input_length == 1)        /* Check for "0"    */
             &&  (user_input [0] == '0'))
    {
        ret_value = CNF_NO;
    }

    else if ((input_length <= 3)        /* Check for "ALL"  */
             &&  (!strncmp ("ALL", user_input, input_length)))
    {
        ret_value = CNF_ALL;
    }

    else if ((input_length <= 4)        /* Check for "QUIT" */
             &&  (!strncmp ("QUIT", user_input, input_length)))
    {
        ret_value = CNF_QUIT;
    }

    else
    {
        ret_value = CNF_PROMPT;         /* All others return error */
    }

    /*
     *    Return the defined status
     */
    return ret_value;
}
