/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECMainFrame.cpp : implementation file
//

#include "stdafx.h"
#include "CECCommander.h"
#include "CECMainFrame.h"
#include "Solutions.h"
#include "CYRWizardStartDlg.h"
#include "../Common/CECRegistryTools.h"
#include "HtmlHelp.h"
#include "CECUserToolManager.h"
#include "BCGWindowsManagerDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECMainFrame

CECMainFrame::CECMainFrame()
{
    // Default the MDI restore rect
    m_rcRestoreRect = CRect(0,0,0,0);
    // And get hold of the Trace Object
    CECCommanderApp *pApp = (CECCommanderApp*)AfxGetApp();
    ASSERT (pApp != NULL);

    m_pTrace = pApp->GetTrace();
    m_bAutoSaveWorkspace=true;

    m_pFullScreenDlg = NULL;
}

CECMainFrame::~CECMainFrame()
{
    m_pTrace = NULL;
}

IMPLEMENT_DYNAMIC(CECMainFrame, CECGlobalCmdTarget)


BEGIN_MESSAGE_MAP(CECMainFrame, CECGlobalCmdTarget)
//{{AFX_MSG_MAP(CECMainFrame)
ON_WM_CREATE()
ON_COMMAND(ID_VIEW_FULLSCREEN, OnViewFullscreen)
ON_UPDATE_COMMAND_UI(ID_VIEW_FULLSCREEN, OnUpdateViewFullscreen)
ON_WM_CLOSE()
ON_COMMAND(ID_PROJECT_WORKSPACE_SAVE, OnProjectWorkspaceSave)
ON_COMMAND(ID_PROJECT_WORKSPACE_RESTORE, OnProjectWorkspaceRestore)
ON_WM_DESTROY()
ON_COMMAND(ID_VIEW_FLOORVIEWER, OnViewFloorviewer)
ON_UPDATE_COMMAND_UI(ID_VIEW_FLOORVIEWER, OnUpdateViewFloorviewer)
ON_COMMAND(ID_VIEW_OUTPUTWINDOW, OnViewOutputwindow)
ON_UPDATE_COMMAND_UI(ID_VIEW_OUTPUTWINDOW, OnUpdateViewOutputwindow)
ON_COMMAND(ID_HELP_CYRANOONTHEWEB, OnVisitWebSite)
ON_COMMAND(ID_HELP_CONTENTS, OnHelpContents)
ON_COMMAND(ID_HELP_INDEX, OnHelpIndex)
ON_COMMAND(ID_HELP_SEARCH, OnHelpSearch)
ON_COMMAND(ID_HELP_OPENSTAPRODUCTWEBSITE, OnHelpOpenSTAProductWebsite)
ON_COMMAND(ID_WINDOW_MANAGE, OnWindowManage)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CECMainFrame message handlers

void CECMainFrame::ShowFullScreen(BOOL bShow)
{
    CRect rect(0,0,0,0);

    // Set full screen status
    CECWorkspace* pWorkspace = CECCommanderApp::GetWorkspace();
    ASSERT (pWorkspace != NULL);

    pWorkspace->SetFullScreenMode(bShow);

    if ( bShow == FALSE )
    {
        //Hide the dialog to end full screen mode
        if(m_pFullScreenDlg)
        {
            m_pFullScreenDlg->DestroyWindow();
            delete m_pFullScreenDlg;
            m_pFullScreenDlg = NULL;
        }

        // If we do not have a rectangle to restore the window to, we
        // shouldn't do it.  If we do the window will have very strange
        // extents and disappear.
        if ( m_rcRestoreRect.IsRectEmpty() == FALSE )
        {
            // Restore caption
            ModifyStyle( 0, WS_CAPTION|WS_THICKFRAME );

            // Restore the menu
            m_wndMenuBar.ShowWindow(SW_SHOW);

            // Restore screen size
            rect = m_rcRestoreRect;

            // If the current child window is maximised we need to add
            // the controls onto the newly created menu.  The easiest
            // way to do this is to post a maximise message to the child.
            BOOL bMax = FALSE;
            CMDIChildWnd* pMDIChild = MDIGetActive( &bMax );
            if ( bMax == TRUE )
            {
                // As window is already maximised (and will therefore do nothing if it
                // receives purely a maximise message) we post a restore message first.
                pMDIChild->PostMessage( WM_SIZE, SIZE_RESTORED, MAKELONG( rect.Width(), rect.Height() ));
                pMDIChild->PostMessage( WM_SIZE, SIZE_MAXIMIZED, MAKELONG( rect.Width(), rect.Height() ));

                // Fudge: The above two functions leave the system boxes for the window
                // in an unpredictable state.  Basically the close button is greyed out,
                // but it can still be clicked on.  To get round this we modify the style
                // of the window to remove the system menu and then re-add it.  This is a
                // workaround for the problem.
                pMDIChild->ModifyStyle( WS_SYSMENU, 0 );
                pMDIChild->ModifyStyle( 0, WS_SYSMENU );
            }
        }
    }
    else
    {
        //Show the dialog to end full screen mode
        if(!m_pFullScreenDlg)
            m_pFullScreenDlg = new CFullScreenDlg;
        m_pFullScreenDlg->Create(IDD_FULLSCREEN,this);

        // Before we make any changes, let's get the current status so we can restore
        // it when and if necessary.

        // Get current size of window
        GetWindowRect( m_rcRestoreRect );

        // Remove caption
        ModifyStyle( WS_CAPTION|WS_THICKFRAME,0 );

        // Hide menu (effectively we destroy it here and recreate it later).
        m_wndMenuBar.ShowWindow(SW_HIDE);

        // Maximize window by get current size of desktop
        GetDesktopWindow()->GetWindowRect( rect );
    }

    // If we have a window size then set it
    if ( rect.IsRectEmpty() == FALSE )
    {
        SetWindowPos( &wndTop, rect.left, rect.top, rect.Width(), rect.Height(),
                      SWP_FRAMECHANGED | SWP_SHOWWINDOW );
    }
}

int CECMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CECGlobalCmdTarget::OnCreate(lpCreateStruct) == -1)
        return -1;
    //
    //    Load in the resources
    //
    LoadResources();

    //
    //    ...then the floor windows
    //
    AddFloorWnds();

    //
    //    ..then output views
    //
    AddOutputViews();
    //
    //    ...and finally document templates.
    //
    AddDocumentTemplates();

    AddToolbars();

    CBCGUserToolsManager* pUserToolsManager = theApp.GetUserToolsManager ();
    CString szKey = (CString)SOLUTIONS_FRAMEWORK_KEY + "\\UserTools";
    pUserToolsManager->LoadState(szKey);

    //
    //    Create output view [PLUG_INS]
    //
    if (!CreateStudioOutputView())
        return -1;
    //
    //    Create output view [PLUG_INS]
    //
    if (!CreateStudioFloorView())
        return -1;

    return 0;
}

//
// CreateStudioFloorView
// =====================
//
// This function is used to create the standard output view

BOOL CECMainFrame::CreateStudioFloorView()
{

    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    CPtrArray& FloorWndArray = pApp->GetFloorWndArray ();

    // Only create the Floor Window if a snapin actually wants it
    if (FloorWndArray.GetSize() > 0)
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CMainFrame", "CreateStudioFloorView", "Creating a Floor View");

        // Create the tree view window as a Floor Window so that we can dock it and resize
        // it. The tree view will create the tab control and views required to display the
        // trees for the repository and execution tree view.
        // Dockable window
        DWORD dwStyle = WS_CHILD|WS_VISIBLE|CBRS_LEFT|CBRS_TOOLTIPS|CBRS_SIZE_DYNAMIC;

        m_wndFloor = (CECFloorWnd*)FloorWndArray[0];

        if (!m_wndFloor->Create( _T("Floor Window"), this, CSize (150, 150),
                                 TRUE /* Has gripper */, ID_VIEW_OUTPUT,
                                 WS_CHILD | WS_VISIBLE | CBRS_LEFT))
        {
            if (m_pTrace != NULL) m_pTrace->Log(10, "CMainFrame", "CreateStudioFloorView", "Failed to create the Floor View");
            return FALSE;
        }

        // Enable and dock the floor wnd
        m_wndFloor->EnableDocking(CBRS_ALIGN_LEFT|CBRS_ALIGN_RIGHT);
        DockControlBar(m_wndFloor);

    }
    return TRUE;
}

void CECMainFrame::OnViewFullscreen()
{
    // Toggle full screen status
    CECWorkspace* pWorkspace = CECCommanderApp::GetWorkspace();
    ASSERT(pWorkspace != NULL);

    bool bShowFullScreen = !pWorkspace->IsFullScreen();
    ShowFullScreen(bShowFullScreen);
}

void CECMainFrame::OnUpdateViewFullscreen(CCmdUI* pCmdUI)
{
    // Add a check mark next to entry if we are in full screen mode
    CECWorkspace* pWorkspace = CECCommanderApp::GetWorkspace();
    ASSERT (pWorkspace != NULL);

    pCmdUI->SetCheck( pWorkspace->IsFullScreen() );
}

void CECMainFrame::ActivateOutputWnd(CWnd *pOutWnd)
{
    m_wndOutputView.ActivateTab(pOutWnd);
}

//
// CreateStudioOutputView
// ======================
//
// This function is used to create the standard output view

BOOL CECMainFrame::CreateStudioOutputView()
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    CPtrArray& OutputWndArray = pApp->GetOutputWndArray ();

    // Only create the Output Window if a snapin actually wants it
    if (OutputWndArray.GetSize() > 0)
    {
        if (m_pTrace != NULL) m_pTrace->Log(10, "CMainFrame", "CreateStudioOutputView", "Creating an Output View");

        // Create the output window within which the results of command
        // executions will be displayed.
        if (!m_wndOutputView.Create( _T("Output"), this, CSize (150, 150),
                                     TRUE /* Has gripper */, ID_VIEW_OUTPUT,
                                     WS_CHILD | WS_VISIBLE | CBRS_LEFT))
        {
            if (m_pTrace != NULL) m_pTrace->Log(10, "CMainFrame", "CreateStudioOutputView", "Failed to create the Floor View");
            return FALSE;
        }
        m_wndOutputView.EnableDocking(CBRS_ALIGN_ANY);
        DockControlBar(&m_wndOutputView, AFX_IDW_DOCKBAR_BOTTOM);
    }
    return TRUE;
}

void CECMainFrame::OnClose()
{
    CECUserToolManager::SaveUserTools();

    CMDIFrameWnd::OnClose();
}

void CECMainFrame::OnProjectWorkspaceSave()
{
    BeginWaitCursor();

    //don't automatically save the workspace on shutdown
    m_bAutoSaveWorkspace=false;

    //
    // Hurray --- hang on le'me save the window positions first.....
    CECWorkspace* pWorkspace = CECCommanderApp::GetWorkspace();
    ASSERT (pWorkspace != NULL);

    if (IsZoomed() )
    {
        //
        // OK the window is maximised, therefore we only need to save the
        // zoomed state of the window
        pWorkspace->SetWindowZoomState(TRUE);
    } // if the window is maximised
    else if ( !IsIconic() )
    {
        //
        // The window is neither maximised/minimised, therefore we DO SAVE the window size/pos
        CRect r;
        this->GetWindowRect(&r);
        pWorkspace->SetWindowZoomState(FALSE);
        pWorkspace->SetMainWindowRect(r);

        // If we are currently in full screen mode then we need to update
        // the application window - otherwise it will restore to a maximised
        // position.
        if (pWorkspace->IsFullScreen())
        {
            pWorkspace->SetMainWindowRect(m_rcRestoreRect);
        }

    } // ...else if the window is not minimised - ie the window is neither minimised/maximised

    //
    //    ...and save the current project and update the ini file with the settings on exit.
    //
    {
        // If we are currently in full screen mode then we need to update
        // the application window - otherwise it will restore to a maximised
        // position.
        if (pWorkspace->IsFullScreen())
        {
            pWorkspace->SetMainWindowRect(m_rcRestoreRect);
        }

        // And finally the current Project Name
        CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
        ASSERT (pApp != NULL);

        pWorkspace->SetCurrentProject(pApp->GetServer()->GetOpenProjectName());

        pWorkspace->SaveSettings();
    } // Local unconditional block - to automate the Cursor/Status bar statu

    EndWaitCursor();

    return;
}

void CECMainFrame::OnProjectWorkspaceRestore()
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    // For this we need to perform a recursive delete as NT insists on this and close
    // commander to save the changes
    CString szMessage;
    AfxFormatString1 (szMessage, IDS_RESTORE_WORKSPACE, pApp->GetCoreProductTitle());

    // See what the user wants to do
    UINT nAction = AfxMessageBox(szMessage, MB_YESNOCANCEL);

    if (nAction == IDYES)
    {
        // We have the go ahead so reset the workspace
        ResetProjectWorkspace();

        // And close TestCommander
        m_bAutoSaveWorkspace = false;
        SendMessage(WM_CLOSE);
    }
    else if (nAction == IDNO)
    {
        // Simply reset the workspace - won't have any effect on the windows positions
        // as we will always do a save workspace at the end of this session, but it will effect
        // all test run dialogs etc.
        ResetProjectWorkspace();
    }
}

//
// ResetProjectWorkspace
// =====================
//
//  This routine performs the recursive registry delete of the user key
//
void CECMainFrame::ResetProjectWorkspace()
{
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    BeginWaitCursor();

    // Construct the appropriate register key for the current product
    CString szMainKey = "SOFTWARE\\" + pApp->ConstructUserKey() + "\\Commander";

    // And blitz that key ...
    CECRegistryTools Registry(HKEY_CURRENT_USER);
    Registry.DeleteKey(szMainKey);

    // Set this so that workspace is not written on shutdown
    m_bAutoSaveWorkspace=false;

    EndWaitCursor();
}

void CECMainFrame::OnDestroy()
{
    //If the workspace was restored or save previously don't save it again
    if(m_bAutoSaveWorkspace)
    {
        // And save the Current Workspace for next time in
        OnProjectWorkspaceSave();
    }
    CECGlobalCmdTarget::OnDestroy();
}

void CECMainFrame::OnViewFloorviewer()
{
    ShowControlBar (m_wndFloor,
                    !(m_wndFloor->GetStyle () & WS_VISIBLE),
                    FALSE);
    RecalcLayout ();

}

void CECMainFrame::OnUpdateViewFloorviewer(CCmdUI* pCmdUI)
{
    pCmdUI->SetCheck (m_wndFloor->GetStyle () & WS_VISIBLE);
}

void CECMainFrame::OnViewOutputwindow()
{
    ShowControlBar (&m_wndOutputView,
                    !(m_wndOutputView.GetStyle () & WS_VISIBLE),
                    FALSE);
    RecalcLayout ();
}

void CECMainFrame::OnUpdateViewOutputwindow(CCmdUI* pCmdUI)
{
    pCmdUI->SetCheck (m_wndOutputView.GetStyle () & WS_VISIBLE);
}

void CECMainFrame::OnVisitWebSite()
{
    ShellExecute (NULL ,NULL ,"www.cyrano.com" ,NULL ,NULL ,0);
}

void CECMainFrame::OnHelpContents()
{
    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();
    CString helpPath = pApp->GetInstallationDirectory();
    helpPath += "\\help\\Commander.chm";

    HtmlHelp(NULL, helpPath, HH_DISPLAY_TOC, 0);
}

void CECMainFrame::OnHelpIndex()
{
    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();
    CString helpPath = pApp->GetInstallationDirectory();
    helpPath += "\\help\\Commander.chm";

    HtmlHelp(NULL, helpPath, HH_DISPLAY_INDEX, 0);
}

void CECMainFrame::OnHelpSearch()
{
    CECCommanderApp* pApp = (CECCommanderApp*)AfxGetApp();
    CString helpPath = pApp->GetInstallationDirectory();
    helpPath += "\\help\\Commander.chm";

    HH_FTS_QUERY q ;
    q.cbStruct         = sizeof(HH_FTS_QUERY) ;
    q.fUniCodeStrings  = FALSE ;
    q.pszSearchQuery   = "";
    q.iProximity       = HH_FTS_DEFAULT_PROXIMITY ;
    q.fStemmedSearch   = FALSE ;
    q.fTitleOnly       = FALSE ;
    q.fExecute         = FALSE ;
    q.pszWindow        = NULL ;
    HtmlHelp(NULL, helpPath, HH_DISPLAY_SEARCH, (DWORD)&q);
}

BOOL CECMainFrame::LoadFrame(UINT nIDResource, DWORD dwDefaultStyle, CWnd* pParentWnd, CCreateContext* pContext)
{
    if (!CECGlobalCmdTarget::LoadFrame(nIDResource, dwDefaultStyle, pParentWnd, pContext))
    {
        return FALSE;
    }

    CECUserToolManager::LoadUserTools();

    return TRUE;
}

void CECMainFrame::OnHelpOpenSTAProductWebsite()
{
    ShellExecute (NULL ,NULL ,"www.opensta.org" ,NULL ,NULL ,0);
}

void CECMainFrame::OnWindowManage()
{
    CBCGWindowsManagerDlg dlg(this);
    dlg.DoModal();
}
