/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECOTAThresholds.cpp: implementation of the CECOTAThresholds class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CECOTAThresholds.h"

#include "CECRegistryTools.h"
#include "..\..\include\solutions.h"
#include "..\interfaces\CYRPlugins.h"
#include "..\..\CYRFileManagement\server.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
const int nDefaultNetUseThreshold = 10000;
const int nDefaultCPUThreshold =  90;
const int nDefaultPageSwapThreshold = 2000;

CECOTAThresholds::CECOTAThresholds(const CString& strPhysicalNode)
{
    m_strPhysicalNode = strPhysicalNode;
    if(!m_strPhysicalNode)
        return;

    m_nCPUThreshold = nDefaultCPUThreshold;
    m_nNetThreshold = nDefaultNetUseThreshold;
    m_nSwapThreshold = nDefaultPageSwapThreshold;

    //Get Server pointer from App
    CYRServer *pServer = (CYRServer *)AfxGetMainWnd()->SendMessage (WMU_GET_SERVERPTR);
    CString strWorkDir = pServer->GetFolderPath(CYRServer::enRootFolder);
    m_strThresholdFile = strWorkDir + "ThresholdSettings";
    LoadThresholds();
}

CECOTAThresholds::~CECOTAThresholds()
{
    if(!m_strPhysicalNode)
        return;

    SaveThresholds();

    CYRServer *pServer = (CYRServer *)AfxGetMainWnd()->SendMessage (WMU_GET_SERVERPTR);
    pServer->CheckInFile(CYRServer::enRootFolder,"","ThresholdSettings","OTA Threshold Settings");
}

void CECOTAThresholds::LoadThresholds()
{
    m_nCPUThreshold = GetPrivateProfileInt(m_strPhysicalNode,"CPU Threshold",m_nCPUThreshold,m_strThresholdFile);
    m_nNetThreshold = GetPrivateProfileInt(m_strPhysicalNode,"NET Threshold",m_nNetThreshold,m_strThresholdFile);
    m_nSwapThreshold = GetPrivateProfileInt(m_strPhysicalNode,"Swap Threshold",m_nSwapThreshold,m_strThresholdFile);
}

void CECOTAThresholds::SaveThresholds()
{
    CString str;
    str.Format("%d",m_nCPUThreshold);
    WritePrivateProfileString(m_strPhysicalNode,"CPU Threshold",str,m_strThresholdFile);

    str.Format("%d",m_nNetThreshold);
    WritePrivateProfileString(m_strPhysicalNode,"NET Threshold",str,m_strThresholdFile);

    str.Format("%d",m_nSwapThreshold);
    WritePrivateProfileString(m_strPhysicalNode,"Swap Threshold",str,m_strThresholdFile);
}
