/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#include "stdafx.h"
#include "CYRTestProperties.h"
#include "solutions.h"
#include "..\interfaces\cyrplugins.h"
#include "cecRegistrytools.h"
#include "Server.h"

CYRTests::CYRTests(CYRServer *pServer)
{
    m_pServer = pServer;
}

void CYRTests::GetSimpleTests(CStringArray& arstrTests)
{
    CStringArray arstrAvailableTests;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,"",arstrAvailableTests);

    for(int nNdx=0;nNdx<arstrAvailableTests.GetSize();nNdx++)
    {
        CString& strTestName = arstrAvailableTests[nNdx];
        CString strWizFile = strTestName + ".WIZ";
        strWizFile.MakeUpper();

        //      m_pServer->CheckOutFile(CYRServer::enTestsFolder,strTestName,strWizFile);

        if(IsSimpleTest(strTestName))
        {
            arstrTests.Add(strTestName);
        }
    }
}

void CYRTests::GetGroupTests(CStringArray& arstrTests)
{
    CStringArray arstrAvailableTests;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,"",arstrAvailableTests);

    for(int nNdx=0;nNdx<arstrAvailableTests.GetSize();nNdx++)
    {
        CString& strTestName = arstrAvailableTests[nNdx];
        CString strWizFile = strTestName + ".WIZ";
        strWizFile.MakeUpper();

        //      m_pServer->CheckOutFile(CYRServer::enTestsFolder,strTestName,strWizFile);

        if(IsGroupTest(strTestName))
        {
            arstrTests.Add(strTestName);
        }
    }
}

BOOL CYRTests::IsSimpleTest(const CString& strTestName)
{
    CString strTestsPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
    CString strWizFile(strTestsPath + strTestName +"\\" + strTestName + ".WIZ");

    DWORD dwMaxBufferSize=50;
    CString strType;
    GetPrivateProfileString("Wizard Test","Type","",strType.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizFile);
    strType.ReleaseBuffer();
    return (strType == "Simple Test");
}

BOOL CYRTests::IsGroupTest(const CString& strTestName)
{
    CString strTestsPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
    CString strWizFile(strTestsPath + strTestName +"\\" + strTestName + ".WIZ");

    DWORD dwMaxBufferSize=50;
    CString strType;
    GetPrivateProfileString("Wizard Test","Type","",strType.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizFile);
    strType.ReleaseBuffer();
    return (strType == "Group Test");
}

int CYRTests::GetNumVUsForSimpleTest(const CString& strTestName)
{
#ifdef _DEBUG
    ASSERT( IsSimpleTest(strTestName));
#endif

    CString strTestsPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
    CString strWizFile(strTestsPath + strTestName +"\\" + strTestName + ".WIZ");

    DWORD dwMaxBufferSize=50;
    CString strNum;
    GetPrivateProfileString("Wizard Test","Num V Users","1",strNum.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizFile);
    strNum.ReleaseBuffer();

    return atoi(strNum);
}

BOOL CYRTests::DoesTestExist(const CString& strTestName)
{
    CString strTestsPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
    CString strWizFile(strTestsPath + strTestName +"\\" + strTestName + ".WIZ");

    DWORD dwMaxBufferSize=50;
    CString strType;
    GetPrivateProfileString("Wizard Test","Type","",strType.GetBuffer(dwMaxBufferSize),dwMaxBufferSize,strWizFile);
    strType.ReleaseBuffer();
    return !strType.IsEmpty();
}
