/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECExecuteTestDlg.cpp : implementation file
//

#include "stdafx.h"
#include "CECExecuteTestDlg.h"
#include "..\..\Common\CECXServerError.h"
#include "..\..\Common\CECRegistryTools.h"
#include "Solutions.h"

#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif


#include "..\..\CECCommander\CECCommander.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "..\..\Interfaces\CyrPlugins.h"

#define WMU_TIMEOUT_TIMER                   (WM_USER+9800)

/////////////////////////////////////////////////////////////////////////////
UINT CECExecuteTestDlg::execute(LPVOID pParam)
{
    // Firstly get hold of the main app - for the product name
    CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
    ASSERT (pApp != NULL);

    CECExecuteTestDlg *pExecTest = (CECExecuteTestDlg*)pParam;

    TestInitiator* tinit=NULL;
    try
    {
        int createStatus;

        //parameters
        //Project        Name
        //Test Name
        //Node on which to execute

        if (pExecTest->m_szNodeFile == "<Local>")
        {
            CECInetInfo inet;

            CECNodeDefinition def("localhost");

            tinit=new TestInitiator(pExecTest->m_szProjectName.GetBuffer(0),
                                    pExecTest->m_szTestName.GetBuffer(0),
                                    def);
        }
        else
        {
            CString physNode, user, pass;
            pExecTest->GetNodeInfo(physNode, user, pass);
            CECNodeDefinition def(physNode.GetBuffer(0));

            tinit=new TestInitiator(pExecTest->m_szProjectName.GetBuffer(0),
                                    pExecTest->m_szTestName.GetBuffer(0),
                                    def);
        }
        //parameters
        //Node on which data server resides
        //username,password
        tinit->SetRepositoryInformation(pExecTest->m_szHost.GetBuffer(0),
                                        pExecTest->m_szUsername.GetBuffer(0),
                                        pExecTest->m_szPassword.GetBuffer(0));

        createStatus = tinit->CreateTestManager();

        createStatus = tinit->ExecuteTest();

        pExecTest->m_bExecuted = TRUE;
        pExecTest->SetExecuted(TRUE);
        return 0;
    }
    catch (CyrException e)
    {
        // Corba exception ...
        CString szErrorMsg;

        // As these messages can be java exceptions we do not really want to display these to
        // the user. Instead we should process the error string returning from CyrException
        // and present a "nice" user friendly message to the user instead
        processException(e.what().c_str(), szErrorMsg);

        // Output all Corba details we have ...
        CECXServerError error;
        error.DisplayErrorMessage(pApp->GetCoreProductTitle(), szErrorMsg);

    }
    catch(...)
    {
        // Corba exception ...

        // Unknown error - but it is probably the lack of omninames which caused it so ...
        CString szErrorMsg;

        szErrorMsg = "Failed to execute test\x0D\x0A\x0D\x0A";
        szErrorMsg += "There may be a problem with the configuration of Omninames.\x0D\x0A\x0D\x0A";
        szErrorMsg += "Please ensure that OmniNames is only running on the server machine ";
        szErrorMsg += "and that the port number in your daemon configuration is correct.\x0D\x0A\x0D\x0A";
        szErrorMsg += "Please call CYRANO Support for further assistance.";

        // Output all Corba details we have ...
        CECXServerError error;
        error.DisplayErrorMessage(pApp->GetCoreProductTitle(), szErrorMsg);
    }

    delete tinit;
    return -1;
}

//
// processTestManagerException
// ===========================
//
// This function accepts a Test Manager exception and converts it into something
// suitable to display to the end user.
void CECExecuteTestDlg::processException(const CString& rwError, CString& szDisplayText)
{
    CString szErrorString = rwError;

    // First we try for no Omninames ...
    if (szErrorString.Find("NameService") != -1)
    {
        szDisplayText = "Failed to Execute Test.\x0D\x0A\x0D\x0A";
        szDisplayText += "Please verify that OmniNames is only running on the ";
        szDisplayText += "same machine as the CYRANO Repository and that the ";
        szDisplayText += "correct port number is specified in the configuration ";
        szDisplayText += "panel on the Login Screen. (You may want to turn off the ";
        szDisplayText += "Auto Login feature to confirm this).\x0D\x0A\x0D\x0A";
    }

    // Now for the Daemon
    else if (szErrorString.Find("resolve object") != -1)
    {
        szDisplayText = "The CYRANO Daemon is not running.\x0D\x0A\x0D\x0A";
        szDisplayText += "The CYRANO Daemon is not running on the node on which ";
        szDisplayText += "you wish to execute a test - or part of it.\x0D\x0A\x0D\x0A";
        szDisplayText += "Please ensure that the CYRANO Daemon is running on ";
        szDisplayText += "all nodes used for this test. If you have just restarted the ";
        szDisplayText += "Daemon on this machine then you will need to restart TestCommander ";
        szDisplayText += "first. ";
    }

    // Now for missing exes
    else if (szErrorString.Find("AttachToObject") != -1)
    {
        // Firstly get hold of the main app - for the product name
        CECCommanderApp* pApp = (CECCommanderApp*) AfxGetApp();
        ASSERT (pApp != NULL);

        szDisplayText = "Failed to locate the Test Manager.\x0D\x0A\x0D\x0A";
        szDisplayText += "Please check that the CYRANO_HOME environmental variable ";
        szDisplayText += "is set up on all nodes being used in this test and that ";
        szDisplayText += "a compact Installation of " + pApp->GetCoreProductTitle() + " has been performed on ";
        szDisplayText += "this node.";
    }

    // Now for the rest ...
    else
    {
        szDisplayText = "Login Failure\x0D\x0A\x0D\x0A";
        szDisplayText += szErrorString;
    }

}

/////////////////////////////////////////////////////////////////////////////
// CECExecuteTestDlg dialog


CECExecuteTestDlg::CECExecuteTestDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECExecuteTestDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECExecuteTestDlg)
    //}}AFX_DATA_INIT
    m_bExecuted = FALSE;
    m_bExecuting = FALSE;

    CWnd* pMainWnd = AfxGetMainWnd();
    m_pServer = (CYRServer*) pMainWnd->SendMessage (WMU_GET_SERVERPTR);
}


void CECExecuteTestDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECExecuteTestDlg)
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECExecuteTestDlg, CDialog)
//{{AFX_MSG_MAP(CECExecuteTestDlg)
ON_WM_HELPINFO()
ON_WM_TIMER()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECExecuteTestDlg message handlers

BOOL CECExecuteTestDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    //Initialise html help handler

    // Get a handle to the Mainframe
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    // Now get the server login information from the framework
    m_szHost = m_pServer->GetHostName();
    m_szUsername = m_pServer->GetUserName();
    m_szPassword = m_pServer->GetPassword();

    GetListOfTests();
    GetListOfNodes();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECExecuteTestDlg::SetExecuteInfo(CString ProjectName, CString TestName)
{
    m_szProjectName = ProjectName;
    m_szTestName = TestName;
}

BOOL CECExecuteTestDlg::IsExecuted()
{
    return m_bExecuted;
}

void CECExecuteTestDlg::OnOK()
{
    CButton* pBOK = (CButton*)GetDlgItem(IDOK);
    pBOK->EnableWindow(FALSE);

    CComboBox* pTestCombo = (CComboBox*)GetDlgItem(IDC_TEST_NAME);
    int index = pTestCombo->GetCurSel();
    pTestCombo->GetLBText(index, m_szTestName);

    CComboBox* pNodeCombo = (CComboBox*)GetDlgItem(IDC_NODENAME);
    index = pNodeCombo->GetCurSel();
    pNodeCombo->GetLBText(index, m_szNodeFile);

    // Save the name of this test to the regisrty
    CString KeyName = SOLUTIONS_FRAMEWORK_KEY;
    CString ValueName = "LastExec";
    CString Value = m_szProjectName+"|"+m_szTestName;

    CECRegistryTools reg;
    reg.SetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName, Value.GetBuffer(0));


    BeginWaitCursor();

    m_pThread = AfxBeginThread(execute,this);
    m_bExecuting = TRUE;

    SetTimer(WMU_TIMEOUT_TIMER, 30000, NULL);

}

void CECExecuteTestDlg::GetListOfTests()
{
    CComboBox* pTestCombo = (CComboBox*)GetDlgItem(IDC_TEST_NAME);

    CStringArray arstrTests;
    m_pServer->GetFolderList(CYRServer::enTestsFolder,"",arstrTests);
    //
    //    ...and add them to the tree beneath the project.
    //
    for(int nNdx=0;nNdx<arstrTests.GetSize();nNdx++)
    {
        pTestCombo->AddString(arstrTests[nNdx]);
    }

    int index = pTestCombo->FindStringExact(0,m_szTestName);
    if (index != CB_ERR)
    {
        pTestCombo->SetCurSel(index);
    }
    else
    {
        pTestCombo->SetCurSel(0);
    }
}

void CECExecuteTestDlg::GetListOfNodes()
{
    //
    // Add the available node names to the combo box
    //
    CComboBox* pNodesCombo = (CComboBox*) GetDlgItem (IDC_NODENAME);
    ASSERT (pNodesCombo != NULL);

    pNodesCombo->AddString ("<Local>");

    //
    // Look add the nodes defined in the node folder for the active project
    //
    //    Recover a list of the nodes in that project...
    CStringArray arstrNodes;
    m_pServer->GetFileList(CYRServer::enNodesFolder,"",arstrNodes);

    //    ...and add them to the combo-box.
    for(int nNdx=0;nNdx<arstrNodes.GetSize();nNdx++)
    {
        pNodesCombo->AddString (arstrNodes[nNdx]);
    }
    pNodesCombo->SetCurSel(0);
}

BOOL CECExecuteTestDlg::HasSpawnProc()
{
    return TRUE;
}

BOOL CECExecuteTestDlg::GetNodeInfo(CString& physNode, CString& userName, CString& password)
{
    // Get the work directory
    CString nodePath = m_pServer->GetFolderPath(CYRServer::enNodesFolder) + m_szNodeFile;

    try
    {
        CStdioFile f(nodePath, CFile::modeRead);
        f.ReadString(physNode);
    }
    catch (CFileException e)
    {}

    return TRUE;
}


BOOL CECExecuteTestDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    //return false so html help tool handles message
    return false;
}

void CECExecuteTestDlg::OnCancel()
{
    if (m_bExecuting)
    {
        DWORD dwExitCode=0;
        m_bExecuted=FALSE;
        TerminateThread(m_pThread->m_hThread,dwExitCode);
    }

    CDialog::OnCancel();
}

void CECExecuteTestDlg::SetExecuted(BOOL bExecuted)
{
    KillTimer(WMU_TIMEOUT_TIMER);

    if (bExecuted)
    {
        EndDialog(IDOK);
    }
    else
    {
        EndDialog(IDCANCEL);
    }

    EndWaitCursor();
}

void CECExecuteTestDlg::OnTimer(UINT nIDEvent)
{
    SetExecuted(FALSE);

    CDialog::OnTimer(nIDEvent);
}
