/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// @doc     Document
// @module  CECFilterDlg | Allows the user to filter the test run data
//
////////////////////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "resource.h"
#include "CECGlobalSettings.h"
#include "CECFilterDlg.h"
#include "htmlhelp.h"
#include "CECVirtGridView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "..\..\..\HTMLHelp\CYRAnalyzer\Stat_Filter.inl"

extern CECGlobalSettings* g_pGlobalSettings;
/////////////////////////////////////////////////////////////////////////////
// CECFilterDlg dialog

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Constructor
CECFilterDlg::CECFilterDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECFilterDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECFilterDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_pGrid = NULL;
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}


void CECFilterDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECFilterDlg)
    // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECFilterDlg, CDialog)
//{{AFX_MSG_MAP(CECFilterDlg)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECFilterDlg message handlers

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Initialize the dialog, filling the list boxes.
// @rdesc   Return Value Desc
BOOL CECFilterDlg::OnInitDialog()
{
    CDialog::OnInitDialog();


    //
    // Make sure the grid pointer has been initilaized and the data area has been
    // selected.
    //
    if (m_pGrid == NULL || !m_pGrid->GetNumTimers(m_strTestRun))
    {
        return FALSE;
    }

    //
    // Scan the data set loading the info into the list boxes
    //
    ListBoxLoader();
    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Provides access to the hidden grid for data processing
// @parm    CECVirtGridView* | pGrid | Grid pointer

void CECFilterDlg::SetGridPtr(CECVirtGridView *pGrid)
{
    ASSERT (pGrid != NULL);
    m_pGrid = pGrid;
}

void CECFilterDlg::SetDataRun(const CString& strTestRun)
{
    m_strTestRun = strTestRun;
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Applies the filters to the selections.
void CECFilterDlg::OnOK()
{
    BeginWaitCursor();
    //
    // Apply the filters
    //
    ListBoxFilter();

    ArrangeTimers();

    EndWaitCursor();

    CDialog::OnOK();
}


/////////////////////////////////////////////////////////////////////////////
// @mfunc   Applies a filter to a specified grid column using the items
//          selected in the given list control.
// @parm    CListBox* | pLB | List box containing the selections
// @parm    int | FilterCol | Column in the grid to filter
void CECFilterDlg::ListBoxFilter()
{
    const int nNumNameFields = 4;
    CListBox* pNameList[nNumNameFields] = {
        (CListBox*)GetDlgItem(IDC_TEST_LIST),
        (CListBox*)GetDlgItem(IDC_THREAD_LIST),
        (CListBox*)GetDlgItem(IDC_SCRIPT_LIST),
        (CListBox*)GetDlgItem(IDC_TIMER_LIST)};

    CMapStringToString mapItems[nNumNameFields];

    CString strItem;
    bool bAllFieldsSelected=true;
    for(int nField=0;nField<nNumNameFields;nField++)
    {
        int nNumSelItems = pNameList[nField]->GetSelCount();
        int nTotalItems = pNameList[nField]->GetCount();

        if(nNumSelItems != nTotalItems)
            bAllFieldsSelected = false;

        int *pSelItems = new int[nNumSelItems];
        pNameList[nField]->GetSelItems(nNumSelItems,pSelItems);

        for(int nNdx=0;nNdx<nNumSelItems;nNdx++)
        {
            pNameList[nField]->GetText(pSelItems[nNdx],strItem);
            mapItems[nField].SetAt(strItem,"");
        }

        delete []pSelItems;
    }

    TimerData *pTimerData = m_pGrid->GetTimerData(m_strTestRun);

    //All items are selected so don't bother checking
    if(bAllFieldsSelected)
    {
        POSITION pos = pTimerData->listTimers.GetHeadPosition();
        while(pos)
        {
            pTimerData->listTimers.GetNext(pos)->bInFilter = true;
        }
        return;
    }

    // Loop through the data set...
    CString strName,strTemp;
    POSITION pos = pTimerData->listTimers.GetHeadPosition();
    while(pos)
    {
        Timer *pTimer = pTimerData->listTimers.GetNext(pos);

        // ... retrieve the item name and check to see if it exists in the array
        if(!mapItems[0].Lookup(pTimer->strTestName,strTemp) ||
                !mapItems[1].Lookup(pTimer->strThreadName,strTemp) ||
                !mapItems[2].Lookup(pTimer->strScriptName,strTemp) ||
                !mapItems[3].Lookup(pTimer->strTimerName,strTemp) )
        {
            //Found an unselected item so remove it
            pTimer->bInFilter = false;
        }
        else
            pTimer->bInFilter = true;
    }
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Loads all unique item in a grid column into a list box control
// @parm    CListBox* | pLB | List Box to be filled
// @parm    int | ColIndex | Column index
void CECFilterDlg::ListBoxLoader()
{
    CListBox* pTestList = (CListBox*)GetDlgItem(IDC_TEST_LIST);
    CListBox* pThreadList = (CListBox*)GetDlgItem(IDC_THREAD_LIST);
    CListBox* pScriptList = (CListBox*)GetDlgItem(IDC_SCRIPT_LIST);
    CListBox* pTimerList = (CListBox*)GetDlgItem(IDC_TIMER_LIST);

    //
    // Loop through the data set...
    //
    CString itemName;
    TimerData *pTimerData = m_pGrid->GetTimerData(m_strTestRun);

    int nNumTimers = pTimerData->listTimers.GetCount();
    pThreadList->InitStorage(nNumTimers/2,30);

    CMap <CString,LPCTSTR,bool,bool> m_mapTests;
    CMap <CString,LPCTSTR,bool,bool> m_mapThreads;
    CMap <CString,LPCTSTR,bool,bool> m_mapScripts;
    CMap <CString,LPCTSTR,bool,bool> m_mapTimers;

    POSITION pos =  pTimerData->listTimers.GetHeadPosition();
    while(pos)
    {
        Timer *pTimer = pTimerData->listTimers.GetNext(pos);

        // ... and retrieve the data item in the specified column
        m_mapTests.SetAt(pTimer->strTestName,false);
        m_mapThreads.SetAt(pTimer->strThreadName,false);
        m_mapScripts.SetAt(pTimer->strScriptName,false);
        m_mapTimers.SetAt(pTimer->strTimerName,false);
    }

    bool bDummy;
    CString str;
    pos = m_mapTests.GetStartPosition();
    pTestList->InitStorage(m_mapTests.GetCount(),30);
    while(pos)
    {
        m_mapTests.GetNextAssoc(pos,str,bDummy);
        pTestList->AddString(str);
    }

    pos = m_mapThreads.GetStartPosition();
    pThreadList->InitStorage(m_mapThreads.GetCount(),30);
    while(pos)
    {
        m_mapThreads.GetNextAssoc(pos,str,bDummy);
        pThreadList->AddString(str);
    }

    pos = m_mapScripts.GetStartPosition();
    pScriptList->InitStorage(m_mapScripts.GetCount(),30);
    while(pos)
    {
        m_mapScripts.GetNextAssoc(pos,str,bDummy);
        pScriptList->AddString(str);
    }

    pos = m_mapTimers.GetStartPosition();
    pTimerList->InitStorage(m_mapTimers.GetCount(),30);
    while(pos)
    {
        m_mapTimers.GetNextAssoc(pos,str,bDummy);
        pTimerList->AddString(str);
    }

    //
    // After all of the item have been added, pre-select them all.
    //
    int nCount=0;
    if((nCount = pTestList->GetCount()) > 1)
        pTestList->SelItemRange(TRUE,0,nCount-1);
    else
        pTestList->SetSel(0);

    if((nCount = pThreadList->GetCount()) > 1)
        pThreadList->SelItemRange(TRUE,0,nCount-1);
    else
        pThreadList->SetSel(0);

    if((nCount = pScriptList->GetCount()) > 1)
        pScriptList->SelItemRange(TRUE,0,nCount-1);
    else
        pScriptList->SetSel(0);

    if((nCount = pTimerList->GetCount()) > 1)
        pTimerList->SelItemRange(TRUE,0,nCount-1);
    else
        pTimerList->SetSel(0);
}

void CECFilterDlg::ArrangeTimers()
{
    /*
        CListBox* pTimerList = (CListBox*)GetDlgItem(IDC_TIMER_LIST);
        int nTimers = pTimerList->GetCount();
        for (int TimerCount = 0; TimerCount<nTimers; TimerCount++)
        {
            CString timerName;
            pTimerList->GetText(TimerCount, timerName);
     
            int nAdded = 0;
            for (int isub = 0; isub<m_nRows; isub++)
            {
                CString expression;
                expression.Format("=IF(H%d==\"%s\",B%d,\"\")",m_nStartRow+isub,timerName, m_nStartRow+isub);
                m_pGrid->SetExpressionRowCol(m_nStartRow+nAdded,RT_START_COL+(TimerCount*2),expression);
     
                CString expression2;
                expression2.Format("=IF(H%d==\"%s\",C%d,\"\")",m_nStartRow+isub,timerName, m_nStartRow+isub);
                m_pGrid->SetExpressionRowCol(m_nStartRow+nAdded,RT_START_COL+(TimerCount*2+1),expression2);
     
                CString retVal = m_pGrid->GetValueRowCol(m_nStartRow+nAdded,RT_START_COL+(TimerCount*2));
                if (!retVal.IsEmpty())
                {
                    nAdded++;
                }
            }
        }
    */
}

BOOL CECFilterDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
