/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

////////////////////////////////////////////////////////////////////////////////
// @doc     Document
// @module  CECRunList | Floating window showing test runs loaded into the hidden
//          grid. Graph displays will show results for the selected runs in this
//          window.
//
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "CECRunList.h"
#include "CYREventSink.h"
#include "CECWorkspace.h"
#include "CECGlobalSettings.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CECGlobalSettings* g_pGlobalSettings;
extern CYREventSink *pEventSink;

/////////////////////////////////////////////////////////////////////////////
// CECRunList dialog

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Constructor
CECRunList::CECRunList(CWnd* pParent /*=NULL*/)
        : CDialog(CECRunList::IDD, pParent),m_nSplitterWidth(4)
{
    //{{AFX_DATA_INIT(CECRunList)
    //}}AFX_DATA_INIT


    m_nSplitPos=-1;
    m_bDraggingSplitter=false;

    //Add all graphs
    m_mapGraphs.SetAt("Cumulative Frequency",enCumFreqGraph);
    m_mapGraphs.SetAt("Percentage Frequency",enPercentFreqGraph);
    m_mapGraphs.SetAt("Standard Frequency",enStdFreqGraph);
    m_mapGraphs.SetAt("Response Time / Active Users",enRTATGraph);
    m_mapGraphs.SetAt("Response Time / Elapsed Time",enRTTimeGraph);
    //  m_mapGraphs.SetAt("Active Users / Elapsed Time",enUsersTimeGraph);
    m_mapGraphs.SetAt("Percent CPU Usage",enPercentCPUGraph);
    m_mapGraphs.SetAt("Page Swapping",enPageSwapGraph);
    m_mapGraphs.SetAt("Percent Network Usage",enPercentNetUseGraph);
    m_mapGraphs.SetAt("Total Bytes / Second",enKbPerSecondGraph);
    m_mapGraphs.SetAt("Average Hit Response Time / Number Of Hits",enAVPRTPerUserGraph);
    m_mapGraphs.SetAt("Errors / Active Connection",enErrorsPerUserGraph);
    m_mapGraphs.SetAt("Errors / Elapsed Time",enErrorsTimeGraph);
    m_mapGraphs.SetAt("Total Hits / Second",enHitsPerSecondGraph);
    m_mapGraphs.SetAt("Average Hit Response Time / Second",enPagesPerSecondGraph);
    m_mapGraphs.SetAt("Custom Performance Monitor Graph",enSimpleOTAGraph);
}


void CECRunList::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECRunList)
    DDX_Control(pDX, IDC_GRAPHS, m_ctlGraphs);
    DDX_Control(pDX, IDC_RUNS, m_ctlTestRuns);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECRunList, CDialog)
//{{AFX_MSG_MAP(CECRunList)
ON_WM_DESTROY()
ON_WM_SIZE()
ON_WM_CLOSE()
ON_LBN_SELCHANGE(IDC_RUNS, OnSelchangeRuns)
ON_LBN_SELCHANGE(IDC_GRAPHS, OnSelchangeGraphs)
ON_WM_LBUTTONDOWN()
ON_WM_LBUTTONUP()
ON_WM_MOUSEMOVE()
ON_WM_CAPTURECHANGED()
ON_LBN_DBLCLK(IDC_GRAPHS, OnDblclkGraphs)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECRunList message handlers

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Called when a test run has been loaded into the grid, to display
//          that this data set is available for graphing.
// @parm    CString | strRunName | The run name (date) to be displayed
void CECRunList::AddRunName(CString strRunName)
{
    int nPos = strRunName.ReverseFind('.');
    strRunName = strRunName.Left(nPos);

    int newRun = m_ctlTestRuns.AddString(strRunName);

    //
    // Select the item in the list.
    //
    m_ctlTestRuns.SetSel(newRun,TRUE);

    OnSelchangeRuns();
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Delete all items from the Display
void CECRunList::Clear()
{
    m_ctlTestRuns.ResetContent();
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Over-ride the on destroy so that the display is just hidden instead
//          of being destroyed
void CECRunList::OnDestroy()
{
    //
    // Save the window size/pos
    //
    CECWorkspace*   pWorkspace      = g_pGlobalSettings->GetWorkSpace();
    CRect r;
    GetWindowRect(&r);
    //  pMainFrame      ->ScreenToClient( &r );
    //  pWorkspace      ->SaveWindowPos(pWorkspace->m_RunList, r);
    pWorkspace->m_RunList.Format("%d,%d,%d,%d,%d",r.left,r.top,r.right,r.bottom,m_nSplitPos);
    pWorkspace      ->SaveSettings();

    CDialog::OnDestroy();
}

/////////////////////////////////////////////////////////////////////////////
// @mfunc   Helps to find which runs have been selected
// @rdesc   Returns a string containing the indices of the selected list items
CString CECRunList::GetConfigString()
{
    CString strConfig;

    //
    // Allocate buffer to hold indicies of selected items
    //
    int* pSelIndices = new int[10];
    ASSERT( pSelIndices != NULL );

    //
    // Get selected item indicies
    //
    int nSelected = m_ctlTestRuns.GetSelItems(10, pSelIndices);

    //
    // Add the selected indices to the config string separated by '|' charactesrs
    //
    for (int count = 0; count<nSelected; count++)
    {
        CString tempString;
        int sel = pSelIndices[count];
        tempString.Format("%d|", sel);
        strConfig = strConfig+tempString;
    }

    //
    // Strip off the trailing '|' character
    //
    int nLen = strConfig.GetLength();
    strConfig = strConfig.Left(nLen-1);

    delete []pSelIndices;

    return strConfig;
}

BOOL CECRunList::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Obtain saved window position
    CECWorkspace* pWorkspace = g_pGlobalSettings->GetWorkSpace();
    CRect r;
    //  pWorkspace->ParseWindowSizePos( pWorkspace->m_RunList, r);
    CString strCopy(pWorkspace->m_RunList);
    char *pszToken = strtok(strCopy.GetBuffer(0),",");
    if(pszToken)
        r.left = atoi(pszToken);

    pszToken = strtok(NULL,",");
    if(pszToken)
        r.top = atoi(pszToken);

    pszToken = strtok(NULL,",");
    if(pszToken)
        r.right = atoi(pszToken);

    pszToken = strtok(NULL,",");
    if(pszToken)
        r.bottom = atoi(pszToken);

    pszToken = strtok(NULL,",");
    if(pszToken)
        m_nSplitPos = atoi(pszToken);

    // Resize the view (no move)
    SetWindowPos( NULL, r.left, r.top, r.Width(), r.Height(), SWP_NOZORDER);

    m_ctlGraphs.SetCheckStyle(BS_AUTOCHECKBOX);
    UpdateGraphsList();

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECRunList::OnSize(UINT nType, int cx, int cy)
{
    CDialog::OnSize(nType, cx, cy);

    ResizeWindows();
}

void CECRunList::OnClose()
{
    //
    // Just Hide the Window instead of killing it.
    //
    pEventSink->ShowRunList(FALSE);
}

void CECRunList::OnSelchangeRuns()
{
    pEventSink->OnRunListSelChange();

    UpdateGraphsList();
}

void CECRunList::OnSelchangeGraphs()
{
    int nNumItems = m_ctlGraphs.GetCount();
    for(int nItem=0;nItem<nNumItems;nItem++)
    {
        int nCheck = m_ctlGraphs.GetCheck(nItem);

        CString strGraph;
        m_ctlGraphs.GetText(nItem,strGraph);

        GraphTypesEnum GraphType;
        if(m_mapGraphs.Lookup(strGraph,GraphType))
        {
            pEventSink->ShowGraph(GraphType,nCheck);
        }
    }
}

void CECRunList::OnDblclkGraphs()
{
    int nNumItems = m_ctlGraphs.GetCount();
    for(int nItem=0;nItem<nNumItems;nItem++)
    {
        int nCheck = m_ctlGraphs.GetCheck(nItem);

        CString strGraph;
        m_ctlGraphs.GetText(nItem,strGraph);

        GraphTypesEnum GraphType;
        if(m_mapGraphs.Lookup(strGraph,GraphType))
        {
            pEventSink->ShowGraph(GraphType,nCheck);
        }
    }
}

void CECRunList::UpdateGraphsList()
{
    m_ctlGraphs.SetRedraw(FALSE);

    //Clear the list
    m_ctlGraphs.ResetContent();

    //Refill the list with the enabled graph types
    CString strGraph;
    int nItem;
    GraphTypesEnum GraphType;
    POSITION pos = m_mapGraphs.GetStartPosition();
    while(pos != NULL)
    {
        m_mapGraphs.GetNextAssoc(pos,strGraph,GraphType);

        if(pEventSink->IsGraphEnabled(GraphType))
        {
            nItem = m_ctlGraphs.AddString(strGraph);

            //Set the check state
            if(pEventSink->IsGraphVisible(GraphType))
                m_ctlGraphs.SetCheck(nItem,1);
            else
                m_ctlGraphs.SetCheck(nItem,0);
        }
    }
    m_ctlGraphs.SetRedraw(TRUE);
}


void CECRunList::OnLButtonDown(UINT nFlags, CPoint point)
{
    if((point.y >= m_nSplitPos) && (point.y < m_nSplitPos+m_nSplitterWidth))
    {
        SetCapture();
        SetCursor(AfxGetApp()->LoadStandardCursor(IDC_SIZENS));
        m_bDraggingSplitter = true;
        DrawMovingSplitter();
    }

    CDialog::OnLButtonDown(nFlags, point);
}

void CECRunList::OnLButtonUp(UINT nFlags, CPoint point)
{
    if(m_bDraggingSplitter)
    {
        ReleaseCapture();
        m_bDraggingSplitter = false;
        DrawMovingSplitter();
        ResizeWindows();
    }

    CDialog::OnLButtonUp(nFlags, point);
}

void CECRunList::OnMouseMove(UINT nFlags, CPoint point)
{
    if((point.y >= m_nSplitPos) && (point.y < m_nSplitPos+m_nSplitterWidth))
    {
        SetCursor(AfxGetApp()->LoadStandardCursor(IDC_SIZENS));
    }

    if(m_bDraggingSplitter)
    {
        CRect rcClient;
        GetClientRect(&rcClient);
        if(point.y < 10)
            point.y = 10;

        if(point.y >= rcClient.Height()-10)
            point.y = rcClient.Height()-10;

        DrawMovingSplitter();
        m_nSplitPos += ( point.y-m_nSplitPos);
        DrawMovingSplitter();
    }

    CDialog::OnMouseMove(nFlags, point);
}

void CECRunList::ResizeWindows()
{
    if(m_ctlTestRuns.GetSafeHwnd() && m_ctlGraphs.GetSafeHwnd())
    {
        CRect rcClient;
        GetClientRect(&rcClient);

        if(m_nSplitPos == -1)
            m_nSplitPos = rcClient.Height()/2;

        if(m_nSplitPos-m_nSplitterWidth > rcClient.Height()-10)
            m_nSplitPos = rcClient.Height()-10;

        if(m_nSplitPos < 10)
            m_nSplitPos = 10;

        const int nSeparator =2;
        m_ctlGraphs.SetWindowPos(NULL,nSeparator,nSeparator,rcClient.Width()-2*nSeparator,m_nSplitPos-nSeparator,SWP_NOZORDER);

        m_ctlTestRuns.SetWindowPos(NULL,nSeparator,m_nSplitPos+m_nSplitterWidth,rcClient.Width()-2*nSeparator,rcClient.Height()-m_nSplitPos-m_nSplitterWidth-nSeparator,SWP_NOZORDER);
    }
}

void CECRunList::DrawMovingSplitter()
{
    CRect rcClient;
    GetClientRect(&rcClient);

    CRect rcSplitter(0,m_nSplitPos,rcClient.Width(),m_nSplitPos+m_nSplitterWidth);

    CClientDC dc(this);
    dc.InvertRect(rcSplitter);

}

void CECRunList::OnCaptureChanged(CWnd *pWnd)
{
    m_bDraggingSplitter=false;

    CDialog::OnCaptureChanged(pWnd);
}

