/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECSummaryView.cpp : implementation of the CECSummaryView class
//

#include "stdafx.h"
#include "resource.h"
#include "CECGlobalSettings.h"
#include "AnalyzerDoc.h"
#include "CECSummaryView.h"
#include "CYREventSink.h"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
#include "..\..\..\HTMLHelp\CYRAnalyzer\Stat_Summary.inl"

extern CYREventSink*        pEventSink;
extern CECGlobalSettings* g_pGlobalSettings;
/////////////////////////////////////////////////////////////////////////////
// CECSummaryView

IMPLEMENT_DYNCREATE(CECSummaryView, CFormView)

BEGIN_MESSAGE_MAP(CECSummaryView, CFormView)
//{{AFX_MSG_MAP(CECSummaryView)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
// Standard printing commands
ON_COMMAND(ID_FILE_PRINT, CFormView::OnFilePrint)
ON_COMMAND(ID_FILE_PRINT_DIRECT, CFormView::OnFilePrint)
ON_COMMAND(ID_FILE_PRINT_PREVIEW, CFormView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECSummaryView construction/destruction

CECSummaryView::CECSummaryView()
        : CFormView(CECSummaryView::IDD)
{
    //{{AFX_DATA_INIT(CECSummaryView)
    m_LowerQ = 0.0;
    m_Max = 0.0;
    m_Mean = 0.0;
    m_Median = 0.0;
    m_Min = 0.0;
    m_Range = 0.0;
    m_Sample = 0;
    m_StdDev = 0.0;
    m_UpperQ = 0.0;
    //}}AFX_DATA_INIT
    // TODO: add construction code here

    m_pGrid = NULL;
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}

CECSummaryView::~CECSummaryView()
{}


void CECSummaryView::DoDataExchange(CDataExchange* pDX)
{
    CFormView::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECSummaryView)
    DDX_Text(pDX, IDC_LOWERQ, m_LowerQ);
    DDX_Text(pDX, IDC_MAX, m_Max);
    DDX_Text(pDX, IDC_MEAN, m_Mean);
    DDX_Text(pDX, IDC_MEDIAN, m_Median);
    DDX_Text(pDX, IDC_MIN, m_Min);
    DDX_Text(pDX, IDC_RANGE, m_Range);
    DDX_Text(pDX, IDC_SAMPLE, m_Sample);
    DDX_Text(pDX, IDC_STDDEV, m_StdDev);
    DDX_Text(pDX, IDC_UPPERQ, m_UpperQ);
    //}}AFX_DATA_MAP
}


BOOL CECSummaryView::PreCreateWindow(CREATESTRUCT& cs)
{
    // TODO: Modify the Window class or styles here by modifying
    //  the CREATESTRUCT cs

    return CFormView::PreCreateWindow(cs);
}

void CECSummaryView::OnInitialUpdate()
{
    CFormView::OnInitialUpdate();


    ResizeParentToFit();

    GetParentFrame()->SetTitle("Statistical Summary");
}



/////////////////////////////////////////////////////////////////////////////
// CECSummaryView printing

BOOL CECSummaryView::OnPreparePrinting(CPrintInfo* pInfo)
{
    // default preparation
    return DoPreparePrinting(pInfo);
}

void CECSummaryView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
    // TODO: add extra initialization before printing
}


void CECSummaryView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
    // TODO: add cleanup after printing
}


void CECSummaryView::OnPrint(CDC* pDC, CPrintInfo* /*pInfo*/)
{
    // TODO: add customized printing code here
}

/////////////////////////////////////////////////////////////////////////////
// CECSummaryView diagnostics


#ifdef _DEBUG
void CECSummaryView::AssertValid() const
{
    CFormView::AssertValid();
}

void CECSummaryView::Dump(CDumpContext& dc) const
{
    CFormView::Dump(dc);
}

CAnalyzerDoc* CECSummaryView::GetDocument() // non-debug version is inline
{
    ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CAnalyzerDoc)));
    return (CAnalyzerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CECSummaryView message handlers


void CECSummaryView::Initialize(int RunIndex)
{
    m_pGrid = pEventSink->m_pVirtGrid;

    CString strTestRun;
    g_pGlobalSettings->GetRunInfo(RunIndex, strTestRun);

    TimerData *pTimerData = m_pGrid->GetTimerData(strTestRun);

    m_Sample = m_pGrid->GetNumTimers(strTestRun);

    m_StdDev = pTimerData->dStdDev;
    m_Mean = pTimerData->dMean;
    m_Max = pTimerData->dMax;
    m_Min = pTimerData->dMin;
    m_Median = pTimerData->dMedian;
    m_LowerQ = pTimerData->dLowerQuartile;
    m_UpperQ = pTimerData->dUpperQuartile;

    m_Range = m_Max - m_Min;

    UpdateData(FALSE);

}

BOOL CECSummaryView::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}
