/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECVirtGridView.cpp: implementation of the CECVirtGridView class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "CECGlobalSettings.h"
#include "CECVirtGridView.h"
#include "CECFilterDlg.h"
#include "..\..\interfaces\cyrPlugins.h"
#include "..\studio\Common\CECXServerError.h"
#include "CYREventSink.h"
#include "Solutions.h"
#include "Server.h"
#include "CYRWraplib\CECTestOutput.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

extern CECGlobalSettings* g_pGlobalSettings;
extern CYREventSink* pEventSink;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CECVirtGridView::CECVirtGridView()
{
    m_pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);
}

CECVirtGridView::~CECVirtGridView()
{
    POSITION pos = m_mapTestRunToData.GetStartPosition();
    TestRunData *pTestRunData;
    CString strTestRun;
    while(pos != NULL)
    {
        m_mapTestRunToData.GetNextAssoc(pos,strTestRun,pTestRunData);

        HttpDataArray *parHttpData = pTestRunData->parHttpData;

        for(int nStat=0;nStat<parHttpData->GetSize();nStat++)
            delete (*parHttpData)[nStat];
        delete parHttpData;

        OtaQuery *pQuery;
        OtaQueryArray *parOtaData = pTestRunData->parOtaData;
        if(parOtaData != NULL)
        {
            for(int nNdx=0;nNdx<parOtaData->GetSize();nNdx++)
            {
                pQuery = parOtaData->GetAt(nNdx);

                POSITION pos2 =pQuery->listQueryData.GetHeadPosition();
                while(pos2)
                    delete pQuery->listQueryData.GetNext(pos2);

                delete pQuery;
            }
            delete parOtaData;
        }

        TimerData *pTimerData = pTestRunData->pTimerData;
        if(pTimerData)
        {
            for(int nNdx=0;nNdx<pTimerData->arTimerIntervals.GetSize();nNdx++)
                delete pTimerData->arTimerIntervals[nNdx];

            POSITION pos2 = pTimerData->listTimers.GetHeadPosition();
            while(pos2)
                delete pTimerData->listTimers.GetNext(pos2);

            pos2 = pTimerData->listRtAtData.GetHeadPosition();
            while(pos2)
                delete pTimerData->listRtAtData.GetNext(pos2);

            delete pTimerData;
        }

        delete pTestRunData;
    }
}

int CECVirtGridView::GetNextField(char sep, CString &inStr, CString &field)
{
    if (inStr == "")
    {
        return -1;
    }

    int len = inStr.GetLength();
    int nPos = inStr.Find(sep, 0);

    if (nPos == -1)
    {
        field = inStr;
        inStr = "";
        return 0;
    }
    else
    {
        field = inStr.Left(nPos);
        inStr = inStr.Right(len - nPos - 1);
        return 0;
    }
}


void CECVirtGridView::ModifySelection()
{
    CString configStr;
    configStr = pEventSink->GetSelectedRuns();

    CString field;
    while (GetNextField('|', configStr, field) != -1)
    {
        int RunIndex = atoi(field);

        CString strTestRun;
        g_pGlobalSettings->GetRunInfo(RunIndex, strTestRun);

        ApplyFilters(strTestRun);
        CalculateTimerStats(strTestRun);
    }
}

BOOL CECVirtGridView::AddTestRun(CString strTestName, CString strTestRunName, BOOL &bTimers, BOOL &bOTA,BOOL &bHttp)
{
    m_strTestName = strTestName;

    LoadHttpData(strTestRunName);

    bHttp = GetHttpData(strTestRunName)->GetSize() ? TRUE : FALSE;

    LoadOTAData(strTestRunName);
    bOTA = GetOtaData(strTestRunName)->GetSize() ? TRUE : FALSE;

    LoadTimerData(strTestRunName);

    if(GetNumTimers(strTestRunName))
    {
        if(!ApplyFilters(strTestRunName))
            return false;
        bTimers = (GetNumTimers(strTestRunName) ? TRUE : FALSE);
    }

    if(!bHttp && !bOTA && !bTimers)
        return true;

    g_pGlobalSettings->AddRunInfo(strTestRunName);

    return TRUE;
}

void CECVirtGridView::LoadTimerData(const CString& strTestRun)
{
    TestRunData *pTestRunData=NULL;
    if(!m_mapTestRunToData.Lookup(strTestRun,pTestRunData))
    {
        pTestRunData = new TestRunData;
        m_mapTestRunToData.SetAt(strTestRun,pTestRunData);
    }
    pTestRunData->pTimerData = new TimerData;

    const CString strTimerFile("Timer.txt");

    if(!m_pServer->CheckOutFile(CYRServer::enTestsFolder,m_strTestName + "\\" + strTestRun,strTimerFile))
        return;

    CString strTimerPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
    strTimerPath += m_strTestName + "\\" + strTestRun + "\\" + strTimerFile;

    try
    {
        CStdioFile TimerFile(strTimerPath,CFile::modeRead);

        CString strDescription;
        CString strTimer;
        while(TimerFile.ReadString(strTimer))
        {
            CECTimerOutput TimerOut(strTimer);

            Timer *pTimer = new Timer;

            pTimer->dwTimeStamp = TimerOut.GetCreatedTime().getSeconds();

            pTimer->dwActiveThreads = TimerOut.GetActiveThreads();

            strDescription = TimerOut.GetDescription().c_str();
            char *pszToken = strtok(strDescription.GetBuffer(0),"|");
            if(pszToken)
                pTimer->strTestName = pszToken;

            pszToken = strtok(NULL,"|");
            if(pszToken)
                pTimer->strThreadName = pszToken;

            pszToken = strtok(NULL,"|");
            if(pszToken)
                pTimer->strScriptName = pszToken;

            pszToken = strtok(NULL,"|");
            if(pszToken)
                pTimer->strTimerName = pszToken;

            //Timer values are in centi-seconds for some unknown historical reason
            pTimer->dValue = (double)TimerOut.GetValue() /100;

            pTestRunData->pTimerData->listTimers.AddTail(pTimer);
        }
    }
    catch (...)
    {}


    DWORD dwStart = pTestRunData->pTimerData->listTimers.GetHead()->dwTimeStamp;
    DWORD dwEnd = pTestRunData->pTimerData->listTimers.GetTail()->dwTimeStamp;

    if(dwStart<pTestRunData->dwStartTime)
        pTestRunData->dwStartTime = dwStart;

    if(dwEnd> pTestRunData->dwEndTime)
        pTestRunData->dwEndTime = dwEnd;
}

int CECVirtGridView::GetHttpFiles(const CString &strTestRun,CStringArray& arstrHttpFiles)
{
    CStringArray arstrResultFiles;
    m_pServer->GetFileList(CYRServer::enTestsFolder,m_strTestName + "\\" + strTestRun,arstrResultFiles);

    CString strPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
    strPath += m_strTestName + "\\" + strTestRun + "\\";

    int nTotalNumStats = 0;
    for(int nFile=0;nFile<arstrResultFiles.GetSize();nFile++)
    {
        //Look for the http stat files named
        //TEW_XXXXXXXXX.stat and TEW_XXXXXXXXX.url
        CString &strFile = arstrResultFiles[nFile];
        int nFilenameLength = strFile.GetLength();
        if(nFilenameLength < 8)
            continue;

        if(strFile.Left(4).CompareNoCase("TEW_") != 0)
            continue;

        //Check the file out of the server
        m_pServer->CheckOutFile(CYRServer::enTestsFolder,m_strTestName + "\\" + strTestRun,strFile);

        if(strFile.Right(5).CompareNoCase(".stat") != 0)
            continue;

        //Check the file exists
        CFileStatus fs;
        if(!CFile::GetStatus(strPath + strFile,fs))
            continue;

        if(fs.m_size)
        {
            nTotalNumStats += fs.m_size/(sizeof(HttpData)-sizeof(int));

            arstrHttpFiles.Add(strPath + strFile.SpanExcluding("."));
        }
    }
    return nTotalNumStats;
}

void CECVirtGridView::LoadHttpData(const CString& strTestRun)
{
    //Setup Http data arrays
    TestRunData *pTestRunData=NULL;
    if(!m_mapTestRunToData.Lookup(strTestRun,pTestRunData))
    {
        pTestRunData = new TestRunData;
        m_mapTestRunToData.SetAt(strTestRun,pTestRunData);
    }
    HttpDataArray *parHttpData = new HttpDataArray;
    pTestRunData->parHttpData = parHttpData;

    //Check out and get list of all http files
    CStringArray arstrHttpFiles;
    int nTotalNumStats = GetHttpFiles(strTestRun,arstrHttpFiles);
    if(!nTotalNumStats)
        return;

    HttpDataArray &arHttpData = (*parHttpData);
    arHttpData.SetSize(nTotalNumStats);

    CMap <CString ,LPCTSTR,int,int> &mapURLs = pTestRunData->mapUrls;
    mapURLs.InitHashTable(nTotalNumStats);

    //Load urls and stats together
    int nCount=0;
    int nStatSize = sizeof(HttpData)-sizeof(int);
    for(int nFile=0;nFile<arstrHttpFiles.GetSize();nFile++)
    {
        CString strStatFile = arstrHttpFiles[nFile] + ".stat";
        CString strUrlFile = arstrHttpFiles[nFile] + ".urls";

        CFileStatus fsStats,fsUrls;
        if(!CFile::GetStatus(strStatFile,fsStats) || !CFile::GetStatus(strUrlFile,fsUrls))
            continue;

        //From the file length we can calculate the number of stats
        int nNumStatsInFile = fsStats.m_size/nStatSize;

        //Open the http stat file
        HANDLE hStatFile = CreateFile(strStatFile,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
        if(hStatFile == INVALID_HANDLE_VALUE)
            return;

        HANDLE hStatMap = CreateFileMapping(hStatFile,NULL,PAGE_READONLY,0,0,"StatsData");
        if(hStatMap == NULL)
            return;

        LPBYTE pStatBuffer = (LPBYTE)MapViewOfFile(hStatMap,FILE_MAP_READ,0,0,0);
        LPBYTE pStatBufPos = pStatBuffer;

        //Open the http url file
        HANDLE hUrlFile = CreateFile(strUrlFile,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
        if(hUrlFile == INVALID_HANDLE_VALUE)
            return;

        HANDLE hUrlMap = CreateFileMapping(hUrlFile,NULL,PAGE_READONLY,0,0,"UrlData");
        if(hUrlMap == NULL)
            return;

        LPBYTE pUrlBuffer = (LPBYTE)MapViewOfFile(hUrlMap,FILE_MAP_READ,0,0,0);
        char *pUrlBufPos = (char*)pUrlBuffer;

        for(int nStat=0;nStat<nNumStatsInFile;nStat++)
        {
            int nIndex=-1;
            if(!mapURLs.Lookup(pUrlBufPos,nIndex))
            {
                nIndex = mapURLs.GetCount();
                mapURLs.SetAt(pUrlBufPos,nIndex);
            }
            pUrlBufPos += strlen(pUrlBufPos)+1;

            //Do the load
            HttpData *pData = arHttpData[nStat+nCount] = new HttpData;
            memcpy(pData,&pStatBuffer[nStatSize*nStat],nStatSize);

            pData->nURLNdx = nIndex;
        }
        nCount += nNumStatsInFile;

        //Clean up
        UnmapViewOfFile(pStatBuffer);
        CloseHandle(hStatMap);
        CloseHandle(hStatFile);
        UnmapViewOfFile(pUrlBuffer);
        CloseHandle(hUrlMap);
        CloseHandle(hUrlFile);
    }
    arHttpData.SetSize(nCount);

    //Sort data into time order
    qsort((*parHttpData).GetData(),(size_t)(*parHttpData).GetSize(),sizeof(HttpData**),SortHttpData);

    //Record first and last stat times
    int nNumStats = (*parHttpData).GetSize();
    if(nNumStats)
    {
        DWORD dwStart = (*parHttpData)[0]->dwTimeStamp;
        if(dwStart< pTestRunData->dwStartTime)
            pTestRunData->dwStartTime = dwStart;

        DWORD dwEnd = (*parHttpData)[nNumStats-1]->dwTimeStamp;
        if(dwEnd > pTestRunData->dwEndTime)
            pTestRunData->dwEndTime = dwEnd;
    }
}

int CECVirtGridView::SortHttpData( const void *arg1, const void *arg2 )
{
    DWORD dwTime1 = *(DWORD*)(*(DWORD*)arg1);
    DWORD dwTime2 = *(DWORD*)(*(DWORD*)arg2);

    if(dwTime1>dwTime2)
        return 1;

    return ((dwTime2>dwTime1) ? -1 : 0);
}

void CECVirtGridView::LoadOTAData(const CString& strTestRun)
{
    TestRunData *pTestRunData=NULL;
    if(!m_mapTestRunToData.Lookup(strTestRun,pTestRunData))
    {
        pTestRunData = new TestRunData;
        m_mapTestRunToData.SetAt(strTestRun,pTestRunData);
    }

    OtaQueryArray *parOtaQueries = new OtaQueryArray;
    pTestRunData->parOtaData = parOtaQueries;

    const CString strOTAHeaderFile("OtaHeader.txt");
    if(!m_pServer->CheckOutFile(CYRServer::enTestsFolder,m_strTestName + "\\" + strTestRun,strOTAHeaderFile))
        return;

    CString strTestRunPath = m_pServer->GetFolderPath(CYRServer::enTestsFolder);
    strTestRunPath += m_strTestName + "\\" + strTestRun + "\\";

    CString strHeaderPath = strTestRunPath + strOTAHeaderFile;


    // Get the Query Headers
    CArray <CECOtaHeader*,CECOtaHeader*> arHeaders;
    try
    {
        CStdioFile HeaderFile(strHeaderPath,CFile::modeRead);

        CString strHeader;
        while(HeaderFile.ReadString(strHeader))
        {
            CECOtaHeader *pHeader = new CECOtaHeader(strHeader);
            arHeaders.Add(pHeader);
        }
    }
    catch(...)
    {}


    CMap <CString,LPCTSTR,int,int> mapNodeToOffset;
    CString strCurrentNode,strLastNode;
    int nHeader=0,nOffset=0;
    int nNumHeaders = arHeaders.GetSize();
    while(nNumHeaders)
    {
        strCurrentNode = arHeaders[nHeader]->GetNodeName().c_str();

        if(strLastNode == strCurrentNode)
        {
            OtaQuery *pOtaQuery = new OtaQuery;
            parOtaQueries->Add(pOtaQuery);

            pOtaQuery->strQuery = arHeaders[nHeader]->GetDescription().c_str();
            pOtaQuery->strNode = strCurrentNode;

            delete arHeaders[nHeader];
            arHeaders.RemoveAt(nHeader--);
            nOffset++;
        }
        else
            if(nHeader==0)
            {
                OtaQuery *pOtaQuery = new OtaQuery;
                parOtaQueries->Add(pOtaQuery);

                strLastNode = strCurrentNode;
                mapNodeToOffset.SetAt(strCurrentNode,nOffset);

                pOtaQuery->strQuery = arHeaders[nHeader]->GetDescription().c_str();
                pOtaQuery->strNode = strCurrentNode;

                delete arHeaders[nHeader];
                arHeaders.RemoveAt(nHeader--);
                nOffset++;
            }

        nNumHeaders = arHeaders.GetSize();
        nHeader++;
        if(nHeader >= nNumHeaders)
            nHeader=0;
    }


    // Get the Query Data
    const CString strOTAStatFile("OtaStat.txt");
    if(!m_pServer->CheckOutFile(CYRServer::enTestsFolder,m_strTestName + "\\" + strTestRun,strOTAStatFile))
        return;

    CString strStatPath = strTestRunPath + strOTAStatFile;

    DWORD &dwStartTime = pTestRunData->dwStartTime;
    DWORD &dwEndTime = pTestRunData->dwEndTime;

    BOOL bAsked = false;
    try
    {
        CStdioFile StatFile(strStatPath,CFile::modeRead);

        CString strStat;

        while(StatFile.ReadString(strStat))
        {
            CECOtaSimpleQuery Query(strStat);

            if(mapNodeToOffset.Lookup(Query.GetNodeName().c_str(),nOffset))
            {
                int nIndex = Query.GetIndex();
                nOffset+= nIndex;

                OtaQuery *pOtaQuery = (*parOtaQueries)[nOffset];

                OtaQueryData *pOtaData = new OtaQueryData;
                pOtaData->dwStartTime = Query.GetStartTime();

                if(dwStartTime>pOtaData->dwStartTime)
                    dwStartTime=pOtaData->dwStartTime;
                if(dwEndTime<pOtaData->dwStartTime)
                    dwEndTime = pOtaData->dwStartTime;

                pOtaData->nValue = Query.GetResult();

                if(pOtaData->nValue < 0)
                {
                    if(!bAsked)
                    {
                        AfxMessageBox("Some invalid OTA data was found in this test run.\n\nThis data will be set to zero",MB_ICONWARNING|MB_OK);
                        bAsked = true;
                    }
                    pOtaData->nValue = 0;
                }

                pOtaQuery->listQueryData.AddTail(pOtaData);
            }
        }
    }
    catch (...)
    {}

}

BOOL CECVirtGridView::ApplyFilters(const CString& strTestRun)
{
    CECFilterDlg filterDlg;
    filterDlg.SetGridPtr(this);
    filterDlg.SetDataRun(strTestRun);
    if(filterDlg.DoModal() == IDCANCEL)
        return false;

    if(!GetNumTimers(strTestRun))
        return true;

    CalculateTimerStats(strTestRun);

    return true;
}

HttpDataArray* CECVirtGridView::GetHttpData(const CString& strTestRun)
{
    TestRunData *pTestRun;
    if(m_mapTestRunToData.Lookup(strTestRun,pTestRun))
        return pTestRun->parHttpData;
    return NULL;
}

OtaQueryArray* CECVirtGridView::GetOtaData(const CString& strTestRun)
{
    TestRunData *pTestRun;
    if(m_mapTestRunToData.Lookup(strTestRun,pTestRun))
        return pTestRun->parOtaData;
    return NULL;
}

TimerData* CECVirtGridView::GetTimerData(const CString& strTestRun)
{
    TestRunData *pTestRun;
    if(m_mapTestRunToData.Lookup(strTestRun,pTestRun))
        return pTestRun->pTimerData;
    return NULL;
}

void CECVirtGridView::GetURLs(const CString& strTestRun, CStringArray& arstrURLs)
{
    arstrURLs.RemoveAll();

    TestRunData *pTestRun;
    if(m_mapTestRunToData.Lookup(strTestRun,pTestRun))
    {
        arstrURLs.SetSize(pTestRun->mapUrls.GetCount());

        int nIndex;
        CString strURL;
        POSITION pos = pTestRun->mapUrls.GetStartPosition();
        while(pos)
        {
            pTestRun->mapUrls.GetNextAssoc(pos,strURL,nIndex);
            arstrURLs[nIndex] = strURL;
        }

    }
}

void CECVirtGridView::CalculateTimerStats(const CString& strTestRun)
{
    TimerData *pTimerData = GetTimerData(strTestRun);

    double dSum = 0;
    double &dMax = pTimerData->dMax;
    double &dMin = pTimerData->dMin;
    dMax = 0;
    dMin = 0xffffffff;

    int nNumTimers = 0;

    ///
    POSITION pos = pTimerData->listTimers.GetHeadPosition();
    while(pos)
    {
        Timer *pTimer = pTimerData->listTimers.GetNext(pos);
        if(!pTimer->bInFilter)
            continue;

        double &dValue = pTimer->dValue;

        //Find largest timer value
        if(dValue > dMax) dMax = dValue;

        //Find smallest timer value
        if(dValue < dMin) dMin = dValue;

        //Generate the sum of all timer values
        dSum += dValue;

        nNumTimers++;
    }

    //Calculate Mean
    double dMean = dSum / nNumTimers;
    pTimerData->dMean = dMean;

    //Calculate Standard deviation
    double dSumOfSqOfDeviations=0;
    double dDeviation;
    pos = pTimerData->listTimers.GetHeadPosition();
    while(pos)
    {
        Timer *pTimer = pTimerData->listTimers.GetNext(pos);
        if(!pTimer->bInFilter)
            continue;
        dDeviation = pTimer->dValue - dMean;
        dSumOfSqOfDeviations += dDeviation*dDeviation;
    }
    pTimerData->dStdDev = sqrt( dSumOfSqOfDeviations/nNumTimers);

    double dRange = dMax - dMin;
    double dInterval;
    if (dRange <= 0.1)  dInterval = 0.01;
    else if (dRange <= 1.0) dInterval = 0.1;
    else if (dRange <= 360.0) dInterval = 1.5;
    else dInterval = 60.0;

    int nNumIntervals = (dMax / dInterval);
    if((nNumIntervals * dInterval) < dMax)
        nNumIntervals++;

    if(!dRange)
        nNumIntervals = 0;

    //Calculate Frequency, cumulative frequency and percentage frequency for intervals
    double dPercentFreq=0;
    DWORD dwCumFrequency=0;
    int nCount=0;

    for(int nInterval=0;nInterval<pTimerData->arTimerIntervals.GetSize();nInterval++)
        delete pTimerData->arTimerIntervals[nInterval];
    pTimerData->arTimerIntervals.RemoveAll();

    double dIntervalLower=0;
    double dIntervalUpper=dInterval;
    for(nInterval=0;nInterval<nNumIntervals;nInterval++)
    {
        TimerInterval *pInterval = new TimerInterval;
        pInterval->dInterval = dIntervalUpper;

        nCount = 0;
        pos = pTimerData->listTimers.GetHeadPosition();
        while(pos)
        {
            Timer *pTimer = pTimerData->listTimers.GetNext(pos);
            if(!pTimer->bInFilter)
                continue;

            double &dValue = pTimer->dValue;
            if(dValue >= dIntervalLower  && dValue < dIntervalUpper)
                nCount++;
        }

        pInterval->dFrequency = nCount;

        dwCumFrequency += nCount;
        pInterval->dCumFreq = dwCumFrequency;

        pInterval->dPercentFreq = (double)dwCumFrequency*100/nNumTimers;

        pTimerData->arTimerIntervals.Add(pInterval);

        dIntervalLower = dIntervalUpper;
        dIntervalUpper += dInterval;
    }

    double *pValues = new double[nNumTimers];
    int nIndex=0;
    pos = pTimerData->listTimers.GetHeadPosition();
    while(pos)
    {
        Timer *pTimer = pTimerData->listTimers.GetNext(pos);
        if(!pTimer->bInFilter)
            continue;

        pValues[nIndex++] = pTimer->dValue;
    }

    qsort(pValues,nNumTimers,sizeof(double),SortTimers);

    int nMidNdx = nNumTimers/2;
    if(nNumTimers % 2)  //if odd
        pTimerData->dMedian = pValues[nMidNdx];
    else        //even num timers so take mean of middle two values
        pTimerData->dMedian = (pValues[nMidNdx] + pValues[nMidNdx+1]) / 2;

    pTimerData->dLowerQuartile = pValues[nNumTimers/4];
    pTimerData->dUpperQuartile = pValues[3*nNumTimers/4];

    delete []pValues;
}

int CECVirtGridView::SortTimers( const void *arg1, const void *arg2 )
{
    if(*(double*)arg1 > *(double*)arg2)
        return 1;

    return (*(double*)arg1 < *(double*)arg2) ? -1 : 0;
}

int CECVirtGridView::GetNumTimers(const CString& strTestRun)
{
    int nCount=0;
    TimerData *pTimerData = GetTimerData(strTestRun);
    if(pTimerData)
    {
        POSITION pos = pTimerData->listTimers.GetHeadPosition();
        while(pos)
        {
            if(pTimerData->listTimers.GetNext(pos)->bInFilter)
                nCount++;
        }
    }
    return nCount;
}

BOOL CECVirtGridView::GetHasHttp(int nRunIndex)
{
    CString strTestRun;
    g_pGlobalSettings->GetRunInfo(nRunIndex,strTestRun);

    HttpDataArray* pHttpData = GetHttpData(strTestRun);
    return (pHttpData->GetSize() ? TRUE:FALSE);
}

BOOL CECVirtGridView::GetHasOTA(int nRunIndex)
{
    CString strTestRun;
    g_pGlobalSettings->GetRunInfo(nRunIndex,strTestRun);

    OtaQueryArray *pOTA = GetOtaData(strTestRun);
    return (pOTA->GetSize() ? TRUE : FALSE);
}

BOOL CECVirtGridView::GetHasTimers(int nRunIndex)
{
    CString strTestRun;
    g_pGlobalSettings->GetRunInfo(nRunIndex,strTestRun);

    TimerData* pTimerData = GetTimerData(strTestRun);
    if(pTimerData && pTimerData->listTimers.GetCount())
        return true;

    return false;
}

DWORD CECVirtGridView::GetStartTime(int nRunIndex)
{
    CString strTestRun;
    g_pGlobalSettings->GetRunInfo(nRunIndex,strTestRun);

    TestRunData *pTestRun;
    if(m_mapTestRunToData.Lookup(strTestRun,pTestRun))
        return pTestRun->dwStartTime;

    return 0;
}

DWORD CECVirtGridView::GetStopTime(int nRunIndex)
{
    CString strTestRun;
    g_pGlobalSettings->GetRunInfo(nRunIndex,strTestRun);

    TestRunData *pTestRun;
    if(m_mapTestRunToData.Lookup(strTestRun,pTestRun))
        return pTestRun->dwEndTime;

    return 0;
}
