/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECWorkspace.cpp : implementation file
//

#include "stdafx.h"
#include "AnalyzerDefs.h"
#include "CECWorkspace.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECWorkspace

CECWorkspace::CECWorkspace()
{
    m_szWindowsPositions    = "Window Size/Position";

    //
    // Window size/position.
    m_szMainWindow                      = "Main Window";
    m_MainWindow =                      _T("0,0,700,500");
    m_szMainWindowZoomState             = "Main Window Zoom State";

    // MDI window size/position
    m_szCumFreqGraph                    = "CumFreq Graph";
    m_CumFreqGraph                      = _T("82, 150, 834, 562");
    m_szStdFreqGraph                    = "StdFreq Graph";
    m_StdFreqGraph                      =_T("82, 150, 834, 562");
    m_szPercentFreqGraph                = "PercentFreqGraph";
    m_PercentFreqGraph                  = _T("82, 150, 834, 562");
    m_szRTATGraph                       = "RTATGraph";
    m_RTATGraph                         = _T("82, 150, 834, 562");
    m_szSummary                         = "Summary Screen";
    m_Summary                           = _T("82, 150, 834, 562");
    m_szSortedList                      = "Sorted List";
    m_SortedList                        = _T("82, 150, 834, 562");
    m_szOTAGraph                        = "OTAGraph";
    m_OTAGraph                          = _T("82, 150, 834, 562");
    m_szPageSwapGraph                   = "Page Swap Graph";
    m_PageSwapGraph                     = _T("82, 150, 834, 562");
    m_szNetUseGraph                     = "Net Use Graph";
    m_NetUseGraph                       = _T("82, 150, 834, 562");
    m_szPercentCPUGraph                 = "Percent CPU Graph";
    m_PercentCPUGraph                   = _T("82, 150, 834, 562");
    m_szRunList                         = "Run List";

    int nScreenHeight = GetSystemMetrics(SM_CYSCREEN);
    m_RunList.Format("20,%d,250,%d,150",nScreenHeight-300,nScreenHeight-30);

    RestoreWorkspace();
}

CECWorkspace::~CECWorkspace()
{}


/////////////////////////////////////////////////////////////////////////////
// CECWorkspace message handlers


BOOL CECWorkspace::RestoreWorkspace()
{
    CWinApp* pApp = AfxGetApp();

    CString         str;
    int             n;

    // Pick up the main window size-pos + zoom state
    str = pApp->GetProfileString(m_szWindowsPositions, m_szMainWindow);
    if (!str.IsEmpty() ) m_MainWindow = str;
    n = pApp->GetProfileInt(m_szWindowsPositions, m_szMainWindowZoomState,          0);
    m_bMainWindowZoomState = n;


    // Get Other window positions
    str = pApp->GetProfileString(m_szWindowsPositions, m_szRunList);
    if (!str.IsEmpty() ) m_RunList = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szCumFreqGraph);
    if (!str.IsEmpty() ) m_CumFreqGraph = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szStdFreqGraph);
    if (!str.IsEmpty() ) m_StdFreqGraph = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szPercentFreqGraph);
    if (!str.IsEmpty() ) m_PercentFreqGraph = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szRTATGraph);
    if (!str.IsEmpty() ) m_RTATGraph = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szSummary);
    if (!str.IsEmpty() ) m_Summary = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szSortedList);
    if (!str.IsEmpty() ) m_SortedList = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szOTAGraph);
    if (!str.IsEmpty() ) m_OTAGraph = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szPageSwapGraph);
    if (!str.IsEmpty() ) m_PageSwapGraph = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szNetUseGraph);
    if (!str.IsEmpty() ) m_NetUseGraph = str;

    str = pApp->GetProfileString(m_szWindowsPositions, m_szPercentCPUGraph);
    if (!str.IsEmpty() ) m_PercentCPUGraph = str;

    return TRUE;
}

//
// ParseWindowSizePos
// ==================
//
// Parses the co-ordinates of a window. Co-ordinates delimited by
// comma.
void CECWorkspace::ParseWindowSizePos(const CString& strSizePos,
                                      CRect& r)
{
    CString strLocalCopy = strSizePos;
    int nPos;

    //
    // Initialise r
    r.left = r.right = r.top = r.bottom = 0;

    //
    // Retrieve left
    r.left = atoi( strLocalCopy);

    //
    // Retrieve top
    nPos = strLocalCopy.Find(",");
    if (nPos == -1) return;
    strLocalCopy = strLocalCopy.Mid(nPos + 1);
    r.top = atoi( strLocalCopy );

    //
    // Retrive the right
    nPos = strLocalCopy.Find(",");
    if (nPos == -1) return;
    strLocalCopy = strLocalCopy.Mid(nPos + 1);
    r.right = atoi( strLocalCopy );

    //
    // Retrive the bottom
    nPos = strLocalCopy.Find(",");
    if (nPos == -1) return;
    strLocalCopy = strLocalCopy.Mid(nPos + 1);
    r.bottom = atoi( strLocalCopy );

}

//
// SaveWindowPos
// =============
//
// Overloaded.
//
// Saves the position of a window, supplied in r, in the string
// strSizePos, delimited by commas.
//
// Converse of ParseWindowSizePos functions.
void CECWorkspace::SaveWindowPos(CString& strSizePos, const CRect& r)
{
    strSizePos.Format("%d,%d,%d,%d", r.left, r.top, r.right, r.bottom);
}

//
//
//
//    SaveProjectSettings
//    ===================
//
//    This routine is called when we wish to save the project settings (on exit from commander)
//
void CECWorkspace::SaveSettings ()
{
    CWinApp* pApp = AfxGetApp();
    //
    // Save mainframe window size - together with state of the zoom
    CMDIFrameWnd* pMainWnd = (CMDIFrameWnd*)AfxGetMainWnd();
    pApp->WriteProfileString(m_szWindowsPositions,  m_szMainWindow,                 m_MainWindow);
    pApp->WriteProfileInt(m_szWindowsPositions,     m_szMainWindowZoomState,        pMainWnd->IsZoomed());

    // Save MDI window size/pos and zoom
    pApp->WriteProfileString(m_szWindowsPositions,  m_szRunList,                    m_RunList);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szCumFreqGraph,               m_CumFreqGraph);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szStdFreqGraph,               m_StdFreqGraph);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szPercentFreqGraph,           m_PercentFreqGraph);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szRTATGraph,                  m_RTATGraph);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szSummary,                    m_Summary);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szSortedList,                 m_SortedList);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szOTAGraph,                   m_OTAGraph);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szPageSwapGraph,              m_PageSwapGraph);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szNetUseGraph,                m_NetUseGraph);
    pApp->WriteProfileString(m_szWindowsPositions,  m_szPercentCPUGraph,            m_PercentCPUGraph);
}
