/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYREventSink.cpp
// ----------------
//
//    This is the Implementation file for the CYREventSink DLL.
//
//=========================================================================


#include "stdafx.h"
#include "CYREventSink.h"
#include "resource.h"
#include "..\..\Interfaces\CYRPlugIns.h"
#include "CECProjectViewerDlg.h"
#include "TabbedDoc.h"
#include "AnalyzerDefs.h"
#include "CECRunList.h"
#include "CECGlobalSettings.h"
#include "CECVirtGridView.h"
#include "CECDocManager.h"
#include "AnalyzerSheet.h"
#include "TabbedChildFrame.h"
#include "Server.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CMultiDocTemplate*       pAnalyzerDocTemplate;
extern  CECProjectViewerDlg*    pProjectViewerDlg;
extern CECGlobalSettings*       g_pGlobalSettings;
extern CECDocManager*           g_pDocManager;
/////////////////////////////////////////////////////////////////////////////
// CYREventSink

CYREventSink::CYREventSink()
{
    // In release we have discovered that our extension DLL's behave in a different
    // manner to the debug builds. It appears that any member variables are not
    // persistant for the entirety of this class and unless static they lose their data
    // after leaving the constructor.

    // As a workaround we have found that if we set up a 16-byte char value ALL other
    // member variables then retain their values during this objects lifetime.

    // Don't ask me why - it's an MFC Release Build Memory Management thing ...
    (void) strcpy (EventSink, "CYRAnalyzer");

    m_bHasTimers = FALSE;
    m_bHasOTA = FALSE;
    m_bHasHttp = FALSE;

    m_pVirtGrid=NULL;
    m_pRunList = NULL;

    m_bRunListVisible = FALSE;

    m_hParent = NULL;

    for(int nNdx=0;nNdx<enNumGraphTypes;nNdx++)
        m_arpGraphDocs[nNdx] = NULL;

    // Get and store the parent window for this event sink
    CWnd* pParentWnd = AfxGetMainWnd();
    if (pParentWnd != NULL)
    {
        m_hParent = pParentWnd->GetSafeHwnd();

        if (m_hParent != NULL)
        {
            // Create a default window size
            CRect rect(0,0,0,0);
            Create(NULL, NULL, WS_CHILD, rect, pParentWnd, ID_ANALYZER_EVENT_SINK);
        }
    }
}

CYREventSink::~CYREventSink()
{
    for(int nNdx=0;nNdx<enNumGraphTypes;nNdx++)
    {
        delete m_arpGraphDocs[nNdx];
        m_arpGraphDocs[nNdx] = NULL;
    }

    if (m_pRunList)
        delete m_pRunList;

    if(m_pVirtGrid)
        delete m_pVirtGrid;
}


BEGIN_MESSAGE_MAP(CYREventSink, CWnd)
//{{AFX_MSG_MAP(CYREventSink)
ON_COMMAND(ID_HTTP_LIST,OnHttpList)
ON_COMMAND(ID_SORTED_LIST,OnSortedList)
ON_COMMAND(ID_SUMMARY,OnSummary)
ON_COMMAND(ID_VIEW_RUNLIST,OnViewRunList)
ON_MESSAGE(STUDIO_SWITCH_PROJECTS, OnSwitchProjects)
ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
ON_WM_PARENTNOTIFY()
ON_COMMAND(ID_EDIT_SELECTION,OnEditSelection)
//}}AFX_MSG_MAP
ON_MESSAGE(WMU_OPEN_TEST_RUN,OnOpenTestRun)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CYREventSink message handlers

//
// IsEnabled
// =========
//
// Handler for ALL the Plug-Ins resources
//
BOOL CYREventSink::IsEnabled(UINT nID)
{
    BOOL bState = FALSE;
    //should be using extended handler
    ASSERT(FALSE);
    return bState;
}

//
// OnSwitchProjects
// ================
//
// The function is called whenever switching projects - All we have to do is obtain
// the new project name which will already of been set up for us in the framework
//
LRESULT CYREventSink::OnSwitchProjects(WPARAM,LPARAM)
{
    //
    // Recover the active project from the framework
    //
    CYRServer *pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);

    if (pProjectViewerDlg != NULL)
    {
        if(pServer->IsThereAnOpenProject())
        {
            // Obtain the new project name
            CString szProjectName = pServer->GetOpenProjectName();

            // And set it to be active
            pProjectViewerDlg->SetActiveProject(szProjectName);
        }
        else
        {
            pProjectViewerDlg->SetActiveProject("<No Project Selected>");
        }
    }

    //
    // Clear the run list and remove all DataRun Information
    //
    if(m_pRunList)
        m_pRunList->Clear();
    g_pGlobalSettings->RemoveAllRunInfo();
    ShowRunList(FALSE);

    return 0;
}

void CYREventSink::OnViewRunList()
{
    ShowRunList(!m_bRunListVisible);
}

void CYREventSink::OnSummary()
{
    OnGraphButton(enSummaryScreen,"Graph");
}

void CYREventSink::OnSortedList()
{
    OnGraphButton(enSortedList,"Sorted");
}

void CYREventSink::OnEditSelection()
{
    if (m_pVirtGrid != NULL)
    {
        m_pVirtGrid->ModifySelection();
    }
}

void CYREventSink::ShowGraph(GraphTypesEnum nGraphType,BOOL bShow)
{
    bool bVisible = IsGraphVisible(nGraphType);

    if((bVisible && !bShow) || (!bVisible && bShow))
    {
        OnGraphButton(nGraphType,"Graph");
    }
}

void CYREventSink::OnGraphButton(GraphTypesEnum nGraphType,const CString &strIdent)
{
    CMDIFrameWnd *pParent = (CMDIFrameWnd*)AfxGetMainWnd();
    if(m_arpGraphDocs[nGraphType] == NULL)
    {
        CTabbedDoc *pTabbedDoc = (CTabbedDoc*)OnFileNew(strIdent);
        if(pTabbedDoc!=NULL)
        {
            CString str;
            CWnd *pMDIClient = CWnd::FromHandle(pParent->m_hWndMDIClient);

            //Find the sheet we've just created
            CWnd *pChild = pMDIClient->GetWindow(GW_CHILD);
            while(pChild)
            {
                CTabbedChildFrame *pChildFrame = dynamic_cast<CTabbedChildFrame *>(pChild);
                if((pChildFrame != NULL) && !pChildFrame->IsInitialised())
                {
                    pTabbedDoc->SetupGraph(nGraphType,pChildFrame);
                    m_arpGraphDocs[nGraphType] = pTabbedDoc;
                    pChildFrame->SetOwner(this);
                    pChildFrame->SetInitialised();

                    if(m_pRunList && m_pRunList->GetSafeHwnd())
                        m_pRunList->UpdateGraphsList();

                    break;
                }

                pChild = pChild->GetWindow(GW_HWNDNEXT);
            }
        }

    }
    else
    {
        m_arpGraphDocs[nGraphType]->GetFrame()->DestroyWindow();
        delete m_arpGraphDocs[nGraphType];
        m_arpGraphDocs[nGraphType] = NULL;
    }

    pParent->Invalidate();
}

CDocument *CYREventSink::OnFileNew(LPCSTR DocIdent)
{
    if (g_pDocManager != NULL)
        return g_pDocManager->OnFileNew(DocIdent);
    return NULL;
}

void CYREventSink::ShowRunList(BOOL bShow)
{
    if(!bShow)
    {
        if(g_pGlobalSettings->GetLoadedCount())
        {
            if(AfxMessageBox("Closing the test run list will unload all the current test results\n\nDo you want to do this?",MB_ICONWARNING|MB_YESNO)==IDNO)
                return;
        }

        ResetGrid();
        if(m_pVirtGrid)
        {
            delete m_pVirtGrid;
            m_pVirtGrid = NULL;
        }
        m_bRunListVisible = false;

        if (m_pRunList != NULL)
        {
            m_pRunList->ShowWindow(SW_HIDE);
            m_pRunList->DestroyWindow();
            delete m_pRunList;
            m_pRunList = NULL;
        }
    }
    else
        if (m_pRunList != NULL)
        {
            m_bRunListVisible = true;
            m_pRunList->ShowWindow(SW_SHOW);
        }
}

void CYREventSink::AddToRunList(CString strRunName)
{
    if (m_pRunList == NULL)
    {
        m_pRunList = new CECRunList;
        m_pRunList->Create(IDD_RUNLIST);
    }

    m_pRunList->ShowWindow(SW_SHOW);
    m_pRunList->AddRunName(strRunName);
    m_bRunListVisible = TRUE;
}

void CYREventSink::ClearRunList()
{
    if (m_pRunList != NULL)
    {
        m_pRunList->Clear();

    }
}

CString CYREventSink::GetSelectedRuns()
{
    return m_pRunList->GetConfigString();
}

LRESULT CYREventSink::OnOpenTestRun(WPARAM wParam,LPARAM)
{
    CString strCombined((LPCTSTR)wParam);

    CString strTestName,strTestRunName;

    int nPos = strCombined.Find('|');
    if(nPos != -1)
    {
        strTestName = strCombined.Left(nPos);
        strTestRunName = strCombined.Right(strCombined.GetLength()-nPos-1);
    }

    OpenTestRun(strTestName,strTestRunName);
    return 1;
}

void CYREventSink::OpenTestRun(CString strTestName, CString strTestRunName)
{
    AfxGetMainWnd()->SendMessage(STUDIO_BEGIN_WAIT);

    try
    {
        if(m_pVirtGrid == NULL)
            OpenCalcGrid();

        //
        // Check that the selected run is not a run of the same test. If it is reset the grid.
        //
        if (g_pGlobalSettings->GetLoadedCount() > 0)
        {
            CString strLoadedTest;
            m_pVirtGrid->GetTestName(strLoadedTest);
            if(strLoadedTest != strTestName)
            {
                AfxGetMainWnd()->SendMessage(STUDIO_END_WAIT);
                int retVal = AfxMessageBox("Opening test run from a different test\nwill unload all data from previous test.\n\nContinue?",MB_YESNO);

                if (retVal == IDNO)
                    throw -1;

                AfxGetMainWnd()->SendMessage(STUDIO_BEGIN_WAIT);
                ResetGrid();
                OpenCalcGrid();
            }

            for(int nRun=0;nRun < g_pGlobalSettings->GetLoadedCount();nRun++)
            {
                CString strTestRun;
                g_pGlobalSettings->GetRunInfo(nRun,strTestRun);
                if(strTestRun == strTestRunName)
                {
                    //remove .xxx from end of name
                    CString strRunName(strTestRunName);
                    int nPos = strRunName.ReverseFind('.');
                    strRunName = strRunName.Left(nPos);

                    CString strMessage;
                    strMessage.Format("The test run \"%s\" is already open",strRunName);
                    throw strMessage;
                }
            }
        }

        //
        // Now we can add this Test Run to the grid...
        //

        // ... first load and parse data ...

        if(!m_pVirtGrid->AddTestRun(strTestName, strTestRunName, m_bHasTimers, m_bHasOTA,m_bHasHttp))
        {
            if(!g_pGlobalSettings->GetLoadedCount())
                ShowRunList(FALSE);
            throw -1;
        }

        if (!m_bHasTimers && !m_bHasOTA && !m_bHasHttp)
        {
            if (m_pRunList != NULL)
                OnRunListSelChange();

            if(!g_pGlobalSettings->GetLoadedCount())
                ShowRunList(FALSE);

            throw CString("No data found");
        }

        AddToRunList(strTestRunName);
    }
    catch(CString str)
    {
        AfxMessageBox(str);
    }
    catch(int)
    {}


    AfxGetMainWnd()->SendMessage(STUDIO_END_WAIT);
}

void CYREventSink::ResetGrid()
{
    //
    // Clear the run list and remove all DataRun Information
    //
    if(m_pRunList)
        m_pRunList->Clear();
    g_pGlobalSettings->RemoveAllRunInfo();

    //
    // Close All Analyzer Windows
    //
    CMDIFrameWnd *pParent = (CMDIFrameWnd*)AfxGetMainWnd();
    CWnd *pMDIClient = CWnd::FromHandle(pParent->m_hWndMDIClient);

    CWnd *pChild = pMDIClient->GetWindow(GW_CHILD);
    while(pChild)
    {
        CAnalyzerSheet *pAnalyzerSheet = dynamic_cast<CAnalyzerSheet*>(pChild);
        pChild = pChild->GetWindow(GW_HWNDNEXT);

        if(pAnalyzerSheet != NULL)
        {
            pAnalyzerSheet->SendMessage(WM_CLOSE);
        }
    }

    m_bHasTimers = FALSE;
    m_bHasOTA = FALSE;
    m_bHasHttp = FALSE;
}

void CYREventSink::OpenCalcGrid()
{
    if(!m_pVirtGrid)
        m_pVirtGrid = new CECVirtGridView;
}

BOOL CYREventSink::OnCommandUIUpdate(UINT uiID,CCmdUI *pCmdUI)
{
    switch(uiID)
    {
    case ID_SUMMARY:
        return  OnUpdateGraphButton(TRUE,FALSE,FALSE,m_arpGraphDocs[enSummaryScreen]!=NULL,pCmdUI);
    case ID_SORTED_LIST:
        return OnUpdateGraphButton(TRUE,FALSE,FALSE,m_arpGraphDocs[enSortedList]!=NULL,pCmdUI);
    case ID_EDIT_SELECTION:
        return OnUpdateGraphButton(TRUE,FALSE,FALSE,FALSE,pCmdUI);
    case ID_VIEW_RUNLIST:
        return OnUpdateViewRunlist(pCmdUI);
    case ID_HTTP_LIST:
        return OnUpdateGraphButton(FALSE,FALSE,TRUE,m_arpGraphDocs[enHttpList]!=NULL,pCmdUI);
    default:
        ASSERT(FALSE);
        return false;
    }
}

BOOL CYREventSink::OnUpdateGraphButton(BOOL bNeedsTimers,BOOL bNeedsOTA,BOOL bNeedsHttp,BOOL bIsVisible,CCmdUI* pCmdUI)
{
    BOOL bEnabled = TRUE;
    bEnabled &= ((m_bHasTimers && bNeedsTimers) || !bNeedsTimers);
    bEnabled &= ((m_bHasOTA && bNeedsOTA) || !bNeedsOTA);
    bEnabled &= ((m_bHasHttp && bNeedsHttp) || !bNeedsHttp);
    bEnabled &= (g_pGlobalSettings->GetLoadedCount() > 0);
    BOOL bCheck = bEnabled && bIsVisible;

    pCmdUI->SetCheck(bCheck);
    return bEnabled;
}

BOOL CYREventSink::IsGraphEnabled(GraphTypesEnum GraphType)
{
    switch(GraphType)
    {
    case enCumFreqGraph:
    case enStdFreqGraph:
    case enPercentFreqGraph:
    case enRTATGraph:
    case enRTTimeGraph:
    case enUsersTimeGraph:
    case enSummaryScreen:
    case enSortedList:
        return (m_bHasTimers);
        break;

    case enPercentCPUGraph:
    case enPageSwapGraph:
    case enPercentNetUseGraph:
    case enSimpleOTAGraph:
        return (m_bHasOTA);
        break;

    case enKbPerSecondGraph:
    case enAVPRTPerUserGraph:
    case enErrorsPerUserGraph:
    case enErrorsTimeGraph:
    case enHitsPerSecondGraph:
    case enPagesPerSecondGraph:
    case enHttpList:
        return (m_bHasHttp);
        break;
    default:
        ASSERT(FALSE);
        break;
    }
    return false;
}

BOOL CYREventSink::IsGraphVisible(GraphTypesEnum GraphType)
{
    return (m_arpGraphDocs[GraphType] != NULL);
}

BOOL CYREventSink::OnUpdateViewRunlist(CCmdUI* pCmdUI)
{
    BOOL bEnabled = (m_pRunList != NULL);
    BOOL bState = (bEnabled && m_bRunListVisible);
    pCmdUI->SetCheck(bState);
    return bEnabled;
}

void CYREventSink::OnUpdateFileNew(CCmdUI *pCmdUI)
{
    pCmdUI->Enable(FALSE);
}

void CYREventSink::OnRunListSelChange()
{
    m_bHasTimers = false;
    m_bHasOTA=false;
    m_bHasHttp = false;

    //Search through the selected items in the test run list
    //and see if they have any timers or ota data, so we know
    //whether to enable the toolbar buttons or not
    CString strTestRun;
    CString strSelRuns = GetSelectedRuns();
    char *pszToken = strtok(strSelRuns.GetBuffer(0),"|");
    while(pszToken != NULL)
    {
        int nNdx = atoi(pszToken);

        if(m_pVirtGrid->GetHasTimers(nNdx))
            m_bHasTimers = true;

        if(m_pVirtGrid->GetHasOTA(nNdx))
            m_bHasOTA = true;

        if(m_pVirtGrid->GetHasHttp(nNdx))
            m_bHasHttp = true;

        pszToken = strtok(NULL,"|");
    }

    //Tell the worksheets that the test run selection has changed
    for(int nDoc=0;nDoc<enNumGraphTypes;nDoc++)
    {
        if(m_arpGraphDocs[nDoc] != NULL)
        {
            m_arpGraphDocs[nDoc]->UpdateTabs();
        }
    }

    AfxGetMainWnd()->Invalidate();
}

void CYREventSink::OnParentNotify(UINT message, LPARAM lParam)
{
    CWnd::OnParentNotify(message, lParam);
    switch(message)
    {
    case WM_DESTROY:
        for(int nNdx=0;nNdx<enNumGraphTypes;nNdx++)
        {
            if(m_arpGraphDocs[nNdx]==NULL)
                continue;

            if(m_arpGraphDocs[nNdx]->GetFrame()->GetSafeHwnd() == (HWND)lParam)
            {
                //              m_arpGraphDocs[nNdx]->RemoveAllTabs();
                delete m_arpGraphDocs[nNdx];
                m_arpGraphDocs[nNdx] = NULL;
                break;
            }
        }
        if(m_pRunList && m_pRunList->GetSafeHwnd())
            m_pRunList->UpdateGraphsList();

        break;
    }
}

void CYREventSink::OnHttpList()
{
    OnGraphButton(enHttpList,"Http");
}

