/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYREventSink.h
// --------------
//
//    This is the include file for the CYREventSink.
//
//=========================================================================

#if !defined(AFX_CYREVENTSINK_H__ED9AF3A3_F889_11D2_B516_00600856D1F5__INCLUDED_)
#define AFX_CYREVENTSINK_H__ED9AF3A3_F889_11D2_B516_00600856D1F5__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// CYREventSink.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CYREventSink window
#include "analyzerdefs.h"

class CTabbedDoc;
class CECRunList;
class CECVirtGridView;

class CYREventSink : public CWnd
{
    // Construction
public:
    CYREventSink();

    // Workaround for the MFC Memory Management Release Build Issue
    char EventSink[16];

    // Attributes
public:
    // @cmember Called to retrieve a set of data from the repository
    void OpenTestRun(CString strTestName, CString strTestRunName);

    BOOL m_bRunListVisible;

    // @cmember Reference to the hidden calculation grid
    CECVirtGridView* m_pVirtGrid;


    // @cmember Get the configuration string which contains the index positions
    //          of the data sets selected in the run list
    CString GetSelectedRuns();

    // @cmember Show/Hide the run selection list window
    void ShowRunList(BOOL bShow);
    void OnRunListSelChange();

    // Operations
public:

    // Overrides
    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CYREventSink)
    //}}AFX_VIRTUAL

    // Implementation
public:
    virtual ~CYREventSink();

    BOOL IsEnabled(UINT nID);
    BOOL OnCommandUIUpdate(UINT uiID,CCmdUI *pCmdUI);

    void ShowGraph(GraphTypesEnum nGraphType,BOOL bShow);
    BOOL IsGraphEnabled(GraphTypesEnum GraphType);
    BOOL IsGraphVisible(GraphTypesEnum GraphType);

    BOOL GetHasTimers()
    {
        return m_bHasTimers;
    }
    BOOL GetHasOTA()
    {
        return m_bHasOTA;
    }
    BOOL GetHasHttp()
    {
        return m_bHasHttp;
    }

    // Generated message map functions
protected:
    //{{AFX_MSG(CYREventSink)
    afx_msg void OnSortedList();
    afx_msg void OnSummary();
    afx_msg void OnViewRunList();
    afx_msg LRESULT OnSwitchProjects(WPARAM,LPARAM);
    afx_msg void OnUpdateFileNew(CCmdUI *pCmdUI);
    afx_msg void OnParentNotify(UINT message, LPARAM lParam);
    afx_msg LRESULT OnOpenTestRun(WPARAM wParam,LPARAM);
    afx_msg void OnEditSelection();
    afx_msg void OnHttpList();
    afx_msg void OnCustomOTA();
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()

    HWND m_hParent;
    BOOL m_bHasOTA;
    BOOL m_bHasTimers;
    BOOL m_bHasHttp;

    // @cmember Reference to the graph Tab windows
    CTabbedDoc* m_arpGraphDocs[enNumGraphTypes];

    // @cmember Pointer to the run selection list window
    CECRunList* m_pRunList;

    CDocument *OnFileNew(LPCSTR DocIdent);

    // @cmember Called to clear the run list box
    void ClearRunList();

    // @cmember Called to add a test run name to the run list
    void AddToRunList(CString strRunName);

    void ResetGrid();

    void OpenCalcGrid();

    //Toolbar Update functions
    BOOL OnUpdateViewRunlist(CCmdUI* pCmdUI);
    BOOL OnUpdateGraphButton(BOOL bNeedsTimer,BOOL bNeedsOTA,BOOL bNeedsHttp,BOOL bIsVisible,CCmdUI* pCmdUI);

    void OnGraphButton(GraphTypesEnum nGraphType,const CString &strIdent);
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_CYREVENTSINK_H__ED9AF3A3_F889_11D2_B516_00600856D1F5__INCLUDED_)
