/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECSetWorkFolderDlg.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "Solutions.h"
#include "CECSetWorkFolderDlg.h"
#include "..\..\Common\CECRegistryTools.h"
#include "..\..\interfaces\cyrplugins.h"
#include "..\..\..\HTMLHelp\CYRGeneric\Set_Workfolder.inl"
#include "htmlhelp.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif
/////////////////////////////////////////////////////////////////////////////
int CALLBACK BrowseCallback(HWND hwnd,UINT uMsg,LPARAM lp, LPARAM /*pData*/)
{
    TCHAR szDir[MAX_PATH];

    switch(uMsg)
    {
    case BFFM_INITIALIZED:
        {
            if (GetCurrentDirectory(sizeof(szDir)/sizeof(TCHAR), szDir))
            {
                // WParam is TRUE since you are passing a path.
                // It would be FALSE if you were passing a pidl.
                SendMessage(hwnd,BFFM_SETSELECTION,TRUE,(LPARAM)szDir);
            }
            break;

        }
    case BFFM_SELCHANGED:
        {
            // Set the status window to the currently selected path.
            if (SHGetPathFromIDList((LPITEMIDLIST) lp ,szDir))
            {
                SendMessage(hwnd,BFFM_SETSTATUSTEXT,0,(LPARAM)szDir);
            }
            break;
        }
    default:
        {
            break;
        }
    }
    return 0;
}
/////////////////////////////////////////////////////////////////////////////
// CECSetWorkFolderDlg dialog


CECSetWorkFolderDlg::CECSetWorkFolderDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CECSetWorkFolderDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECSetWorkFolderDlg)
    m_WorkFolder = _T("");
    //}}AFX_DATA_INIT
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}


void CECSetWorkFolderDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECSetWorkFolderDlg)
    DDX_Text(pDX, IDC_FOLDER_NAME, m_WorkFolder);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECSetWorkFolderDlg, CDialog)
//{{AFX_MSG_MAP(CECSetWorkFolderDlg)
ON_BN_CLICKED(IDC_FOLDER_BROWSE, OnFolderBrowse)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECSetWorkFolderDlg message handlers

BOOL CECSetWorkFolderDlg::OnInitDialog()
{
    CDialog::OnInitDialog();
    //
    //    Recover the current working folder from the registry
    //
    CEdit* pWorkingFolder = (CEdit*) GetDlgItem (IDC_FOLDER_NAME);
    ASSERT (pWorkingFolder != NULL);

    //
    //    Recover the working directory from the registry.
    //
    CString KeyName = SOLUTIONS_SERVER_KEY;
    CString ValueName = "ProjectDir";
    //
    CECRegistryTools reg;
    CString WorkFolder;
    WorkFolder = reg.GetStringValue(HKEY_CURRENT_USER ,KeyName ,ValueName);

    pWorkingFolder->SetWindowText(WorkFolder);


    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECSetWorkFolderDlg::OnFolderBrowse()
{
    CEdit* pWorkFolder = (CEdit*) GetDlgItem (IDC_FOLDER_NAME);
    ASSERT (pWorkFolder != NULL);

    CString WorkFolder;


    // Declare the BROWSEINFO which does the trick
    BROWSEINFO bi;
    TCHAR szDir[MAX_PATH];
    LPITEMIDLIST pidl;
    LPMALLOC pMalloc;

    if (SUCCEEDED(SHGetMalloc(&pMalloc)))
    {
        ZeroMemory(&bi,sizeof(bi));
        bi.hwndOwner = NULL;
        bi.pszDisplayName = 0;
        bi.pidlRoot = 0;
        bi.lpszTitle = "Please choose a directory you wish to export into;";
        bi.lpfn = BrowseCallback;
        bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_STATUSTEXT;

        // And browse for folders only
        pidl = SHBrowseForFolder(&bi);

        if (pidl)
        {
            if (SHGetPathFromIDList(pidl, szDir))
            {
                // Signal a successful directory capture by filling in szLocalDir
                WorkFolder = szDir;
            }

            // Free up the allocated memory
            pMalloc->Free(pidl);
            pMalloc->Release();
        }
    }

    pWorkFolder->SetWindowText(WorkFolder);
}

void CECSetWorkFolderDlg::OnOK()
{
    CEdit* pWorkingFolder = (CEdit*) GetDlgItem (IDC_FOLDER_NAME);
    ASSERT (pWorkingFolder != NULL);

    CString InitialFolder;
    pWorkingFolder->GetWindowText(InitialFolder);

    if(InitialFolder.IsEmpty())
    {
        AfxMessageBox("Please enter a folder name");
        return;
    }

    if(InitialFolder.GetAt(InitialFolder.GetLength()-1) != '\\')
    {
        InitialFolder += "\\";
    }

    //Check if the folder exists...
    CFileStatus fs;
    if(!CFile::GetStatus(InitialFolder+".", fs))
    {
        //if it doesn't ask whether to create it or not
        CString strMessage;
        AfxFormatString1(strMessage,IDS_CREATE_NEW_FOLDER,InitialFolder);
        if(AfxMessageBox(strMessage,MB_YESNO)==IDNO)
            return;

        //Create the directory
        if(!CreateDirectory(InitialFolder, NULL))
        {
            AfxFormatString1(strMessage,IDS_CANT_CREATE_FOLDER,InitialFolder);
            AfxMessageBox(strMessage,MB_OK);
            return;
        }
    }

    //
    //    Save the new working directory back into the registry
    //
    CString KeyName = SOLUTIONS_SERVER_KEY;
    CString ValueName = "ProjectDir";
    //
    CECRegistryTools reg;
    reg.SetStringValue (HKEY_CURRENT_USER ,KeyName ,ValueName ,InitialFolder.GetBuffer(0));

    CWnd *pMainWnd = AfxGetMainWnd();
    ASSERT(pMainWnd != NULL);
    pMainWnd->SendMessage(WMU_FIRE_GLOBAL_EVENT,STUDIO_SWITCH_PROJECTS);

    CDialog::OnOK();
}

BOOL CECSetWorkFolderDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";

    if (pHelpInfo->iCtrlId == IDC_STATIC)
        return true;

    CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);

    HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);

    return true;
}


