/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// CECProjectSettingsDlg.cpp : implementation file
//

#include "stdafx.h"
#pragma warning(disable:4786)
#include "CyrStAPI\CECComServer.h"
#include "CyrStAPI\CECComProject.h"
#include "CyrStAPI\CECComImpactProject.h"

#include "CECProjectSettingsDlg.h"
#include "..\..\Interfaces\CYRPlugIns.h"
#include "htmlhelp.h"

/*
#include "..\..\..\HTMLHelp\CYRImpact\Project_Settings_dialog_box.inl"
*/
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CECProjectSettingsDlg dialog


CECProjectSettingsDlg::CECProjectSettingsDlg(BOOL bIsCreating, CWnd* pParent /*=NULL*/)
        : CDialog(CECProjectSettingsDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CECProjectSettingsDlg)
    // NOTE: the ClassWizard will add member initialization here
    //}}AFX_DATA_INIT

    m_bIsCreating = bIsCreating;
    m_szHtmlHelpFile = getenv("CYRANO_HOME");
    m_szHtmlHelpFile += "\\Framework\\Help\\Commander.chm";
}


void CECProjectSettingsDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CECProjectSettingsDlg)
    // NOTE: the ClassWizard will add DDX and DDV calls here
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CECProjectSettingsDlg, CDialog)
//{{AFX_MSG_MAP(CECProjectSettingsDlg)
ON_BN_CLICKED(IDC_ADVANCED_BUTTON, OnAdvancedButton)
ON_WM_HELPINFO()
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CECProjectSettingsDlg message handlers

BOOL CECProjectSettingsDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    //Setup html help handler
    //
    // If we are editing the project settings load the current settings.
    //
    if (!m_bIsCreating)
    {
        RestoreSettings();
    }

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}


void CECProjectSettingsDlg::RestoreSettings()
{
    //
    // Get the active project
    //
    CECComProject* pProject = GetProject();
    ASSERT (pProject != NULL);

    //
    // Get handles to dlg controls
    //
    CEdit* pName = (CEdit*)GetDlgItem(IDC_IMPACT_PROJECT_NAME);
    ASSERT (pName != NULL);

    pName->SetReadOnly(TRUE);

    CEdit* pDesc = (CEdit*)GetDlgItem(IDC_IMPACT_PROJECT_DESC);
    ASSERT (pDesc != NULL);

    //
    // Set project name
    //
    CString name = pProject->GetName();

    pName->SetWindowText(name);

    //
    // Set Description
    //
    pDesc->SetWindowText(pProject->GetDescription());
    pDesc->SetReadOnly(TRUE);
}

void CECProjectSettingsDlg::OnOK()
{

    //
    // Get the project name and description from the dialog controls
    //
    CEdit* pName = (CEdit*)GetDlgItem(IDC_IMPACT_PROJECT_NAME);
    ASSERT (pName != NULL);

    CEdit* pDesc = (CEdit*)GetDlgItem(IDC_IMPACT_PROJECT_DESC);
    ASSERT (pDesc != NULL);

    CString strName;
    pName->GetWindowText(strName);

    CString strDesc;
    pDesc->GetWindowText(strDesc);

    if (m_bIsCreating)
    {

        //
        // Check for existing project name
        //
        BOOL bValid = ValidateProjectName(strName);

        //
        // Create the project and set the description
        //
        if (bValid)
        {
            //
            // Get Hold of the Current Server Object
            //
            CWnd* pMainWnd = AfxGetMainWnd();
            ASSERT (pMainWnd != NULL);

            CECComServer* pServer = (CECComServer*) pMainWnd->SendMessage (WMU_GET_SERVERPTR);
            ASSERT (pServer != NULL);

            CECComImpactProject impactProject(pServer, strName, strDesc);
        }

        else
        {
            AfxMessageBox("Invalid Project Name");
            return;
        }
    }
    CDialog::OnOK();
}

//
// ValidateProjectName
// ===================
//
// Check for existing project name
//
BOOL CECProjectSettingsDlg::ValidateProjectName(CString ProjectName)
{
    //
    // Get Hold of the Current Server Object
    //
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    CECComServer* pServer = (CECComServer*) pMainWnd->SendMessage (WMU_GET_SERVERPTR);
    ASSERT (pServer != NULL);

    char*       pProjList;

    //
    // Get the list of all projects
    //
    try
    {
        long lCount;
        CString szProjectList= pServer->GetProjects(lCount);

        pProjList = szProjectList.GetBuffer(0);
    }

    catch(CyrRepositoryException &e)
    {
        CString str1;
        str1.Format("where: %s what: %s",e.where(),e.what());
        AfxMessageBox(str1);
    }
    catch(...)
    {
        AfxMessageBox("Dunno");
    }

    BOOL bValid = TRUE;

    //
    // Tokenize the list and check for an existing project with the given name
    //
    char* pProjName = strtok (pProjList ,"|");
    while (pProjName != NULL)
    {
        if (pProjName == ProjectName)
        {
            bValid = FALSE;
            break;
        }

        pProjName = strtok (NULL ,"|");
    }

    return bValid;
}

//
// OnAdvancedButton
// ================
//
// Invoke the Build Settings Dialog. As this is housed inside the Generic
// DLL we shall fire a global event. To do this we must reserve an ID value
// in CYRPlugIns.h - currently STUDIO_CUSTOM_LINK_1
//
void CECProjectSettingsDlg::OnAdvancedButton()
{
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    pMainWnd->SendMessage (WMU_FIRE_GLOBAL_EVENT ,(LPARAM)STUDIO_CUSTOM_LINK_1);
}


//
//    GetProject
//
//    Recover the current project by sending a message to the main application window
//
CECComProject* CECProjectSettingsDlg::GetProject()
{
    CWnd* pMainWnd = AfxGetMainWnd();
    ASSERT (pMainWnd != NULL);

    CECComProject* pProject = (CECComProject*) pMainWnd->SendMessage (WMU_GET_PROJECT);
    ASSERT (pProject != NULL);

    return pProject;
}

BOOL CECProjectSettingsDlg::OnHelpInfo(HELPINFO* pHelpInfo)
{
    // Dialog not really used!
    /*
        CString contextHelp = m_szHtmlHelpFile+"::whatisthis.txt";
     
        if (pHelpInfo->iCtrlId == IDC_STATIC)
            return true;
     
        CWnd* pWnd = GetDlgItem(pHelpInfo->iCtrlId);
     
        HtmlHelp(pWnd->m_hWnd, contextHelp, HH_TP_HELP_CONTEXTMENU, (DWORD)(LPVOID)g_TopicIdArray);
    */
    return true;
}
