/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYREventSink.cpp
// ----------------
//
//    This is the implementation file for the CYREventSink class.
//
//=========================================================================

#include "stdafx.h"
#include "CYREventSink.h"
#include "resource.h"

//
//    Product includes
//
#include "..\..\Interfaces\CyrCtrls.h"
#include "..\..\Interfaces\CyrPlugins.h"
#include "..\..\Common\CECDLLFunctions.h"
#include "..\..\Common\CECCyranoHome.h"
#include "..\..\Common\CECTargetEnvironment.h"
#include "solutions.h"
#include "Server.h"

//
//    Project includes
//
#include "CYRTGEnvironmentDlg.h"
#include "CECWebExecDlg.h"
#include "CECInitiatorDlg.h"
#include "CYRSimpleTestWizard.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern  CECDLLFunctions*    pDllFunctions;

/////////////////////////////////////////////////////////////////////////////
// CYREventSink

CYREventSink::CYREventSink()
{
    // In release we have discovered that our extension DLL's behave in a different
    // manner to the debug builds. It appears that any member variables are not
    // persistant for the entirety of this class and unless static they lose their data
    // after leaving the constructor.

    // As a workaround we have found that if we set up a 16-byte char value ALL other
    // member variables then retain their values during this objects lifetime.

    // Don't ask me why - it's an MFC Release Build Memory Management thing ...
    (void) strcpy (EventSink, "CYRImpact");

    // Get the parent window for this event sink
    CWnd* pParentWnd = AfxGetMainWnd();
    if (pParentWnd != NULL)
    {
        m_hParent = pParentWnd->GetSafeHwnd();

        // Create a default window size
        if (m_hParent != NULL)
        {
            // Create a default window size
            CRect rect(0,0,0,0);
            Create(NULL, NULL, WS_CHILD, rect, pParentWnd, ID_IMPACT_EVENT_SINK);
        }
    }

    //
    //    Get CYRANO_HOME directory
    //
    CECCyranoHome CyranoHome;
    CyranoHome.GetCyranoHome(m_CyranoHome);
    m_pServer = (CYRServer*)AfxGetMainWnd()->SendMessage(WMU_GET_SERVERPTR);
}

CYREventSink::~CYREventSink()
{}



BEGIN_MESSAGE_MAP(CYREventSink, CWnd)
//{{AFX_MSG_MAP(CYREventSink)
ON_COMMAND(IDB_RECORDER, OnRecorder)
ON_COMMAND(IDB_DESIGNER, OnDesigner)
ON_MESSAGE(WMU_EXECUTE_WEB, OnExecuteWeb)
ON_MESSAGE(STUDIO_SWITCH_PROJECTS, OnSwitchProjects)
//}}AFX_MSG_MAP
ON_COMMAND(ID_NEW_SIMPLE_TEST, OnNewSimpleTest)
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CYREventSink message handlers

BOOL CYREventSink::IsEnabled(UINT nID)
{
    BOOL bState = TRUE;

    switch (nID)
    {
    case IDB_DESIGNER:
    case ID_NEW_SIMPLE_TEST:
        bState = m_pServer->IsThereAnOpenProject();
        break;
    case IDB_RECORDER:
    default:
        {
            bState = TRUE;
        }
    }
    return bState;
}

//
// OnRecorder
// ==========
//
// This function is called when trying to invoke the
// CYRANO Transaction Recorder
//
void CYREventSink::OnRecorder()
{
    CWaitCursor wc;

    CECTargetEnvironment te;
    te.InvokeRecorder();
}

//
// OnDesigner
// ==========
//
// This function is called when trying to invoke the
// CYRANO Transaction Designer
//
void CYREventSink::OnDesigner()
{
    CWaitCursor wc;

    CECTargetEnvironment te;
    te.InvokeModeller();
}

//
// OnSwitchProjects
// ================
//
// The function is called whenever switching projects
//
LRESULT CYREventSink::OnSwitchProjects(WPARAM,LPARAM)
{

    return 0;
}

LRESULT CYREventSink::OnExecuteWeb(WPARAM wParam, LPARAM)
{
    CString* pParams = (CString*) &wParam;

    int nPos = pParams->Find("|", 0);
    int nLen = pParams->GetLength();
    ;
    if (nPos == -1 || nLen ==0)
        return (LRESULT)FALSE;

    CString strProjName = pParams->Left(nPos);
    CString strTestName = pParams->Right(nLen - nPos -1);

    CECWebExecDlg dlg(strProjName, strTestName);
    int nRet = dlg.DoModal();
    if (nRet == IDOK)
    {
        BeginWaitCursor();
        CString strTest = dlg.GetTestToExecute();
        CECInitiatorDlg init( strTest);
        int nExec = init.DoModal();
        EndWaitCursor();

        if (nExec == IDOK)
        {
            CWnd* pMainWnd = AfxGetMainWnd();
            ASSERT (pMainWnd != NULL);

            CString szDll(SOLUTIONS_GENERIC_DLL);
            pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,WMU_REFRESH_TREE_VIEWS,(LPARAM)&szDll);
            return (LRESULT)TRUE;
        }


    }

    return (LRESULT)FALSE;
}

void CYREventSink::OnNewSimpleTest()
{
    CWnd *pMainWnd = AfxGetMainWnd();

    CYRSimpleTestWizardSheet dlg("Test Wizard",pMainWnd,0,"");
    if(dlg.DoModal()==ID_WIZFINISH)
    {
        CString szDll(SOLUTIONS_GENERIC_DLL);
        pMainWnd->SendMessage(WMU_FIRE_SPECIFIC_EVENT,WMU_REFRESH_TREE_VIEWS,(LPARAM)&szDll);
    }
}
