/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRTGEnvironmentDlg.cpp
// -----------------------
//
//    This is the implementation file for the CYRTGEnvironmentDlg class
//
//=========================================================================

// Include required header files
//
#include "stdafx.h"

#include "CYRTGEnvironmentDlg.h"
#include "..\..\Common\CECRegistryTools.h"
#include "..\..\Common\CECTargetEnvironment.h"
#include "..\..\Interfaces\CyrPlugIns.h"
#include "Solutions.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CYRTGEnvironmentDlg dialog


CYRTGEnvironmentDlg::CYRTGEnvironmentDlg(CWnd* pParent /*=NULL*/)
        : CDialog(CYRTGEnvironmentDlg::IDD, pParent)
{
    //{{AFX_DATA_INIT(CYRTGEnvironmentDlg)
    m_szStatic1 = _T("");
    //}}AFX_DATA_INIT
}



void CYRTGEnvironmentDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CYRTGEnvironmentDlg)
    DDX_Control(pDX, IDC_CONTEXT_COMBO, m_NewContext);
    DDX_Text(pDX, IDC_RUNTIME_STATIC1, m_szStatic1);
    //}}AFX_DATA_MAP
}



BEGIN_MESSAGE_MAP(CYRTGEnvironmentDlg, CDialog)
//{{AFX_MSG_MAP(CYRTGEnvironmentDlg)
//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CYRTGEnvironmentDlg message handlers

BOOL CYRTGEnvironmentDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Now set up the Text Statics with the current choice
    CString szContext = GetCurrentContext();

    if (szContext.IsEmpty())
    {
        m_szStatic1.Format(IDS_NOTARGETENVIRONMENT);
    }
    else
    {
        AfxFormatString1 (m_szStatic1 ,IDS_TARGETENVIRONMENT ,szContext);
    }

    //
    //    Fill out the combo box with all the other available contexts
    //
    CComboBox* pCombo = (CComboBox*) GetDlgItem(IDC_CONTEXT_COMBO);
    ASSERT (pCombo != NULL);

    CECTargetEnvironment TE;
    TE.GetTargets(*pCombo);

    //
    //    ...and pre-select the previous value if any
    //
    int SelectThis = pCombo->FindStringExact(0 ,szContext);
    if (SelectThis == CB_ERR) SelectThis = 0;

    pCombo->SetCurSel(SelectThis);

    UpdateData(FALSE);

    return TRUE;  // return TRUE unless you set the focus to a control
    // EXCEPTION: OCX Property Pages should return FALSE
}

//
// GetDatabaseContext
// ==================
//
// This function will determine whether we have chosen a database as our default context
//

CString CYRTGEnvironmentDlg::GetCurrentContext()
{
    CECTargetEnvironment TE;
    CString CurrentContext;
    TE.GetCurrentContext(CurrentContext);
    return CurrentContext;
}


//
// OnOk
// ====
//
// Rewrite the context in the registry
//
void CYRTGEnvironmentDlg::OnOK()
{
    // TODO: Add extra validation here

    UpdateData(TRUE);

    CString szContext;
    int     nContext = m_NewContext.GetCurSel();

    // Get the selected Text
    m_NewContext.GetLBText(nContext, szContext);

    // And set it in the registry
    CECTargetEnvironment TE;
    TE.SetCurrentContext(szContext);

    CDialog::OnOK();
}
