/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

/////////////////////////////////////////////////////////////////////////
//
// CYRWorkbenchDetector.cpp
// ------------------------
//
//    This is the implementation file for the CYRWorkbenchDetector class
//
//=========================================================================

#include "stdafx.h"
#include "CYRWorkbenchDetector.h"
#include "CYRTGEnvironmentDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CYRWorkbenchDetector

CYRWorkbenchDetector::CYRWorkbenchDetector()
{}


CYRWorkbenchDetector::~CYRWorkbenchDetector()
{}

/////////////////////////////////////////////////////////////////////////////
// CYRWorkbenchDetector message handlers

//
// IsWorkbenchInstalled
// ====================
//
// This function determines whether CYRANO Workbench is installed on the system
//


BOOL CYRWorkbenchDetector::IsWorkbenchInstalled()
{
    // Defines for the Registry Workbench Key Checker
    HKEY    hRegKey;
    char chTargetString [32];

    // Obtain the current context
    CYRTGEnvironmentDlg environment;

    CString szContext = environment.GetCurrentContext();

    // Convert the context into a Target ID.
    TARGETID TargetID = Tgt_GetTargetID (szContext.GetBuffer(0));

    // And if Unknown we automatically disable the facility
    if (TargetID == TARGET_UNKNOWN)
        return FALSE;

    // And get from the ID the displayed text.
    Tgt_GetTarget (TargetID ,chTargetString ,sizeof (chTargetString) ,TRUE);

    CString  szRegKey("SOFTWARE\\CYRANO\\CYRANO Workbench for ");
    szRegKey += chTargetString;

    // Assume that Workbench is not present on the system. We shall look this up in the
    // registry. If found we then check for the presence of the Workbench Client. If found
    // we assume that workbench is present and configured to run
    BOOL bWorkbenchInstalled = FALSE;

    // First lets open the key

    // As we only need read to query the key's presence to find out if installed
    if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_QUERY_VALUE, &hRegKey) == ERROR_SUCCESS)
    {
        CString szLocation = GetWorkbenchLocation();
        if (!szLocation.IsEmpty())
        {
            bWorkbenchInstalled = TRUE;
        }
    }
    return bWorkbenchInstalled;
}

//
// GetWorkbenchLocation
// ====================
//
// This function gets the actual location of the Workbench Client
//
CString CYRWorkbenchDetector::GetWorkbenchLocation()
{
    // Defines for the Registry Workbench Key Checker
    HKEY    hRegKey;
    char    dwKeyValueData[_MAX_PATH];
    DWORD   dwKeyLen = sizeof(dwKeyValueData);

    char chTargetString [32];

    // Obtain the current context
    CYRTGEnvironmentDlg environment;

    CString szContext = environment.GetCurrentContext();

    // Convert the context into a Target ID.
    TARGETID TargetID = Tgt_GetTargetID (szContext.GetBuffer(0));

    // And get from the ID the displayed text.
    Tgt_GetTarget (TargetID ,chTargetString ,sizeof (chTargetString) ,TRUE);

    char    szRegKey[] = "SOFTWARE\\CYRANO\\CYRANO Workbench for ";
    (void) strcat(szRegKey, chTargetString);

    // Find out the location from the registry entry
    CString szWorkbenchLocation(_T(""));
    CString szWorkbenchClient(_T(""));

    // First lets open the key

    // As we only need read to query the key's presence to find out if installed
    if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, szRegKey, 0, KEY_ALL_ACCESS, &hRegKey) == ERROR_SUCCESS)
    {
        // And now obtain the Value
        if (RegQueryValueEx(hRegKey, "PathName", 0, NULL, (unsigned char *) &dwKeyValueData, &dwKeyLen) == ERROR_SUCCESS)
        {
            szWorkbenchLocation = dwKeyValueData;

            // And load the executable name
            VERIFY(szWorkbenchClient.LoadString(IDS_WORKBENCH_EXE));

            // And make sure that the client exists in this directory
            CFileStatus FileStatus;
            if (!CFile::GetStatus(szWorkbenchLocation + "\\" + szWorkbenchClient, FileStatus))
            {
                // No Executable so return sfa
                szWorkbenchLocation.Empty();
            }
        }
    }
    return szWorkbenchLocation;
}
