/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TFakeRecords.h
#ifndef _CYR_TFAKERECORDS_H
#define _CYR_TFAKERECORDS_H
/**
 * Header file to define some 'fake' TRecords
 * these are intended as examples to those people defining a real file
 * format with some functionality.  The examples I have chosen are
 * part of the tof specification.
 **/
#include <string>
#ifdef unix
#elif defined (WIN32)
#pragma warning(disable:4786)
using std::string;
#else //unknown
#error Unknown Platform
#endif // unknown

// Local includes
#include "TRecord.h"
#include "TConnect.h"

/**
 * Define slightly more meaningful class to use for connections
 **/
class DConn : public TConnect
{
    const string _name;
public:
DConn(const string& name) : _name(name)
    {}

    string name() const
    {
        return _name;
    };
}
; // DConn


/**
 * Definition of Connect Record class
 **/
class TRcrdConn : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
TRcrdConn(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}
    // Accessors

    virtual int execute(TContext& ctxt);
    virtual TRecord* duplicate() const
    {
        return new TRcrdConn(*this);
    };
    // Mutators
}

; // TRcrdConn

/**
 * Definition of Language Record class
 **/
class TRcrdLang : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
TRcrdLang(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}
    // Accessors

    virtual int execute(TContext& ctxt);
    virtual TRecord* duplicate() const
    {
        return new TRcrdLang(*this);
    };
    // Mutators
}

; // TRcrdLang

/**
 * Definition of Disconnect Record class
 **/
class TRcrdDisc : public TRecord
{
public:
    static const AtvfKey KEY;
    // Constructors & Destructor
TRcrdDisc(const TParamRule* headRuleP) : TRecord(headRuleP)
    {}
    // Accessors

    virtual int execute(TContext&);
    virtual TRecord* duplicate() const
    {
        return new TRcrdDisc(*this);
    };
    // Mutators
}

; // TRcrdDisc

#endif // _CYR_TFAKERECORDS_H
