/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TParam.h
#ifndef CYR_TPARAM_H
#define CYR_TPARAM_H
/**
 * Object to represent a parameter
 **/
#ifdef WIN32
#pragma warning(disable:4786)
#endif
#include <string>
#include <map>
#include <vector>
#ifdef unix
#elif defined (__VMS)
#elif defined (WIN32)
using namespace std;
#else //unknown
#error Unknown Platform
#endif // unknown

#include "AtvfRecord.h"
#include "TParamRule.h"

// Forward Declarations...
class CRegExp;


// class definition
class TParam
{
    const TParamRule* _prmRP;
    vector<AtvfRecord*> _recs;
    vector<int> _ints;
    vector<TSegStr*> _segs;

    bool caseIndValMatch(const string& text, unsigned int idx) const;

public:
    // construtors & destructor
    TParam(const TParamRule* prmRP);
    virtual ~TParam();

    // accessors
    const AtvfRecRule* innerRule() const;
    const TParamRule* rule() const;
    const TOptions& options() const;
    bool hasAttr(const string& attr) const;
    string token() const;
    unsigned int numVals() const;
    string strVal(unsigned int idx = 0) const;
    int intVal(unsigned int idx = 0) const;
    int intVal(const string& text) const;
    bool findInVal(const string& text, bool matchCase = true) const;


    bool findInVal(CRegExp& regex, bool matchCase = true) const;

    const TSegStr* segStr(unsigned int idx = 0) const;
    const CConfigTable* config() const;

    // mutators
    AtvfRecord* setData(AtvfRecord* recP);


    AtvfRecord* deleteData(unsigned int idx);

    AtvfRecord* setVal(const string& val, unsigned int idx = 0);

    // Parameter name string constants
    static const string stDelay;
    static const string stConnid;
    static const string stSessid;
    static const string stCmdid;
    static const string stServer;
    static const string stUser;
    static const string stPasswd;
    static const string stSql;
    static const string stAutoFetch;
    static const string stName;
    static const string stValue;
    static const string stCount;
    static const string stCancel;
    static const string stMaxLong;
    static const string stBatchSize;
    static const string stIterations;
    static const string stDefered;
    static const string stAssign;
    static const string stFPrecision;
    static const string stParseOnly;
    static const string stParse;
    static const string stBinds;
    static const string stValues;
    static const string stRow;
    static const string stColumn;
    static const string stLogMsg;
    static const string stCode;
    static const string stAction;

    static const string stExpression;
    static const string stDepth;
    static const string stInitial;
    static const string stEndValue;
    static const string stStep;
    static const string stRetvar;
    static const string stOptions;
    static const string stDimensions;
    static const string stIvalue;
    static const string stLvalues;
    static const string stSvalues;
    static const string stSize;
    static const string stFIdent;
    static const string stFile;
    static const string stDestination;
    static const string stType;
    static const string stStatusvar;
    static const string stexpr;
    static const string stlabel;
    static const string stErrlab;
    static const string stTmolab;
    static const string stGolab;
    static const string stTarget;
    static const string stMode;
    static const string stFormat;
    static const string stParameters;
    static const string stTo;
    static const string stString;

}
; // TParam

ostream& operator<<(ostream& os, const TParam& p);

// map to hold these parameter objects referenced by key
typedef map<AtvfKey, TParam*, less<AtvfKey> > prmp_cont_t;

#endif // CYR_TPARAM_H
