/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TScript.h
#ifndef _CYR_TSCRIPT_H
#define _CYR_TSCRIPT_H

// Define some objects that we need to know exist
class TContext;

// local includes
#include "TRecHolder.h"
#include "TScriptRule.h"
#include "CECTofTxt.h"
#include <stack>

// type define a set of records for use as declarations
typedef set <TRecord*, less<TRecord*> > trecset_t;
typedef map <string, int, less<string> >    label_map_t;
// Map: subroutine name, pointer to object describing formal subroutine parameters.
typedef map <string, TtValList*>        subrtn_map_t;
typedef stack   <TRecord*>          rcrd_stack_t;

///////////////////////////////////////////////////////////////////////////////
// @class   TScript | This class is used to process a TOF file (transaction
//      script).
//
// @base    public | TRecHolder
//
class TScript : public TRecHolder
{
    // @access Private Members.
private:
    static const string _current_ver;
    const TScriptRule* _ruleP;
    TScriptHead* _headP;
    // @cmember Set to hold all script declaration TRecords.
    trecset_t _decls;
    label_map_t _labels;
    label_map_t _sublabels;
    string m_cur_subroutine;
    // @cmember Map of subroutine names/formal parameter details.
    subrtn_map_t _subroutines;
    rcrd_stack_t _openObjects;
    // TestCase Preparation
    bool m_have_valid_testcase;
    //@cmember vector of record_indexes of end test cases
    vector<int> m_endtestcase_list;


    // @access Public Members.
public:
    // @cmember Constructor.
    TScript(const TScriptRule* ruleP);
    // @cmember Constructor.
    TScript(const TScriptRule* ruleP, AtvfInput& in);
    // @cmember Destructor.
    virtual ~TScript();

    //Accessors
    TRecord* header() const;
    const trecset_t& declarations() const;
    const TOptions& options() const;
    virtual void save(ostream& out) const;
    int run(TContext& ctxt);
    int prepare(bool verbose = false);
    // @cmember Accessor function to get pointer to variable definition record.
    TRecord* GetVardef(const string& varnam) const;
    subrtn_map_t& Subroutines()
    {
        return _subroutines;
    }

    //Mutators
    void declarations(const trecset_t& decls);
    void AddLabel(const string& label, int recIndex);
    int GetLabelIndex(const string& label,const string& subroutine) const;
    rcrd_stack_t& OpenObjects()
    {
        return _openObjects;
    }
    void AddSubroutineLabel(const string& label, int recIndex);
    int GetSubroutineLabelIndex(const string& label) const;
    // @cmember Add entry subroutine name/parameters map.
    int AddSubroutineName(string subNam, TtValList* prmListP);
    inline string GetCurrentSubroutineName()const
    {
        return m_cur_subroutine;
    }
    ;//

    //TestCase Methods all here to make most of prepare statement
    inline void TestCaseStarted()
    {
        m_have_valid_testcase=true;
    }
    inline bool IsTestCaseStarted()const
    {
        return m_have_valid_testcase;
    }
    //@cmember add index of endtestcase, relies on insertion in numeric order
    void AddEndTestCase(int index);
    //@cmember Get the Index of the Next EndTestCase Statement based on current index in script
    int GetNextEndTestCaseIndex(int _cur_index);

}
; // End class TScript

#endif // _CYR_TSCRIPT_H
