/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// TScriptHead.cpp
/**
 * Method defintions for the TScriptHead class
 * This class holds all the header information for a Transaction
 * Script.
 **/

// Local includes
#include "TScriptHead.h"

/**
 * Constructors
 **/
TScriptHead::TScriptHead(TParamRule*)
        :   _verP(NULL)
{}

/**
 * Destructor
 **/
TScriptHead::~TScriptHead()
{
    delete _verP; // version string
}

/**
 * Accessor to find out the version of the tof file
 **/
string TScriptHead::version() const
{
    return *_verP;
}

/**
 * Overridden accessor to add useful info to description of header.
 **/
string TScriptHead::description() const
{
    return string("Tof version " + *_verP + " header");
}

/**
 * Overidden accessor to show script header is known even though
 * it doesn't have the header rule
 **/
bool TScriptHead::isKnown() const
{
    return true;
}

/**
 * Mutator to set the header part of the record,
 * The version string in this specific case
 **/
void TScriptHead::setVer(string* verP)
{
    delete _verP;
    _verP = verP;
}
