/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// MDB Return/Status Codes

#ifndef MDBSTATUS_H
#define MDBSTATUS_H

// Status text access function prototypes
const char *   mdb_stext(int);
const char *   mdb_ptext(int);

// status codes
#define MDB_FAIL        -1
#define MDB_OK          0
#define MDB_EOF         2
#define MDB_NULLNAME    3
#define MDB_FNF         4
#define MDB_PEND        5
#define MDB_DONE        6
#define MDB_RESDONE     7
#define MDB_CANCEL      8
#define MDB_START       9
#define MDB_FAILDONE    10
#define MDB_SLEEP       11
#define MDB_ABORT       12
#define MDB_COMPLETE    13
#define MDB_IDLE        14
#define MDB_ABORTING    99
#define MDB_EXCEPT      199

// Wait states for external events
#define MDB_VWAIT       15        /* waiting for variable value */
#define MDB_SWAIT       16        /* waiting for script name */
#define MDB_FOWAIT      17        /* waiting for open datafile */
#define MDB_FCWAIT      18        /* waiting for close datafile */
#define MDB_TSWAIT      19        /* waiting for timer start */
#define MDB_TEWAIT      20        /* waiting for timer end */
#define MDB_WMWAIT      21        /* waiting for write message */
#define MDB_SCWAIT      22        /* waiting for sql complete */
#define MDB_VSWAIT      23        /* waiting for set variable */
#define MDB_DELAY       24        /* initial delay */
#define MDB_SRSIG       25        /* got a script name signalled */
#define MDB_VRSIG       26        /* got a variable value signalled */

// MDB Return/Status Code Text

#define MDB_FAIL_TXT        "Fail"
#define MDB_OK_TXT          "Ok"
#define MDB_EOF_TXT         "End Of File"
#define MDB_NULLNAME_TXT    "Null Filename"
#define MDB_FNF_TXT         "File Not Found"
#define MDB_PEND_TXT        "Pending"
#define MDB_DONE_TXT        "Done"
#define MDB_RESDONE_TXT     "Results Done"
#define MDB_CANCEL_TXT      "Cancel"
#define MDB_START_TXT       "Starting"
#define MDB_FAILDONE_TXT    "Fail Done"
#define MDB_SLEEP_TXT       "Sleeping"
#define MDB_ABORT_TXT       "Aborting"
#define MDB_COMPLETE_TXT    "Complete"
#define MDB_IDLE_TXT        "Idle"
#define MDB_VWAIT_TXT       "Var Get Wait"
#define MDB_SWAIT_TXT       "Script Get Wait"
#define MDB_FOWAIT_TXT      "File Open Wait"
#define MDB_FCWAIT_TXT      "File Close Wait"
#define MDB_TSWAIT_TXT      "Timer Start Wait"
#define MDB_TEWAIT_TXT      "Timer Stop Wait"
#define MDB_WMWAIT_TXT      "Write Msg Wait"
#define MDB_SCWAIT_TXT      "Sql Comp Wait"
#define MDB_VSWAIT_TXT      "Var Set Wait"
#define MDB_DELAY_TXT       "Start Delay"
#define MDB_SRSIG_TXT       "Script Ret Sig"
#define MDB_VRSIG_TXT       "Var Ret Sig"
#define MDB_EXCEPT_TXT      "Exception"

// Transaction Object Execution Phases

// Initial Phase
#define PHASE_NONE          0

// For TCONNECT
#define LOGIN_CONNECT       201
#define LOGIN_CHECK         202

// For TDISCONNECT
#define LOGOUT_DISCONNECT   301
#define LOGOUT_CHECK        302

// For TLANG
#define LANGCMD_SEND_PREP   501
#define LANGCMD_SEND        502
#define LANGCMD_SEND_CHECK  503
#define LANGCMD_PARSE       510
#define LANGCMD_PARSE_CHECK 511

// For Results (TLANG etc)
#define RESULTS             601
#define RESULTS_CHECK       602

// For TFETCH (And TLANG result fetch etc)
#define FETCH               611
#define FETCH_CHECK         612

// Special phases for aborting a connection

#define CONABORT              1001
#define CONABORT_CANCEL_CHECK 1002
#define CONABORT_CLOSE        1003
#define CONABORT_CLOSE_CHECK  1004

// Phase code text
#define PHASE_NONE_TXT        "None"
#define LOGIN_CONNECT_TXT     "Connect"
#define LOGIN_CHECK_TXT       "Connect Check"
#define LOGOUT_DISCONNECT_TXT "Discon"
#define LOGOUT_CHECK_TXT      "Discon Check"
#define LANGCMD_SEND_PREP_TXT "Sql Prep"
#define LANGCMD_SEND_TXT      "Sql Send"
#define LANGCMD_SEND_CHECK_TXT "Sql Send Check"
#define LANGCMD_PARSE_TXT      "Sql Parse"
#define LANGCMD_PARSE_CHECK_TXT "Sql Parse Check"
#define RESULTS_TXT           "Results"
#define RESULTS_CHECK_TXT     "Results Check"
#define FETCH_TXT             "Fetch"
#define FETCH_CHECK_TXT       "Fetch Check"
#define CONABORT_TXT          "Abort"
#define CONABORT_CANCEL_CHECK_TXT "Abort Cancel Chk"
#define CONABORT_CLOSE_TXT        "Abort Close"
#define CONABORT_CLOSE_CHECK_TXT  "Abort Close Chk"


#endif // MDBSTATUS_H
