/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

// Cresultcol : implementation file

#include "rescol.h"
#include "mdbstl.h"

extern "C"
{
#include <stdlib.h>
#include <malloc.h>
}

/////////////////////////////////////////////////////////////////////////////
// Cresultset

// Default constructor
Cresultcol::Cresultcol()
{
    // Initialize member variables
    BSCLEAR(colname);
    colwid  = 32;
    nrows   = 0;
    rowarrsz= 0;
    rowdata = NULL;
}

// Constructor
Cresultcol::Cresultcol(char * name, int width)
{
    // Initialize member variables
    colname = name;
    colwid  = width;
    // Initialize member variables
    nrows   = 0;
    rowarrsz= 0;
    // Allocate column array of default initial size
    rowdata = (char **)malloc(ROWFAC * sizeof(char *));
    if(rowdata != NULL)
    {
        rowarrsz = ROWFAC;
    }
}

// Destructor
Cresultcol::~Cresultcol()
{
    if(rowdata)
    {
        int i;
        // Free row data
        for(i=0 ; i < nrows ; i++)
        {
            // Free result value
            free((void *)rowdata[i]);
        }
        free((void *)rowdata);
    }
}

char * Cresultcol::nextaddr(void)
{
    char **nrp;

    // If current array is full
    if(nrows >= rowarrsz)
    {
        // extend array by ROWFAC entries
        nrp = (char **)realloc((void *)rowdata,
                               (rowarrsz + ROWFAC) * sizeof(char *));
        // Extend ok
        if(nrp != NULL)
        {
            // Set pointer to possibly new array and set new size
            rowdata = nrp;
            rowarrsz += ROWFAC;
        }
        // Extend failed, return fail
        else
        {
            return NULL;
        }
    }
    // Allocate buffer for new value
    rowdata[nrows] = (char *)malloc(colwid + 2);
    *(rowdata[nrows]) = '\0';

    nrows++;
    // Return pointer to buffer allocated above
    return rowdata[nrows-1];
}

// Return defined width for column
int Cresultcol::getwidth()
{
    return colwid;
}

// Return display width for column
int Cresultcol::getdisplaywidth()
{
    // If the column name is wider than the data field return the
    // former instead.
    return (int)colname.length() > colwid ? (int)colname.length() : colwid;
}

// Return result value in specified row
char * Cresultcol::getvalue(int row)
{
    static char *invrow="<Invalid Row>";

    if(row > nrows)
    {
        // Return a string indicating invalid row index
        return invrow;
    }
    return rowdata[row-1];
}

// Return the name of this column
char * Cresultcol::getname(void)
{
    return((char *)colname.c_str());
}
