/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/
#include "CORBARepository.h"
#include "CECMutex.h"
extern CORBAServer*     pCServer;//This has become a Global variable due to
//compilation problems if included in some headers
CECMutex::CECMutex(string name,int scope):m_key(0)
{
    switch(scope)
    {
    case TOF_MTXSCP_TESTWIDE:
        name=Repository->GetPrimaryTestRunString()+"."+name+".TEST-WIDE";
        break;
    case TOF_MTXSCP_LOCAL:
    default:
        name=Repository->GetTestRunString()+"."+name+".LOCAL";
        break;
    }
    CORBA::Object_var obj;
    try
    {
        obj=pCServer->GetObjectFromClass(TPMutex::CLASS_NAME,name.c_str());
    }
    catch(...)//no mutex of that name available so get one from TestManager Variable Factory
    {
        string factory_regname;
        if(scope == TOF_MTXSCP_TESTWIDE)//Test-Wide
        {
            factory_regname=Repository->GetPrimaryTestRunString();
        }
        else//Local
        {
            factory_regname=Repository->GetTestRunString();
        }
        CORBA::Object_var Ofac;
        Ofac=pCServer->GetObjectFromClass(TMVarFactory::CLASS_NAME,factory_regname.c_str());
        TMVarFactory_var TMVF=narrow_object((TMVarFactory_ptr)NULL,Ofac.inout());
        CORBA::String_var cb_name=CORBA::string_dup(name.c_str());
        obj=TMVF->CreateMutex(cb_name.in());

    }
    m_mutex=narrow_object((TPMutex_ptr)NULL,obj.inout());
}
CECMutex::~CECMutex()
{}

bool CECMutex::lock()
{
    try
    {
        m_key=m_mutex->lock();
    }
    catch(...)
    {
        return false;
    }
    return true;
}
bool CECMutex::unlock()
{
    if(m_key==0)//haven't ever locked it
        return false;
    try
    {
        m_mutex->unlock(m_key);
    }
    catch(...)
    {
        return false;
    }
    return true;
}
bool CECMutex::trylock()
{
    try
    {
        m_key=m_mutex->trylock();
    }
    catch(...)
    {
        return false;
    }
    if(m_key==-1)//failed to lock
        return false;
    else
        return true;
}
