/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/

#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
#include "TEThread.h"

#include "TExecuterRcrd.h"
#include "TExecuterRcrd.inl"
///////////////////////////////////////////////////////////////////////////////
// PREPARE method for Call Script Record
// ================================
///////////////////////////////////////////////////////////////////////////////
int TRcrdCscr::prepare(TScript* script, int objIndex, bool verbose)
{
    // Parameters must be parsed ready fo execute (where they can be evaluated)
    // errlab must exist
    // retvar should be declared (should be able to check this now as
    // all prepares are done after reading in all TRecords
    //parse parameters
    const TParam* prmP = param(TParam::stParameters);
    string l_parameters = prmP->strVal();

    _prmListP = new TtValList(script);
    if (!_prmListP->Parse(l_parameters , 0))
    {
        MSGERR ("Call Script: parameters: " + l_parameters +
                " Failed during PREPARE of parameters with: " + _prmListP->Reason());
        return -1;  // Return 'failure' status
    }

    _labelIndex=PrepareGoto(this,TParam::stErrlab,script,false);
    return 0;
}
///////////////////////////////////////////////////////////////////////////////
// EXECUTE method for Call Script Record
// ================================
///////////////////////////////////////////////////////////////////////////////
int TRcrdCscr::execute(TContext& ctxt)
{
    ctxt.set_phase_text("Call Script");
    //Cast context to be TE thread
    TEThread* TEctxt=static_cast<TEThread*>(&ctxt);
    //if not cast safely then this is definitely NOT a test Executer
    if(!TEctxt)
        throw mdbParseException("Invalid TestExecuter Execution Method Called");
    string realscriptname=strPrmVal(TParam::stName, ctxt);
    if(realscriptname=="")
        throw mdbParseException("NULL Script Name Called");
    logr->AddTrace(DBGSTD,DETAILLEVEL,"Lock TtValList Mutex");
    TtValList *PList=NULL;
    {
        CTLockGuard<CTMutex> guard(m_mutex);
        PList=new TtValList(*_prmListP);//make local copy to give to thread
        logr->AddTrace(DBGSTD,DETAILLEVEL,"Try TtValList Mutex Unlocked");
    }
    logr->AddTrace(DBGSTD,DETAILLEVEL,"TtValList Mutex Unlocked");
    int retvar=TEctxt->CallScript(realscriptname,*PList);
    if(PList)delete PList;
    //After call script Thread is in state it was before
    //set the return variable to be the current return value of thread
    string return_var_name = strPrmVal(TParam::stRetvar, ctxt);
    if(return_var_name!="")//return values used
    {
        retvar=ctxt.set_variable(return_var_name, ctxt.GetCurrentReturnValue());
    }
    //if error goto error label if there is one
    int destination=0;
    if(retvar==-1)
    {
        destination=ExecuteGoto(this,ctxt,TParam::stErrlab,&_labelIndex,&m_mutex);//if no error label pass onto ON ERROR
        if(destination==0)//no jump
        {
            string msg=TEctxt->GetLastError();
            throw mdbParseException("Error in Called Script: "+msg);
        }
    }
    else
        destination=0;
    ctxt.set_phase_text(TP_PHASE_NONE);
    return destination;
}
///////////////////////////////////////////////////////////////////////////////
// PREPARE method for Entry Record
// ================================
///////////////////////////////////////////////////////////////////////////////
int TRcrdEntry::prepare(TScript* script, int objIndex, bool verbose)
{
    const TParam* prmP = param(TParam::stParameters);
    string l_parameters = prmP->strVal();

    _prmListP = new TtValList(script);
    if (!_prmListP->Parse(l_parameters , 0))
    {
        MSGERR ("Entry: parameters: " + l_parameters +
                " Failed during PREPARE of parameters with: " + _prmListP->Reason());
        return -1;  // Return 'failure' status
    }

    return 0;
}
///////////////////////////////////////////////////////////////////////////////
// EXECUTE method for Entry Record
// ================================
///////////////////////////////////////////////////////////////////////////////
int TRcrdEntry::execute(TContext& ctxt)
{
    ctxt.set_phase_text("Entry");
    //Cast context to be TE thread
    TEThread* TEctxt=static_cast<TEThread*>(&ctxt);
    //if not cast safely then this is NOT a test Executer
    if(!TEctxt)
        throw mdbParseException("Invalid TestExecuter Execution Method Called");
    {
        CTLockGuard<CTMutex> guard(m_mutex);
        TtValList PList(*_prmListP);//make local copy to give to thread
        TEctxt->Entry(PList);
    }//End of mutex guard
    ctxt.set_phase_text(PHASE_NONE);
    return 0;
}
