/* Copyright (C) 2000 Cyrano SA
 
   This file is part of OpenSTA
 
   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.
 
   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.
 
   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html
 
   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


///////////////////////////////////////////////////////////////////////////////
// @doc     TMThread - Thread Implementation for TestManager
///////////////////////////////////////////////////////////////////////////////
#ifdef WIN32
#pragma warning(disable:4786)
#pragma warning(disable:4503)
#endif
#include "CORBARepository.h"
#include <CORBA/CORBAServer.h>
#include "TMThread.h"
#include "Objects/SpawnProc.h"
#include "TestInitiator.h"
#include "TContext.h"
#include "TMthreadcontrol.h"
#include "ociphases.h"
#ifdef HPUX
#include <iostream.h>
#else
#include <iostream>
#endif
extern CORBAServer*    pCServer;
extern TestManager_ptr      TManager_ptr;
//Global Thread Control Object

TScriptRule* TMThread::thrule=NULL;//("ThreadDefinition.odl");
ScriptCache* TMThread::ThreadCache=NULL;//(&TMThread::thrule);
CTMutex TMThread::ThrAccessMtx;
TMThread::ThreadMap_t TMThread::ThreadMap;

TMThread::TMThread(TMthreadcontrol* threadcontrol,string filename):
        Cmdbthread(threadcontrol,filename)
        ,m_TestLevel(TestLevel::_nil())
        ,m_ParentTestManager(TestManager::_nil())
        ,m_ParentTestLevel(TestLevel::_nil())
        ,m_TestLevelImpl(NULL)
        ,m_tmthreadcontrol(threadcontrol)
{
    localThreadMap.clear();
}
TMThread::~TMThread()
{
    //CleanUp();
}

void TMThread::SetThreadInfo(TestLevel_ptr TL,TestManager_ptr TM)
{
    m_ParentTestManager=TM;
    m_ParentTestLevel=TL;
    Initialise();
}
int TMThread::TestExecution(string name,string node)
{
    if(node==""||node=="<local>")
    {
        set_phase_text(PHASE_EXECUTE_L);
        // Get the Test file 'name'.tof from the repository and copied
        // to local disk space. Full path of the file is returned.
        string TestFilename = Repository->GetTofFile(name);

        // Attach a new thread for this TOF file and commence execution of it
        m_tmthreadcontrol->AttachThread(TestFilename.c_str(),m_TestLevel);
    }
    else
    {
        set_phase_text(PHASE_EXECUTE_R);
        string strmsg="Execute Remote Test "+name+" on Node "+node;
        logr->AddTrace(DBGSTD,DEBUGLEVEL,strmsg.c_str());
        TestInitiator* TI=new TestInitiator(Repository->GetProjectName(),name,Repository->GetNodeDefinition(node),pCServer);
        TI->SetRepositoryInformation(Repository->GetDataServerNodeName(),Repository->GetUserName(),Repository->GetPassword());
        TI->CreateTestManager(TestLevel::_duplicate(m_TestLevel),Repository->GetTestRunString());//Create new TestManager with current TestLevel as parent TBD
        TestManager_var TM=TI->GetTestManager();
        TManager_ptr->AddSubTestManager(TM.inout());
        TI->ExecuteTest();//Execute Remote Test
        delete TI;
    }
    return 0;

}// End of TContext::TestExecution
///////////////////////////////////////////////////////////////////////////////
// ThreadExecution
// ===============
///////// //////////////////////////////////////////////////////////////////////
// @mfunc
// @parm string | name |
// @rdesc
int TMThread::ThreadExecution(string threadname,string scriptname,string nodename,unsigned int number_of_threads,unsigned int interval_btw_threads,unsigned int delay_before_first_thread)
{
    if(number_of_threads<1)number_of_threads=1;//if no threads specified make sure at least 1
    ThreadMap_t::iterator i=localThreadMap.find(threadname);
    if(i==localThreadMap.end())return -1;//or throw some exception
    //
    // LINUX Port Changes
    //
    // Cannot use the arrow operator on this object
    ThreadDef thd=(*i).second;//make local copy of ThreadDef
    //deal with delay first
    delay_before_first_thread*=1000;//convert to milliseconds
    CyrSleep(delay_before_first_thread);
    CORBA::Long  status;
    interval_btw_threads*=1000;//convert interval to milliseconds
    for(unsigned int threadno=0;threadno<number_of_threads;threadno++)
    {
        status=m_TestLevelImpl->ExecuteThreadScript(scriptname.c_str(),nodename.c_str(),thd,m_TestLevel.inout());
        CyrSleep(interval_btw_threads);//wait between starting thread
    }

    return 0;

}//end of TContext::ThreadExecution
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Clear up this thread on exit from script with/without keep alive options set
// @parm    true - with keep alive option set
int TMThread::ExitCommand(bool keepalive,const string& retvar)
{
    if(m_ContextDataStack.top()->exit_called_flag==true)
        return 0;//Have already exited don't try and do it again
    //Deregister ScriptObject from TestManagerVariableFactories for
    string Pfactory_regname=Repository->GetPrimaryTestRunString();
    string factory_regname=Repository->GetTestRunString();
    CORBA::Object_var obj;
    obj=pCServer->GetObjectFromClass(TMVarFactory::CLASS_NAME,Pfactory_regname.c_str());
    TMVarFactory_var TMVF=narrow_object((TMVarFactory_ptr)NULL,obj.inout());
    CORBA::String_var cb_scriptname=CORBA::string_dup(ScriptName().c_str());
    TMVF->DeregisterScript(cb_scriptname.in(),keepalive);
    if(Pfactory_regname!=factory_regname)//if the two TestManager Factories are not the same one
    {
        obj=pCServer->GetObjectFromClass(TMVarFactory::CLASS_NAME,factory_regname.c_str());
        TMVF=narrow_object((TMVarFactory_ptr)NULL,obj.inout());
        TMVF->DeregisterScript(cb_scriptname.in(),keepalive);
    }
    TContext::ExitCommand(keepalive,retvar);
    m_ContextDataStack.top()->exit_called_flag=true;//have explicitly called exit in Script

    return 0;//Got here without throwing an exception so must be successful
}

void TMThread::Initialise()
{
    string scriptname=strip_extension(filename_base(get_file()));
    CECInetInfo info;
    string Node=info.GetIPAddressAsString();
    if(!CORBA::is_nil(m_ParentTestManager))
    {
        MakeTestLevel(m_ParentTestManager,scriptname,Node,m_ParentTestLevel);
    }
    else
    {
        MakeTestLevel(m_ParentTestLevel,scriptname,Node);
    }

}
void TMThread::MakeTestLevel(TestManager_ptr TM,string scriptname,string Node,TestLevel_ptr pTL)
{
    set_phase_text("MakeTestLevel");
    if(m_TestLevelImpl)
        throw mdbRuntimeError("Cannot kick off more than 1 test from TestManager");
    //if object already has been used then somehow
    //Execute Thread has been called on TestManager Twice , throw exception
    m_TestLevelImpl=new TestLevelImpl(scriptname,Node,pTL,m_ThreadID);
    pCServer->ReadyObject(m_TestLevelImpl);
    m_TestLevel=m_TestLevelImpl->_this();
    TM->SetTestLevel(m_TestLevel.inout());
    if(!CORBA::is_nil(pTL))//If we also have a parent TestLevel for this (i.e. is first Level in Distributed TM)
    {
        m_ParentTestLevel=TestLevel::_duplicate(pTL);
        m_ParentTestLevel->AddChildTestLevel(m_TestLevel.inout());
    }
    set_phase_text(TP_PHASE_NONE);
}
void TMThread::MakeTestLevel(TestLevel_ptr TL,string scriptname,string Node)
{
    //if this thread has already been used then dispose of old object
    //remove from old parent and create new object for new parent
    set_phase_text("MakeTestLevel");
    if(!CORBA::is_nil(m_TestLevel))
    {
        m_ParentTestLevel->RemoveChildTestLevel(m_TestLevel.inout());
        m_TestLevel=TestLevel::_nil();
        m_TestLevelImpl->_dispose();
        m_TestLevelImpl=NULL;
    }
    m_TestLevelImpl=new TestLevelImpl(scriptname,Node,TL,m_ThreadID);
    pCServer->ReadyObject(m_TestLevelImpl);
    m_TestLevel=m_TestLevelImpl->_this();
    m_ParentTestLevel=TestLevel::_duplicate(TL);
    m_ParentTestLevel->AddChildTestLevel(m_TestLevel.inout());
    set_phase_text(TP_PHASE_NONE);
}
void TMThread::CleanUp()
{
    set_phase_text("DestroyTestLevel");
    logr->AddTrace(DBGSTD,DEBUGLEVEL,"Try destroying TestLevel");
    if(!CORBA::is_nil(m_TestLevel))
    {
        logr->AddTrace(DBGSTD,DEBUGLEVEL,"m_TestLevel is not _nil, wait for empty");
        //Do not clear up until lower levels are cleared up then set to close down!
        if(m_TestLevelImpl)
        {
            m_TestLevelImpl->WaitForEmpty();
        }
        string msg="End Test "+get_test_name();
        Repository->LogMessage(msg,AUDITMSG);
        logr->AddTrace(DBGSTD,DEBUGLEVEL,"Now empty");
        if(!CORBA::is_nil(m_ParentTestManager))
        {
            m_ParentTestManager->SetTestLevel(TestLevel_var(TestLevel::_nil()));
            CORBA::release(m_ParentTestManager);
            m_ParentTestManager=TestManager::_nil();
            logr->AddTrace(DBGSTD,DEBUGLEVEL,"m_ParentTestManager updated");
        }
        if(!CORBA::is_nil(m_ParentTestLevel))
        {
            m_ParentTestLevel->RemoveChildTestLevel(m_TestLevel.inout());
            logr->AddTrace(DBGSTD,DEBUGLEVEL,"m_ParentTestLevel updated");
            CORBA::release(m_ParentTestLevel);
            m_ParentTestLevel=TestLevel::_nil();
        }
        logr->AddTrace(DBGSTD,DEBUGLEVEL,"and NULL Impl ptr");
        if(m_TestLevelImpl)
        {
            m_TestLevelImpl->SetToCloseDown();
            m_TestLevelImpl->_dispose();
            m_TestLevelImpl=NULL;
        }
        else
        {
            logr->AddTrace(DBGSTD,DEBUGLEVEL,"NULL Impl");
        }
        logr->AddTrace(DBGSTD,DEBUGLEVEL,"_nil this TestLevel");
        m_TestLevel=TestLevel::_nil();


    }
    logr->AddTrace(DBGSTD,DEBUGLEVEL,"Leave now");
    set_phase_text(TP_PHASE_NONE);
}
void TMThread::SetNoteText(const string& mesg)
{
    m_TestLevelImpl->SetNoteText(mesg);
}

///////////////////////////////////////////////////////////////////////////////
// @mfunc   deal with Thread declarations then pass on to TContext::declarations
//      to deal with standard declarations (e.g. Vardefs)
// @parm    0 for success
int  TMThread::declarations(trecset_t _decls)
{
    logr->AddTrace(DBGSTD,DEBUGLEVEL+500,"Got in that bit that has to do with declarations");
    for(trecset_t::iterator it=_decls.begin() ; it != _decls.end() ; it++)
    {
        // NOTE: case insensitive compare as long as constants (e.g. DECL_VARDEF)
        // are uppercase;
        string strtkn=(*it)->token();
        string_toupper(strtkn);
        if(strtkn == DECL_THRDEF)
        {
            string  l_thname;

            const TParam * par = (*it)->param("name");
            if(par != NULL)
                l_thname=par->strVal();
            else
                l_thname="";
            const TParam* prmPtarget =  (*it)->param("target");
            int l_target;
            if(prmPtarget != NULL)
                l_target=prmPtarget->intVal();
            else
                l_target=0;

            if(l_target == 0) //No target specified so use Global thread defintions from repository
            {
                // lock mutex for insertion of new ThreadDefinition
                // update the cache and makeTask for the file
                CTLockGuard<CTMutex> guard(ThrAccessMtx);//lock prepare method, use for global cache
                ThreadMap_t::iterator i=ThreadMap.find(l_thname);//do we have this thread def
                if(i==ThreadMap.end())//if not there copy over file and read in thread def
                {
                    logr->AddTrace(DBGSTD,DEBUGLEVEL+500,string("Retrieve file "+l_thname).c_str());
                    string ThreadFilename = Repository->GetThreadFile(l_thname);
                    logr->AddTrace(DBGSTD,DEBUGLEVEL+500,string("Retrieve file "+ThreadFilename).c_str());
                    // Make a task, this will call the prepare method on the thread definition
                    // record and set up a thread object!
                    ThreadTask* ThrDefTask=ThreadCache->makeTask(ThreadFilename);
                    delete ThrDefTask;
                    //now we have inserted new thrdef set the interator to point to it
                    i=ThreadMap.find(l_thname);
                    if(i==ThreadMap.end())return -1;//or throw some exception
                }
                //
                // LINUX Port Changes
                //
                // Cannot use the arrow operator on this object

                ThreadDef thrdef=(*i).second;//make local copy of ThreadDef
                //copy into local map of thread defs for this thread
                localThreadMap.insert(TMThread::ThreadMap_t::value_type(l_thname,thrdef));
                //now the thread def has been read in we could
            }

            else
            {
                //insert directly into local map
                ThreadDef thrdef;
                thrdef.ThreadName=l_thname.c_str();
                thrdef.Target=l_target;
                localThreadMap.insert(TMThread::ThreadMap_t::value_type(l_thname,thrdef));
            }
        }
    }
    return TContext::declarations(_decls);
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Any Set up required specifically for this class of thread at end of a script
// @parm    Name of Script Being started
void    TMThread::StartScript(const string& scriptname)
{
    m_TestLevelImpl->ScriptName(scriptname);
    m_ScriptNameStack.push(scriptname);
    m_ContextDataStack.top()->exit_called_flag=false;
    //Register script for Variable factory
    string Pfactory_regname=Repository->GetPrimaryTestRunString();
    string factory_regname=Repository->GetTestRunString();
    CORBA::Object_var obj;
    obj=pCServer->GetObjectFromClass(TMVarFactory::CLASS_NAME,Pfactory_regname.c_str());
    TMVarFactory_var TMVF=narrow_object((TMVarFactory_ptr)NULL,obj.inout());
    CORBA::String_var cb_scriptname=CORBA::string_dup(ScriptName().c_str());
    TMVF->RegisterScript(cb_scriptname.in());
    if(Pfactory_regname!=factory_regname)//if the two TestManager Factories are not the same one
    {
        obj=pCServer->GetObjectFromClass(TMVarFactory::CLASS_NAME,factory_regname.c_str());
        TMVF=narrow_object((TMVarFactory_ptr)NULL,obj.inout());
        TMVF->RegisterScript(cb_scriptname.in());
    }
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Any Clear up required specifically for this class of thread at end of a script
void    TMThread::EndScript()
{
    m_TestLevelImpl->ScriptName(ScriptName());
    //at end of script default to ending without keepalive option
    if(!m_ContextDataStack.top()->exit_called_flag)ExitCommand(false,"");
    if(m_ScriptNameStack.size()>1)//if still have scripts to return to remove scriptname from stack,should never occur in TestManager||
    {
        m_ScriptNameStack.pop();
    }
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Is named test running
// @rdesc   true if running
// @parm    name of test
bool TMThread::isTestRunning(const string& testname)
{
    return m_TestLevelImpl->isTestRunning(testname);
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Is named Thread running
// @rdesc   true if running
// @parm    name of Thread
bool TMThread::isThreadRunning(const string& threadname)
{
    return m_TestLevelImpl->isThreadRunning(threadname);
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Get Name of test Currently running (not script)
// @rdesc   Name of Test Running
string TMThread::get_test_name(void)
{
    static string TestName("");
    try
    {
        TestName=m_TestLevel->TestName();
    }
    catch(const CyrClosingDownException&)
    {
        //if closing down return the stored test name
    }

    return TestName;
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc Log a message saying that a thread is starting
void TMThread::LogStartMessage()
{
    string msg="Start Test "+get_test_name();
    Repository->LogMessage(msg,AUDITMSG);
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc Log a message saying that a thread is ending
void TMThread::LogEndMessage()
{
    //this message is done in CleanUp as otherwise the text will
    // appear before threads have finished if no WaitForThread is present
    //string msg="End Test "+get_test_name();
    //Repository->LogMessage(msg,AUDITMSG);
}
///////////////////////////////////////////////////////////////////////////////
// @mfunc   Terminates the current test case, as either passed or failed depending
//      on the status of the flag passed in.
//
// @parm    int | result | Flag determining the state of termination
//      (TOF_TCTRES_PASS/FAIL)
// @parm    description of TestCase passed or failed (taken from start test case
//      record at prepare time).
// @rdesc   integer 0 if the test case is terminated successfully, otherwise false.

int TMThread::TCterminate(int result,string desc)
{
    if(result==TOF_TCTRES_FAIL)
        m_TestLevelImpl->AddTestCase(FAILED);
    else
        m_TestLevelImpl->AddTestCase(PASSED);
    return TContext::TCterminate(result,desc);
}
