/* Copyright (C) 2000 Cyrano SA

   This file is part of OpenSTA

   OpenSTA is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free
   Software Foundation; either version 2, or (at your option) any later
   version.

   OpenSTA is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
   for more details.

   You should have received a copy of the GNU General Public License along
   with OpenSTA; see the file COPYING.  If not, write to the Free Software
   Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

   For more information about the GNU Public license see:
   http://www.gnu.org/copyleft/gpl.html

   OpenSTA is at:
   http://www.opensta.org/
   http://opensta.sourceforge.net/
*/


%{
#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include "Tof2Scl.cpp.h"
#ifdef _DEBUG
#include "Windows.h"
#endif
bool bResetSC=false;
bool bInloadResponseInfo=false;

%}

%x StringIsWord
%x StringIsLine
%x QuotedString
%x Integer

%%

^CONNECT {
#ifdef _DEBUG
	OutputDebugString("Connect\n");
#endif
	return CONNECT;
}

^DISCONNECT	{
#ifdef _DEBUG
	OutputDebugString("Disconnect\n");
#endif
	return DISCONNECT;
}

^WAIT {
#ifdef _DEBUG
	OutputDebugString("Wait\n");
#endif
	return WAIT;
}

^REQUEST {
#ifdef _DEBUG
	OutputDebugString("Request\n");
#endif
	return REQUEST;
}

^"COMMENT ~~~" {
#ifdef _DEBUG
	OutputDebugString("Commentt\n");
#endif
	BEGIN StringIsLine;
	bResetSC=false;
	return COMMENT;
}

^LLOAD {
#ifdef _DEBUG
	OutputDebugString("Load Response Info\n");
#endif
	return LOAD_RESPONSE_INFO;
}

^TYPE {
#ifdef _DEBUG
	OutputDebugString("Type\n");
#endif

	BEGIN Integer;
	return _TYPE;
}

^IDENTIFIER {
#ifdef _DEBUG
	OutputDebugString("Identifier\n");
#endif

	BEGIN QuotedString;
	bResetSC=true;
	return IDENTIFIER;
}

^INTO {
#ifdef _DEBUG
	OutputDebugString("Into\n");
#endif

	BEGIN StringIsWord;
	return INTO;
}

^TDELAY	{
#ifdef _DEBUG
	OutputDebugString("TDELAY\n");
#endif
	BEGIN Integer;
	return TDELAY;
}

^STIMER {
#ifdef _DEBUG
	OutputDebugString("STIMER\n");
#endif
	return STIMER;
}

^ETIMER {
#ifdef _DEBUG
	OutputDebugString("ETIMER\n");
#endif
	return ETIMER;
}

^DATA {
#ifdef _DEBUG
	OutputDebugString("DATA\n");
#endif
	BEGIN Integer;
	return DATA;
}

^CONID {
#ifdef _DEBUG
	OutputDebugString("CONID\n");
#endif

	BEGIN Integer;
	return CONID;
}

^SERVER {
#ifdef _DEBUG
	OutputDebugString("Server\n");
#endif

	BEGIN QuotedString;
	bResetSC=true;
	return SERVER;
}

^HEADER {
#ifdef _DEBUG
	OutputDebugString("Header\n");
#endif
	bResetSC = false;
	return HEADER;
}

^BODY {
#ifdef _DEBUG
	OutputDebugString("Body\n");
#endif
	return BODY;
}

^URI { 
#ifdef _DEBUG
	OutputDebugString("URI\n");
#endif
	BEGIN QuotedString;
	bResetSC = true;
	return URI;
}

<QuotedString,StringIsLine>~~~ {
#ifdef _DEBUG
	OutputDebugString("<Line>~~~\n");
#endif

	BEGIN 0;
	return SECTION_MARKER;
}

~~~ {
#ifdef _DEBUG
	OutputDebugString("~~~\n");
#endif
	BEGIN QuotedString;
	return SECTION_MARKER;
}

^VARDEF {
#ifdef _DEBUG
	OutputDebugString("vardef\n");
#endif
	return VARDEF;	
}

^TIMDEF {
#ifdef _DEBUG
	OutputDebugString("timdef\n");
#endif
	return TIMDEF;
}

^NAME {
#ifdef _DEBUG
	OutputDebugString("name\n");
#endif
	BEGIN StringIsWord;
	return NAME;	
}

^OPTION[ ] {
#ifdef _DEBUG
	OutputDebugString("Option\n");
#endif
	BEGIN Integer;
	return OPTION;
}

^OPTIONS {
#ifdef _DEBUG
	OutputDebugString("Options\n");
#endif
	BEGIN Integer;
	return OPTIONS;
}

^SIZE {
#ifdef _DEBUG
	OutputDebugString("size\n");
#endif
	BEGIN Integer;
	return _SIZE;
}

^SYNCHRONIZE {
#ifdef _DEBUG
	OutputDebugString("Synchronize\n");
#endif
	return _SYNCHRONIZE;
}

<Integer>[0-9]+ {
#ifdef _DEBUG
	OutputDebugString("Integer\n");
#endif
	/*Match an integer*/
	BEGIN 0;
	Tof2Scllval.integer_type = atoi(yytext);
	return INTEGER;
}

<StringIsWord>[^\n ]+ {		
#ifdef _DEBUG
	OutputDebugString("Word\n");
#endif
	BEGIN 0;
	/*Match everything except spaces and newline*/

	Tof2Scllval.string_type = _strdup(yytext);
	return STRING_WORD;
}

<StringIsLine>[^\n]+ {
#ifdef _DEBUG
	OutputDebugString("Line\n");
#endif

	bInloadResponseInfo=false;
	
	/* Match everything except ~~~ and newline*/
	if(bResetSC)
		BEGIN 0;

	char *pMark = strstr(yytext,"~~~");
	if(pMark)
	{
		int nResult = pMark - yytext;
		yyless(nResult);
	}

	int nLeadingSpaces=0;
	while(yytext[nLeadingSpaces]==' ') nLeadingSpaces++;

	Tof2Scllval.string_type = _strdup(&yytext[nLeadingSpaces]);
	return STRING_LINE;
}

<QuotedString>\"[^\"]+\" {
#ifdef _DEBUG
	OutputDebugString("Quoted String\n");
#endif
	if(bResetSC)
		BEGIN 0;

	/* Match everything except ~~~ , newline*/
	int nLeadingSpaces=0;
	while(yytext[nLeadingSpaces]==' '||yytext[nLeadingSpaces]=='"') nLeadingSpaces++;
	
	int nTrailingSpaces=yyleng-1;
	while(yytext[nTrailingSpaces]==' '||yytext[nTrailingSpaces]=='"') nTrailingSpaces--;
	
	yytext[nTrailingSpaces+1] = '\0';
	Tof2Scllval.string_type = _strdup(&yytext[nLeadingSpaces]);
	return QUOTED_STRING;
}

<*>[^\n] {/*Eat up all unused characters*/}
<*>[\n] {/*Eat up all newlines*/}

%%

